#include "queue.h"
#include "tftp_client.h"
#include "MainComponent.h"
#include "ec_voipglobals.h"
#include "ColorWindow.h"

extern struct Ifacelist *ifaceHead;
extern struct Ifacelist *ifaceTail;

extern int 	structure_size;
extern struct corporate_directory *directory;
extern int targetcount;
int myColor = 0;

/* Mutex variable for accessing live monitor */
extern pthread_mutex_t livemonitor_mutex;		/* Extern from ec_sip.c */
#define LIVEMONITOR_LOCK    do{ pthread_mutex_lock(&livemonitor_mutex); } while(0)
#define LIVEMONITOR_UNLOCK  do{ pthread_mutex_unlock(&livemonitor_mutex); } while(0)

MainPage *mainPage;

char *getCorporateDirectoryValues(int, int);

extern "C" { void getInterfaceList(); }

extern "C" { void startUCSniff(); }

extern "C" { void stopUCSniff(); }

extern "C" { void printToOutput(char *); }

extern "C" { void addToQueue(void(*funcPtr)(void)); }

extern "C" {
	int getNumberOfHosts();
}
extern "C" {
	int getNumberOfActiveCalls();
}
extern "C" {
	char* getHostsList(int, int);
}
extern "C" {
	char* getActiveCalls(int, int);
}

extern "C"{
	char* getTargetTable(int,int);
}

extern "C"{
	int getNumberOfFiles();
}

extern "C"{
	char* getMediaFilesTable(int,int);
}

extern "C"{

	void copyTargets(int);
}

extern "C"{

	void *getActiveCallPtr(int,int,int*);
}

extern "C" {
	
	void newLiveMonitorCall(void *,int);
}

extern "C" {

	void stopMonitoringCall(void *, int);
}

void printToOutput(char *msg){

	mainPage->printToOutputEditor(msg);
}

char *getCorporateDirectoryValues(int rowNumber, int columnID){

	int i = 0;

	for( i = 0; i < structure_size; i++){

		if((i+1) == rowNumber){

			if(columnID == 1)
				return directory[i].name;
			else if(columnID == 2)
				return directory[i].telephone_number;
		}
	}
	return NULL;
}
		

class fileMediaPlayer : public Component, 
						public ButtonListener,
						public ChangeListener,
						public AudioIODeviceCallback,
						public AudioDeviceManager
{

private:

	TextButton* playButton;
	TextButton* stopButton;

	AudioDeviceManager audioDeviceManager;

	// this allows an audio source to be streamed to the IO device
	AudioSourcePlayer audioSourcePlayer;

	// this controls the playback of a positionable audio stream, handling the
	// starting/stopping and sample-rate conversion
	AudioTransportSource transportSource;

	// this is the actual stream that's going to read from the audio file.
	AudioFormatReaderSource* currentAudioFileSource;

	File currentFile;

	String mediaFileName;

public:

	fileMediaPlayer(String fileName){

	mediaFileName = fileName;
		
	addAndMakeVisible (playButton = new TextButton (T("play"),T("click here to play the current audio file")));
        playButton->addButtonListener (this);
        playButton->setColour (TextButton::buttonColourId, Colours::lightgreen);
        playButton->setColour (TextButton::buttonOnColourId, Colours::lightgreen);

	addAndMakeVisible (stopButton = new TextButton (T("stop"),
                                                       T("click here to play the current audio file")));
        stopButton->addButtonListener (this);
        stopButton->setColour (TextButton::buttonColourId, Colours::red);
        stopButton->setColour (TextButton::buttonOnColourId, Colours::red);

	playButton->setBounds (12, 74, 88, 48);
	stopButton->setBounds (224, 74, 88, 48);
	//pauseButton->setBounds(108, 74, 88, 48);
        // register for start/stop messages from the transport source..
        transportSource.addChangeListener (this);

        // and initialise the device manager with no settings so that it picks a
        // default device to use.
        const String error (audioDeviceManager.initialise (1, /* number of input channels */
                                                           2, /* number of output channels */
                                                           0, /* no XML settings.. */
                                                           true  /* select default device on failure */));

        if (error.isNotEmpty())
        {
            AlertWindow::showMessageBox (AlertWindow::WarningIcon,
                                         T("Media Playback"),
                                         T("Couldn't open an output device!\n\n") + error);
        }
        else
        {  
           // ..and connect the mixer to our source player.
            audioSourcePlayer.setSource (&transportSource);

            // start the IO device pulling its data from our callback..
            audioDeviceManager.addAudioCallback (this);
            
        }

		File audioFile (fileName);

        // unload the previous file source and delete it..
        transportSource.stop();
        transportSource.setSource (0);
     
        // create a new file source from the file..

        // get a format manager and set it up with the basic types (wav and aiff).
        AudioFormatManager formatManager;
        formatManager.registerBasicFormats();

        AudioFormatReader* reader = formatManager.createReaderFor (audioFile);

        if (reader != 0)
        {
            currentFile = audioFile;

            currentAudioFileSource = new AudioFormatReaderSource (reader, true);

            // ..and plug it into our transport source
            transportSource.setSource (currentAudioFileSource,
                                       32768, // tells it to buffer this many samples ahead
                                       reader->sampleRate);
        }

        updateButtons();

		setSize(350,150);

	}

	void updateButtons()
    {
        playButton->setEnabled (currentAudioFileSource != 0 && ! transportSource.isPlaying());
        stopButton->setEnabled (transportSource.isPlaying());
        repaint();
    }

	void paint (Graphics& g)
    {
        // print some text to explain what state we're in.

        g.setColour (Colours::black);
        g.setFont (14.0f);

        String s;

        if (transportSource.isPlaying())
            s = T("playing");
        else
            s = T("stopped");

        
        s += T(" - file: \"") + mediaFileName;

        g.drawText (s, 5, 5, getWidth()-25, 24, Justification::centredLeft, true);
    }

	void resized()
    {
             
        //stopButton->setBounds (156, 58, 104, 40);
        
        updateButtons();
    }

	void buttonClicked (Button* button) {

		if (button == playButton)
        {
            transportSource.setPosition (0.0);
            transportSource.start();
        }
        else if (button == stopButton)
        {
            transportSource.stop();
        }
	}

	
	void changeListenerCallback (void*){

		updateButtons();
	}

	
	void audioDeviceIOCallback (const float** inputChannelData,
                                int totalNumInputChannels,
                                float** outputChannelData,
                                int totalNumOutputChannels,
								int numSamples){
									
		audioSourcePlayer.audioDeviceIOCallback (inputChannelData, totalNumInputChannels, outputChannelData, totalNumOutputChannels, numSamples);
	}

	void audioDeviceAboutToStart (AudioIODevice* device){
	
		audioSourcePlayer.audioDeviceAboutToStart (device);
	}

	void audioDeviceStopped(){

		audioSourcePlayer.audioDeviceStopped();
	}

	~fileMediaPlayer(){

        audioDeviceManager.addAudioCallback (0);

        transportSource.removeChangeListener (this);

        transportSource.setSource (0);
        deleteAndZero (currentAudioFileSource);

        audioSourcePlayer.setSource (0);

        deleteAllChildren();

	}
};

class MediaFilesPage : public Component, public TableListBoxModel
{
private:
	TableListBox *table;
	int numRows;

public:

	MediaFilesPage(){

		addAndMakeVisible(table = new TableListBox(T("MediaFiles Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
        table->setOutlineThickness (1);

		table->getHeader()->addColumn ("Media Files",
                                           1,
                                           800,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		setSize(800,670);

		numRows = getNumberOfFiles();
	}

	~MediaFilesPage(){

	}

	void paint(Graphics &g){

		numRows = getNumberOfFiles();
		repaint();
	}

	void resized(){

		table->setBounds(0,2,getWidth(),getHeight() - 2);
	}

	int getNumRows()
    {
        return numRows;
    }

	void paintRowBackground (Graphics& g, int rowNumber, int width, int height, bool rowIsSelected)
    {
        if (rowIsSelected)
            g.fillAll (Colours::lightblue);
    }

	void paintCell (Graphics& g,
                    int rowNumber,
                    int columnId,
                    int width, int height,
                    bool rowIsSelected)
    {
        g.setColour (Colours::black);


        const String text = getMediaFilesTable(rowNumber,columnId);
        g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
        g.setColour (Colours::black.withAlpha (0.2f));
        g.fillRect (width - 1, 0, 1, height);
    }

	void cellDoubleClicked (int rowNumber,int columnId,const MouseEvent &e){

		String text = getMediaFilesTable(rowNumber,columnId);
		DialogWindow::showModalDialog (T("Player"),
                                           new fileMediaPlayer(text),
                                           this,
                                           Colours::azure,
                                           true);
	}


};

class TargetPage : public Component, public TableListBoxModel
{
private:
	TableListBox *table;
	int numRows;

public:

	TargetPage(){

		addAndMakeVisible(table = new TableListBox(T("Target Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
        table->setOutlineThickness (1);


		table->getHeader()->addColumn ("Extension",
                                           1,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Name",
                                           2,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("IP Address",
                                           3,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Protocol",
                                           4,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		numRows = targetcount;
		setSize(800,670);

	}

	TargetPage(String targetedMode){

		addAndMakeVisible(table = new TableListBox(T("Target Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
        table->setOutlineThickness (1);


		table->getHeader()->addColumn ("Extension",
                                           1,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Name",
                                           2,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("IP Address",
                                           3,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Protocol",
                                           4,
                                           200,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		numRows = targetcount;
		setSize(800,500);

	}

	~TargetPage(){

	}

	void paint(Graphics &g){

		numRows = targetcount;
	}

	void resized(){

		table->setBounds(0,2,getWidth(),getHeight() - 2);
	}

	int getNumRows()
    {
        return numRows;
    }

	void paintRowBackground (Graphics& g, int rowNumber, int width, int height, bool rowIsSelected)
    {
		if (rowIsSelected){
            g.fillAll (Colours::lightblue);

			if(target_yes == 1){
				copyTargets(rowNumber);		
			}
		}

    }

	void paintCell (Graphics& g,
                    int rowNumber,
                    int columnId,
                    int width, int height,
                    bool rowIsSelected)
    {
        g.setColour (Colours::black);
        //g.setFont (font);


        //if (rowElement != 0)
        //{
            const String text = getTargetTable(rowNumber,columnId);
	
            g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
        //}

        g.setColour (Colours::black.withAlpha (0.2f));
        g.fillRect (width - 1, 0, 1, height);
    }
};

class ActiveCallsPage : public Component, public TableListBoxModel
{
private:
	TableListBox *table;
	int numRows;

    // This is a custom component containing a combo box, which we're going to put inside
    // our table's "rating" column.
    class MonitorColumnCustomComponent    : public Component,
                                           public ComboBoxListener
    {
    public:
        MonitorColumnCustomComponent (ActiveCallsPage& owner_)
            : owner (owner_)
        {
            // just put a combo box inside this component
            addAndMakeVisible (comboBox = new ComboBox (String::empty));
            comboBox->addItem (T("Start"), 1);
            comboBox->addItem (T("Stop"), 2);

            // when the combo is changed, we'll get a callback.
            comboBox->addListener (this);
            comboBox->setWantsKeyboardFocus (false);
        }

        ~MonitorColumnCustomComponent()
        {
        }

        void resized()
        {
            comboBox->setBoundsInset (BorderSize (2));
        }
		
        void setRowAndColumn (const int newRow, const int newColumn)
        {
            row = newRow;
            columnId = newColumn;
            //comboBox->setSelectedId (owner.getRating (row, columnId), true);
        }
		
        void comboBoxChanged (ComboBox* comboBoxThatHasChanged)
        {
            owner.setRating (row, columnId, comboBox->getSelectedId());
        }

    private:
        ActiveCallsPage& owner;
        ComboBox* comboBox;
        int row, columnId;
    };

public:

	ActiveCallsPage(){

		addAndMakeVisible(table = new TableListBox(T("ActiveCalls Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
        table->setOutlineThickness (1);

		table->getHeader()->addColumn ("Source Username",
                                           1,
                                           133,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Source Extension",
                                           2,
                                           133,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Destination Username",
                                           3,
                                           133,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Destination Extension",
                                           4,
                                           133,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Start Time",
                                           5,
                                           133,
                                           50, 130,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("Duration",
                                           6,
                                           50,
                                           40, 50,
                                           TableHeaderComponent::defaultFlags);


		table->getHeader()->addColumn("Live Monitor",
											7,
											80,
											50, 80,
											TableHeaderComponent::defaultFlags);

		numRows = getNumberOfActiveCalls();
		setSize(800,670);

	}

	~ActiveCallsPage(){

	}

	void paint(Graphics &g){

		numRows = getNumberOfActiveCalls();
	}

	void resized(){

		table->setBounds(0,2,getWidth(),getHeight() - 2);
	}

	int getNumRows()
    {
        return numRows;
    }

	void paintRowBackground (Graphics& g, int rowNumber, int width, int height, bool rowIsSelected)
    {
        if (rowIsSelected)
            g.fillAll (Colours::lightblue);
    }

	// This is overloaded from TableListBoxModel, and must update any custom components that we're using
    Component* refreshComponentForCell (int rowNumber, int columnId, bool isRowSelected,
                                        Component* existingComponentToUpdate)
    {
        if (columnId == 7) // If it's the monitor column, we'll return our custom component..
        {
            MonitorColumnCustomComponent* monitorsBox = (MonitorColumnCustomComponent*) existingComponentToUpdate;

            // If an existing component is being passed-in for updating, we'll re-use it, but
            // if not, we'll have to create one.
            if (monitorsBox == 0)
                monitorsBox = new MonitorColumnCustomComponent (*this);

            monitorsBox->setRowAndColumn (rowNumber, columnId);

            return monitorsBox;
        }
        else
        {
            // for any other column, just return 0, as we'll be painting these columns directly.

            jassert (existingComponentToUpdate == 0);
            return 0;
        }
    }

	// A couple of quick methods to set and get the "rating" value when the user
    // changes the combo box
	/*
    int getRating (const int rowNumber, const int columnId) const
    {
        return dataList->getChildElement (rowNumber)->getIntAttribute (T("Rating"));
    }
	*/

    void setRating (const int rowNumber, const int columnId, const int newRating)
    {
printf("%s:%d newRating = %d\n", __FILE__, __LINE__, newRating);
#ifndef OS_WINDOWS

		if(newRating == 1){
printf("%s:%d starting new live monitor\n", __FILE__, __LINE__);

			int callType = 0;
        	void *callPtr = getActiveCallPtr(rowNumber,columnId, &callType);
        	if(callPtr != NULL){
				
            	LIVEMONITOR_LOCK;

            	newLiveMonitorCall(callPtr,callType);

            	LIVEMONITOR_UNLOCK;
        	}
			else {
				printf("%s:%d cannot get active call\n", __FILE__, __LINE__);
			}

		}
		else if(newRating == 2){

			int callType = 0;
            void *callPtr = getActiveCallPtr(rowNumber,columnId, &callType);

            if(callPtr != NULL){
			
				LIVEMONITOR_LOCK;

				stopMonitoringCall(callPtr,callType);

				LIVEMONITOR_UNLOCK;
			}
		}
#endif

    }

	void paintCell (Graphics& g,
                    int rowNumber,
                    int columnId,
                    int width, int height,
                    bool rowIsSelected)
    {
        g.setColour (Colours::black);
        //g.setFont (font);


        //if (rowElement != 0)
        //{
            const String text = getActiveCalls(rowNumber,columnId);
			

            g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
        //}

        g.setColour (Colours::black.withAlpha (0.2f));
        g.fillRect (width - 1, 0, 1, height);
    }
	/*
	void cellDoubleClicked (int rowNumber,int columnId,const MouseEvent &e){

		int callType = 0;
        void *callPtr = getActiveCallPtr(rowNumber,columnId, &callType);
	
		if(callPtr != NULL){
			
			LIVEMONITOR_LOCK;

			newLiveMonitorCall(callPtr,callType);

			LIVEMONITOR_UNLOCK;
		}
    }
	*/
};


class HostListPage : public Component, public TableListBoxModel
{

private:
	TableListBox *table;
	int numRows;

public:
	HostListPage()
	{
		addAndMakeVisible(table = new TableListBox(T("HostsList Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
        table->setOutlineThickness (1);

		table->getHeader()->addColumn ("IP Address",
                                           1,
                                           400,
                                           50, 400,
                                           TableHeaderComponent::defaultFlags);

		table->getHeader()->addColumn("MAC Address",
                                           2,
                                           400,
                                           50, 400,
                                           TableHeaderComponent::defaultFlags);


		numRows = getNumberOfHosts();
		setSize(800,670);
	}

	~HostListPage(){

	}

	void paint(Graphics &g){

		numRows = getNumberOfHosts();
	}

	void resized(){

		table->setBounds(0,2,getWidth(),getHeight() - 2);
	}

	int getNumRows()
    {
        return numRows;
    }

	void paintRowBackground (Graphics& g, int rowNumber, int width, int height, bool rowIsSelected)
    {
        if (rowIsSelected)
            g.fillAll (Colours::lightblue);
    }

	void paintCell (Graphics& g,
                    int rowNumber,
                    int columnId,
                    int width, int height,
                    bool rowIsSelected)
    {
        g.setColour (Colours::black);
        //g.setFont (font);


        //if (rowElement != 0)
        //{
            const String text = getHostsList(rowNumber,columnId);
			

            g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
        //}

        g.setColour (Colours::black.withAlpha (0.2f));
        g.fillRect (width - 1, 0, 1, height);
    }
};
	

class TempPage  : public Component, public TableListBoxModel
{
public:
    TempPage()
    {
		
		addAndMakeVisible(table = new TableListBox(T("Corporate Table"),this));
		table->setColour (ListBox::outlineColourId, Colours::black);
		table->setOutlineThickness (1);
		table->getHeader()->addColumn ("Name",
                                           1,
                                           390,
                                           50, 400,
                                           TableHeaderComponent::defaultFlags);	
		table->getHeader()->addColumn("Number",
                                           2,
                                           390,
                                           50, 400,
                                           TableHeaderComponent::defaultFlags);

		numRows = structure_size + 1;
		setSize (800, 670);
		
    }

    ~TempPage()
    {
    }

	void paint(Graphics &g){

		numRows = structure_size + 1;
		
		//g.fillAll (Colour (0xff702f2d));
	}

	void resized(){

		table->setBounds(0,2,getWidth(),getHeight() - 2);
	}

	int getNumRows()
    {
        return numRows;
    }

    // This is overloaded from TableListBoxModel, and should fill in the background of the whole row
    void paintRowBackground (Graphics& g, int rowNumber, int width, int height, bool rowIsSelected)
    {
        if (rowIsSelected)
            g.fillAll (Colours::lightblue);
    }

    // This is overloaded from TableListBoxModel, and must paint any cells that aren't using custom
    // components.
    void paintCell (Graphics& g,
                    int rowNumber,
                    int columnId,
                    int width, int height,
                    bool rowIsSelected)
    {
        g.setColour (Colours::black);
        //g.setFont (font);


        //if (rowElement != 0)
        //{
            const String text = getCorporateDirectoryValues(rowNumber,columnId);
            g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
        //}

        g.setColour (Colours::black.withAlpha (0.2f));
        g.fillRect (width - 1, 0, 1, height);
    }


private:
	ComboBox* comboBox;
	TableListBox *table;
	int numRows;
};

MainPage::MainPage(TabbedComponent *p) : quitButton (0),
	startUCSniffButton (0),
	outputEditor (0),
	comboBox (0),
	topLogoimageucsniffdarkblue (0),
	topLogoimageucsniffdarkred (0),
	topLogoimageucsniffdarkgrey (0),
	tb(0),
	themes(0),
	labelInterface(0),
	labelMode(0),
	labelOandStats(0),
	labelStartUCS(0),
	labelStopUCS(0),
	labelEnterMAC(0)
{

	// Constructor for MainPage
	addAndMakeVisible(tb = new ToolbarWorker(this));

	aceSuccess = 0;
	vlanOptionEnabled = 0;
	getVlanIDFromEditor = 0;
	mitmYes = 0;
	ace_yes = 0;
	monitor_yes = 0;
	target_yes = 0;
	
	mainTabbedComponent = p;
	
	backgroundColour = getRandomBrightColour();

	addAndMakeVisible (labelInterface = new Label (T("new label"),
                                                   T("1.  Select Interface")));
	labelInterface->setFont (Font (15.0000f, Font::bold));
	labelInterface->setJustificationType (Justification::centredLeft);
	labelInterface->setEditable (false, false, false);
	labelInterface->setColour (TextEditor::textColourId, Colours::black);
	labelInterface->setColour (TextEditor::backgroundColourId, Colour (0x0));


	    addAndMakeVisible (labelMode = new Label (T("new label"),
                                              T("2.  Select Mode")));
    labelMode->setFont (Font (15.0000f, Font::bold));
    labelMode->setJustificationType (Justification::centredLeft);
    labelMode->setEditable (false, false, false);
    labelMode->setColour (TextEditor::textColourId, Colours::black);
    labelMode->setColour (TextEditor::backgroundColourId, Colour (0x0));

	    addAndMakeVisible (labelOandStats = new Label (T("new label"),
                                                   T("Output and Status")));
    labelOandStats->setFont (Font (15.0000f, Font::bold));
    labelOandStats->setJustificationType (Justification::centred);
    labelOandStats->setEditable (false, false, false);
    labelOandStats->setColour (TextEditor::textColourId, Colours::black);
    labelOandStats->setColour (TextEditor::backgroundColourId, Colour (0x0));



    addAndMakeVisible (labelStartUCS = new Label (T("new label"),
                                                  T("3.  Start UCSniff")));
    labelStartUCS->setFont (Font (15.0000f, Font::bold));
    labelStartUCS->setJustificationType (Justification::centredLeft);
    labelStartUCS->setEditable (false, false, false);
    labelStartUCS->setColour (TextEditor::textColourId, Colours::black);
    labelStartUCS->setColour (TextEditor::backgroundColourId, Colour (0x0));

    addAndMakeVisible (labelStopUCS = new Label (T("new label"),
                                                 T("4.  Stop UCSniff")));
    labelStopUCS->setFont (Font (15.0000f, Font::bold));
    labelStopUCS->setJustificationType (Justification::centredLeft);
    labelStopUCS->setEditable (false, false, false);
    labelStopUCS->setColour (TextEditor::textColourId, Colours::black);
    labelStopUCS->setColour (TextEditor::backgroundColourId, Colour (0x0));


        addChildComponent (aceOptionsGroupComponent = new GroupComponent (T("new group"),
                                                                      T("ACE Options")));
        addChildComponent (macAceEditor = new TextEditor (T("macAceEditor")));
        macAceEditor->setMultiLine (false);
        macAceEditor->setReturnKeyStartsNewLine (false);
        macAceEditor->setReadOnly (false);
        macAceEditor->setScrollbarsShown (true);
        macAceEditor->setCaretVisible (true);
        macAceEditor->setPopupMenuEnabled (true);
        macAceEditor->setText (String::empty);
	macAceEditor->setTooltip  (T("Format Example: 00:50:60:03:88:11"));

        addChildComponent (vlanOptionGroupComponent = new GroupComponent (T("vlan option group component"),
                                                             T("VLAN Options")));
        addChildComponent (spoofCDPButton = new ToggleButton (T("spoof cdp button")));
        spoofCDPButton->setButtonText (T("Spoof CDP"));
        spoofCDPButton->addButtonListener (this);
        spoofCDPButton->setRadioGroupId(2222);

        addChildComponent (sniffCDPButton = new ToggleButton (T("sniff cdp button")));
        sniffCDPButton->setButtonText (T("Sniff CDP"));
        sniffCDPButton->addButtonListener (this);
        sniffCDPButton->setRadioGroupId(2222);

        addChildComponent (specifyVlanIDButton = new ToggleButton (T("specify vlan button")));
        specifyVlanIDButton->setButtonText (T("Specify VLAN ID"));
        specifyVlanIDButton->addButtonListener (this);
        specifyVlanIDButton->setRadioGroupId(2222);

        addChildComponent (vlanEditor = new TextEditor (T("vlan Editor")));
        vlanEditor->setMultiLine (false);
        vlanEditor->setReturnKeyStartsNewLine (false);
        vlanEditor->setReadOnly (false);
        vlanEditor->setScrollbarsShown (true);
        vlanEditor->setCaretVisible (true);
        vlanEditor->setPopupMenuEnabled (true);
        vlanEditor->setText (String::empty);


        addChildComponent(vlanHopGroupComponent = new GroupComponent(T("vlanHop Group Component"),T("VLAN Hop?")));
        addChildComponent (yesVlanButton = new ToggleButton (T("yes Vlan Button")));
        yesVlanButton->setButtonText (T("Yes"));
        yesVlanButton->addButtonListener (this);
        yesVlanButton->setRadioGroupId(1111);

        addChildComponent (noVlanButton = new ToggleButton (T("no vlan button")));
        noVlanButton->setButtonText (T("No"));
        noVlanButton->addButtonListener (this);
        noVlanButton->setRadioGroupId(1111);
        noVlanButton->setToggleState(true,false);

        addChildComponent (aceGroupComponent = new GroupComponent (T("new group"),
                                                               T("ACE?")));
        addChildComponent (yesAceButton = new ToggleButton (T("yesAceButton")));
        yesAceButton->setButtonText (T("Yes"));
        yesAceButton->addButtonListener (this);
        yesAceButton->setRadioGroupId(3333);

        addChildComponent (noAceButton = new ToggleButton (T("noAceButton")));
        noAceButton->setButtonText (T("No"));
        noAceButton->addButtonListener (this);
        noAceButton->setRadioGroupId(3333);
        noAceButton->setToggleState(true,false);


        addChildComponent (mitmModeGroupComponent = new GroupComponent (T("new group"),
                                                                T("MitM Mode")));
        addChildComponent( learningButton = new ToggleButton(T("learningButton")));
        learningButton->setButtonText (T("Learning"));
        learningButton->addButtonListener (this);
        learningButton->setRadioGroupId(5555);

        addChildComponent (targetButton = new ToggleButton (T("targetButton")));
        targetButton->setButtonText (T("Target"));
        targetButton->addButtonListener (this);
        targetButton->setRadioGroupId(5555);

	addAndMakeVisible (startUCSniffButton = new TextButton (T("start ucsniff button"), T("Click here to start UCSniff")));
	startUCSniffButton->setButtonText (T("Start UCSniff"));
	startUCSniffButton->setTooltip	(T("Click here to start UCSniff"));
	startUCSniffButton->addButtonListener (this);

	addAndMakeVisible (clearButton = new TextButton (T("new button")));
	clearButton->setButtonText (T("Clear Screen"));
	clearButton->setTooltip(T("Click here to clear the output & status screen"));
	clearButton->addButtonListener (this);

	addAndMakeVisible (stopUCSniffButton = new TextButton (T("new button")));
	stopUCSniffButton->setButtonText (T("Stop UCSniff"));
	stopUCSniffButton->setTooltip(T("Click here to stop UCSniff"));
	stopUCSniffButton->addButtonListener (this);

	addAndMakeVisible (modeGroupComponent = new GroupComponent (T("new group"),
                                                                T("Mode")));
	addAndMakeVisible (mitmButton = new ToggleButton (T("mitmButton")));
	mitmButton->setButtonText (T("MitM"));
	mitmButton->addButtonListener (this);
	mitmButton->setRadioGroupId(4444);

	addAndMakeVisible (monitorButton = new ToggleButton (T("monitorButton")));
	monitorButton->setButtonText (T("Monitor"));
	monitorButton->addButtonListener (this);
	monitorButton->setRadioGroupId(4444);

	addChildComponent (labelEnterMAC = new Label (T("new label"),
                                                  T("Enter MAC Address")));
	labelEnterMAC->setFont (Font (15.0000f, Font::bold));
	labelEnterMAC->setJustificationType (Justification::centredLeft);
	labelEnterMAC->setEditable (false, false, false);
	labelEnterMAC->setColour (TextEditor::textColourId, Colours::black);
	labelEnterMAC->setColour (TextEditor::backgroundColourId, Colour (0x0));
	labelEnterMAC->setTooltip  (T("Format Example: 00:50:60:03:88:11"));

	getInterfaceList();
	//addToQueue(&getInterfaceList);
	
	addAndMakeVisible (comboBox = new ComboBox (T("interface combo box")));
	comboBox->setEditableText (false);
	comboBox->setJustificationType (Justification::centredLeft);
	comboBox->setTextWhenNothingSelected (String::empty);
	comboBox->setTextWhenNoChoicesAvailable (T("No choices"));
	comboBox->addListener (this);

	addAndMakeVisible (outputEditor = new TextEditor (T("output editor")));
	outputEditor->setMultiLine (true);
	outputEditor->setReturnKeyStartsNewLine (true);
	outputEditor->setReadOnly (false);
	outputEditor->setScrollbarsShown (true);
	outputEditor->setCaretVisible (true);
	outputEditor->setPopupMenuEnabled (true);
	outputEditor->setText (String::empty);

	addAndMakeVisible (miscGroupComponent = new GroupComponent (T("new group"),
                                                                T("Miscellaneous Options")));

	addAndMakeVisible (garpButton = new ToggleButton (T("garpButton")));
	garpButton->setButtonText (T("Enable arp request poisoning"));
	garpButton->setTooltip(T("Enable unicast arp request poisoning"));
	garpButton->addButtonListener (this);

	addAndMakeVisible (sipLogButton = new ToggleButton (T("sipLogButton")));
	sipLogButton->setButtonText (T("Enable SIP log"));
	sipLogButton->addButtonListener (this);

	addAndMakeVisible (verboseButton = new ToggleButton (T("verboseButton")));
	verboseButton->setButtonText (T("Enable Verbose mode"));
	verboseButton->addButtonListener (this);

	addAndMakeVisible (garpdButton = new ToggleButton(T("garpdButton")));
	garpdButton->setButtonText (T("Bypass of GARP Disablement"));
	garpdButton->setTooltip (T("Enable Bypass of GARP Disablement on Cisco Unified IP phones"));
	garpdButton->addButtonListener(this);

	addAndMakeVisible (tftpmButton = new ToggleButton (T("tftpmButton")));
	tftpmButton->setButtonText (T("Enable TFTP Modify Attack"));
	tftpmButton->setTooltip (T("Enable TFTP File Modification on Cisco Unified IP phones"));
	tftpmButton->addButtonListener(this);	

	addAndMakeVisible (mixAvButton = new ToggleButton (T("mixAvButton")));
	mixAvButton->setButtonText (T("Enable Mixing A/V"));
	mixAvButton->setTooltip (T("Enable mixing of audio/video files into AVI file at end of call(s)"));
	mixAvButton->addButtonListener(this);	


	struct Ifacelist *currentiface = ifaceHead;
	int i = 0;
	
	while(currentiface != NULL){

		i += 1;
		comboBox->addItem(currentiface->description,i);

		currentiface = currentiface->next;
	}

	topLogoimageucsniffdarkblue = ImageCache::getFromMemory (ucsniffdarkblue_png, ucsniffdarkblue_pngSize);
	topLogoimageucsniffdarkred = ImageCache::getFromMemory (ucsniffdarkred_png, ucsniffdarkred_pngSize);
	topLogoimageucsniffdarkgrey = ImageCache::getFromMemory (ucsniffdarkgrey_png, ucsniffdarkgrey_pngSize);

	setSize (800, 670);
}

MainPage::~MainPage(){

    deleteAndZero (startUCSniffButton);
    deleteAndZero (outputEditor);
    deleteAndZero (comboBox);
    ImageCache::release (topLogoimageucsniffdarkblue);
    ImageCache::release (topLogoimageucsniffdarkred);
    ImageCache::release (topLogoimageucsniffdarkgrey);
	deleteAndZero (labelInterface);
	deleteAndZero (labelMode);
	deleteAndZero (labelOandStats);
	deleteAndZero (labelStartUCS);
	deleteAndZero (labelStopUCS);
}

MainComponent::MainComponent ()
    : tabbedComponent (0)
{
	
    addAndMakeVisible (tabbedComponent = new TabbedComponent (TabbedButtonBar::TabsAtTop));
    tabbedComponent->setTabBarDepth (30);
    tabbedComponent->addTab (T("Get Started"), Colours::lightgrey, (mainPage = new MainPage(tabbedComponent)), true,0);
    tabbedComponent->setCurrentTabIndex (0);
    
    setSize (800, 670);

}

MainComponent::~MainComponent()
{

    deleteAndZero (tabbedComponent);

}
/*void MainComponent::themeChanged()
{

	repaint();

}*/
void MainPage::themeChanged()
{

	repaint();

}
void MainPage::freeTargetsMemory(){


}
void MainPage::printToOutputEditor(char *msg){

	outputEditor->insertTextAtCaret(msg);
}
/*static const*/Colour MainPage::getRandomBrightColour()
{
        return Colour (Random::getSystemRandom().nextFloat(), 0.1f, 0.97f, 1.0f);
}
void MainPage::paint(Graphics& g){

	if(myColor == 0) {
		g.fillAll(backgroundColour);//getRandomBrightColour());
	} else {
		setBackground(g);
	}
	//printf("myColor:  %d\n",myColor);

	// If Sipera, or default (no theme)
	if ((myColor >= 0) && (myColor < 6)) {
		g.drawImageWithin (topLogoimageucsniffdarkblue,
		proportionOfWidth (0.4198f), proportionOfHeight (0.0488f), proportionOfWidth (0.1481f), proportionOfHeight (0.0519f),
		RectanglePlacement::centred,
		false);
		
		if(myColor == 1) {
			paintFont(T("aceOptWhite"));
		} else if(myColor == 2) {
			paintFont(T("allWhite"));
		} else if (myColor == 5) {
			paintFont(T("whiteLeft"));
		} else {
			paintFont(T("allBlack"));

		}

 	// VIPER VAST chosen	
	} else if ((myColor >= 6) && (myColor < 11)) {

                g.drawImageWithin (topLogoimageucsniffdarkblue,
                proportionOfWidth (0.4198f), proportionOfHeight (0.0488f), proportionOfWidth (0.1481f), proportionOfHeight (0.0519f),
                RectanglePlacement::centred,
                false);

		if(myColor == 9) {
			paintFont(T("allWhite"));
		} else if (myColor == 6) {
			paintFont(T("whiteLeft"));
		} else if (myColor == 7) {
			paintFont(T("aceOptWhite"));
		} else {
			paintFont(T("allBlack"));
		}


	// If LAVA chosen
	} else if ((myColor >= 11) && (myColor < 16)) {
                g.drawImageWithin (topLogoimageucsniffdarkred,
                proportionOfWidth (0.4198f), proportionOfHeight (0.0488f), proportionOfWidth (0.1481f), proportionOfHeight (0.0519f),
                RectanglePlacement::centred,
                false);

		paintFont(T("allWhite"));

	// Classic theme chosen
	} else {
                g.drawImageWithin (topLogoimageucsniffdarkgrey,
                proportionOfWidth (0.4198f), proportionOfHeight (0.0488f), proportionOfWidth (0.1481f), proportionOfHeight (0.0519f),
                RectanglePlacement::centred,
                false);

		if((myColor == 16) || (myColor == 19)) {
			//Theme is black
	                g.drawImageWithin (topLogoimageucsniffdarkred,
			proportionOfWidth (0.4198f), proportionOfHeight (0.0488f), proportionOfWidth (0.1481f), proportionOfHeight (0.0519f),
			RectanglePlacement::centred,
			false);
			paintFont(T("allWhite"));
		} else if (myColor == 18) {
			paintFont(T("whiteLeft"));
		} else if (myColor == 20) {
			paintFont(T("whiteLeft"));
		} else {
			paintFont(T("allBlack"));
		}
	} 
				   
	if(ace_yes == 1){

		labelEnterMAC->setVisible(true);
		resized();
	}

}
void MainPage::paintFont(String color)
{

	if(color.compare(T("allWhite")) == 0) {

		modeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        modeGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        mitmModeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        mitmModeGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        miscGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        miscGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        aceGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        aceGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        aceOptionsGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        aceOptionsGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        vlanHopGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        vlanHopGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        vlanOptionGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        vlanOptionGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        labelEnterMAC->setColour (Label::textColourId, Colours::white);
			labelInterface->setColour(Label::textColourId, Colours::white);
			labelMode->setColour(Label::textColourId, Colours::white);
			labelOandStats->setColour(Label::textColourId, Colours::white);
			labelStartUCS->setColour(Label::textColourId, Colours::white);
			labelStopUCS->setColour(Label::textColourId, Colours::white);
                        mitmButton->setColour (ToggleButton::textColourId, Colours::white);
                        learningButton->setColour (ToggleButton::textColourId, Colours::white);
                        targetButton->setColour (ToggleButton::textColourId, Colours::white);
                        monitorButton->setColour (ToggleButton::textColourId, Colours::white);
                        garpButton->setColour (ToggleButton::textColourId, Colours::white);
                        sipLogButton->setColour (ToggleButton::textColourId, Colours::white);
                        garpdButton->setColour (ToggleButton::textColourId, Colours::white);
                        tftpmButton->setColour (ToggleButton::textColourId, Colours::white);
                        mixAvButton->setColour (ToggleButton::textColourId, Colours::white);
                        yesAceButton->setColour (ToggleButton::textColourId, Colours::white);
                        noAceButton->setColour (ToggleButton::textColourId, Colours::white);
                        yesVlanButton->setColour (ToggleButton::textColourId, Colours::white);
                        noVlanButton->setColour (ToggleButton::textColourId, Colours::white);
                        spoofCDPButton->setColour (ToggleButton::textColourId, Colours::white);
                        sniffCDPButton->setColour (ToggleButton::textColourId, Colours::white);
                        specifyVlanIDButton->setColour (ToggleButton::textColourId, Colours::white);
                        verboseButton->setColour (ToggleButton::textColourId, Colours::white);

	} else if ( color.compare(T("allBlack")) == 0) {

		modeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                modeGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        mitmModeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        mitmModeGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        miscGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        miscGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        aceGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        aceGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        aceOptionsGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        aceOptionsGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        vlanHopGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanHopGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        labelEnterMAC->setColour (Label::textColourId, Colours::black);
                        labelInterface->setColour(Label::textColourId, Colours::black);
                        labelMode->setColour(Label::textColourId, Colours::black);
                        labelOandStats->setColour(Label::textColourId, Colours::black);
                        labelStartUCS->setColour(Label::textColourId, Colours::black);
                        labelStopUCS->setColour(Label::textColourId, Colours::black);
                        mitmButton->setColour (ToggleButton::textColourId, Colours::black);
                        learningButton->setColour (ToggleButton::textColourId, Colours::black);
                        targetButton->setColour (ToggleButton::textColourId, Colours::black);
                        monitorButton->setColour (ToggleButton::textColourId, Colours::black);
                        garpButton->setColour (ToggleButton::textColourId, Colours::black);
                        sipLogButton->setColour (ToggleButton::textColourId, Colours::black);
                        garpdButton->setColour (ToggleButton::textColourId, Colours::black);
                        tftpmButton->setColour (ToggleButton::textColourId, Colours::black);
                        mixAvButton->setColour (ToggleButton::textColourId, Colours::black);
                        yesAceButton->setColour (ToggleButton::textColourId, Colours::black);
                        noAceButton->setColour (ToggleButton::textColourId, Colours::black);
                        yesVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        noVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        spoofCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        sniffCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        specifyVlanIDButton->setColour (ToggleButton::textColourId, Colours::black);
                        verboseButton->setColour (ToggleButton::textColourId, Colours::black);

	} else if ( color.compare(T("whiteLeft")) == 0) {

	                modeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        modeGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        mitmModeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        mitmModeGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        miscGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        miscGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        aceGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        aceGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        aceOptionsGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        aceOptionsGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        vlanHopGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanHopGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        labelEnterMAC->setColour (Label::textColourId, Colours::black);
                        labelInterface->setColour(Label::textColourId, Colours::white);
                        labelMode->setColour(Label::textColourId, Colours::white);
                        labelOandStats->setColour(Label::textColourId, Colours::black);
                        labelStartUCS->setColour(Label::textColourId, Colours::white);
                        labelStopUCS->setColour(Label::textColourId, Colours::white);
                        mitmButton->setColour (ToggleButton::textColourId, Colours::white);
                        learningButton->setColour (ToggleButton::textColourId, Colours::white);
                        targetButton->setColour (ToggleButton::textColourId, Colours::white);
                        monitorButton->setColour (ToggleButton::textColourId, Colours::white);
                        garpButton->setColour (ToggleButton::textColourId, Colours::white);
                        sipLogButton->setColour (ToggleButton::textColourId, Colours::white);
                        garpdButton->setColour (ToggleButton::textColourId, Colours::white);
                        tftpmButton->setColour (ToggleButton::textColourId, Colours::white);
                        mixAvButton->setColour (ToggleButton::textColourId, Colours::white);
                        yesAceButton->setColour (ToggleButton::textColourId, Colours::black);
                        noAceButton->setColour (ToggleButton::textColourId, Colours::black);
                        yesVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        noVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        spoofCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        sniffCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        specifyVlanIDButton->setColour (ToggleButton::textColourId, Colours::black);
                        verboseButton->setColour (ToggleButton::textColourId, Colours::white);

	} else if ( color.compare(T("aceOptWhite")) == 0) {

	                modeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
       		        modeGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        mitmModeGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        mitmModeGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        miscGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        miscGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        aceGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        aceGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        aceOptionsGroupComponent->setColour(GroupComponent::outlineColourId,Colours::white);
                        aceOptionsGroupComponent->setColour(GroupComponent::textColourId,Colours::white);
                        vlanHopGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanHopGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::outlineColourId,Colours::black);
                        vlanOptionGroupComponent->setColour(GroupComponent::textColourId,Colours::black);
                        labelEnterMAC->setColour (Label::textColourId, Colours::white);
                        labelInterface->setColour(Label::textColourId, Colours::black);
                        labelMode->setColour(Label::textColourId, Colours::black);
                        labelOandStats->setColour(Label::textColourId, Colours::black);
                        labelStartUCS->setColour(Label::textColourId, Colours::black);
                        labelStopUCS->setColour(Label::textColourId, Colours::black);
                        mitmButton->setColour (ToggleButton::textColourId, Colours::black);
                        learningButton->setColour (ToggleButton::textColourId, Colours::black);
                        targetButton->setColour (ToggleButton::textColourId, Colours::black);
                        monitorButton->setColour (ToggleButton::textColourId, Colours::black);
                        garpButton->setColour (ToggleButton::textColourId, Colours::black);
                        sipLogButton->setColour (ToggleButton::textColourId, Colours::black);
                        garpdButton->setColour (ToggleButton::textColourId, Colours::black);
                        tftpmButton->setColour (ToggleButton::textColourId, Colours::black);
                        mixAvButton->setColour (ToggleButton::textColourId, Colours::black);
                        yesAceButton->setColour (ToggleButton::textColourId, Colours::white);
                        noAceButton->setColour (ToggleButton::textColourId, Colours::white);
                        yesVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        noVlanButton->setColour (ToggleButton::textColourId, Colours::black);
                        spoofCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        sniffCDPButton->setColour (ToggleButton::textColourId, Colours::black);
                        specifyVlanIDButton->setColour (ToggleButton::textColourId, Colours::black);
                        verboseButton->setColour (ToggleButton::textColourId, Colours::black);

	}

}


void MainComponent::paint (Graphics& g)
{
	//setBackground(g);

  	if(ucsniffInstantCtr == 0){

		if(aceSuccess == 1){
			tabbedComponent->addTab (T("Directory List"), Colours::lightgrey, new TempPage(), true);
			aceSuccess = 0;
		}
	}
	
	if(ucsniffInstantCtr == 1){

		if(mitmSuccess == 1){

			tabbedComponent->addTab (T("Hosts List"), Colours::lightgrey, new HostListPage(), true);
			tabbedComponent->addTab (T("Active Calls"), Colours::lightgrey, new ActiveCallsPage(), true);
			tabbedComponent->addTab (T("Media Files"), Colours::lightgrey, new MediaFilesPage(), true);
			tabbedComponent->addTab (T("Targets"), Colours::lightgrey, new TargetPage(), true);
			mitmSuccess = 0;
		}

	}

}

void MainPage::removeAceOptionGroup(){

	aceOptionsGroupComponent->setVisible(false);
	macAceEditor->setVisible(false);
	labelEnterMAC->setVisible(false);
	ace_yes = 0;
	mac_yes = 0;
}

void MainPage::removeAce(){

	aceGroupComponent->setVisible(false);
	yesAceButton->setVisible(false);
	noAceButton->setVisible(false);
	if(ace_yes){
		removeAceOptionGroup();
	}
}
void MainPage::removeVlan(){

	vlanHopGroupComponent->setVisible(false);
	yesVlanButton->setVisible(false);;
	noVlanButton->setVisible(false);;
	if(vlanOptionEnabled){
		removeVlanOptionGroup();
	}
}

void MainPage::removeVlanOptionGroup(){

	vlanOptionGroupComponent->setVisible(false);
	spoofCDPButton->setVisible(false);
	sniffCDPButton->setVisible(false);
	specifyVlanIDButton->setVisible(false);
	vlanEditor->setVisible(false);
	vlanOptionEnabled = 0;
	cdp_mode_yes = 0;
	cdp_mode_type = 0;
	vlan_yes = 0;
	getVlanIDFromEditor = 0;
}

void MainPage::removeMitmMode(){

	//deleteAndZero(mitmModeGroupComponent);
	mitmModeGroupComponent->setVisible(false);;
	//deleteAndZero(targetButton);
	targetButton->setVisible(false);;
	//deleteAndZero(learningButton);
	learningButton->setVisible(false);;
}

void MainPage::displayMitmMode(){

	mitmModeGroupComponent->setVisible(true);
	learningButton->setVisible(true);
	targetButton->setVisible(true);
}
void MainPage::displayAceAndVlan(){

	vlanHopGroupComponent->setVisible(true);
	aceGroupComponent->setVisible(true);
	yesVlanButton->setVisible(true);
	noVlanButton->setVisible(true);
	yesAceButton->setVisible(true);
	noAceButton->setVisible(true);
}

void MainPage::displayAceOptions(){

	aceOptionsGroupComponent->setVisible(true);
	macAceEditor->setVisible(true);

}

void MainPage::displayVlanHopOptions(){

	vlanOptionGroupComponent->setVisible(true);
	spoofCDPButton->setVisible(true);
	sniffCDPButton->setVisible(true);
	specifyVlanIDButton->setVisible(true);
	vlanEditor->setVisible(true);

}

void MainPage::resized(){

	startUCSniffButton->setBounds (proportionOfWidth (0.0432f), proportionOfHeight (0.8244f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	stopUCSniffButton->setBounds(proportionOfWidth (0.0432f), proportionOfHeight (0.9374f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	// comboBox is for the interface selection
	comboBox->setBounds (proportionOfWidth (0.0329f), proportionOfHeight (0.0977f), proportionOfWidth (0.2716f), proportionOfHeight (0.0488f));	
	clearButton->setBounds (proportionOfWidth (0.6173f), proportionOfHeight (0.9527f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	modeGroupComponent->setBounds (proportionOfWidth (0.0247f), proportionOfHeight (0.2076f), proportionOfWidth (0.2963f), proportionOfHeight (0.1084f));
	mitmButton->setBounds (proportionOfWidth (0.0494f), proportionOfHeight (0.2443f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	monitorButton->setBounds (proportionOfWidth (0.1975f), proportionOfHeight (0.2443f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	//miscGroupComponent->setBounds (proportionOfWidth (0.0247f), proportionOfHeight (0.4397f), proportionOfWidth (0.3621f), proportionOfHeight (0.3084f));
	miscGroupComponent->setBounds (proportionOfWidth (0.0247f), proportionOfHeight (0.4397f), proportionOfWidth (0.2963f), proportionOfHeight (0.3084f));

	garpButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.4763f), proportionOfWidth (0.2716f), proportionOfHeight (0.0391f));
	sipLogButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.5154f), proportionOfWidth (0.2685f), proportionOfHeight (0.0391f));
	verboseButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.5545f), proportionOfWidth (0.2685f), proportionOfHeight (0.0391f));   
	garpdButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.5936f), proportionOfWidth (0.2685f), proportionOfHeight (0.0391f));
	tftpmButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.6327f), proportionOfWidth (0.2685f), proportionOfHeight (0.0391f));
	mixAvButton->setBounds (proportionOfWidth (0.0476f), proportionOfHeight (0.6718f), proportionOfWidth (0.2685f), proportionOfHeight (0.0391f));

	//outputEditor->setBounds (proportionOfWidth (0.4115f), proportionOfHeight (0.4641f), proportionOfWidth (0.5679f), proportionOfHeight (0.4763f));
	outputEditor->setBounds (proportionOfWidth (0.3374f), proportionOfHeight (0.4397f), proportionOfWidth (0.6420f), proportionOfHeight (0.5008f));
	labelEnterMAC->setBounds (proportionOfWidth (0.7490f), proportionOfHeight (0.2809f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	mitmModeGroupComponent->setBounds(proportionOfWidth (0.0247f), proportionOfHeight (0.3298f), proportionOfWidth (0.2963f), proportionOfHeight (0.0977f));
	learningButton->setBounds(proportionOfWidth (0.0494f), proportionOfHeight (0.3664f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	targetButton->setBounds (proportionOfWidth (0.1975f), proportionOfHeight (0.3664f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	aceGroupComponent->setBounds (proportionOfWidth (0.7078f), proportionOfHeight (0.1099f), proportionOfWidth (0.2058f), proportionOfHeight (0.0977f));
	yesAceButton->setBounds (proportionOfWidth (0.7325f), proportionOfHeight (0.1466f), proportionOfWidth (0.0823f), proportionOfHeight (0.0366f));
	noAceButton->setBounds (proportionOfWidth (0.8231f), proportionOfHeight (0.1466f), proportionOfWidth (0.0730f), proportionOfHeight (0.0366f));
	yesVlanButton->setBounds (proportionOfWidth (0.4609f), proportionOfHeight (0.1466f), proportionOfWidth (0.0576f), proportionOfHeight (0.0366f));
	noVlanButton->setBounds (proportionOfWidth (0.5432f), proportionOfHeight (0.1466f), proportionOfWidth (0.0730f), proportionOfHeight (0.0366f));
	vlanHopGroupComponent->setBounds (proportionOfWidth (0.4362f), proportionOfHeight (0.1099f), proportionOfWidth (0.2058f), proportionOfHeight (0.0977f));
	vlanOptionGroupComponent->setBounds (proportionOfWidth (0.4115f), proportionOfHeight (0.2565f), proportionOfWidth (0.2634f), proportionOfHeight (0.1466f));
	spoofCDPButton->setBounds (proportionOfWidth (0.4280f), proportionOfHeight (0.2931f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));

	sniffCDPButton->setBounds (proportionOfWidth (0.5514f), proportionOfHeight (0.2931f), proportionOfWidth (0.0988f), proportionOfHeight (0.0366f));
	specifyVlanIDButton->setBounds (proportionOfWidth (0.4280f), proportionOfHeight (0.3542f), proportionOfWidth (0.1317f), proportionOfHeight (0.0366f));
	vlanEditor->setBounds (proportionOfWidth (0.5597f), proportionOfHeight (0.3542f), proportionOfWidth (0.0988f), proportionOfHeight (0.0366f));
	aceOptionsGroupComponent->setBounds (proportionOfWidth (0.7243f), proportionOfHeight (0.2443f), proportionOfWidth (0.2058f), proportionOfHeight (0.1466f));

	macAceEditor->setBounds (proportionOfWidth (0.7490f), proportionOfHeight (0.3298f), proportionOfWidth (0.1646f), proportionOfHeight (0.0366f));
	//tb->setBounds(0,0,getWidth(),30);
	tb->setBounds (proportionOfWidth (0.0000f), proportionOfHeight (0.0000f), proportionOfWidth (1.0000f), proportionOfHeight (0.0458f));
	labelInterface->setBounds (proportionOfWidth (0.0411f), proportionOfHeight (0.0488f), proportionOfWidth (0.1811f), proportionOfHeight (0.0488f));
	labelMode->setBounds (proportionOfWidth (0.0411f), proportionOfHeight (0.1588f), proportionOfWidth (0.1811f), proportionOfHeight (0.0366f));
	//labelOandStats->setBounds (proportionOfWidth (0.5597f), proportionOfHeight (0.4153f), proportionOfWidth (0.2685f), proportionOfHeight (0.0488f));labelOandStats->setBounds (proportionOfWidth (0.5185f), proportionOfHeight (0.3908f), proportionOfWidth (0.2685f), proportionOfHeight (0.0488f));
	labelOandStats->setBounds (proportionOfWidth (0.5185f), proportionOfHeight (0.3908f), proportionOfWidth (0.2685f), proportionOfHeight (0.0488f));
	labelStartUCS->setBounds (proportionOfWidth (0.0432f), proportionOfHeight (0.7603f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));
	labelStopUCS->setBounds (proportionOfWidth (0.0432f), proportionOfHeight (0.8885f), proportionOfWidth (0.1543f), proportionOfHeight (0.0366f));


}

void MainComponent::popupMenuClickOnTab(const int newCurrentTabIndex, const String &newCurrentTabName){

	printf("Tab Index is %d \n", newCurrentTabIndex);
}

void MainComponent::currentTabChanged(const int newCurrentTabIndex, const String &newCurrentTabName){

	printf("Tab Index is %d\n",newCurrentTabIndex);

	if(newCurrentTabIndex == 0){

		printf("Get Started Tab\n");
	}
	else if(newCurrentTabIndex == 1){
		printf("Directory list tab \n");
	}
}


void MainComponent::resized()
{
    tabbedComponent->setBounds (0, 0, proportionOfWidth (1.0000f), proportionOfHeight (1.0000f));
}

void MainPage::buttonClicked (Button* buttonThatWasClicked)
{
    //[UserbuttonClicked_Pre]
    //[/UserbuttonClicked_Pre]


	/*	
    if (buttonThatWasClicked == quitButton)
    {
        //[UserButtonCode_quitButton] -- add your button handler code here..

        JUCEApplication::quit();

        //[/UserButtonCode_quitButton]
    }
	*/

    if (buttonThatWasClicked == startUCSniffButton)
    {
        //[UserButtonCode_textButton] -- add your button handler code here..
        //[/UserButtonCode_textButton]
		
		if(getVlanIDFromEditor == 1 && vlan_yes == 1){
			vlan_id = (vlanEditor->getText()).getIntValue();
			
		}
		if(ace_yes == 1){

			String inputText = macAceEditor->getText();
			//macString = inputText;
			char *macstr = new char[(inputText.length()) + 1];
			strncpy(macstr,inputText,inputText.length());
			macString = macstr;
			mac_yes = 1;
		}

		//runAce();
		addToQueue(&startUCSniff);

		/*
		if(ace_yes == 1){
			mainTabbedComponent->addTab (T("Directory List"), Colours::lightgrey, new TempPage(), true);
			//aceSuccess = 0;
		}

		if(monitor_yes != 1){

			mainTabbedComponent->addTab (T("Hosts List"), Colours::lightgrey, new HostListPage(), true);
			mainTabbedComponent->addTab (T("Active Calls"), Colours::lightgrey, new ActiveCallsPage(), true);
			mainTabbedComponent->addTab (T("Media Files"), Colours::lightgrey, new MediaFilesPage(), true);
			mainTabbedComponent->addTab (T("Targets"), Colours::lightgrey, new TargetPage(), true);
			//mitmSuccess = 0;
		}
		*/
		
    }

	else if(buttonThatWasClicked == stopUCSniffButton){

		addToQueue(&stopUCSniff);
	}

	
	else if(buttonThatWasClicked == clearButton){
		outputEditor->clear();
	}

	else if(buttonThatWasClicked == yesVlanButton){

		if(vlanOptionEnabled != 1){
			MainPage::displayVlanHopOptions();
			vlanOptionEnabled = 1;
		}

	}

	else if(buttonThatWasClicked == noVlanButton){

		if(vlanOptionEnabled == 1){

			removeVlanOptionGroup();
		}
	}

	else if(buttonThatWasClicked == spoofCDPButton){

		cdp_mode_yes = 1;
		cdp_mode_type = 1;
		getVlanIDFromEditor = 0;
	}

	else if(buttonThatWasClicked == sniffCDPButton){

		cdp_mode_yes = 1;
		cdp_mode_type = 0;
		getVlanIDFromEditor = 0;
	}

	else if(buttonThatWasClicked == specifyVlanIDButton){

		vlan_yes = 1;
		getVlanIDFromEditor = 1;
	}

	else if(buttonThatWasClicked == mitmButton){

		if(mitmYes != 1){
			displayMitmMode();
			displayAceAndVlan();
			mitmYes = 1;
		}
		target_yes = 0;
		monitor_yes = 0;
		
	}

	else if(buttonThatWasClicked == monitorButton){

		if(mitmYes == 1){
			removeVlan();
			removeAce();
			removeMitmMode();
			mitmYes = 0;
		}
		monitor_yes = 1;
		target_yes = 0;
		learning_yes = 0;
	}

	else if(buttonThatWasClicked == learningButton){

		learning_yes = 1;
		monitor_yes = 0;
		target_yes = 0;

	}

	else if(buttonThatWasClicked == targetButton){

		target_yes = 1;
		monitor_yes = 0;
		learning_yes = 0;
		
		DialogWindow::showModalDialog (T("Targeted Users"),
                                           new TargetPage("Targeted Mode"),
                                           this,
                                           Colours::azure,
                                           true);
		
	}

	else if(buttonThatWasClicked == yesAceButton){
		
		if(ace_yes != 1){
			displayAceOptions();
			ace_yes = 1;
		}
	}
	else if(buttonThatWasClicked == noAceButton){

		if(ace_yes == 1){
			
			removeAceOptionGroup();
		}
	}
	else if(buttonThatWasClicked == sipLogButton){

		if(voipdebug == 1){

			voipdebug = 0;
		}
		else{

			voipdebug = 1;
		}
	}

	else if(buttonThatWasClicked == garpButton){

		if(garpb_yes  == 1){

			garpb_yes = 0;
		}
		else{
			garpb_yes = 1;
		}
	}
	else if(buttonThatWasClicked == garpdButton){
		
		if(garpdb_yes == 1){
			garpdb_yes = 0;
		}
		else{
			garpdb_yes = 1;
		}

	}
	else if(buttonThatWasClicked == tftpmButton){

		if(tftp_modify_yes == 1){

			tftp_modify_yes = 0;
		}
		else{
			tftp_modify_yes = 1;
		}
	}
	else if(buttonThatWasClicked == mixAvButton){

		if(mixAv_modify_yes == 1){

			mixAv_modify_yes = 0;
		}
		else{
			mixAv_modify_yes = 1;
		}
	}
}
void MainPage::comboBoxChanged (ComboBox* comboBoxThatHasChanged)
{
    if (comboBoxThatHasChanged == comboBox)
    {
		ifaceType = comboBox->getSelectedId();
    }

}

// JUCER_RESOURCE: viperlablogosm_gif, 1050, "..\..\VIPER\New Branding\new graphics\viperlab-logo-sm.gif"
static const unsigned char resource_MainComponent_viperlablogosm_gif[] = { 71,73,70,56,57,97,184,0,22,0,213,34,0,181,182,185,211,226,234,192,208,215,240,244,246,208,220,226,16,75,105,160,183,195,64,111,
135,32,87,115,48,99,125,128,159,175,96,135,155,224,232,236,245,248,250,189,211,223,144,171,185,112,147,165,89,145,173,80,123,145,122,167,190,156,189,206,178,204,217,145,182,201,233,241,244,111,160,184,
176,196,205,167,197,212,134,175,195,222,233,239,100,153,179,200,219,228,78,138,168,0,63,95,156,158,161,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,249,4,1,0,0,34,0,44,0,0,0,0,184,0,22,0,0,6,255,64,145,112,72,44,26,133,134,68,97,112,108,58,139,16,137,225,73,
173,90,175,216,172,118,203,21,17,16,160,240,180,123,28,20,194,8,1,121,205,110,187,223,66,194,57,12,58,192,69,6,122,56,51,92,208,153,119,84,13,31,132,22,100,14,132,31,14,129,140,68,18,122,98,111,102,144,
5,67,115,9,141,79,136,132,139,93,22,137,13,153,141,7,144,32,8,128,108,10,165,32,4,94,116,15,162,77,159,132,161,93,17,132,24,177,140,126,165,16,109,4,171,32,186,86,183,31,19,100,1,137,21,195,119,192,171,
106,107,9,171,11,66,96,166,34,16,193,97,9,99,34,10,210,171,7,173,15,116,16,214,97,3,201,132,203,70,27,137,239,132,29,20,66,20,137,28,197,137,24,157,177,0,253,254,255,0,253,13,201,83,234,20,25,85,5,153,
48,40,39,226,92,48,59,34,72,105,3,161,64,196,163,85,152,234,17,186,112,4,158,199,15,28,68,96,136,167,14,222,177,33,1,83,170,92,217,47,132,203,151,48,99,190,36,66,16,18,53,46,206,32,37,0,68,46,140,128,
255,156,218,236,212,212,166,96,128,54,88,29,226,29,241,240,17,94,131,11,137,40,104,52,73,68,166,213,171,88,179,102,45,34,167,84,55,44,3,28,134,145,128,74,98,165,158,32,160,137,96,128,144,162,197,87,173,
214,242,2,145,33,3,157,5,2,242,230,29,0,149,208,60,35,23,2,8,30,44,56,81,7,17,21,18,121,224,48,88,67,212,170,90,35,75,150,108,132,1,56,58,5,226,98,153,75,167,226,16,163,97,168,129,171,68,164,45,1,208,
32,36,176,85,192,90,162,176,185,12,140,36,38,20,224,72,3,15,26,114,231,118,208,247,195,60,119,31,34,20,153,253,161,246,144,201,200,147,199,44,195,25,196,78,44,67,65,20,248,138,135,142,1,212,55,135,128,
67,32,194,174,152,182,122,36,136,184,116,100,2,33,225,70,60,224,83,70,60,192,32,66,27,138,140,12,94,68,185,125,228,78,208,134,190,2,212,185,230,62,127,212,244,213,66,251,205,53,192,101,122,24,144,19,44,
69,188,247,129,33,70,172,151,200,5,230,209,199,20,39,68,56,8,225,113,247,117,184,255,149,19,2,204,17,6,131,79,132,21,30,42,68,204,97,135,129,52,209,193,7,121,7,196,40,99,140,12,100,19,198,127,66,108,162,
136,17,58,118,48,193,143,27,104,40,194,44,31,212,146,99,34,251,8,225,225,146,86,81,49,128,107,172,80,1,37,137,69,8,240,202,120,97,64,36,132,137,232,228,228,139,19,219,29,65,164,145,67,140,73,132,142,139,
20,147,203,16,13,204,87,100,125,76,198,233,146,21,109,21,16,91,19,54,250,231,68,158,12,88,57,226,69,144,136,7,222,42,149,16,8,194,151,69,72,152,200,49,138,122,212,0,7,77,189,19,31,156,114,50,201,31,56,
207,25,81,19,4,40,26,97,13,119,124,138,168,135,26,131,150,66,13,90,124,20,81,146,71,203,68,186,168,8,83,69,106,28,100,149,46,153,5,66,217,13,209,85,26,84,24,234,203,167,222,233,129,192,24,93,105,51,197,
69,164,21,17,43,60,33,1,23,233,50,110,126,20,65,146,28,214,58,83,44,4,144,226,217,150,5,20,176,109,137,122,13,224,107,159,122,169,69,4,185,229,134,235,55,69,94,56,10,193,24,97,132,177,9,47,188,161,204,
75,24,71,204,228,139,196,176,91,38,176,192,157,90,160,101,174,190,4,23,28,72,92,2,0,188,133,89,6,55,236,240,195,69,96,7,241,196,107,4,1,0,59,0,0};

const char* MainPage::viperlablogosm_gif = (const char*) resource_MainComponent_viperlablogosm_gif;
const int MainPage::viperlablogosm_gifSize = 1050;

// JUCER_RESOURCE: viperlablogomed_jpg, 14919, "..\..\VIPER\New Branding\new graphics\viperlab-logo-med.jpg"
static const unsigned char resource_MainComponent_viperlablogomed_jpg[] = { 255,216,255,224,0,16,74,70,73,70,0,1,2,0,0,100,0,100,0,0,255,236,0,17,68,117,99,107,121,0,1,0,4,0,0,0,100,0,0,255,238,0,14,65,
100,111,98,101,0,100,192,0,0,0,1,255,219,0,132,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,2,1,1,2,2,
2,1,2,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,255,192,0,17,8,0,46,1,127,3,1,17,0,2,17,1,3,17,1,255,196,0,198,0,1,0,2,3,1,0,3,1,0,0,0,0,0,0,0,
0,0,9,10,6,7,8,11,1,3,5,4,1,1,0,2,2,3,1,0,0,0,0,0,0,0,0,0,0,0,6,7,4,8,1,3,5,2,16,0,0,6,1,2,4,1,8,5,7,7,13,1,0,0,0,1,2,3,4,5,6,7,0,8,17,18,19,9,20,33,49,34,21,22,150,87,10,84,213,214,23,25,97,50,35,147,
117,151,26,65,81,180,53,182,215,88,98,36,148,37,38,54,118,55,119,183,24,56,216,89,17,0,2,0,5,2,2,6,5,7,10,4,5,5,0,0,0,1,2,0,17,3,4,5,18,6,33,49,65,81,97,34,19,7,113,129,50,21,23,145,161,66,210,147,20,
84,177,82,98,114,35,51,83,163,211,22,146,162,212,85,130,195,36,37,38,193,209,225,67,131,255,218,0,12,3,1,0,2,17,3,17,0,63,0,191,198,144,134,144,134,144,134,144,134,144,136,23,239,91,222,73,14,219,21,250,
174,43,197,21,200,91,182,231,178,157,113,205,158,9,189,152,93,26,167,140,232,222,177,125,6,214,251,98,141,104,118,206,172,175,102,102,162,158,180,137,142,77,195,116,76,179,23,11,185,84,19,64,141,158,79,
246,70,203,59,154,163,221,221,179,38,46,147,105,58,125,167,105,3,160,31,162,0,32,177,145,230,0,19,51,90,191,204,127,48,134,206,164,150,54,40,181,51,85,208,178,234,158,154,105,50,161,216,9,22,36,130,21,
102,4,212,146,100,52,181,39,160,59,178,111,90,75,118,24,115,116,153,151,61,100,236,170,251,22,101,74,222,65,26,51,219,59,152,122,27,136,166,18,73,123,75,87,130,161,66,12,101,30,176,202,205,91,85,212,114,
190,9,131,113,233,56,17,17,19,122,90,187,234,109,28,26,226,43,98,172,173,233,82,90,180,153,53,5,155,204,142,235,23,51,102,42,210,110,36,242,141,112,165,190,247,27,231,109,243,121,27,170,213,218,133,117,
169,160,180,146,64,247,149,80,73,20,50,205,76,148,112,49,234,31,140,50,77,59,49,227,138,38,88,199,147,8,207,209,114,77,74,191,119,168,204,183,242,39,35,94,179,69,182,151,138,114,100,248,137,144,88,205,
29,151,168,145,184,29,37,0,196,48,1,138,32,26,177,117,109,90,202,230,165,165,202,233,175,73,202,176,234,42,100,126,120,221,91,43,203,124,133,157,43,251,70,15,107,90,154,186,30,181,96,8,249,143,46,142,
81,157,107,162,50,161,164,33,164,33,164,33,164,33,164,34,174,191,51,102,252,166,112,14,17,196,219,101,196,151,203,13,51,50,229,219,84,126,75,159,153,165,216,36,171,150,106,174,46,199,239,150,52,83,132,
229,161,158,48,151,138,94,223,144,81,64,25,170,138,156,20,78,9,241,13,195,209,227,106,121,95,128,76,133,245,108,165,229,53,123,42,40,80,6,0,171,59,142,60,8,32,233,73,206,127,156,177,74,121,205,186,42,
98,177,180,48,214,21,94,158,70,225,197,70,40,197,89,105,161,225,196,16,70,186,146,145,29,8,194,33,107,183,191,204,133,186,253,189,89,235,180,205,218,76,73,238,119,5,44,233,171,9,121,169,194,182,83,56,
211,35,206,41,162,121,136,11,152,139,65,189,248,34,152,235,174,202,192,46,93,61,16,4,210,145,104,30,81,155,110,47,45,113,57,26,77,95,16,162,214,254,83,0,126,233,143,81,95,161,62,64,164,128,233,83,21,214,
212,243,127,59,137,172,150,249,230,107,220,97,32,18,210,241,144,117,171,240,215,46,100,60,201,228,29,98,255,0,244,171,253,59,34,208,106,153,70,155,96,143,156,160,93,234,80,151,186,189,161,178,188,145,
114,213,43,28,67,105,232,105,196,86,92,18,233,178,121,16,237,53,192,199,2,137,72,111,72,3,128,235,94,235,219,214,182,184,123,90,202,86,226,155,148,101,233,12,12,136,244,131,194,54,162,218,234,222,238,
214,157,237,187,134,181,171,76,58,183,65,86,26,131,122,8,51,138,204,195,124,210,56,38,227,148,226,177,86,60,218,150,99,186,75,218,175,236,113,245,29,227,75,133,49,130,54,201,41,203,18,85,202,203,150,173,
221,36,43,51,74,117,211,164,12,66,41,233,38,10,128,27,202,3,171,61,252,171,191,163,104,110,238,46,232,162,37,50,236,52,177,210,0,212,222,153,113,138,106,159,157,152,187,139,229,177,180,177,184,169,81,
234,138,104,67,160,212,89,180,169,19,229,168,145,207,174,45,45,170,170,46,216,105,8,105,8,105,8,105,8,105,8,105,8,105,8,105,8,105,8,214,249,131,39,65,97,140,93,125,202,182,84,214,94,22,133,88,149,178,
60,102,216,232,166,238,72,209,237,140,118,145,44,142,224,196,64,175,165,222,138,109,80,231,48,19,172,169,120,136,6,179,177,182,21,178,153,10,56,250,18,21,107,84,10,9,228,38,120,147,46,128,38,79,96,143,
55,49,147,161,134,197,215,202,220,204,209,183,164,206,64,230,100,56,40,159,9,177,146,137,244,145,17,215,183,46,234,180,221,201,230,42,142,28,170,97,139,188,92,165,164,242,135,60,196,140,236,10,209,208,
236,33,161,223,205,72,72,62,34,4,5,76,146,109,152,24,133,41,68,4,234,156,132,15,41,128,53,52,205,249,125,115,131,198,84,201,220,93,82,106,116,229,221,10,211,98,196,40,3,214,126,73,152,174,246,231,154,
150,123,147,49,71,15,105,101,93,106,213,213,222,44,178,80,170,88,147,46,193,235,50,17,43,154,175,98,214,134,144,134,144,134,144,134,144,142,28,222,102,250,40,155,51,78,132,157,158,171,57,115,148,191,30,
124,236,34,160,30,176,100,187,6,21,226,197,149,212,131,213,95,129,147,42,75,184,151,73,52,139,228,49,196,167,16,226,4,55,9,94,216,218,119,123,156,214,52,42,37,42,116,116,204,176,38,101,167,32,37,216,9,
62,174,184,131,239,45,243,99,179,69,184,186,164,245,170,220,107,146,161,0,128,154,102,76,250,203,0,58,248,245,70,65,179,141,222,196,111,22,163,110,186,215,104,22,26,76,37,90,198,222,174,85,167,164,35,
159,250,218,80,99,16,150,126,147,81,143,0,4,189,90,209,243,81,63,63,231,120,146,240,243,14,186,119,54,219,169,182,110,105,218,214,172,149,106,212,66,253,208,68,132,228,39,62,178,15,201,25,27,59,119,209,
222,22,149,175,109,237,234,80,161,74,160,73,177,7,83,105,12,101,46,160,86,126,145,29,135,168,212,76,33,164,33,164,33,164,33,164,33,164,33,164,33,164,33,164,33,164,33,164,35,94,186,203,152,165,147,151,
12,222,100,236,122,209,227,69,213,106,237,163,171,165,109,187,150,174,91,168,100,151,110,225,5,100,136,170,43,162,169,4,167,33,128,12,83,0,128,135,29,125,105,110,163,28,76,71,209,247,203,136,62,43,99,
111,126,107,31,90,105,165,186,140,38,33,247,203,136,62,43,99,111,126,107,31,90,105,165,186,140,38,33,247,203,136,62,43,99,111,126,107,31,90,105,165,186,140,38,35,244,34,178,126,53,157,144,109,19,9,144,
232,211,18,143,12,98,51,141,138,182,192,72,72,58,57,19,58,199,35,102,109,36,22,112,185,136,146,102,48,129,74,34,5,40,143,152,7,92,105,97,204,24,76,69,46,62,106,173,161,100,20,114,190,34,222,204,4,83,217,
124,99,39,143,161,240,133,253,235,52,149,114,74,93,190,2,197,105,158,170,200,76,242,20,124,20,69,202,50,206,118,141,214,31,208,145,236,104,166,115,21,87,77,202,165,229,229,62,98,220,218,86,193,212,33,
110,133,67,85,1,250,74,66,134,3,172,169,89,145,206,70,124,129,150,182,121,225,128,187,23,214,251,146,146,150,178,106,66,141,66,56,232,117,102,101,39,168,56,105,3,202,107,35,197,132,234,11,171,138,40,40,
185,63,203,43,220,209,24,167,10,118,232,204,147,196,65,140,139,169,171,78,216,166,229,93,156,169,161,40,233,71,51,151,76,64,10,44,113,65,34,74,172,103,51,112,196,253,24,25,217,159,161,204,117,92,52,75,
84,191,153,251,92,184,254,228,178,94,240,1,107,129,213,201,106,122,184,43,118,105,60,129,49,176,190,76,239,48,135,251,71,32,210,82,75,91,18,122,76,217,233,122,248,186,118,235,28,202,136,186,158,169,40,
216,200,105,8,105,8,105,8,105,8,211,123,131,207,56,211,108,24,95,34,231,188,193,62,141,111,29,227,26,219,187,29,130,65,67,38,46,28,2,66,155,104,216,72,150,234,40,151,172,44,22,57,119,8,71,199,52,41,129,
71,111,156,164,137,61,35,134,179,113,216,251,172,165,237,60,125,154,234,185,170,193,64,252,164,245,5,19,36,244,0,76,121,217,108,165,158,23,29,91,41,144,112,150,148,80,179,31,200,7,91,49,146,168,233,98,
4,121,74,111,95,118,153,11,123,251,152,202,27,145,201,10,25,9,91,228,217,205,5,94,77,193,156,48,165,210,163,11,224,41,244,184,179,9,83,41,154,87,160,209,73,35,170,4,32,187,115,213,114,160,117,86,80,71,
108,176,120,139,124,22,46,150,50,219,216,166,188,79,75,49,226,204,125,39,228,18,28,132,104,222,227,207,93,238,92,205,124,197,231,7,170,221,213,232,68,28,17,7,98,172,132,250,76,201,226,76,115,133,102,181,
96,185,216,224,106,21,56,105,43,29,166,211,49,27,94,174,87,225,154,44,254,90,110,114,97,226,49,241,81,49,140,91,148,235,187,127,32,249,193,18,73,50,0,152,231,48,0,7,17,215,165,86,173,58,20,218,181,102,
11,73,20,146,79,0,0,19,36,158,160,35,200,163,70,173,197,101,183,160,165,235,187,5,85,2,100,177,50,0,14,146,79,1,30,151,247,123,54,61,237,91,217,178,147,82,220,251,139,181,133,133,23,4,209,240,61,214,47,
23,75,69,181,190,203,92,50,115,20,106,150,104,140,113,56,254,70,54,49,187,138,162,150,9,23,76,222,153,194,98,148,116,80,174,152,29,82,145,35,235,29,10,87,59,175,122,84,173,139,8,173,82,225,170,169,112,
116,5,67,169,75,128,9,239,72,2,37,205,165,203,140,110,69,205,107,77,143,229,237,58,25,163,81,214,149,170,80,113,76,128,229,234,13,44,41,177,32,119,117,49,6,124,21,102,56,240,136,68,236,245,142,251,60,
110,115,121,213,53,54,195,181,253,221,86,50,86,8,138,83,61,71,218,242,246,68,171,204,99,248,39,53,9,216,24,184,53,228,99,226,108,82,110,94,202,251,71,96,104,171,20,65,62,61,84,5,111,33,81,48,132,231,121,
92,239,44,94,21,253,235,117,102,214,183,7,194,43,77,24,57,212,9,50,37,68,132,129,159,166,93,49,91,121,127,105,229,254,107,113,83,56,91,43,244,188,181,95,28,53,90,138,81,74,50,133,152,12,102,117,48,210,
58,196,250,35,189,50,143,205,29,179,28,105,146,242,22,57,38,19,220,45,184,40,23,123,85,40,109,85,246,216,216,32,44,166,171,78,190,131,60,252,23,172,47,141,36,6,22,92,236,69,195,65,93,20,87,51,117,8,42,
38,153,196,72,30,5,175,149,121,187,171,90,119,62,61,178,120,148,213,180,157,115,93,64,25,25,33,19,19,145,145,34,113,40,189,243,179,110,217,222,85,179,251,181,221,79,10,171,38,165,240,244,182,150,43,169,
102,224,200,202,98,96,25,115,2,48,79,226,196,217,159,248,114,220,231,234,49,87,247,135,174,255,0,132,153,191,196,218,255,0,51,234,70,47,199,93,189,248,59,223,229,127,82,63,182,59,230,191,217,26,207,91,
165,41,183,221,211,48,96,115,242,185,120,206,43,19,73,57,64,130,83,114,157,54,74,229,24,210,56,244,248,1,131,172,65,2,136,136,113,16,2,143,13,229,46,116,41,43,113,104,91,211,80,124,254,25,252,145,244,
158,122,109,178,192,61,165,240,78,146,5,35,243,120,131,242,197,132,118,205,186,12,59,187,140,15,76,220,118,23,177,43,47,140,46,236,37,30,48,127,48,201,72,41,40,165,224,37,36,33,44,49,86,24,215,166,3,196,
201,65,75,197,56,69,112,49,140,137,129,48,85,37,21,64,233,170,122,239,41,139,189,195,223,190,54,245,101,117,76,128,64,51,6,96,16,65,28,193,4,17,211,208,64,51,17,107,225,179,88,252,246,46,158,95,28,250,
172,170,2,65,35,73,26,73,12,24,30,69,72,32,244,116,130,65,6,33,195,113,31,50,231,110,156,29,112,153,163,212,135,45,238,22,78,13,234,145,207,108,88,126,181,91,83,31,11,198,170,149,39,169,49,183,93,45,245,
51,78,32,137,185,193,55,113,141,31,48,114,36,226,146,230,76,197,80,102,120,239,44,119,37,245,21,175,91,193,182,86,19,2,163,54,185,116,77,85,90,94,134,32,142,145,56,175,178,222,113,237,28,101,195,91,80,
241,238,221,76,139,82,85,240,230,57,201,221,215,87,165,67,41,232,50,227,28,225,252,88,155,51,255,0,14,91,156,253,70,42,254,240,245,233,124,36,205,254,38,215,249,159,82,60,127,142,187,123,240,119,191,202,
254,164,100,149,79,154,203,98,50,243,109,35,173,24,99,115,245,8,167,71,77,21,44,30,206,227,89,246,177,199,81,100,137,226,36,88,71,100,180,229,61,94,138,38,57,212,51,84,157,184,244,64,164,64,226,110,37,
235,171,229,54,125,16,181,42,246,174,227,163,83,137,250,9,73,79,211,33,219,29,212,60,242,219,21,42,132,175,109,123,78,153,250,90,105,176,30,144,42,78,94,128,79,97,137,103,222,247,115,77,191,236,123,108,
56,247,117,214,182,246,76,155,142,242,204,237,38,27,27,182,199,4,138,86,94,220,218,251,84,150,188,66,79,71,39,100,145,130,108,156,40,213,97,148,118,117,21,81,51,20,14,153,57,121,212,41,70,35,131,218,249,
28,238,82,166,38,142,154,87,52,85,139,235,156,151,67,5,32,233,7,142,163,47,150,39,123,151,121,226,182,214,22,150,114,184,122,214,149,217,5,49,78,83,125,106,92,48,212,84,75,72,159,30,206,147,17,133,143,
254,104,205,131,218,98,111,51,86,218,30,120,199,77,170,16,172,223,197,70,202,66,210,39,44,87,233,151,210,9,180,66,175,79,135,130,187,57,40,200,17,153,87,118,179,185,21,163,226,155,36,135,34,174,136,178,
205,146,90,81,113,229,94,224,164,244,210,141,75,122,133,218,68,130,192,32,3,218,98,87,148,228,0,0,177,159,1,32,72,133,218,121,215,181,171,211,171,82,226,149,213,21,166,160,128,66,51,57,38,90,80,43,243,
148,201,44,85,64,28,88,18,1,214,102,249,176,246,98,6,48,19,110,123,158,49,0,195,200,99,54,197,68,49,139,199,209,19,16,50,41,192,166,16,243,128,8,240,254,113,214,87,194,76,223,226,109,126,90,159,82,48,
190,58,109,223,193,222,255,0,43,250,145,222,125,219,179,218,164,217,254,47,172,165,27,41,83,154,220,52,133,98,122,82,165,59,225,18,158,134,172,87,162,227,46,115,16,179,40,181,89,210,69,148,134,180,190,
134,110,228,17,80,233,2,164,56,1,204,28,57,186,60,182,197,7,220,85,174,137,15,74,205,24,6,19,145,119,37,1,29,133,67,145,62,49,145,230,254,112,211,218,150,246,74,26,157,107,247,86,40,210,212,17,0,168,192,
202,124,67,154,96,203,135,62,49,206,93,143,113,48,63,185,230,28,218,249,183,20,107,144,81,152,230,188,178,133,41,211,52,133,141,210,115,214,53,81,1,32,138,110,163,216,66,49,79,156,4,7,164,252,229,242,
129,135,94,247,154,217,29,22,182,216,164,60,93,205,70,244,40,210,190,162,89,143,165,98,53,228,134,39,196,188,188,205,184,225,78,154,209,79,75,157,111,46,208,21,71,161,140,88,231,84,148,108,100,113,182,
239,119,177,141,118,111,31,72,119,124,131,179,217,94,95,94,77,55,135,137,170,22,32,239,83,107,95,66,61,73,57,23,126,183,147,141,72,141,82,86,85,178,69,229,49,140,99,171,228,14,5,29,73,246,222,214,190,
220,207,85,109,30,157,53,162,20,177,121,202,109,57,1,32,120,240,39,213,16,221,223,189,113,187,58,157,7,191,74,181,94,187,48,85,167,166,114,64,53,19,169,135,14,240,30,184,225,223,199,7,111,223,9,115,31,
234,233,63,106,181,43,248,83,152,252,77,183,249,254,164,66,62,55,237,255,0,194,94,127,47,235,195,241,193,219,247,194,92,199,250,186,79,218,173,62,20,230,63,19,109,254,127,169,15,141,251,127,240,151,159,
203,250,241,220,91,85,223,70,16,221,234,182,56,236,107,237,92,45,142,172,213,188,140,173,90,239,19,31,23,50,49,46,87,6,132,152,96,120,121,153,248,167,209,196,120,98,164,160,145,207,89,35,156,157,68,200,
10,16,77,19,220,27,79,43,182,194,61,247,134,212,42,25,7,66,74,204,113,210,117,42,144,101,199,148,143,25,19,35,19,125,171,190,112,155,188,212,167,141,241,82,230,144,5,146,170,133,109,36,203,80,210,206,
164,79,129,239,76,76,76,9,136,174,247,118,124,178,57,47,120,22,136,54,142,122,208,184,154,10,19,28,176,2,24,192,137,164,27,38,173,130,202,176,166,38,18,131,164,167,231,151,102,161,248,0,152,172,200,30,
80,40,14,174,143,46,177,223,113,219,84,234,176,149,91,151,106,135,208,123,171,234,210,161,135,235,24,215,159,54,114,222,243,221,245,104,33,157,27,74,107,68,117,76,119,223,215,169,138,159,213,17,33,120,
123,119,152,155,183,30,222,112,6,34,189,211,110,182,59,198,66,199,73,103,43,9,106,100,130,31,87,6,70,151,146,119,14,214,100,147,50,209,75,55,148,66,25,154,45,76,144,16,68,133,106,28,195,196,64,70,27,146,
219,121,29,237,154,188,201,90,85,164,150,148,107,248,11,175,87,31,12,0,197,116,131,195,81,38,125,177,97,97,247,118,39,203,157,189,143,196,95,209,173,82,250,226,219,239,47,225,233,225,227,51,21,13,169,
150,76,20,5,151,232,198,116,223,190,6,221,140,178,101,119,138,179,74,45,196,220,22,85,187,58,51,149,136,79,47,164,155,117,46,109,8,169,184,255,0,32,168,64,252,186,196,111,42,115,82,238,220,90,150,237,
53,7,207,160,254,72,206,95,59,182,233,97,174,210,244,47,96,164,79,201,226,15,203,18,85,183,45,201,99,29,210,227,209,201,24,177,228,162,208,237,230,94,87,165,99,167,163,130,46,114,14,113,139,118,79,86,
141,147,104,155,135,141,5,67,48,146,110,185,20,110,187,132,14,154,192,0,126,114,156,165,130,230,240,119,248,11,223,184,228,2,138,165,67,2,166,106,202,73,19,6,64,243,4,113,0,240,229,202,44,173,185,185,
49,155,167,31,239,44,83,57,162,28,163,6,26,89,88,0,100,194,100,114,32,204,18,36,121,206,96,109,155,5,190,165,82,35,85,45,86,138,237,101,55,198,84,140,148,176,77,198,195,17,225,208,2,25,114,53,52,147,150,
197,112,100,74,169,68,224,78,34,80,48,113,243,134,188,144,9,229,30,244,196,99,31,124,184,131,226,182,54,247,230,177,245,166,185,210,221,70,56,152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,
234,48,152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,234,48,152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,234,48,152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,234,48,
152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,234,48,152,135,223,46,32,248,173,141,189,249,172,125,105,166,150,234,48,152,143,181,44,191,137,151,48,145,12,161,142,214,57,82,89,99,17,43,181,
105,67,2,45,145,81,195,133,68,164,147,17,4,144,110,145,148,57,188,196,33,68,195,192,0,71,77,45,212,97,49,21,130,204,223,243,131,43,127,212,155,215,246,162,83,89,107,236,143,68,99,183,180,125,49,173,181,
245,28,67,72,67,72,71,85,236,139,255,0,105,241,15,237,137,159,236,164,254,190,42,123,6,62,147,218,17,99,11,253,2,147,149,41,86,124,115,146,42,208,119,106,37,206,29,229,126,213,84,178,71,161,41,9,59,14,
253,49,73,211,25,6,46,72,116,150,72,229,30,32,60,0,196,56,1,202,37,49,64,67,166,222,226,189,165,117,185,182,118,75,132,96,85,148,200,130,58,65,133,213,173,181,245,179,218,94,34,212,181,168,165,89,88,77,
88,30,96,131,20,18,239,7,216,34,249,180,85,237,27,137,218,100,116,238,74,219,9,150,125,53,102,165,36,14,166,239,248,29,152,245,94,58,51,206,82,42,246,219,140,99,19,1,233,74,254,145,252,107,114,129,100,
65,66,166,105,5,182,7,102,249,131,111,152,9,141,203,149,165,148,224,21,185,37,83,217,208,174,127,55,147,31,103,158,145,171,94,96,121,89,117,128,47,151,193,6,173,133,153,44,156,75,208,28,204,250,90,152,
232,111,105,71,183,57,107,53,189,172,217,108,20,203,28,13,190,167,51,37,92,180,213,166,35,108,53,203,4,51,181,152,75,66,78,67,188,70,66,42,90,49,243,115,17,118,143,227,223,55,34,169,40,65,3,16,229,1,1,
226,26,178,170,210,167,94,155,81,172,161,169,58,144,65,226,8,34,68,17,212,68,84,20,107,85,183,172,183,20,24,165,116,96,202,192,200,134,6,96,131,208,65,226,35,211,163,179,167,115,122,215,113,253,182,49,
150,159,125,21,27,185,28,90,222,58,183,158,41,205,1,187,48,115,34,100,212,74,31,37,87,163,146,18,242,84,239,136,53,58,220,132,33,73,31,36,155,166,96,2,154,72,44,190,175,111,61,175,87,109,100,202,83,4,
227,42,146,212,155,179,165,9,252,228,229,218,178,110,100,129,185,254,95,111,58,59,191,14,42,85,42,185,138,0,45,116,28,56,244,84,81,249,175,41,254,139,106,94,64,19,46,218,135,196,246,26,66,26,66,26,66,
60,242,254,96,254,235,9,111,15,47,134,215,48,109,145,39,251,104,193,118,39,3,49,97,133,123,226,34,179,30,90,143,35,152,217,11,35,119,104,31,195,73,82,105,69,93,120,248,83,167,206,131,213,206,233,249,20,
93,5,153,25,45,138,242,239,105,156,53,159,189,111,214,89,58,235,192,17,198,157,51,196,47,99,55,2,221,32,73,100,8,105,234,119,154,251,228,110,12,135,185,49,143,60,53,171,241,96,120,85,170,56,22,159,74,
39,21,78,130,117,52,200,43,40,12,194,216,75,43,238,39,37,213,176,246,19,162,207,228,108,145,114,126,86,16,21,122,235,65,114,241,193,128,5,71,47,93,172,115,38,206,42,26,49,177,76,187,215,206,148,69,155,
38,196,58,203,170,154,100,49,130,192,189,190,180,198,218,189,229,245,69,167,108,130,101,143,47,71,89,39,144,2,100,158,0,78,42,220,118,54,251,47,120,152,252,109,39,173,121,80,201,85,68,207,105,61,0,14,
101,140,128,28,73,2,61,11,123,65,246,56,198,61,191,227,226,115,78,100,52,30,85,221,204,132,104,255,0,180,8,166,119,148,204,50,148,131,101,219,200,193,99,52,222,32,130,175,102,221,179,116,45,223,207,184,
68,142,85,75,153,6,132,108,221,71,30,43,93,183,142,251,186,220,44,108,172,181,81,195,131,203,233,84,151,34,242,228,58,66,14,29,38,102,82,219,13,131,229,165,150,213,69,200,228,52,215,207,17,237,115,74,
51,28,86,156,249,146,12,153,200,153,228,161,65,58,163,35,230,203,220,113,146,101,181,189,164,196,200,20,5,226,182,29,194,222,163,74,168,149,94,139,111,31,142,177,131,147,166,95,206,110,186,234,90,192,
121,188,130,116,10,32,28,75,196,37,30,81,227,102,215,121,119,28,165,69,79,200,239,255,0,47,229,136,103,158,217,121,45,150,6,153,231,170,187,143,68,233,211,255,0,155,242,70,160,236,184,80,216,255,0,105,
30,227,61,198,228,78,72,139,109,174,61,238,55,196,50,46,8,78,170,82,148,248,179,86,104,207,152,173,232,156,172,172,25,175,42,55,104,228,133,55,57,141,10,3,203,232,144,71,51,123,127,223,119,126,55,109,
175,26,40,67,212,29,140,117,48,61,162,149,50,71,235,71,159,229,215,254,53,176,242,251,189,251,181,234,3,78,145,237,65,165,8,236,106,213,64,63,169,16,77,218,207,101,140,59,129,111,99,24,109,214,211,35,
106,134,160,206,53,183,218,178,117,154,156,188,107,123,44,13,58,169,90,146,149,89,228,115,217,184,153,248,198,175,101,172,33,31,24,146,206,25,58,77,53,159,144,198,76,192,3,169,238,235,205,182,222,193,
213,201,82,8,215,10,85,81,90,122,75,51,1,34,1,6,64,77,164,8,224,34,177,217,27,113,119,86,228,163,136,174,93,109,88,59,84,100,150,165,69,82,102,9,12,1,45,165,65,32,241,110,81,111,191,225,73,237,225,241,
151,121,255,0,188,60,31,255,0,206,186,167,126,44,238,63,224,217,127,130,175,245,162,254,248,27,180,191,17,145,251,74,63,233,226,152,125,195,240,22,40,218,214,244,247,7,183,156,37,109,181,93,241,182,36,
186,35,78,134,177,93,212,136,90,214,180,155,10,252,41,237,241,147,78,32,35,33,98,29,186,174,92,214,144,141,5,208,102,212,171,166,208,170,10,73,152,194,80,186,182,230,66,239,43,132,183,200,223,34,83,185,
172,154,136,89,233,145,39,73,19,36,247,150,77,34,76,167,206,53,219,118,226,172,112,155,142,239,19,141,168,245,108,237,234,104,12,242,213,48,163,88,58,64,4,171,234,89,128,39,41,200,69,164,247,69,108,156,
237,207,242,219,109,227,9,197,200,184,129,202,59,161,128,129,173,200,153,19,168,218,110,62,47,58,189,178,231,124,154,217,110,138,201,244,202,202,150,248,106,238,135,210,229,35,210,151,135,48,243,150,171,
197,81,167,185,60,203,185,190,96,26,214,213,139,14,162,105,5,164,159,43,13,99,209,23,110,110,189,93,163,228,253,166,53,9,91,219,212,85,61,96,86,45,90,160,245,33,240,207,166,32,47,181,63,111,140,73,188,
233,92,255,0,148,119,63,150,44,24,71,105,251,88,160,197,220,179,13,242,170,148,127,180,103,121,103,115,42,218,177,7,18,242,82,18,208,210,63,168,222,2,65,218,235,22,38,85,99,120,84,219,38,223,168,232,138,
167,96,110,205,197,121,133,75,123,92,93,21,175,151,187,168,86,154,52,229,37,150,162,64,43,62,96,14,242,142,36,207,132,141,89,177,182,165,134,226,123,171,220,213,119,182,193,88,210,15,85,214,90,166,211,
210,160,149,96,61,150,36,233,99,192,0,38,211,18,98,109,143,252,178,2,83,1,59,151,238,104,167,224,60,166,51,216,163,20,13,195,209,19,20,54,108,65,48,0,249,195,136,113,254,112,212,99,223,190,104,127,182,
90,252,135,253,76,76,255,0,182,188,152,255,0,121,189,249,71,250,72,173,252,166,57,138,181,103,217,28,71,129,229,36,178,20,37,143,48,59,199,56,102,106,81,146,112,243,23,136,169,139,162,149,156,119,41,35,
28,83,25,40,169,43,51,71,12,213,89,0,17,42,10,174,37,227,192,186,178,146,229,233,99,197,229,248,20,234,45,29,117,0,51,10,66,234,112,15,72,94,60,122,101,21,3,217,165,124,169,176,197,177,171,73,238,13,58,
36,137,23,5,244,211,36,116,22,18,36,116,19,22,29,249,148,242,28,77,22,211,178,157,128,82,228,74,234,165,180,141,185,86,198,76,27,36,131,84,20,155,155,137,134,164,86,153,189,102,138,134,6,207,34,40,248,
225,179,180,146,2,148,137,33,55,192,162,110,97,2,215,62,89,91,61,197,43,237,195,92,74,181,229,203,75,208,9,102,35,176,179,145,233,88,182,124,226,187,167,107,95,27,181,109,140,232,88,89,172,250,56,144,
17,65,29,97,41,130,58,131,198,199,237,27,242,246,225,173,235,109,58,3,115,219,153,201,89,178,158,92,147,55,102,46,55,169,98,135,180,138,201,147,168,86,102,156,214,61,163,159,145,187,81,175,171,62,86,118,
118,37,240,182,69,22,205,8,86,105,162,176,40,177,87,14,92,109,225,230,37,238,15,46,216,188,93,42,15,225,42,235,106,129,155,188,195,84,128,86,73,72,17,57,147,198,98,66,81,153,176,188,168,199,110,60,18,
102,179,53,174,105,248,204,222,26,210,40,189,197,58,117,49,116,121,234,96,210,0,14,18,51,51,137,110,165,252,173,221,188,105,151,26,157,192,50,86,237,44,163,84,178,193,89,66,185,102,190,225,231,21,185,
241,130,148,107,40,16,150,22,241,88,30,30,81,120,57,81,107,208,118,70,206,218,184,50,10,28,19,89,35,136,28,177,10,254,106,110,58,244,94,143,133,102,186,212,172,213,42,106,19,18,152,157,82,38,57,137,130,
39,204,24,158,91,121,39,180,237,238,41,220,120,215,239,225,186,182,150,122,90,91,73,6,77,42,0,233,50,145,145,6,92,136,231,26,19,188,46,90,251,192,221,114,180,150,78,186,176,216,122,165,15,84,4,147,83,
168,216,214,57,162,123,85,97,118,79,32,1,28,2,82,172,216,172,80,17,0,59,14,30,65,1,13,79,60,181,199,125,207,111,125,233,132,170,220,212,103,237,210,189,197,30,142,12,195,245,162,177,243,131,45,239,13,
214,108,144,206,141,157,21,167,217,173,187,238,125,61,229,83,218,177,56,189,177,241,48,226,109,155,226,244,29,53,22,179,89,1,39,217,78,112,4,4,162,178,151,37,19,90,190,169,136,32,6,33,194,148,210,44,134,
1,226,60,228,31,48,112,0,170,55,230,71,222,59,154,224,169,157,42,36,82,95,248,61,175,243,151,139,191,203,44,79,186,118,117,170,184,149,107,128,107,183,255,0,167,20,254,88,65,29,253,168,124,79,226,33,55,
247,67,216,70,85,200,240,14,55,47,185,187,37,14,213,76,174,150,184,210,141,72,179,214,156,250,165,55,143,156,204,173,35,49,0,76,123,122,157,140,152,149,77,226,32,115,46,162,9,168,217,187,113,34,97,229,
58,150,70,207,187,221,248,251,23,92,21,130,86,183,170,250,141,74,138,194,114,1,100,173,226,83,82,162,71,144,38,100,241,232,21,22,254,176,216,89,92,149,54,220,185,58,148,46,168,211,208,41,82,116,58,102,
75,77,147,194,170,202,205,49,204,128,84,44,135,73,224,159,252,113,236,229,254,47,243,31,250,75,47,254,126,212,195,223,126,102,127,182,218,252,255,0,234,34,5,253,185,228,239,251,197,231,202,63,210,70,131,
223,198,212,54,233,183,58,126,9,182,224,188,131,144,110,77,243,52,100,189,153,138,119,101,225,93,166,234,156,139,8,7,176,214,54,30,173,169,83,159,71,33,38,19,132,233,17,203,117,5,201,57,140,65,39,76,192,
111,95,104,110,28,222,110,230,238,219,45,70,141,35,106,193,78,141,66,79,54,12,166,110,224,203,79,25,30,29,179,143,3,126,237,77,187,183,108,236,46,240,87,23,21,150,245,89,199,136,84,206,152,8,85,196,169,
211,32,54,161,32,192,207,152,148,184,245,183,103,24,24,252,119,69,221,6,233,45,32,162,21,170,117,107,217,196,157,0,242,38,163,90,204,83,140,129,118,72,14,97,228,23,8,180,78,35,144,56,8,241,87,242,128,
12,115,204,202,207,123,119,97,128,183,227,94,173,77,82,237,99,225,167,207,175,228,137,111,147,180,41,227,172,114,155,162,235,133,181,26,90,39,216,138,106,212,245,203,195,151,166,34,135,19,213,231,55,67,
185,250,140,4,209,142,238,91,50,229,162,72,91,92,160,10,129,129,181,138,125,105,235,164,153,58,102,235,20,26,70,29,227,143,33,128,64,9,249,193,231,213,135,145,184,165,128,192,212,173,75,133,59,91,105,
32,237,85,210,131,214,116,136,170,113,54,181,247,70,231,163,111,91,141,107,203,185,212,35,169,223,85,70,235,224,186,143,170,44,225,159,123,93,96,93,198,100,201,108,165,120,187,102,104,217,137,54,16,113,
40,66,213,167,232,209,245,168,72,170,244,59,56,104,232,232,70,18,88,234,97,235,70,128,147,78,177,202,119,42,241,93,85,12,2,0,96,40,81,24,125,253,151,194,88,46,62,210,149,171,82,82,199,83,173,66,204,89,
139,18,196,84,80,79,25,114,28,0,141,154,207,249,95,129,220,89,55,202,95,87,189,90,204,170,161,81,233,4,85,69,10,2,131,69,136,28,39,196,158,36,152,174,198,253,54,249,141,118,195,159,222,226,76,97,101,180,
217,98,162,170,149,185,89,149,46,11,68,58,153,138,159,156,77,219,243,69,25,220,36,84,35,39,77,253,70,163,23,36,55,133,72,225,226,68,163,205,202,6,27,167,104,102,111,179,216,113,145,191,167,78,157,70,168,
193,116,76,41,85,144,156,152,177,7,86,161,204,242,141,119,223,155,127,27,182,51,237,137,197,213,171,86,146,82,70,111,16,169,101,118,153,211,53,85,4,105,210,195,186,61,169,118,197,131,123,69,99,23,24,251,
103,176,83,143,208,20,36,50,173,186,203,144,76,69,11,193,114,70,24,204,234,112,133,48,240,15,208,58,143,171,149,226,32,2,33,200,239,155,200,38,16,10,111,204,123,245,188,220,175,69,12,210,222,154,211,236,
159,23,111,144,190,147,232,141,129,242,143,24,216,253,159,78,189,65,42,151,85,158,175,110,158,20,215,212,66,106,29,141,24,143,117,175,247,127,11,126,216,187,127,66,173,106,19,67,166,44,186,156,132,67,
22,178,35,166,26,66,26,66,26,66,26,66,26,66,26,66,51,26,47,245,219,239,248,59,34,127,219,251,62,184,60,189,99,242,199,34,63,99,51,127,206,12,173,255,0,82,111,95,218,137,77,112,190,200,244,65,189,163,233,
141,109,175,168,226,26,66,26,66,58,175,100,95,251,79,136,127,108,76,255,0,101,39,245,241,83,216,49,244,158,208,139,47,235,14,50,35,224,197,41,202,98,28,160,98,24,4,166,41,128,12,83,20,193,192,197,49,71,
136,8,8,15,148,52,135,56,169,175,118,207,151,62,175,152,15,100,220,54,193,161,224,168,89,73,111,23,51,114,219,218,103,103,1,143,178,3,142,2,186,239,241,161,196,168,69,80,45,206,76,6,231,141,56,163,6,248,
230,40,166,44,20,5,12,230,220,218,30,100,85,179,211,142,220,12,213,45,121,45,110,110,157,143,210,235,250,92,88,126,144,148,168,157,249,229,29,12,129,124,182,214,85,165,123,196,189,14,11,77,251,105,244,
35,159,205,224,141,209,164,206,117,50,218,166,229,119,23,219,11,118,241,25,50,14,10,126,153,146,49,164,195,154,174,80,197,23,54,146,85,179,90,106,206,87,108,22,204,109,119,137,122,215,197,179,111,42,221,
20,212,65,85,27,157,70,47,82,108,249,2,138,168,36,109,91,153,108,102,55,116,225,205,173,70,87,182,170,186,146,162,201,180,183,209,117,35,156,186,120,241,19,83,192,152,162,112,121,156,190,203,207,45,237,
37,122,119,148,91,77,74,78,10,234,83,237,83,112,68,196,250,12,187,164,43,14,32,71,168,94,211,247,77,136,247,155,129,168,123,133,194,147,133,152,165,222,35,65,85,25,184,50,36,157,169,216,90,129,81,176,
82,237,108,82,81,95,86,89,107,82,28,200,56,75,152,201,168,0,69,208,58,173,150,69,101,53,103,47,138,188,194,228,42,99,175,151,77,122,103,159,67,14,134,83,210,172,56,143,144,200,130,35,117,112,89,187,13,
197,139,165,150,199,54,171,122,163,151,210,86,30,210,48,232,101,60,15,202,38,8,39,163,117,230,199,175,13,33,21,93,249,136,251,183,151,111,212,185,141,140,109,242,202,84,243,142,73,174,144,153,174,217,
12,236,5,230,38,198,214,54,96,116,234,141,29,53,87,158,58,255,0,145,34,23,230,63,55,5,227,160,92,2,229,41,85,124,205,116,173,127,46,118,135,188,107,174,123,34,191,244,52,155,246,74,121,84,112,125,174,
212,67,234,103,18,228,172,13,33,230,214,252,247,85,179,109,156,75,255,0,220,235,39,237,152,30,52,169,176,246,65,28,158,160,245,170,25,243,101,34,166,221,190,251,100,110,123,184,206,65,45,107,12,214,70,
31,31,195,190,65,28,133,155,45,109,94,180,199,20,86,163,211,85,118,234,72,36,137,143,99,182,42,213,64,51,72,86,29,71,139,9,202,117,69,187,94,171,164,173,189,195,186,49,123,110,223,197,189,109,87,12,59,
148,150,90,219,213,208,189,108,120,14,137,153,3,69,109,77,153,154,221,215,126,14,57,52,218,41,253,165,102,4,83,65,233,250,77,46,72,188,79,76,150,108,61,22,187,122,118,201,219,79,110,28,118,122,198,28,
129,52,222,65,176,48,104,134,72,205,54,102,205,85,191,95,28,32,98,46,102,199,112,144,25,42,229,77,7,133,231,107,14,200,74,213,46,82,157,83,56,115,206,224,250,225,184,183,70,79,114,220,248,183,173,166,
221,79,114,154,251,9,255,0,187,117,177,227,213,33,194,54,223,105,236,204,54,208,180,240,49,235,170,237,192,241,43,48,26,223,179,244,86,124,144,112,28,204,207,19,34,58,142,68,182,60,183,123,214,238,44,
119,59,220,191,115,22,200,231,71,125,89,164,92,11,132,169,68,76,231,93,0,133,196,104,150,148,245,120,227,113,55,59,41,251,116,124,156,154,92,190,137,188,127,16,243,235,106,118,70,55,221,123,98,214,139,
9,85,168,158,43,122,106,119,132,251,66,149,95,84,105,63,152,249,127,125,111,43,202,232,103,70,157,79,5,61,20,187,134,93,140,225,152,126,180,76,191,122,19,151,99,221,163,59,117,246,228,141,0,135,184,91,
88,71,100,92,191,24,217,96,5,10,254,159,23,237,85,241,132,131,96,76,4,88,79,103,12,160,163,198,199,56,148,226,120,65,0,227,202,126,16,189,147,255,0,125,222,25,45,200,221,234,40,74,83,61,140,116,161,29,
162,146,72,254,180,88,126,98,255,0,227,91,11,17,180,19,187,113,80,10,149,64,235,65,169,193,236,106,213,38,63,82,50,223,149,127,19,211,40,80,251,158,222,118,78,157,171,211,98,94,73,86,182,209,143,108,214,
217,168,90,243,5,164,5,170,25,63,36,195,182,146,153,118,208,167,118,225,178,53,149,72,146,102,1,57,16,84,120,27,144,121,58,124,214,187,175,112,246,184,91,85,119,112,26,179,170,130,76,189,132,50,31,241,
252,162,59,252,144,176,183,181,167,123,184,175,89,41,161,43,111,77,152,133,19,151,137,80,2,72,227,251,179,46,195,213,22,155,207,59,240,218,182,2,195,249,19,47,217,179,182,32,121,31,67,169,206,88,16,133,
143,201,116,247,115,54,121,72,232,215,46,226,170,181,248,246,82,174,223,201,207,216,223,162,70,140,219,160,146,170,170,178,165,2,148,117,84,216,96,50,217,11,202,118,116,173,235,6,168,224,76,163,73,65,
60,88,146,36,2,142,36,158,136,187,178,155,159,7,138,199,214,200,86,186,183,41,74,155,52,133,68,37,136,19,10,160,18,75,49,224,0,4,146,99,204,151,106,152,198,213,191,157,255,0,226,106,29,168,203,205,206,
110,63,112,168,79,100,247,173,140,185,92,43,9,63,102,119,121,203,211,169,24,84,89,192,29,149,89,41,103,161,204,113,55,232,252,167,243,155,91,65,150,186,165,183,246,245,107,138,93,218,118,214,210,65,218,
23,77,49,235,109,34,52,203,7,101,95,116,238,170,22,181,251,213,111,46,245,84,61,140,197,234,183,169,117,24,155,63,154,135,112,173,110,187,180,195,27,105,128,114,151,168,182,233,138,79,53,53,30,215,166,
130,17,215,172,190,226,62,81,104,195,181,68,165,40,21,150,63,171,215,151,64,77,228,33,31,156,164,41,64,76,39,131,249,81,142,52,49,21,242,117,7,237,46,107,72,30,181,167,49,63,91,179,131,232,139,35,206,
252,178,220,231,173,176,212,143,236,173,40,76,129,208,245,100,101,46,196,90,100,126,183,203,189,251,109,109,31,24,110,15,178,5,235,6,66,238,235,8,224,92,137,186,140,252,226,227,150,37,45,115,112,18,214,
24,122,94,42,179,70,70,86,241,212,181,99,219,122,172,148,74,242,15,168,237,166,209,50,230,14,164,124,185,132,72,116,156,38,98,249,251,155,49,117,142,223,52,239,222,206,189,197,181,165,190,154,97,65,0,
181,69,37,156,54,150,6,65,138,153,116,175,88,49,234,108,252,13,150,91,203,90,184,202,87,246,214,183,119,215,90,234,150,101,44,18,147,0,180,202,235,82,38,80,56,159,209,126,68,17,26,111,248,107,177,215,
255,0,171,59,101,247,114,15,251,235,214,111,196,203,159,246,155,175,241,31,233,71,157,240,118,211,253,242,203,252,35,250,209,221,125,187,126,94,10,126,15,221,54,33,220,219,141,233,226,221,196,84,176,93,
191,219,7,85,26,13,53,170,96,189,178,54,26,69,122,82,207,44,109,178,37,169,148,65,160,108,199,103,43,202,118,198,81,82,179,228,32,166,38,5,73,224,238,63,49,171,95,226,171,98,197,141,91,106,213,211,78,
167,111,162,72,213,221,208,164,205,102,188,250,125,81,39,218,94,83,91,227,51,118,249,147,145,161,119,66,214,166,178,168,131,218,0,232,155,120,140,6,150,147,114,227,46,142,113,86,93,249,102,105,221,243,
119,10,207,57,46,166,85,237,14,115,38,113,94,167,138,218,181,58,75,47,49,87,143,144,97,141,113,20,123,97,75,130,6,93,229,82,26,45,48,229,17,41,148,55,30,99,113,230,27,91,1,101,79,3,183,109,237,107,119,
5,26,26,170,118,49,5,234,31,83,22,138,71,116,100,106,238,109,217,117,121,67,190,215,23,58,105,1,210,160,138,116,128,244,168,88,245,27,219,118,22,131,219,142,223,240,190,5,174,10,42,67,226,12,99,75,199,
173,157,160,136,183,44,154,181,120,6,49,79,166,148,72,68,199,7,83,143,219,170,241,97,48,137,140,178,230,49,132,68,68,117,170,249,59,218,153,44,133,124,133,79,110,181,86,127,70,162,72,30,161,192,122,35,
117,240,248,234,88,140,85,182,46,140,188,59,122,41,78,125,122,84,2,125,36,204,158,211,27,70,201,96,138,169,215,103,173,83,174,65,156,37,106,22,82,193,48,240,64,76,13,98,161,152,175,35,32,228,74,30,83,
2,13,27,28,220,63,151,134,177,168,81,169,113,89,45,233,9,213,168,193,84,117,150,50,3,229,49,153,115,113,74,210,222,165,213,115,166,133,36,103,99,212,170,9,39,212,4,81,201,186,210,155,151,220,130,110,103,
228,19,138,127,154,242,232,58,155,147,116,241,20,218,192,55,186,90,122,242,79,14,241,241,136,221,8,234,251,7,167,48,9,248,38,154,8,0,0,1,64,3,91,92,69,60,22,15,77,21,212,150,182,210,80,7,22,40,156,4,135,
18,88,143,89,49,163,202,106,238,93,201,170,225,180,84,189,187,155,49,34,74,42,60,201,153,224,2,2,123,0,17,117,104,172,141,134,160,226,227,97,98,114,22,57,97,21,14,193,156,92,99,20,46,85,208,65,148,124,
123,116,218,50,104,136,26,76,194,9,55,108,137,72,94,34,35,192,53,171,117,44,114,149,106,53,90,148,107,154,140,73,39,67,113,36,204,158,93,38,55,78,150,75,13,66,146,208,163,113,108,180,145,66,168,21,18,
64,1,32,61,174,129,26,247,113,89,254,3,20,109,195,45,102,170,220,244,12,216,211,234,242,69,128,123,30,253,132,196,90,151,71,254,30,26,170,193,202,141,22,93,186,128,165,146,93,145,85,75,143,55,76,254,111,
40,107,55,11,135,173,145,205,219,98,235,163,167,139,80,106,4,21,58,4,217,200,159,31,100,52,143,92,121,251,139,63,111,138,219,151,121,171,106,148,223,193,164,116,16,67,47,136,100,168,12,137,30,219,44,199,
81,138,119,225,76,93,63,185,220,208,198,157,35,144,43,213,169,155,139,169,201,235,5,247,35,78,3,118,128,162,40,58,153,151,144,120,237,226,229,117,49,51,32,161,77,200,145,76,42,174,177,249,140,98,16,20,
80,155,45,148,191,163,129,197,155,148,162,245,41,82,10,171,78,154,241,232,80,0,28,21,71,95,32,59,100,35,79,112,184,187,141,207,154,91,58,151,20,233,86,172,89,158,173,102,144,224,11,49,36,153,179,30,129,
204,147,198,66,100,73,186,61,161,169,224,170,66,190,246,113,9,144,5,8,43,21,24,184,178,42,100,185,131,168,84,142,124,130,161,8,160,147,143,40,137,76,0,62,112,31,54,160,135,204,139,153,112,197,92,207,210,
127,167,22,104,242,138,206,99,86,110,207,76,248,247,87,250,177,204,61,207,50,132,21,251,115,110,234,148,183,204,94,99,236,45,76,169,98,154,113,162,29,34,238,32,91,66,199,22,74,88,204,212,108,99,182,80,
205,102,37,149,98,101,8,35,206,70,68,14,60,10,80,15,123,97,216,86,179,192,139,139,160,69,229,213,87,170,243,18,51,99,33,57,241,226,160,52,191,72,196,99,204,236,165,11,253,204,109,108,153,91,31,101,70,
157,10,122,72,43,37,19,105,75,135,6,98,179,253,17,29,225,152,132,219,86,237,13,142,49,210,103,43,11,166,224,86,132,94,95,147,130,82,0,222,238,229,76,141,58,43,38,1,192,124,29,70,57,148,27,158,110,34,82,
174,1,228,48,128,132,71,25,255,0,144,121,145,94,244,241,181,179,13,46,169,160,240,215,229,114,213,7,162,39,121,137,237,95,40,173,177,195,187,123,144,42,91,174,85,79,140,223,37,48,180,155,211,28,229,217,
202,173,82,62,225,45,121,82,229,97,173,192,178,198,84,149,218,193,154,122,102,46,33,69,45,23,165,23,135,65,118,67,36,225,1,92,136,86,89,75,38,175,75,136,148,92,19,152,64,13,192,222,223,153,151,23,62,230,
167,143,181,71,119,175,86,109,165,73,238,83,239,113,144,252,226,146,159,81,136,239,147,182,182,159,220,21,114,183,149,41,211,75,106,4,46,182,85,239,213,238,130,38,68,228,130,160,50,235,30,187,42,76,103,
44,49,1,21,33,55,49,150,49,203,8,168,166,139,190,144,120,189,210,189,211,110,213,178,102,85,101,12,4,144,58,135,16,41,124,133,40,24,230,30,0,80,17,16,13,81,180,177,57,74,213,22,149,43,122,230,163,25,1,
161,184,147,234,141,147,173,156,194,219,210,106,245,174,237,150,146,2,73,53,19,128,30,184,165,238,90,183,78,238,147,115,182,219,36,66,107,185,150,204,153,87,213,245,38,110,138,96,112,155,57,233,180,43,
244,136,149,136,159,88,224,118,49,34,201,176,129,121,135,244,126,78,58,217,252,117,181,44,6,6,157,10,146,20,237,109,230,228,114,154,174,167,62,179,168,198,152,229,174,235,238,157,207,86,230,140,205,107,
203,185,83,7,156,157,130,83,94,158,75,165,125,81,118,124,127,76,137,199,52,74,102,63,130,40,150,22,143,85,175,212,162,121,138,5,56,199,215,98,154,68,180,58,128,2,33,213,81,6,133,49,199,136,136,152,68,
120,143,159,90,179,121,117,82,246,238,173,229,95,222,213,168,206,125,44,73,63,150,55,95,31,101,75,29,97,71,31,67,247,20,41,37,53,244,34,133,31,48,136,192,238,181,254,239,225,111,219,23,111,232,85,173,
124,208,233,142,250,156,132,67,22,178,35,166,26,66,26,66,26,66,26,66,26,66,26,66,51,26,47,245,219,239,248,59,34,127,219,251,62,184,60,189,99,242,199,34,36,47,33,246,233,207,246,139,253,230,205,26,247,
30,150,58,197,112,179,78,199,149,213,142,77,39,37,101,47,52,246,65,168,56,72,149,229,72,154,224,131,130,243,148,12,96,3,113,0,17,243,235,168,85,80,0,227,31,102,153,38,124,35,15,252,51,55,25,244,236,109,
239,52,175,217,173,115,227,39,108,60,54,236,135,225,153,184,207,167,99,111,121,165,126,205,105,227,39,108,60,54,236,135,225,153,184,207,167,99,111,121,165,126,205,105,227,39,108,60,54,236,141,225,182,
237,135,230,220,81,155,104,57,10,208,238,140,164,13,106,66,69,204,137,34,167,164,93,200,25,55,80,114,145,201,120,102,235,65,53,73,83,120,135,132,226,2,161,120,23,136,255,0,39,13,124,189,85,101,32,78,113,
202,161,6,102,38,59,88,241,219,13,33,13,33,17,131,220,87,180,198,213,251,143,86,68,249,50,4,212,124,203,17,26,171,26,102,122,164,178,100,133,226,28,133,76,194,202,42,204,137,193,22,151,250,131,119,34,
6,245,108,129,185,208,41,149,240,78,89,40,178,138,154,83,182,247,118,91,109,85,255,0,165,111,18,200,153,181,38,39,73,235,43,249,141,250,67,159,13,65,128,2,33,123,183,98,96,247,125,31,250,197,240,178,42,
36,149,208,13,99,168,55,67,167,232,183,46,58,74,146,76,86,59,109,208,251,230,249,115,55,40,236,51,213,90,99,40,236,3,47,78,71,195,228,60,137,142,25,201,217,40,141,15,215,43,56,12,153,26,211,129,95,227,
236,153,0,221,80,73,228,84,137,17,36,235,14,171,100,20,122,118,236,93,182,180,50,111,129,243,35,24,61,222,235,75,112,209,82,81,30,74,253,101,9,228,232,122,24,79,73,145,58,102,192,211,24,122,123,155,202,
44,193,247,162,53,125,171,112,192,84,169,76,22,65,208,181,0,231,78,162,242,42,210,214,179,80,90,74,194,241,20,171,165,83,35,84,43,23,234,36,252,101,170,151,115,130,139,179,85,108,176,174,72,242,38,118,
2,105,154,50,17,114,145,238,83,244,86,106,245,154,228,57,71,200,60,7,128,128,15,16,213,23,94,133,107,106,205,111,112,165,43,163,21,101,60,8,32,200,131,232,141,149,182,184,161,121,110,151,86,174,181,45,
234,40,101,96,102,25,72,152,32,245,17,17,107,221,175,185,156,86,192,177,20,125,107,28,199,147,33,238,247,54,17,197,111,111,152,150,49,147,155,4,177,228,222,41,234,176,200,19,117,216,212,93,200,60,175,
64,200,46,82,52,102,4,5,103,100,128,172,208,244,10,237,102,210,173,161,182,31,112,94,26,183,39,195,195,208,239,86,168,76,132,135,29,0,158,0,145,204,253,21,239,30,128,97,27,243,121,166,213,176,20,108,199,
141,159,185,238,208,164,6,163,51,195,89,81,50,85,79,33,205,219,186,62,145,21,237,216,47,203,165,156,119,39,116,95,115,221,207,44,150,122,163,27,188,235,171,212,182,33,9,131,31,53,228,137,121,183,198,153,
125,33,148,236,72,25,84,177,212,124,202,174,57,151,98,221,69,44,2,10,40,146,129,18,178,69,19,88,187,131,204,139,28,101,15,117,237,101,87,100,93,34,164,191,100,128,9,1,76,125,50,58,9,238,114,35,88,49,84,
109,111,40,242,121,139,131,154,222,142,244,214,171,23,52,167,251,106,132,153,147,85,191,250,193,233,81,223,230,15,134,68,93,11,22,226,140,105,132,104,117,236,97,136,104,213,140,113,143,170,140,136,194,
191,82,168,196,52,133,133,142,64,190,146,135,35,86,137,166,11,189,120,176,153,87,46,85,19,184,116,185,204,170,199,58,135,49,198,147,187,187,186,190,184,107,171,202,143,86,225,204,203,49,153,63,252,117,
14,64,112,28,35,98,108,108,108,241,182,169,101,97,73,40,218,32,146,170,0,0,245,14,147,204,147,196,158,36,147,27,7,88,241,151,26,235,48,61,200,113,184,159,38,191,196,80,205,236,89,93,157,2,224,227,25,64,
188,121,25,30,202,95,32,37,95,144,61,58,57,243,249,151,13,98,89,176,117,97,6,228,93,87,10,21,36,210,19,24,220,64,56,14,77,154,219,53,221,37,188,98,182,134,162,235,32,18,66,76,106,32,14,36,202,114,148,
98,100,26,237,44,43,61,130,135,190,20,159,195,82,64,5,244,157,0,147,32,1,105,76,158,18,138,34,108,219,229,228,238,7,29,187,236,9,118,220,214,51,173,69,97,234,238,92,173,95,242,172,226,153,102,129,108,
127,39,17,86,149,45,181,236,99,152,232,123,28,148,228,154,183,7,241,132,143,88,233,144,234,20,30,153,83,24,160,83,28,47,188,215,152,187,121,176,247,20,49,117,88,222,53,22,74,99,195,117,0,176,210,12,200,
0,105,6,99,209,40,214,45,189,229,54,235,76,253,173,206,102,138,46,61,46,21,234,183,138,140,72,83,168,130,3,22,58,200,210,125,51,142,83,249,137,55,29,247,251,220,207,40,192,70,200,11,218,158,222,32,43,
88,30,0,19,85,95,12,18,181,212,92,88,178,1,197,169,184,36,139,246,185,14,209,39,28,170,128,2,101,147,142,75,137,132,165,32,23,214,242,231,27,238,253,177,74,163,9,86,185,102,170,125,7,130,122,180,42,159,
89,143,11,205,172,191,189,119,157,122,72,103,66,209,22,130,245,77,123,213,56,117,138,140,202,122,244,136,145,204,217,218,67,125,57,119,182,95,109,109,188,237,167,23,196,78,215,34,234,86,237,206,231,39,
147,57,18,147,77,112,255,0,46,231,33,99,53,82,139,125,21,99,153,135,93,204,142,61,161,73,171,24,43,149,37,10,82,46,9,2,130,98,28,77,26,177,221,248,27,61,209,147,200,228,234,149,170,93,104,82,146,51,74,
157,41,134,32,128,120,59,141,82,236,156,162,95,146,216,91,154,255,0,102,97,241,56,106,10,212,86,155,220,214,38,162,36,234,214,145,80,67,17,198,154,18,179,151,76,167,206,35,214,59,229,185,238,184,245,226,
45,156,225,236,125,16,138,189,78,121,25,28,215,141,85,102,223,145,35,168,94,177,34,103,165,36,13,213,57,0,133,233,160,126,6,48,115,114,151,137,130,68,222,101,237,37,89,138,213,9,234,20,158,127,56,3,231,
136,154,121,63,190,89,180,155,122,74,58,205,106,114,249,152,159,154,44,129,217,83,177,76,246,193,111,210,91,152,220,157,178,159,112,206,234,214,228,170,184,254,163,69,51,249,74,174,48,142,158,40,33,98,
176,56,179,202,177,137,117,61,116,154,136,227,28,82,183,102,139,56,246,107,59,40,44,244,93,20,237,171,93,239,191,41,238,11,113,139,198,35,165,134,160,206,205,32,206,71,33,164,19,37,7,143,18,73,32,112,
89,113,183,252,185,242,202,174,214,186,108,206,98,165,58,153,77,5,81,18,101,105,134,246,155,81,0,179,145,221,224,0,80,91,139,106,225,193,189,223,59,10,239,123,115,123,215,204,59,160,219,243,172,113,145,
169,249,115,217,57,116,235,179,151,116,234,23,106,212,141,110,139,86,165,185,131,114,157,145,171,106,251,248,243,251,58,10,176,93,41,0,229,110,110,138,137,36,41,20,202,251,251,59,127,224,177,120,58,56,
172,136,171,78,181,29,66,97,117,43,6,102,105,247,120,131,199,136,151,62,32,153,240,139,239,239,43,183,46,107,113,220,102,177,70,141,107,123,141,39,75,62,135,82,168,168,87,188,52,145,221,154,144,220,184,
16,37,198,41,63,135,15,187,31,192,202,95,239,187,18,125,173,212,183,226,86,209,254,59,253,149,79,171,16,111,132,59,235,240,212,254,218,151,214,135,240,225,247,99,248,25,75,253,247,98,79,181,186,124,74,
218,63,199,127,178,169,245,97,240,135,125,126,26,159,219,82,250,209,104,29,132,118,247,221,38,199,187,62,238,7,11,213,105,113,38,222,230,109,107,152,101,201,2,210,237,83,77,8,27,85,202,49,190,44,163,170,
133,239,199,5,104,80,169,211,98,155,216,83,76,93,29,34,188,89,100,64,65,85,12,93,85,187,131,113,98,179,187,202,222,246,179,159,113,208,52,196,202,183,21,83,226,55,114,90,187,204,74,114,229,35,202,46,173,
173,181,51,123,107,203,251,188,117,10,107,253,201,114,42,157,33,215,186,206,60,36,239,207,79,117,0,169,206,83,36,115,49,15,61,175,187,3,111,115,15,239,175,111,185,135,116,152,190,167,90,195,216,146,216,
190,72,146,120,207,35,227,235,115,199,22,170,140,76,132,174,59,108,218,30,6,114,81,241,140,157,249,24,215,39,87,164,37,77,38,231,30,37,55,40,234,101,186,124,193,193,222,96,110,44,177,85,93,175,43,38,128,
10,58,247,88,128,252,72,3,216,212,61,113,95,108,175,43,55,38,63,115,218,100,51,116,41,166,62,133,79,16,145,82,155,157,74,9,166,36,172,79,183,164,206,92,132,94,179,84,52,108,220,114,126,247,105,217,111,
35,237,159,36,227,172,39,14,132,197,230,248,197,141,88,18,115,51,25,6,131,106,228,148,139,80,181,170,163,201,103,76,218,156,29,215,146,112,211,167,206,6,17,115,199,128,129,68,6,69,181,110,113,182,89,218,
23,185,86,43,105,68,151,224,165,166,192,29,28,0,39,131,72,207,178,34,155,218,207,45,145,219,55,56,236,34,7,190,174,161,56,178,168,8,72,241,56,177,3,138,77,101,63,165,21,187,252,37,55,195,240,218,189,251,
198,162,253,123,171,191,226,46,212,254,59,253,157,79,171,26,223,240,155,124,126,26,159,219,82,250,208,252,37,55,195,240,218,189,251,198,162,253,123,167,196,93,169,252,119,251,58,159,86,31,9,183,199,225,
169,253,181,47,173,19,8,27,1,187,188,237,183,19,181,168,233,72,90,198,85,112,229,157,234,194,121,7,135,94,2,66,222,123,65,172,110,160,37,100,226,18,126,30,25,163,14,131,20,220,183,77,116,250,172,146,63,
3,23,137,181,90,255,0,120,218,46,248,108,251,171,84,199,128,105,172,135,120,38,157,33,128,50,226,76,218,68,131,38,34,46,15,236,11,231,242,221,54,181,54,74,89,82,69,87,153,154,154,154,245,148,102,89,240,
2,74,24,2,38,160,241,28,98,32,92,118,143,222,242,11,168,146,120,254,176,241,50,27,148,174,91,228,74,105,80,88,60,158,154,101,119,42,213,200,20,127,203,76,131,249,53,100,47,152,219,84,137,154,213,1,234,
52,223,255,0,64,71,207,21,11,121,73,189,213,136,22,244,136,235,21,169,203,231,96,126,104,250,127,9,77,240,252,54,175,126,241,168,191,94,235,235,226,46,212,254,59,253,157,79,171,31,63,9,183,199,225,169,
253,181,47,173,25,101,11,180,102,237,157,222,41,205,111,116,104,56,154,67,139,68,10,87,25,52,47,245,23,78,24,85,143,40,212,39,221,54,109,31,48,187,231,14,81,138,21,76,153,18,40,156,199,0,0,242,249,117,
141,121,230,62,220,91,74,173,105,89,218,236,83,109,3,195,113,55,145,210,9,43,32,39,41,206,50,236,60,163,221,175,125,69,111,232,34,88,154,171,226,48,171,76,144,154,134,178,0,98,73,211,57,75,140,226,70,
123,159,237,55,115,187,156,187,98,216,172,57,72,137,145,198,184,234,160,252,16,89,123,109,90,186,79,106,108,50,69,74,81,170,17,18,50,108,213,6,113,208,112,17,165,69,78,152,23,153,69,8,80,0,47,19,66,182,
22,226,192,224,109,110,42,100,234,178,223,87,168,62,131,183,113,71,3,48,15,18,204,211,29,128,197,141,230,126,211,220,251,158,246,214,150,30,130,54,54,218,137,148,234,34,119,220,247,128,82,195,128,84,73,
25,116,144,34,46,191,9,77,240,252,54,175,126,241,168,191,94,234,125,241,23,106,127,29,254,206,167,213,138,187,225,54,248,252,53,63,182,165,245,163,228,189,164,247,194,34,0,56,222,186,80,17,0,19,27,35,
81,248,20,4,127,56,121,102,204,110,1,249,0,71,92,124,69,218,159,199,127,179,169,245,99,159,132,187,223,240,212,254,218,151,214,137,30,216,79,106,203,158,25,202,145,25,171,62,203,85,215,145,166,138,239,
40,244,122,203,197,230,202,75,10,169,44,209,11,13,146,85,118,76,217,38,16,168,168,101,89,54,107,226,4,238,140,146,199,89,46,135,69,104,78,239,243,6,215,39,143,108,94,29,106,4,171,194,165,70,26,123,188,
244,168,153,61,238,76,76,184,76,0,103,49,99,108,47,42,239,48,217,84,205,103,222,145,169,70,102,149,36,37,187,252,131,187,16,7,119,154,170,206,109,38,44,52,200,206,174,170,88,189,35,132,55,199,182,252,
133,184,136,172,116,202,130,189,117,21,170,210,22,55,82,99,97,146,117,28,67,39,42,218,29,38,190,20,205,99,164,5,83,1,152,159,156,4,11,192,56,121,248,249,59,105,184,73,206,62,29,75,114,136,240,252,51,55,
25,244,236,109,239,52,175,217,173,118,248,201,219,31,30,27,118,67,240,204,220,103,211,177,183,188,210,191,102,180,241,147,182,30,27,118,67,240,204,220,103,211,177,183,188,210,191,102,180,241,147,182,30,
27,118,67,240,204,220,103,211,177,183,188,210,191,102,180,241,147,182,30,27,118,67,240,204,220,103,211,177,183,188,210,191,102,180,241,147,182,30,27,118,67,240,204,220,103,211,177,183,188,210,191,102,
180,241,147,182,30,27,118,67,240,204,220,103,211,177,183,188,210,191,102,180,241,147,182,30,27,118,70,65,89,237,191,184,56,137,23,46,157,61,199,66,146,181,251,108,97,122,86,73,67,159,196,205,213,102,97,
153,136,129,171,164,0,72,30,63,32,156,120,241,41,56,136,0,136,112,30,13,85,61,112,240,219,178,39,123,88,209,221,13,33,13,33,13,33,13,33,13,33,13,33,13,33,13,33,24,229,195,217,31,100,236,190,223,251,57,
236,55,168,165,125,177,246,195,213,158,201,251,51,224,87,245,231,180,190,186,255,0,83,250,139,213,189,79,23,226,191,205,250,28,221,79,71,142,187,104,248,222,50,253,223,87,143,168,105,211,61,90,167,195,
76,184,206,124,165,198,113,211,113,247,127,1,254,245,163,238,218,78,189,114,211,166,92,117,79,134,153,115,159,9,115,142,103,217,97,118,156,24,150,72,219,42,82,44,216,16,247,235,143,169,19,165,133,183,
238,133,43,9,101,14,23,31,185,113,157,76,181,33,160,158,201,226,4,125,144,49,170,222,181,241,189,14,14,188,102,189,60,223,189,254,248,61,247,63,120,120,107,61,90,124,73,75,187,226,75,189,175,76,191,121,
223,211,166,124,37,30,54,220,247,23,220,15,246,230,159,117,248,175,45,26,188,45,83,239,248,58,187,186,53,79,247,95,179,213,170,93,237,81,248,20,4,246,76,158,239,50,193,96,87,164,184,222,208,176,140,82,
228,165,240,37,75,154,203,69,86,29,53,98,16,198,33,144,81,66,87,238,101,180,105,250,102,26,97,70,178,47,136,184,185,49,228,124,81,135,178,227,223,135,15,71,196,214,48,115,58,116,75,194,215,62,58,244,112,
241,39,252,78,252,165,46,236,163,170,212,109,193,159,175,225,26,103,114,72,107,215,63,27,68,184,120,126,39,31,4,15,224,254,207,84,231,223,213,29,181,175,14,36,144,210,16,210,16,210,17,248,182,71,210,177,
117,217,233,56,40,53,236,211,145,208,178,143,161,171,109,158,71,199,57,176,74,180,98,187,136,232,54,242,18,206,152,197,49,94,89,218,100,64,139,57,93,22,233,25,64,50,135,33,0,76,31,116,149,30,162,173,70,
211,76,176,5,184,153,9,241,50,19,38,92,248,2,122,163,174,179,84,74,46,244,148,189,80,164,133,152,26,136,28,22,100,128,38,120,76,144,7,73,143,36,196,24,163,100,222,218,140,119,155,56,166,60,74,95,115,143,
75,186,137,185,6,115,114,238,234,167,113,147,151,83,53,170,118,53,70,182,105,185,9,70,10,140,145,19,77,153,29,29,71,0,80,3,137,68,84,13,188,44,105,96,245,97,87,196,34,212,120,0,72,106,238,126,203,139,
105,0,30,239,57,112,249,35,67,194,138,219,147,78,226,111,4,53,233,251,201,32,146,191,180,253,183,5,12,73,29,238,83,227,211,211,30,184,140,188,31,131,105,234,239,13,234,255,0,12,135,128,240,93,47,7,224,
250,68,240,190,19,161,250,15,13,208,229,233,242,122,28,156,56,121,53,168,45,171,81,213,61,83,227,62,115,141,242,93,58,70,137,104,151,9,114,151,68,187,35,250,117,196,125,67,72,67,72,67,72,67,72,67,72,67,
72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,67,72,71,255,217,0,0};

const char* MainPage::viperlablogomed_jpg = (const char*) resource_MainComponent_viperlablogomed_jpg;
const int MainPage::viperlablogomed_jpgSize = 14919;
//==============================================================================
// Binary resources - be careful not to edit any of these sections!

// JUCER_RESOURCE: ucSniffblueicon_png, 6338, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC Sniff-Blue-Icon.png"
static const unsigned char resource_MainComponent_ucSniffblueicon_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,61,0,0,0,63,8,6,0,0,0,83,170,192,226,0,0,0,25,116,69,88,116,83,111,102,
116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,24,100,73,68,65,84,120,218,236,91,9,148,84,229,149,254,170,94,237,85,221,213,251,78,111,32,178,245,2,130,
44,110,112,108,92,16,3,17,103,6,151,12,104,18,53,154,68,38,228,76,162,19,21,51,9,137,158,140,98,198,45,137,146,54,9,130,27,136,74,108,16,161,209,102,145,173,23,64,187,161,233,125,161,247,165,246,125,238,
253,223,123,77,119,211,24,52,234,73,206,248,115,110,191,170,87,245,150,239,174,223,189,245,0,190,94,95,175,175,215,215,235,235,245,207,183,52,95,226,185,151,144,20,146,204,39,201,86,100,172,213,160,72,
169,34,123,254,217,64,51,208,149,36,75,71,236,213,155,160,75,202,68,170,221,6,179,78,130,134,174,58,224,245,163,203,225,70,168,171,25,8,120,135,127,187,159,228,77,69,182,254,35,131,94,65,178,102,132,53,
13,38,100,45,189,23,11,175,94,136,217,147,114,113,81,114,12,204,6,61,12,58,45,140,44,122,45,36,66,223,218,239,194,201,134,38,28,60,176,15,187,118,237,66,205,71,100,104,87,223,112,47,224,243,190,244,143,
4,250,42,146,98,21,236,149,87,205,71,94,94,1,60,30,55,142,29,171,66,71,191,3,15,253,97,51,98,98,99,97,34,160,2,176,36,131,54,240,86,210,208,86,51,164,8,131,86,131,198,238,65,188,248,214,14,108,92,255,
59,244,86,236,25,14,126,229,23,229,250,159,23,180,157,100,157,114,35,48,153,76,184,235,238,123,145,158,158,14,73,167,19,95,136,68,34,56,116,240,35,84,214,54,96,233,93,63,70,109,249,126,68,219,99,80,244,
205,229,136,141,137,37,160,26,152,36,25,60,191,54,40,202,80,21,17,10,133,241,242,238,143,240,228,99,191,68,203,129,247,212,235,174,83,44,63,240,85,131,46,80,172,203,73,10,70,163,17,255,190,226,14,100,
100,140,131,205,102,131,70,171,21,95,10,135,195,112,187,221,248,248,196,113,236,223,191,23,57,57,185,56,211,222,14,107,106,22,86,63,249,146,108,225,17,64,101,240,236,9,122,177,95,6,175,33,229,253,126,
115,9,158,124,104,53,6,90,235,248,212,21,138,178,43,191,42,208,5,74,134,141,17,97,107,48,96,249,45,183,33,37,53,21,137,137,73,35,79,76,241,26,12,6,49,56,48,32,20,160,209,200,151,123,231,237,173,40,186,
247,97,76,157,53,79,113,113,217,173,101,151,215,12,133,128,25,1,88,53,126,161,0,157,36,161,207,19,192,143,254,243,39,216,254,234,159,212,100,55,255,243,2,151,46,16,232,79,73,158,87,182,38,182,104,94,126,
1,22,22,93,131,168,168,104,2,156,200,254,44,92,90,221,70,4,80,141,112,125,179,217,12,179,197,2,137,188,192,78,46,190,245,165,231,145,56,121,38,76,49,9,8,208,247,2,225,200,89,9,69,160,245,14,192,228,234,
134,22,33,74,118,34,88,96,33,203,47,185,110,33,172,209,113,248,160,116,151,137,118,46,39,41,33,233,184,64,12,147,20,101,249,52,127,35,73,113,252,204,215,235,245,194,125,147,147,83,200,170,41,228,198,81,
144,72,251,49,49,49,136,79,72,20,96,20,243,142,116,29,141,102,164,75,209,123,135,99,16,117,167,79,227,192,39,167,113,199,186,77,194,202,170,197,217,218,214,144,27,201,206,38,68,217,172,48,146,178,36,73,
7,29,149,57,29,229,10,190,15,45,93,235,149,77,155,240,227,31,175,190,80,139,63,66,199,172,137,176,234,194,97,14,141,249,186,243,124,241,73,146,85,102,179,5,147,39,79,65,118,78,142,176,154,176,24,137,61,
218,142,166,230,38,52,53,54,162,190,161,110,40,74,82,83,82,113,253,13,139,63,21,184,197,98,69,2,41,202,211,254,62,250,124,65,25,172,20,150,193,7,181,48,245,214,195,3,55,116,4,48,64,201,140,129,50,96,21,
56,203,178,155,151,33,64,161,179,246,151,191,136,113,56,28,111,42,192,27,207,147,112,87,165,81,248,245,244,247,35,232,243,21,210,113,133,99,129,46,231,36,149,148,148,140,194,194,66,88,173,54,17,175,108,
85,118,219,61,123,118,163,162,162,28,62,175,247,92,106,85,95,135,75,103,207,65,44,149,40,94,37,239,254,21,215,93,191,72,40,108,184,18,172,86,11,156,190,0,250,252,33,24,181,17,202,226,114,210,138,246,245,
194,223,217,4,111,116,20,52,78,231,48,192,186,17,175,89,150,44,89,42,238,109,203,150,205,217,135,15,29,108,32,240,197,138,103,14,7,191,42,42,42,42,198,229,246,32,218,106,69,183,124,207,253,163,99,250,
17,142,21,6,56,125,250,12,164,165,101,32,119,252,4,178,142,5,199,143,31,195,203,27,254,140,198,198,6,132,72,203,231,196,194,85,87,209,103,141,34,155,143,203,204,20,10,210,147,117,202,203,143,34,51,43,
91,142,115,37,214,183,109,123,27,73,43,31,130,46,49,13,193,8,199,50,16,164,120,142,105,46,135,214,213,11,131,197,38,146,96,40,20,58,71,2,126,31,218,235,106,197,235,44,170,8,181,167,78,97,234,212,105,136,
141,139,43,236,233,238,94,229,247,251,179,149,12,207,107,83,82,98,162,201,225,116,137,16,116,185,221,156,132,31,147,70,185,2,39,6,76,156,56,9,147,38,77,65,82,114,178,200,188,239,188,243,22,202,62,252,
64,196,83,12,215,88,131,81,36,40,159,79,182,182,221,110,71,117,117,181,240,134,151,138,139,49,189,112,6,197,162,150,72,138,23,221,221,93,72,38,79,225,28,160,2,175,169,254,4,206,137,179,17,73,72,69,128,
130,45,64,251,12,131,29,176,55,30,17,86,164,11,33,16,8,32,168,145,224,165,178,119,22,116,16,65,218,239,236,235,66,203,169,106,12,246,116,34,72,65,211,209,213,77,57,32,10,249,148,92,135,129,191,142,172,
156,205,245,222,98,54,161,183,175,143,175,205,165,174,113,56,232,57,42,217,152,119,217,229,72,35,162,193,55,216,76,177,107,37,75,95,50,115,22,38,79,153,42,50,117,32,224,135,151,92,133,153,23,175,148,148,
20,172,89,179,6,253,20,55,167,41,73,241,13,103,101,231,144,251,89,145,153,153,5,45,1,102,229,69,34,178,236,220,185,3,29,101,239,2,179,175,69,192,108,19,160,211,106,74,161,233,109,135,153,110,158,193,249,
252,65,204,251,198,50,100,76,152,8,15,93,171,251,76,187,0,78,103,128,45,54,1,177,20,126,94,183,11,167,170,79,96,71,233,30,68,83,21,97,133,13,3,159,210,214,214,202,214,133,129,66,67,177,242,163,163,75,
86,182,10,122,54,197,37,91,147,111,48,39,119,188,40,51,236,110,199,137,90,238,219,183,15,93,93,157,67,128,121,189,249,230,155,120,254,249,231,69,14,152,52,105,18,54,111,126,67,36,43,210,244,16,80,150,
48,185,48,43,102,223,222,50,68,72,113,174,143,15,67,51,231,122,88,200,202,201,31,151,138,100,165,33,5,5,252,1,164,77,45,68,114,214,120,186,67,9,177,201,169,72,205,157,32,123,14,129,103,165,147,1,17,69,
224,147,211,210,113,186,182,22,109,237,109,72,166,68,202,89,154,239,85,5,207,158,209,208,192,44,86,96,19,241,174,29,43,117,243,205,10,87,10,134,208,65,23,233,235,235,69,89,217,135,232,36,176,9,9,9,66,
216,189,71,47,6,189,106,213,42,113,145,61,165,187,112,134,142,229,115,12,119,207,99,85,103,171,75,168,169,6,125,79,174,66,124,213,251,232,239,233,18,53,219,49,56,136,126,138,193,140,130,153,240,16,50,
47,213,109,31,73,88,210,35,59,127,6,46,93,180,4,209,169,25,232,37,229,245,244,246,162,189,179,155,170,203,120,193,27,152,35,112,77,231,191,97,122,237,241,120,232,158,166,35,53,53,237,188,228,132,181,176,
138,201,71,10,125,137,9,136,106,29,142,229,252,130,2,113,130,52,210,44,127,198,201,141,137,73,111,111,143,56,184,162,162,2,165,165,165,66,14,28,56,64,74,42,67,101,101,37,213,246,100,81,215,249,60,205,
205,205,120,111,199,246,17,55,16,234,110,131,110,176,27,113,113,177,226,58,126,178,242,184,89,87,32,121,226,84,114,101,250,156,16,176,75,135,35,50,28,13,213,237,184,148,116,164,179,219,19,168,205,175,
108,132,75,201,244,145,17,156,64,254,195,202,182,88,204,168,175,175,111,80,27,150,209,37,75,16,250,234,79,62,33,87,73,19,90,83,79,224,114,185,68,93,102,215,236,233,233,30,113,16,3,94,185,114,229,217,210,
37,187,19,1,240,145,171,191,46,60,135,179,122,119,119,247,152,164,128,195,101,124,110,14,6,7,29,194,162,87,205,93,0,71,144,106,55,93,87,79,245,91,22,186,89,210,130,142,74,156,142,202,158,78,50,32,53,111,
38,190,243,235,167,177,249,133,103,113,100,231,54,100,17,129,226,60,18,209,200,152,213,123,31,81,50,199,160,161,156,234,151,19,168,152,68,114,33,115,84,12,105,153,200,32,157,37,72,42,119,186,220,136,177,
71,137,120,141,141,141,35,215,13,80,124,5,5,7,95,183,110,29,178,179,179,133,176,155,151,148,148,136,18,38,119,97,102,10,145,190,243,183,108,148,245,217,115,92,148,152,226,103,47,64,194,197,211,112,232,
141,13,104,61,249,49,188,100,77,63,197,168,49,202,46,44,207,22,15,146,176,82,28,244,66,111,52,161,96,222,21,184,242,198,101,104,108,106,66,213,161,253,176,208,245,134,44,79,127,56,204,218,218,218,134,
166,50,99,209,208,122,78,106,122,2,82,180,248,102,196,198,39,42,187,41,195,38,198,81,215,19,70,31,185,116,123,91,11,58,40,121,184,169,240,215,212,84,227,251,223,191,79,128,85,173,204,110,190,103,207,217,
246,151,179,120,83,83,227,152,160,47,34,46,16,29,29,141,142,190,126,220,254,242,251,104,61,186,31,219,127,190,10,153,25,164,120,202,29,6,146,168,204,139,144,161,119,34,226,25,196,193,142,48,110,184,243,
94,228,205,189,66,112,115,30,70,200,91,192,51,56,128,255,254,206,45,200,73,75,21,231,102,195,108,223,94,194,161,144,125,190,68,102,23,89,220,78,101,73,171,199,206,119,222,64,107,107,43,149,20,13,137,132,
198,206,126,156,106,233,34,10,218,2,151,195,1,3,185,108,56,28,18,53,249,169,167,158,18,89,92,93,243,231,207,23,245,91,93,162,156,80,14,24,189,226,136,189,177,235,115,27,154,115,221,191,64,103,179,163,
241,68,37,122,72,1,31,159,60,133,131,229,21,48,134,6,176,38,123,63,126,57,227,52,214,94,214,133,87,139,218,225,124,225,78,252,118,245,61,240,145,233,253,67,2,81,247,25,56,151,72,150,218,218,83,12,184,
120,56,83,211,140,209,100,148,106,174,90,142,200,34,138,209,245,63,3,106,14,227,226,233,115,133,240,151,179,82,98,97,51,234,4,13,13,81,204,250,189,46,81,179,43,43,43,168,60,248,69,124,171,139,173,189,
96,193,130,161,247,70,227,89,66,3,193,195,205,72,167,220,193,117,188,207,237,197,242,77,31,192,76,188,126,203,138,5,88,152,216,141,159,92,159,138,166,94,63,14,247,196,224,142,124,223,57,10,43,126,191,
21,135,38,221,139,69,100,117,182,50,199,250,107,235,126,133,238,227,71,40,121,90,69,104,148,238,41,229,251,203,30,14,122,180,165,229,25,87,60,185,6,181,112,120,96,61,52,55,221,135,154,242,253,216,253,
230,159,41,46,90,80,211,62,128,163,13,61,56,113,198,137,154,222,0,26,221,122,244,135,77,152,72,12,78,79,181,93,77,104,108,245,225,150,231,53,28,48,103,106,27,113,103,63,41,138,107,126,206,226,91,33,145,
149,43,183,190,140,171,227,206,96,237,77,25,176,155,37,228,165,155,97,128,127,204,176,184,253,242,100,180,110,121,138,74,27,151,53,136,50,118,116,251,91,194,115,56,215,48,81,34,192,197,163,155,145,49,
65,107,226,41,115,27,244,34,25,216,255,237,135,72,127,174,20,145,180,92,28,42,121,21,39,14,236,134,195,75,140,44,76,52,147,196,29,150,208,31,49,161,83,74,64,122,246,69,100,241,42,145,45,217,226,236,226,
187,119,239,70,65,65,193,8,107,139,1,132,209,160,100,120,63,28,196,190,244,4,248,197,27,10,241,30,197,114,201,199,46,97,97,145,217,221,90,204,72,31,187,25,212,145,199,229,199,4,80,123,162,138,238,37,140,
183,95,124,22,201,9,241,162,55,112,144,139,55,202,57,100,205,232,227,198,36,39,204,138,76,4,56,86,47,33,73,175,67,122,90,54,230,62,246,26,102,173,126,18,157,77,167,112,248,173,98,116,182,183,192,19,33,
224,20,239,238,48,9,197,124,183,57,21,41,89,19,48,97,194,69,34,139,47,93,186,84,0,47,38,62,174,198,55,179,37,193,225,201,141,77,38,163,216,55,72,196,100,255,83,15,195,211,211,33,92,189,165,47,128,199,
75,206,136,207,126,187,151,56,184,119,240,83,39,4,125,100,97,182,242,241,237,91,4,229,100,198,214,208,80,207,10,45,30,171,229,28,19,180,158,110,74,5,156,76,146,72,175,227,105,155,191,232,22,172,124,133,
186,166,25,151,227,227,29,27,209,68,52,210,73,192,93,36,78,2,239,32,224,157,182,113,200,153,112,49,30,125,244,231,67,241,205,89,157,129,11,50,66,172,140,123,242,40,114,109,189,78,15,35,149,23,86,50,131,
29,190,202,78,14,98,230,99,117,8,19,31,103,23,31,107,249,93,126,180,12,16,101,189,228,50,236,44,126,142,114,141,1,157,29,29,168,166,106,210,220,210,50,166,149,199,34,39,114,93,37,247,76,36,30,156,64,96,
163,105,27,69,55,100,213,105,97,161,173,217,20,135,219,30,223,128,131,175,60,135,191,174,123,16,142,222,78,196,205,229,193,129,60,53,209,194,72,253,112,14,53,33,61,194,202,44,12,158,59,48,118,115,102,
105,22,158,136,112,143,76,202,245,106,117,99,86,206,102,114,111,201,172,199,22,162,154,247,93,22,141,148,84,27,41,230,172,141,130,190,32,14,157,232,197,209,105,183,35,212,229,198,222,151,95,132,46,224,
21,94,228,162,196,74,205,82,241,121,6,11,99,91,154,153,80,44,177,160,56,2,29,67,160,163,105,27,69,91,155,50,185,100,118,116,229,173,223,195,170,13,31,34,72,77,255,153,125,219,208,67,241,221,75,22,239,
33,105,211,69,193,21,51,78,144,18,149,161,49,112,150,21,43,86,8,206,204,157,88,74,246,120,228,220,245,48,194,68,21,207,9,49,186,121,41,20,64,210,132,2,92,243,76,45,182,239,59,3,103,151,11,158,62,15,28,
29,78,172,223,222,130,187,170,179,144,114,231,131,168,216,240,28,66,30,39,252,116,30,31,157,55,20,8,170,179,114,92,8,104,225,143,145,214,90,68,17,112,59,89,67,88,90,5,204,116,80,165,134,228,13,89,147,
242,240,221,39,54,192,95,95,5,103,245,65,12,82,108,179,12,144,2,170,237,185,148,216,198,139,178,165,18,22,81,102,200,205,111,188,241,70,52,181,52,35,177,232,102,100,92,126,45,102,220,255,139,115,6,19,
177,148,3,184,236,60,241,63,191,193,147,47,108,196,207,14,25,145,254,240,113,228,255,126,0,211,119,101,99,253,204,53,24,255,196,54,4,157,3,104,121,233,113,232,37,73,153,192,134,100,26,38,79,109,47,8,116,
191,0,237,26,132,145,178,161,141,220,201,70,86,230,73,164,202,127,245,67,192,101,143,152,50,235,114,44,190,231,167,64,249,123,128,115,144,220,74,35,36,0,61,122,226,199,11,107,51,69,85,235,54,11,187,58,
91,250,196,145,131,84,91,129,188,91,239,197,204,97,192,121,224,96,162,178,195,93,83,91,91,59,94,171,9,98,230,77,223,195,15,126,248,3,244,247,14,96,242,127,60,134,140,27,110,23,199,54,191,184,86,132,11,
143,137,25,204,48,229,245,95,40,104,193,27,61,181,21,208,144,198,44,228,202,44,6,141,66,248,135,128,67,121,47,43,97,217,189,15,32,33,45,19,56,78,135,147,123,171,210,98,29,135,248,248,4,97,93,6,202,224,
25,52,103,246,37,75,150,224,248,198,103,225,57,89,33,110,62,241,226,188,179,137,134,0,216,169,235,74,167,142,46,121,242,76,196,103,23,160,232,146,137,112,83,203,169,13,248,208,252,202,51,164,248,8,2,237,
141,232,223,249,186,40,173,146,86,18,181,25,67,189,214,249,39,164,99,37,178,10,87,93,101,161,223,235,35,82,16,17,160,84,126,171,83,248,173,150,183,195,246,243,251,101,247,61,128,223,253,215,247,128,130,
235,72,19,102,145,155,66,100,237,96,198,52,120,79,215,98,211,166,141,162,132,141,32,5,4,126,255,131,43,177,224,182,239,66,171,179,141,232,134,152,200,204,156,59,15,47,236,107,39,238,109,195,55,175,155,
139,181,15,172,18,0,7,79,86,137,27,111,92,255,107,226,236,81,226,56,79,32,40,202,161,74,6,63,173,196,141,149,200,196,1,109,135,118,18,205,12,12,209,59,118,105,182,172,164,90,92,35,111,85,107,207,45,186,
81,62,186,233,4,165,86,34,22,1,150,0,122,116,177,194,202,108,225,209,139,7,14,245,245,117,56,86,178,25,161,131,37,184,230,154,107,197,232,201,79,199,185,200,170,149,110,59,204,182,68,20,100,80,108,63,
241,4,89,218,41,70,191,172,154,48,245,225,193,163,187,96,36,22,200,86,230,145,146,214,100,25,145,155,62,139,165,153,59,174,106,252,240,109,236,234,111,69,114,114,18,50,39,229,35,49,61,19,73,233,89,50,
112,141,102,68,119,163,165,45,143,148,114,40,177,213,59,123,168,128,122,135,230,221,61,26,43,114,99,226,71,112,242,225,147,22,94,220,108,244,81,70,231,24,206,155,58,21,211,166,78,163,222,189,1,113,249,
11,197,231,189,53,31,97,227,166,77,162,150,179,53,83,46,185,2,222,29,47,195,76,113,207,67,2,55,1,230,110,202,154,57,25,206,211,194,171,139,63,43,104,142,235,134,174,170,15,178,73,70,124,16,79,113,59,153,
18,215,172,171,111,196,236,162,197,35,220,155,95,231,78,201,71,61,209,78,100,207,80,186,119,101,58,98,142,17,109,38,91,150,173,206,217,92,21,94,121,249,121,184,242,202,43,241,209,129,143,112,242,228,73,
1,62,43,43,11,73,221,135,224,129,11,47,60,255,8,180,33,63,186,123,122,197,8,57,46,57,21,254,61,175,139,154,204,195,75,39,121,64,252,180,203,209,123,234,168,90,170,42,63,43,104,85,83,107,210,126,244,71,
56,244,22,56,137,30,70,78,151,163,167,174,28,101,219,222,64,25,53,5,22,106,234,25,120,254,236,43,144,146,145,37,128,187,137,239,138,177,5,117,95,195,102,54,148,70,101,190,61,60,166,217,202,172,0,222,110,
219,182,13,93,93,93,130,200,204,158,51,123,8,124,223,190,18,234,231,15,97,238,37,51,112,164,252,168,124,195,196,228,198,25,53,56,49,56,72,237,170,94,12,54,152,191,91,233,126,186,169,227,83,166,63,159,
235,87,75,38,202,13,210,184,105,49,154,194,69,8,241,204,90,47,243,100,56,250,232,147,42,160,179,14,24,104,35,51,6,70,30,153,58,153,44,61,115,216,153,53,48,14,182,194,119,252,125,249,7,190,81,139,129,14,
31,54,164,166,166,162,168,168,72,40,227,240,225,35,104,168,175,23,150,143,141,143,23,195,43,79,76,10,236,174,110,52,210,254,32,185,118,83,115,51,162,199,79,199,153,234,195,8,122,156,253,74,211,52,240,
121,127,170,125,68,112,215,139,169,31,142,78,82,105,18,183,71,148,69,136,0,120,149,17,176,143,180,235,87,94,183,144,87,101,207,2,108,241,35,47,209,79,60,248,84,217,152,160,69,198,142,165,174,110,202,149,
64,199,105,74,132,199,233,124,30,193,207,231,204,153,131,201,147,39,139,164,166,186,189,213,102,19,238,28,36,235,186,168,37,61,121,234,20,50,103,47,66,253,190,183,85,174,253,232,223,243,251,180,176,54,
204,228,131,211,174,61,247,83,229,103,217,179,99,77,202,214,245,135,128,9,243,40,179,141,26,189,181,18,144,182,19,98,78,198,46,61,124,128,152,147,147,3,92,52,155,120,237,237,114,242,11,120,128,147,7,129,
35,239,8,240,28,219,119,221,117,151,56,174,172,108,47,154,232,24,158,136,112,2,107,110,165,78,143,40,111,127,243,73,53,150,11,47,228,41,133,191,245,163,252,18,145,205,199,229,3,89,211,201,170,222,177,
191,197,37,138,129,165,76,34,79,48,143,248,165,82,172,234,93,176,107,253,162,46,243,176,159,183,34,99,170,110,125,217,191,2,249,69,178,213,249,231,94,222,18,224,240,230,199,129,238,38,241,131,224,55,136,
186,242,254,122,114,235,48,207,195,125,62,84,86,85,34,34,25,72,79,78,40,79,52,93,208,211,72,218,191,241,249,86,145,212,154,171,16,109,150,160,15,58,168,249,237,28,41,46,18,255,128,172,20,142,251,209,128,
217,253,29,93,226,103,31,46,91,108,93,149,142,222,127,255,253,50,237,188,232,82,49,117,225,201,166,65,17,99,84,44,204,203,31,130,54,41,75,120,200,187,37,37,104,105,109,21,191,87,115,135,198,241,204,115,
55,5,240,186,207,242,248,213,133,60,126,193,110,94,42,25,45,133,51,238,124,20,167,96,70,186,38,140,24,170,145,181,134,104,116,81,22,13,119,80,204,182,83,194,9,6,206,61,250,60,174,205,139,223,15,26,98,
96,90,241,43,81,126,180,148,51,52,138,165,85,118,22,30,232,194,224,51,247,136,215,179,102,93,138,88,62,7,229,148,206,238,30,84,84,86,168,68,100,254,103,121,248,70,123,1,223,225,147,45,13,249,220,253,229,
235,31,198,181,9,122,132,73,87,61,46,31,220,13,53,8,215,148,19,235,63,57,54,96,119,159,0,204,113,57,188,211,82,103,104,3,3,3,48,94,122,3,12,122,3,140,44,6,3,76,100,113,38,29,102,218,90,200,226,182,36,
106,81,39,205,17,199,180,183,183,35,41,49,17,38,186,107,5,112,195,103,5,252,105,117,122,244,226,102,124,126,208,231,41,253,240,143,107,99,174,185,227,39,168,108,104,135,163,238,228,249,143,224,56,167,
140,205,99,34,46,75,60,48,228,120,86,107,53,55,33,26,163,5,58,75,52,116,174,62,24,227,83,137,237,73,98,176,160,19,132,231,172,181,3,25,19,9,94,133,248,97,49,218,32,97,203,222,125,106,23,181,244,243,60,
94,245,121,158,46,42,182,90,173,133,247,60,248,27,248,140,113,248,107,217,1,212,181,182,157,107,225,58,202,192,158,126,60,242,200,35,2,168,74,57,213,159,128,120,130,162,49,90,161,79,205,69,202,183,31,
163,94,93,130,65,146,69,79,110,206,188,94,0,167,127,134,189,127,192,194,75,243,113,188,170,66,52,46,68,69,43,20,192,141,95,213,115,100,118,133,159,207,191,249,182,111,99,217,183,238,70,75,91,11,246,236,
218,129,119,182,190,161,128,238,63,151,180,140,142,43,123,18,44,211,175,70,204,188,155,168,169,176,195,196,61,52,37,41,51,109,141,36,81,6,29,166,196,91,49,209,226,134,183,253,24,158,123,246,105,148,31,
61,162,246,6,43,255,158,7,232,254,158,199,36,5,121,73,75,75,195,218,95,253,26,133,51,231,162,119,192,137,247,62,170,196,222,211,109,56,218,218,13,39,63,87,193,19,13,178,34,119,66,220,48,72,150,40,24,82,
199,203,207,145,16,72,147,78,34,160,188,213,33,195,102,194,228,132,40,76,75,180,97,98,156,153,18,127,39,54,20,63,139,13,127,249,19,255,128,216,175,144,143,167,190,210,199,36,185,195,161,70,93,67,29,141,
158,182,166,213,171,87,79,122,253,245,215,158,113,56,28,51,249,41,133,111,221,121,15,138,174,189,1,81,209,118,193,91,218,6,220,104,115,184,113,184,165,71,100,102,126,172,162,182,199,5,187,201,128,116,
187,85,60,7,18,77,22,157,16,103,67,150,221,12,171,94,38,53,45,77,245,216,242,250,95,176,117,243,38,145,188,226,226,226,75,111,185,245,150,159,81,217,171,38,101,121,244,58,189,95,175,215,5,121,168,255,
165,129,230,209,14,137,142,250,92,171,223,31,136,34,130,111,83,132,223,91,158,126,250,127,103,111,125,115,203,242,142,142,142,139,57,113,205,187,236,10,220,176,120,177,120,56,54,47,63,95,40,32,172,16,
56,230,207,242,179,39,195,201,93,4,101,31,126,136,19,199,42,80,242,238,219,212,112,136,68,69,89,63,187,234,250,69,139,222,185,251,238,187,15,234,245,6,183,193,160,119,25,12,6,22,39,137,131,222,59,245,
122,189,215,108,50,133,191,80,208,220,193,16,96,139,207,231,183,19,11,138,242,249,124,54,146,33,224,180,223,234,83,94,31,59,118,44,253,213,77,27,47,175,174,174,206,117,56,6,109,234,57,184,109,84,31,200,
225,49,48,3,87,217,216,0,101,116,126,98,88,93,137,137,137,253,228,53,117,139,23,223,120,116,202,212,105,109,12,140,74,153,139,172,234,84,192,58,141,70,3,239,115,26,77,70,7,237,31,160,215,131,4,62,168,
211,233,190,24,208,4,84,242,251,124,12,56,218,235,21,160,73,252,2,60,1,85,148,64,224,253,67,251,132,52,52,212,39,28,171,170,76,235,233,238,137,174,171,63,205,201,79,234,236,232,48,208,119,180,202,131,
180,225,164,228,228,0,221,64,56,59,39,119,48,62,62,193,57,45,47,175,61,51,51,171,71,6,38,131,52,42,32,13,70,2,40,239,119,136,247,6,163,195,100,82,196,104,236,39,79,32,15,208,127,49,117,154,106,101,72,
171,149,188,146,36,25,117,58,201,24,10,233,124,68,246,13,180,213,19,241,215,211,107,29,109,165,112,36,34,69,120,12,170,120,108,78,78,110,40,35,99,156,131,188,196,76,185,192,20,98,9,133,248,24,157,114,
222,8,37,182,0,157,215,79,22,242,178,144,181,220,44,4,150,183,46,245,61,245,205,20,199,242,119,232,30,124,68,69,125,242,150,68,43,121,52,90,173,79,171,213,124,113,228,132,89,18,89,209,77,37,211,79,84,
209,69,23,138,162,11,138,155,242,251,245,108,13,107,32,224,183,112,108,115,24,80,130,51,7,130,1,83,144,182,193,80,208,24,12,178,130,66,6,6,203,202,33,197,104,85,208,36,97,58,103,144,69,71,224,137,83,251,
4,120,161,128,33,37,120,84,37,40,49,173,196,179,97,208,160,215,59,104,191,207,98,54,71,190,180,236,205,15,193,80,230,214,16,48,157,2,204,18,8,6,45,138,53,133,69,101,9,153,101,111,16,150,85,69,0,166,156,
165,149,199,180,2,116,68,163,37,79,210,16,112,73,27,160,108,206,150,15,144,82,201,179,84,203,82,182,214,11,69,184,117,108,125,249,189,159,94,135,204,99,60,225,244,165,255,119,6,69,9,92,198,180,100,85,
77,40,24,162,109,72,10,201,162,37,160,90,5,172,20,9,71,52,148,179,53,195,7,9,74,99,17,145,67,72,203,194,150,15,19,240,144,36,105,67,58,73,23,38,5,144,232,35,4,52,98,250,156,32,191,94,255,95,215,255,9,
48,0,42,200,112,186,204,62,57,79,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* MainPage::ucSniffblueicon_png = (const char*) resource_MainComponent_ucSniffblueicon_png;
const int MainPage::ucSniffblueicon_pngSize = 6338;

// JUCER_RESOURCE: ucsniffshield_png, 4843, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC-Sniff-Shield.png"
static const unsigned char resource_MainComponent_ucsniffshield_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,106,0,0,0,34,8,6,0,0,0,183,42,76,150,0,0,0,25,116,69,88,116,83,111,102,
116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,18,141,73,68,65,84,120,218,236,90,121,112,92,119,125,255,188,115,239,75,183,172,107,45,249,144,28,130,54,
7,56,142,147,122,115,147,4,136,184,91,210,192,134,246,15,142,150,10,58,67,11,237,76,76,167,109,160,5,108,96,134,99,218,12,74,8,12,165,33,145,67,78,156,99,157,196,9,4,7,175,124,200,150,172,99,117,223,187,
239,237,253,246,93,253,190,39,217,150,98,27,39,109,96,96,170,159,230,59,111,223,241,187,190,159,239,241,249,190,39,96,189,173,183,245,246,214,53,238,2,215,91,72,186,72,146,36,202,5,238,199,72,74,36,115,
23,24,227,14,146,118,146,129,117,53,255,223,27,115,129,235,207,95,123,197,37,209,217,37,25,167,146,147,201,21,192,78,183,48,195,176,225,182,10,39,102,179,101,228,202,154,117,45,190,186,179,223,31,136,
110,174,240,227,232,244,156,84,46,151,195,116,73,94,87,245,91,15,212,174,214,198,250,248,51,95,188,1,94,179,132,35,211,26,38,83,69,12,207,72,54,48,79,14,46,192,37,58,112,67,131,27,183,120,12,36,38,179,
228,86,28,102,116,32,228,113,163,150,99,224,47,22,32,144,175,62,172,185,240,244,209,227,61,52,230,221,235,170,126,235,67,95,207,189,31,187,41,188,145,205,34,35,41,16,74,5,4,202,69,72,57,5,247,29,75,99,
186,250,114,164,66,155,113,104,201,64,50,157,198,140,193,34,226,113,96,51,171,163,26,58,92,166,6,147,254,202,170,142,250,160,15,175,166,114,17,242,170,158,117,175,122,107,129,234,108,104,104,252,202,103,
47,245,33,47,23,49,154,210,240,212,80,22,63,62,182,128,251,134,74,144,195,215,0,109,111,39,215,169,1,124,21,152,210,61,24,21,106,48,81,200,224,114,81,135,166,169,208,77,147,4,208,200,87,141,92,14,70,125,
35,121,227,172,68,99,31,88,87,247,255,190,241,175,59,143,189,119,91,3,250,167,10,248,143,19,105,252,38,203,195,108,189,12,90,71,19,76,119,8,112,56,41,88,18,2,4,6,92,30,160,186,129,16,81,113,98,64,196,
47,242,9,236,34,222,161,233,58,166,200,155,84,195,68,133,83,68,187,200,33,238,112,196,20,69,249,242,186,186,223,58,160,186,124,161,58,124,118,255,203,136,196,254,25,91,107,90,49,62,49,132,220,43,189,208,
135,14,1,217,37,242,164,74,8,159,248,58,244,249,49,48,85,77,118,39,243,210,171,161,188,54,128,156,86,192,193,156,1,119,100,59,88,142,197,192,240,32,188,227,99,8,55,54,134,7,134,135,59,87,230,216,77,210,
77,50,246,86,111,230,189,151,182,182,148,56,33,92,48,184,228,75,71,250,223,208,248,31,218,82,219,162,136,238,112,142,119,226,185,196,137,3,127,12,64,181,52,54,53,133,123,127,149,128,190,227,131,152,8,
53,33,123,242,85,100,158,188,15,166,78,204,78,116,217,15,181,86,7,49,191,56,6,182,162,134,174,43,96,252,85,208,150,198,80,161,230,48,165,168,40,52,182,34,232,112,128,37,207,115,108,104,134,217,180,17,
66,114,212,54,2,142,227,162,150,80,206,178,192,26,75,220,217,222,249,157,215,22,247,206,228,52,148,121,14,6,47,66,103,121,168,46,95,247,139,125,253,125,27,3,238,61,21,110,71,68,32,242,194,11,130,13,62,
245,79,60,223,63,242,185,211,139,190,119,123,211,174,87,102,114,221,37,19,93,185,212,18,57,187,73,241,156,193,229,53,126,24,28,223,171,58,61,61,199,71,39,246,189,126,227,223,186,182,249,158,103,70,228,
152,92,80,194,70,201,202,171,57,92,89,23,132,84,214,226,69,135,183,123,106,102,182,239,141,42,241,171,87,109,216,243,210,120,54,162,112,60,133,124,30,227,172,171,123,40,57,110,247,255,100,91,229,61,99,
138,222,165,57,156,145,76,217,144,150,114,133,158,161,116,206,94,255,215,222,217,212,249,194,164,180,187,236,112,70,75,172,16,76,203,217,196,184,46,196,210,233,84,223,111,3,42,220,238,115,227,153,254,
1,212,239,172,65,165,192,99,73,202,128,109,124,27,116,2,2,51,131,104,239,216,70,212,219,143,244,139,15,34,248,241,175,128,117,120,96,208,22,83,51,67,120,98,120,2,219,154,26,224,113,58,176,237,230,59,48,
59,120,28,233,190,67,152,153,159,67,32,24,196,251,222,255,129,221,253,199,143,99,96,224,164,69,229,237,133,12,47,169,193,193,130,25,5,1,100,50,44,41,153,129,105,24,144,75,229,160,77,243,221,174,136,207,
201,71,13,150,181,9,138,149,255,86,211,212,127,217,213,250,241,231,103,148,158,146,224,58,71,121,222,149,8,97,50,70,215,53,151,108,233,121,233,248,224,25,230,249,195,219,54,253,224,193,83,249,88,201,177,
182,159,135,140,133,136,107,84,97,248,120,211,165,29,209,95,30,61,241,134,192,234,91,82,35,57,78,136,46,135,23,29,227,211,115,246,250,255,237,154,230,61,79,142,231,186,65,235,135,90,134,155,65,208,244,
186,34,72,231,240,194,7,183,180,60,120,50,27,207,112,98,16,154,97,151,171,33,151,24,201,192,21,36,160,206,153,131,93,245,59,26,36,139,109,168,174,198,198,64,0,109,161,42,188,115,251,117,208,105,2,84,134,
193,69,110,66,94,53,236,7,155,43,188,144,31,217,131,122,162,226,245,44,131,98,226,89,100,10,5,36,229,28,170,58,34,184,228,134,219,241,238,191,250,59,252,249,151,191,142,237,55,191,27,126,159,15,162,32,
226,178,203,175,64,83,115,243,153,154,236,208,56,145,13,193,65,158,36,64,183,132,44,82,231,4,204,165,150,9,34,67,158,169,209,125,141,128,180,68,101,5,104,43,75,254,246,45,91,238,120,110,17,61,89,214,1,
85,112,94,80,52,158,198,55,180,216,245,111,223,186,203,234,71,94,188,235,167,147,122,44,199,90,99,59,109,81,40,236,41,220,217,115,142,231,131,130,161,117,191,81,143,90,82,153,229,125,172,136,40,10,246,
245,254,37,45,102,88,243,175,72,145,230,228,172,60,111,21,158,131,217,232,201,34,19,52,40,90,88,162,82,191,50,137,223,227,186,120,142,10,57,5,220,210,92,139,199,158,253,49,26,35,215,161,166,170,14,127,
114,235,71,241,194,79,190,65,33,137,133,76,160,117,109,223,1,69,41,161,239,251,223,197,75,95,188,101,205,96,146,148,134,184,165,19,15,254,211,23,176,177,125,43,54,84,248,176,235,206,191,196,85,119,124,
8,47,254,244,135,144,168,118,214,84,237,204,243,99,233,2,116,98,143,38,133,73,219,163,232,104,144,207,44,44,45,45,3,37,88,224,112,80,137,187,148,53,162,252,68,84,220,172,105,223,59,184,160,237,157,46,
153,107,230,55,200,27,53,85,221,203,114,124,140,231,185,224,26,111,49,84,75,241,7,246,207,232,209,73,229,172,125,58,120,62,89,54,16,29,155,153,237,110,174,173,58,3,142,195,68,228,124,10,171,168,168,8,
100,179,217,136,207,231,75,166,82,41,59,15,242,14,161,135,229,196,120,170,80,70,174,88,34,175,244,36,115,196,120,167,203,102,208,50,64,171,165,75,42,100,13,209,122,143,199,62,63,153,46,134,117,247,217,
37,142,166,114,61,1,191,175,39,36,10,201,139,2,165,146,146,54,25,5,236,244,243,120,225,190,47,161,237,79,191,132,156,211,143,218,107,63,132,185,209,19,200,167,199,241,253,239,125,7,213,228,117,62,159,
31,217,108,102,205,96,111,187,245,3,40,12,191,134,143,251,95,70,131,122,28,29,196,8,31,125,224,33,184,62,245,83,123,236,233,169,73,204,204,76,239,62,253,188,230,114,216,57,201,2,71,179,40,189,190,12,134,
64,0,89,198,80,54,89,228,21,202,95,4,174,162,170,80,202,42,130,34,139,95,125,100,107,231,187,158,158,11,159,83,107,112,220,222,197,165,165,207,53,214,215,39,242,114,166,103,245,61,39,207,218,90,57,56,
145,199,66,254,44,192,60,25,224,244,252,98,56,20,10,246,100,74,106,68,164,240,71,129,2,110,42,92,126,116,211,198,209,222,1,41,156,231,56,219,163,75,162,59,89,74,167,194,26,231,132,192,211,24,109,77,113,
209,235,239,82,12,37,86,212,217,104,70,41,128,119,186,112,119,179,51,122,42,36,68,231,24,138,18,204,178,81,136,14,22,181,110,62,238,100,17,191,103,103,107,244,149,57,98,200,236,89,245,7,125,222,152,215,
33,196,156,172,25,61,31,209,90,3,84,201,202,15,20,47,195,170,68,10,84,112,224,219,159,132,239,182,79,3,53,141,168,244,133,80,74,109,66,158,186,180,86,233,200,16,72,125,137,195,103,250,110,217,178,21,67,
135,94,193,232,115,179,248,194,231,58,232,169,178,253,226,227,82,247,52,30,29,29,197,240,171,7,49,50,50,220,179,122,17,58,133,182,2,57,152,74,224,156,17,77,131,203,229,178,129,42,40,10,228,92,129,0,42,
131,8,8,213,105,116,207,231,194,119,19,139,209,180,36,225,60,64,245,90,71,159,199,221,59,57,51,19,125,221,109,187,67,70,206,74,115,233,210,234,235,22,224,113,166,92,148,222,187,177,162,247,55,69,38,126,
160,175,255,126,235,70,187,94,247,200,156,224,90,54,8,59,141,148,194,78,23,133,46,151,243,244,14,162,140,166,236,149,13,198,54,48,191,207,107,151,47,125,11,26,166,77,7,229,171,213,239,128,136,44,81,201,
82,166,61,60,59,167,97,209,16,105,60,125,205,75,34,50,20,132,196,139,179,62,73,165,252,144,163,193,116,154,172,82,205,227,61,33,47,94,221,247,175,24,223,112,25,156,55,222,13,177,178,14,166,47,136,23,127,
249,40,110,172,175,198,109,183,191,7,79,60,254,115,180,182,182,129,167,34,56,63,246,107,123,160,79,254,104,6,15,197,106,160,22,57,60,151,105,192,127,125,103,47,38,143,246,65,150,229,221,171,39,183,172,
52,147,45,217,224,88,27,176,60,167,76,94,19,10,248,109,173,230,243,5,88,128,92,179,181,6,205,110,1,188,172,80,194,231,177,189,45,20,252,204,13,151,64,215,168,200,86,13,91,116,221,192,187,30,34,198,74,
10,59,49,52,44,95,168,192,174,12,186,123,55,168,230,110,29,122,240,218,230,16,174,164,124,91,73,250,82,100,37,152,147,178,49,159,97,198,90,182,54,239,78,57,132,46,37,159,73,220,20,244,116,205,82,61,120,
104,169,108,175,177,64,113,50,184,42,143,56,24,166,75,42,150,19,214,30,236,115,34,97,37,129,116,72,99,22,105,125,22,56,182,162,201,43,69,203,85,73,183,37,203,240,12,22,197,85,105,192,235,16,97,81,38,193,
224,46,10,84,98,76,206,194,109,26,54,80,214,219,5,147,8,194,165,30,30,77,165,81,28,124,224,239,161,191,231,243,96,40,167,112,59,222,135,196,145,199,209,202,44,216,33,144,138,89,92,255,143,223,66,97,207,
61,152,255,213,115,48,188,13,248,114,241,99,120,97,255,83,168,254,196,63,32,249,189,27,81,44,41,61,175,119,233,50,185,190,148,77,17,64,42,84,18,141,54,102,210,252,13,181,85,246,253,79,237,104,193,187,
54,108,195,200,201,5,204,46,20,32,17,245,93,32,143,63,220,63,143,169,33,217,102,130,150,161,91,245,183,165,14,15,197,127,233,60,158,182,186,253,247,224,220,216,205,157,91,163,87,184,153,94,215,82,54,172,
78,231,49,76,32,231,73,161,121,235,72,26,46,234,70,24,34,223,91,116,9,61,170,92,192,166,130,130,118,191,3,15,47,40,61,139,58,211,173,22,202,146,110,24,43,76,145,13,206,45,73,96,216,229,16,231,35,80,157,
181,129,158,6,134,219,221,55,147,138,43,43,96,84,120,93,137,134,128,175,219,97,106,18,35,56,131,169,180,18,75,73,114,236,244,186,58,58,90,187,157,166,150,224,76,61,113,49,160,146,57,26,180,192,208,230,
45,177,137,240,178,53,120,75,57,236,16,13,28,236,253,119,148,111,249,140,93,83,45,182,237,68,235,200,227,136,221,253,23,216,191,255,105,28,251,218,223,162,137,188,42,227,116,162,249,186,59,161,230,41,
127,209,166,11,79,62,64,245,143,229,246,231,46,160,76,225,64,206,100,109,112,172,118,231,206,141,184,109,115,53,238,125,121,153,158,202,163,18,30,31,152,131,68,0,202,36,89,18,157,214,22,240,184,18,169,
162,2,195,52,237,21,154,43,96,213,144,146,44,152,174,139,108,107,17,50,114,140,179,118,65,10,101,13,29,140,174,39,127,62,149,190,255,214,206,45,157,20,170,130,223,59,49,27,43,150,74,209,142,144,63,92,
71,180,152,87,149,72,153,136,128,70,128,89,227,178,101,53,204,123,93,209,19,180,190,32,121,73,69,89,195,161,197,124,146,136,132,124,245,198,70,28,159,95,68,158,214,83,225,118,34,95,40,89,251,91,86,168,
225,38,175,9,37,159,238,27,56,192,147,247,159,246,40,39,12,233,23,19,211,7,86,133,233,168,229,253,86,251,147,142,90,2,194,72,60,209,119,234,192,27,41,120,199,102,83,82,114,91,83,77,88,43,228,237,226,242,
116,179,148,224,35,101,111,247,49,120,241,55,79,1,111,191,9,140,203,135,83,170,7,139,143,61,10,31,213,86,253,135,95,195,93,119,221,133,142,15,188,31,41,189,136,248,195,255,9,79,91,7,244,215,158,71,129,
60,243,245,159,66,150,115,34,107,131,116,115,184,2,119,93,178,129,10,231,18,166,94,153,66,59,231,136,28,163,208,53,81,40,69,180,98,209,178,112,202,95,198,10,75,99,144,117,186,146,121,105,209,174,185,204,
85,121,32,228,242,88,223,208,14,48,146,20,83,41,204,170,171,230,18,56,198,154,255,126,119,177,184,215,161,168,81,158,194,147,69,247,103,121,177,251,240,200,228,221,22,184,204,226,66,210,204,23,86,24,164,
73,4,135,152,110,177,12,153,8,205,130,192,193,235,118,19,129,202,98,43,145,160,59,55,215,98,176,84,198,146,147,195,67,131,42,74,202,178,231,88,107,226,77,125,69,111,230,25,0,77,67,95,179,247,118,191,7,
87,251,68,116,214,249,201,144,76,60,182,82,250,188,161,87,72,68,175,227,106,91,115,140,52,187,162,128,181,95,65,66,180,128,192,100,2,242,230,29,48,139,68,165,55,180,163,51,28,66,99,99,19,170,171,170,241,
204,179,207,96,231,213,59,176,193,156,193,237,183,220,136,254,227,253,152,42,149,64,150,37,145,245,156,83,60,22,84,45,241,233,214,58,180,144,18,78,12,44,33,77,22,154,81,41,244,152,197,221,215,111,168,
138,165,228,108,240,156,239,50,134,153,216,119,108,180,239,250,250,202,164,89,42,173,97,126,78,173,220,29,173,175,12,155,249,124,244,220,141,154,54,237,77,105,166,196,102,115,224,236,55,24,64,189,169,
197,154,59,183,37,248,178,18,214,203,103,161,181,210,201,105,221,169,22,201,82,12,120,168,190,92,30,67,195,113,242,162,10,98,136,126,234,242,209,166,106,132,137,59,204,230,203,40,57,120,50,212,101,208,
190,116,99,7,170,169,156,40,230,85,204,22,53,236,249,117,238,108,174,164,60,87,160,231,94,152,72,83,249,97,130,175,174,120,83,239,250,122,103,13,54,54,77,9,94,244,248,236,186,196,77,225,195,42,132,235,
104,96,189,84,68,189,75,128,60,55,10,4,136,162,87,55,225,213,253,251,48,69,180,59,147,201,64,150,100,98,118,73,10,29,6,26,27,26,236,130,220,65,161,144,114,88,239,249,38,63,124,98,80,174,173,13,197,211,
60,27,205,209,230,75,22,57,88,14,21,22,64,145,243,125,60,163,234,222,30,43,237,242,118,123,139,165,94,126,149,45,177,148,231,40,217,117,157,239,19,129,11,166,77,215,71,114,165,222,102,195,232,226,153,
229,142,142,124,54,130,82,33,174,89,57,103,149,123,18,141,150,70,117,46,94,3,156,1,221,122,235,98,181,28,205,51,146,43,98,158,66,162,71,228,49,74,211,78,73,37,8,84,252,59,52,1,156,103,121,156,227,39,22,
232,92,37,226,64,121,95,88,75,18,20,171,92,41,156,101,159,33,77,121,83,64,237,27,28,25,77,94,245,249,251,194,39,179,69,154,60,13,109,114,0,56,22,135,56,63,137,0,61,157,183,172,174,46,79,177,196,133,98,
193,143,106,10,123,13,13,141,216,190,189,9,223,254,214,30,170,202,69,212,214,214,33,183,66,171,75,121,219,138,246,94,104,1,47,23,141,88,187,160,199,61,134,30,190,216,23,78,63,135,221,63,91,204,219,113,
252,240,200,216,190,54,143,43,86,39,114,61,34,204,11,246,35,221,193,197,34,246,179,197,130,221,111,108,126,241,126,49,224,237,110,100,204,179,134,160,175,13,75,2,187,220,103,98,110,62,82,179,74,191,238,
21,101,51,100,136,138,110,73,153,200,135,6,86,112,18,61,55,72,40,239,18,216,167,41,246,18,233,138,85,150,1,16,205,181,161,205,193,177,107,141,73,85,223,20,80,68,161,165,221,71,31,249,102,207,180,123,69,
111,150,229,133,175,68,185,212,129,5,235,60,187,96,149,226,86,77,129,165,227,47,163,145,106,33,43,244,89,117,143,213,198,55,223,136,241,137,99,224,15,63,134,141,141,141,152,95,90,58,243,110,239,124,45,
147,145,199,228,214,141,17,78,45,118,59,212,114,23,99,121,146,69,117,73,121,28,111,41,198,148,120,150,137,83,1,186,247,225,233,204,154,100,59,156,47,146,210,107,147,85,134,22,227,76,163,139,49,205,32,
121,20,41,142,183,0,146,200,219,122,29,172,185,183,119,58,179,102,254,146,199,23,77,139,124,119,192,80,99,108,185,28,182,104,49,99,133,31,210,29,107,154,189,126,17,187,127,50,46,247,81,194,15,202,126,
119,92,20,4,88,133,48,209,107,59,124,210,186,226,44,17,23,139,168,16,75,3,231,16,19,22,120,44,253,182,0,97,20,213,166,158,156,211,17,231,104,33,22,153,17,137,44,172,1,70,20,146,172,219,25,231,12,139,240,
208,24,101,69,122,211,255,51,225,247,251,71,249,200,237,225,148,238,176,74,106,2,165,184,98,106,2,197,156,89,138,1,21,203,0,206,15,161,93,144,236,23,181,139,11,11,24,49,125,96,111,255,107,10,121,44,132,
201,126,248,14,62,128,249,185,185,232,31,250,71,195,119,119,110,234,164,156,149,220,119,100,248,15,246,43,52,127,126,43,207,196,154,166,19,241,108,221,149,80,61,126,180,92,182,211,46,216,134,79,29,39,
223,46,47,131,100,181,137,62,108,255,179,143,88,255,204,98,83,116,71,120,7,28,94,191,69,61,17,50,50,24,153,155,219,251,199,240,101,247,177,190,161,190,63,244,53,94,232,223,197,198,50,169,197,96,125,192,
115,85,214,112,64,158,28,65,122,98,152,18,84,230,44,72,167,14,162,181,202,139,116,58,141,99,71,143,224,212,169,65,184,218,34,168,216,212,9,31,1,56,241,179,111,38,136,233,197,112,254,127,55,91,111,111,
178,49,23,185,255,131,202,142,237,49,57,180,153,106,156,73,114,181,185,229,242,210,229,5,138,178,13,154,85,145,115,84,83,57,91,59,81,245,142,91,225,95,56,133,177,159,220,43,201,178,28,249,93,124,197,93,
7,234,183,128,85,93,93,19,147,93,53,40,135,175,160,96,73,209,146,23,151,243,21,29,197,134,77,16,125,33,56,69,39,60,175,245,98,122,255,131,9,85,85,99,191,141,64,172,183,223,93,251,27,151,203,157,118,187,
221,102,40,20,50,119,237,218,101,118,118,118,218,199,64,32,96,242,84,230,123,43,107,44,142,252,8,73,96,93,93,191,199,102,189,197,78,165,211,226,196,196,100,205,201,129,193,75,238,253,202,87,239,106,111,
239,248,53,213,73,38,177,58,251,245,90,36,18,33,144,120,211,231,247,75,31,254,240,71,190,209,119,228,232,142,83,67,195,109,211,51,51,129,76,38,203,174,107,241,247,16,250,228,76,198,41,203,153,86,73,146,
91,36,73,106,150,228,76,19,177,193,134,129,147,39,55,61,245,212,19,219,142,244,37,130,205,45,45,165,43,223,177,125,238,186,235,174,63,74,20,125,138,188,107,50,20,12,76,208,113,34,24,12,12,5,252,190,41,
203,11,215,213,252,59,4,42,159,207,35,155,203,123,114,185,92,13,201,6,250,93,159,207,229,171,11,197,98,165,162,40,126,77,211,68,134,97,76,65,16,10,78,167,67,34,64,150,188,30,207,156,215,235,157,241,122,
61,211,244,123,177,186,186,74,91,87,241,239,143,76,216,160,17,64,76,177,88,98,21,165,204,106,154,202,158,249,32,198,115,22,88,134,195,225,48,92,46,167,81,17,10,173,107,117,189,253,255,109,255,35,192,0,
132,197,232,205,237,15,109,54,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* MainPage::ucsniffshield_png = (const char*) resource_MainComponent_ucsniffshield_png;
const int MainPage::ucsniffshield_pngSize = 4843;

// JUCER_RESOURCE: ucSniffdarkgreyicon_png, 5965, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC Sniff-Dark-Grey-Icon.png"
static const unsigned char resource_MainComponent_ucSniffdarkgreyicon_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,61,0,0,0,63,8,6,0,0,0,83,170,192,226,0,0,0,25,116,69,88,116,83,111,
102,116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,22,239,73,68,65,84,120,218,236,90,9,116,92,213,121,254,102,159,209,104,31,237,178,118,203,146,140,37,
203,182,100,203,6,27,203,6,35,2,193,132,37,77,66,22,19,104,221,156,146,96,232,201,105,14,167,237,129,156,198,129,230,132,210,52,129,176,5,40,161,96,147,128,77,240,2,120,195,6,175,88,171,101,73,182,150,
209,190,75,179,105,246,165,255,127,223,204,104,100,203,198,91,211,166,199,247,156,95,247,205,232,189,55,239,251,215,239,191,247,1,215,199,245,113,125,92,31,215,199,95,222,144,253,25,127,43,135,36,55,226,
179,137,164,225,255,19,232,133,36,171,131,82,126,14,216,115,7,131,175,39,57,16,148,79,255,146,188,38,142,228,81,146,46,146,64,164,200,100,178,11,202,185,231,146,76,146,188,22,84,220,255,89,75,51,216,77,
65,137,143,252,199,61,247,220,139,181,107,111,65,94,126,62,82,83,83,161,211,105,33,151,203,197,255,124,62,63,166,166,166,208,219,219,75,210,131,134,134,6,52,146,156,60,249,69,228,45,216,242,79,94,107,
235,95,45,232,71,131,15,21,15,133,138,156,184,2,136,74,148,254,99,25,66,180,185,11,127,220,186,21,209,209,209,151,124,67,171,213,134,131,7,15,96,207,158,61,56,126,236,104,232,235,215,131,74,53,255,111,
130,102,235,110,11,198,44,4,224,194,85,64,108,50,16,19,4,109,157,32,224,163,72,181,180,97,109,117,53,78,53,53,34,37,37,21,223,127,232,97,68,233,245,151,244,35,125,125,189,248,96,251,118,236,219,187,7,
94,175,135,99,255,238,107,97,117,217,21,38,169,3,97,87,14,1,78,43,0,146,51,1,118,95,142,76,191,15,232,111,39,224,99,128,109,20,80,235,197,124,211,220,100,60,244,240,223,92,214,15,50,248,87,95,126,9,70,
35,167,11,97,241,127,191,26,208,138,203,180,238,55,72,222,9,3,214,197,2,101,183,3,169,121,64,38,129,86,42,9,52,137,66,41,29,199,25,36,189,70,211,156,144,6,104,98,209,211,113,6,201,81,74,196,199,199,195,
229,114,205,42,78,167,19,110,183,27,1,191,159,46,151,65,173,214,160,162,114,41,233,209,143,206,206,142,154,96,53,216,126,165,160,149,151,112,206,122,146,13,65,215,146,0,164,19,192,140,98,32,133,230,0,
61,88,76,66,208,194,129,224,37,1,201,218,42,53,85,231,98,233,123,254,236,156,18,58,248,253,182,29,120,88,171,65,90,90,250,121,63,198,167,105,212,106,161,20,6,28,160,107,53,26,13,84,164,196,117,183,213,
144,62,21,216,189,107,231,134,224,233,15,94,194,179,179,103,24,131,185,167,251,203,220,251,102,146,231,184,206,234,242,202,160,42,171,134,63,99,30,188,186,56,120,60,110,248,72,192,150,240,210,60,214,19,
124,90,29,96,152,19,124,250,64,4,146,8,101,152,200,221,79,31,69,161,167,11,247,222,123,255,121,144,249,212,228,228,100,200,21,231,59,161,146,128,251,125,62,124,252,209,110,236,220,249,225,151,185,58,123,
166,49,129,180,103,182,90,249,58,230,2,139,46,102,105,206,202,207,233,13,25,168,248,222,83,80,230,46,192,208,148,21,163,84,98,108,78,7,100,131,103,33,107,62,136,64,95,43,37,172,241,233,171,212,58,40,54,
252,130,192,71,241,227,35,48,66,202,72,202,154,97,253,64,114,6,80,176,16,246,147,167,68,201,154,1,153,188,70,167,211,9,23,167,26,30,204,58,178,176,101,92,193,185,122,205,90,196,39,36,224,192,129,125,207,
13,244,247,151,71,90,49,98,108,162,170,17,239,231,88,140,139,195,196,196,68,249,197,220,91,0,214,144,11,151,255,232,55,144,233,98,208,53,57,129,81,199,20,188,221,205,112,125,246,46,252,189,167,103,87,
149,219,1,121,79,19,20,55,172,146,116,96,172,131,67,165,130,60,57,7,129,48,240,0,188,195,109,200,167,218,61,53,101,155,21,180,195,225,128,132,89,22,158,167,83,175,116,156,155,155,135,91,111,189,141,192,
140,111,160,218,190,129,192,191,30,1,94,112,135,24,42,149,22,27,253,134,92,17,42,123,179,38,50,230,199,187,249,64,255,221,205,112,80,162,26,38,176,46,114,99,231,161,119,225,216,241,27,4,168,12,205,54,
86,174,188,89,148,162,209,150,147,208,87,173,135,130,18,154,210,50,12,231,137,29,208,150,223,2,5,37,55,5,185,172,220,235,130,250,200,22,84,46,44,165,50,228,13,139,135,18,151,92,38,39,208,81,68,92,124,
84,175,173,176,219,167,40,85,40,196,255,125,62,175,248,190,245,84,3,140,93,157,132,93,46,220,157,149,88,56,175,8,217,57,57,229,54,155,109,19,93,199,73,174,138,172,188,218,71,207,173,39,37,142,79,8,111,
188,59,84,231,207,5,205,9,162,6,233,133,208,47,189,3,177,244,0,42,122,88,235,238,151,225,34,43,35,41,15,202,249,55,195,159,183,24,88,64,33,79,0,96,26,18,23,38,36,38,34,157,18,83,103,75,19,52,134,116,104,
231,20,66,150,81,8,77,201,114,250,17,63,148,26,45,148,172,8,154,205,59,94,20,201,74,79,74,10,129,230,108,205,159,153,177,185,92,78,20,87,44,195,178,181,235,208,213,126,6,227,35,195,66,105,12,60,99,78,
150,56,175,183,187,155,74,88,39,44,22,43,216,162,106,74,154,17,224,171,68,232,144,66,56,227,83,184,176,149,223,184,144,165,107,4,225,200,45,71,73,85,13,180,228,154,90,178,142,47,202,0,147,147,98,52,38,
5,126,59,41,107,178,31,168,221,1,140,247,137,139,138,75,230,11,192,163,99,163,240,121,125,48,181,28,67,242,242,245,208,82,98,211,144,168,213,90,161,60,53,101,94,142,243,169,47,118,98,112,112,0,113,148,
161,213,4,222,37,172,76,222,69,15,207,244,212,74,15,252,181,191,126,4,134,180,12,148,175,88,137,212,57,217,232,60,211,134,177,97,73,193,156,228,50,9,60,199,106,99,67,61,6,135,134,144,145,145,41,146,92,
8,188,33,41,9,221,70,35,108,82,8,221,29,201,230,206,5,45,58,163,196,236,18,204,173,184,133,0,147,251,184,28,112,76,12,35,141,110,98,253,244,63,225,235,170,37,176,189,68,158,189,225,139,242,11,40,254,23,
45,70,78,78,46,74,75,203,16,67,28,187,227,196,65,164,87,172,19,74,211,144,27,106,73,52,116,108,62,184,21,142,222,22,113,221,48,61,108,76,76,12,220,148,184,98,227,98,132,251,59,41,81,22,86,222,132,194,
197,149,224,36,228,15,200,16,151,148,130,37,43,87,35,133,192,183,156,106,194,8,41,140,61,66,65,247,204,202,202,38,203,147,71,170,85,225,216,103,240,81,81,81,200,205,203,67,107,75,11,103,237,103,46,70,
78,152,116,124,67,77,106,191,97,205,55,161,35,203,232,181,81,72,77,203,130,103,202,2,91,124,14,28,6,170,205,185,75,128,210,53,80,150,222,12,13,124,208,185,44,162,150,90,44,22,33,124,220,119,186,22,83,
131,93,72,47,90,138,40,186,7,131,247,12,117,162,123,251,175,17,224,50,199,15,71,49,55,58,58,42,154,145,216,216,88,17,179,227,38,51,238,221,244,4,228,100,49,63,229,61,95,80,248,56,145,104,236,178,53,183,
34,133,128,158,170,175,197,40,129,23,217,152,60,146,115,228,116,206,147,5,235,189,6,102,139,57,205,108,54,51,101,30,190,80,246,102,150,99,50,247,157,137,183,116,212,35,181,168,50,76,86,125,144,99,78,102,
46,148,214,81,12,159,62,76,241,236,160,44,220,9,182,119,39,221,156,31,58,52,66,224,221,199,119,97,188,233,16,178,170,238,20,223,247,30,253,16,62,231,204,140,237,241,120,152,87,139,152,230,216,46,88,177,
6,1,173,30,118,47,229,1,202,212,10,234,62,21,244,251,114,113,44,205,249,101,75,240,216,194,37,56,125,242,56,94,254,229,211,112,155,39,68,54,103,188,220,172,138,4,79,226,37,37,198,199,197,163,7,221,241,
95,70,67,91,217,218,150,209,62,204,161,135,229,12,232,165,82,98,72,74,131,210,109,195,64,253,62,216,135,137,224,208,131,10,50,226,178,11,11,221,113,231,87,201,189,243,48,135,98,173,160,96,46,122,186,141,
148,129,45,240,147,85,77,147,99,48,181,30,21,199,179,13,206,7,28,219,35,147,147,184,235,31,54,99,114,108,4,117,251,63,18,138,83,71,199,66,78,57,33,100,237,233,89,70,204,54,3,43,215,127,29,41,185,249,56,
246,217,1,140,15,13,138,60,17,166,118,148,200,136,182,194,100,50,61,119,49,75,35,72,217,48,122,230,36,106,183,253,6,115,191,242,176,164,65,82,223,24,153,117,84,79,77,69,105,166,116,102,106,54,20,9,73,
208,183,31,197,49,106,3,243,243,11,194,55,137,97,203,247,7,63,248,220,144,197,38,81,185,27,59,159,252,147,219,51,197,100,5,229,87,223,9,127,84,12,126,249,87,235,144,160,83,11,47,224,178,20,151,59,79,60,
131,157,206,201,43,158,143,165,183,220,142,210,101,55,18,240,128,80,192,252,101,55,225,169,170,149,104,58,122,8,191,255,197,79,81,144,149,197,43,23,112,80,201,235,239,239,55,158,187,44,53,27,232,213,33,
202,215,177,235,21,184,169,91,202,88,183,129,156,91,6,77,114,58,110,188,231,33,76,182,30,67,231,193,237,112,118,30,135,111,106,2,22,243,144,184,107,127,95,159,72,106,60,216,221,185,81,112,187,169,172,
217,38,144,185,238,65,12,31,250,3,60,14,235,140,31,203,204,200,144,98,153,250,232,59,191,254,125,56,201,148,35,189,70,88,40,47,112,105,74,73,73,193,223,102,117,225,43,139,211,40,113,105,48,62,217,132,
23,95,251,16,191,221,125,59,190,245,196,207,132,203,135,196,144,153,13,135,217,4,127,102,166,224,236,109,173,45,172,184,39,47,165,203,226,78,170,138,9,63,91,119,188,245,4,76,237,117,80,21,87,193,77,68,
193,102,179,160,175,187,19,30,109,28,252,84,194,144,58,151,154,143,249,212,92,232,16,229,54,9,138,24,114,115,37,89,177,167,71,98,135,238,201,33,168,147,179,104,14,123,25,82,146,146,201,29,227,4,237,204,
190,237,62,228,173,88,139,230,125,187,136,226,126,130,39,238,202,193,211,247,101,163,121,76,137,77,213,113,20,203,148,203,3,148,149,53,114,172,44,73,196,100,243,81,236,235,241,33,175,188,146,60,89,74,
92,175,111,254,71,24,162,52,130,208,88,44,102,52,52,54,24,41,89,62,120,41,253,244,126,178,242,234,80,98,10,177,35,230,213,113,213,15,32,122,249,221,97,106,200,11,63,94,98,108,206,137,17,76,182,159,130,
111,172,15,197,190,30,220,90,93,77,89,121,68,196,83,125,93,157,100,237,144,107,233,162,233,26,27,162,136,41,37,83,25,100,143,178,186,189,248,214,150,207,224,177,154,177,227,159,255,14,141,27,167,29,112,
127,151,12,213,121,129,153,116,149,217,25,17,152,123,95,236,197,119,182,181,18,233,145,17,117,232,193,175,31,186,7,69,243,138,201,229,3,104,106,108,64,111,95,239,134,72,82,114,49,75,111,160,56,203,101,
58,200,84,143,99,142,143,185,204,88,91,142,194,209,217,64,77,67,54,124,196,201,61,148,228,124,164,85,153,62,14,218,244,28,184,40,230,70,39,173,56,190,227,109,145,57,99,99,98,169,126,47,66,103,71,135,80,
158,120,96,234,171,101,14,139,96,81,156,104,56,110,19,43,171,209,117,120,47,118,61,245,67,216,70,6,144,157,168,198,130,12,141,104,91,15,119,251,80,154,226,151,90,88,22,191,87,88,92,174,144,227,244,153,
81,88,11,171,17,77,117,124,235,211,255,132,120,21,5,33,41,192,68,189,66,107,91,155,145,92,252,193,75,93,68,216,68,64,211,180,90,237,140,47,85,84,55,53,68,33,221,227,3,176,159,216,9,191,138,254,159,150,
47,24,20,3,98,34,161,162,14,202,227,39,87,83,168,80,146,164,23,132,37,54,54,142,220,61,7,167,136,84,136,65,140,142,235,184,129,186,36,86,168,141,216,87,127,115,45,38,58,219,164,118,146,220,104,103,163,
9,27,87,25,240,214,177,73,40,201,195,202,82,124,211,160,49,109,245,253,141,19,176,221,80,35,8,204,161,23,254,21,137,116,79,31,149,189,51,196,222,172,196,195,47,180,174,62,27,232,223,74,214,213,137,152,
142,20,102,65,204,116,248,255,246,182,227,240,14,117,1,89,55,8,107,251,131,192,149,9,41,228,5,9,232,110,164,108,158,28,39,146,17,11,19,5,142,111,190,79,146,33,9,58,82,42,223,199,229,166,58,77,138,11,183,
146,193,241,193,105,23,234,122,221,248,151,219,98,232,218,243,163,208,227,240,224,189,102,55,242,54,110,198,7,207,60,1,133,117,130,44,60,137,142,206,78,140,141,143,25,47,182,192,48,27,232,213,211,238,
61,19,52,63,164,130,220,138,189,128,193,59,7,141,112,183,215,194,95,176,4,126,25,113,116,202,244,126,178,134,140,24,88,32,54,21,103,118,191,141,158,206,118,81,206,236,118,187,176,48,119,78,73,6,131,200,
236,134,204,28,244,15,15,195,239,57,191,126,143,155,166,48,58,97,65,166,202,143,178,44,61,20,42,197,12,192,199,155,198,241,74,234,3,152,72,206,199,137,159,81,88,80,246,103,158,109,181,77,113,139,186,233,
98,187,39,23,137,105,221,140,22,150,193,202,229,50,145,25,89,148,74,149,88,218,117,140,15,193,219,81,135,64,222,34,170,141,114,65,45,133,3,82,204,251,220,78,100,42,28,98,133,164,100,254,13,130,151,51,
225,232,164,214,144,9,73,193,55,126,0,103,20,53,248,228,222,231,90,154,201,74,69,197,18,52,152,245,248,236,139,118,164,168,2,212,36,251,209,222,107,195,243,251,6,241,43,229,45,72,123,228,25,180,61,247,
19,184,140,173,162,102,179,199,248,36,5,62,25,73,70,206,29,242,217,190,228,7,103,144,33,225,12,203,32,37,81,10,81,17,223,101,203,101,103,231,64,195,171,157,187,159,39,19,208,15,114,166,38,74,201,241,39,
91,180,14,173,29,70,81,62,66,131,27,255,196,68,3,134,70,70,168,239,174,193,178,191,127,6,57,183,221,47,194,35,146,176,36,39,25,80,181,108,25,30,255,201,19,24,203,89,131,251,223,30,67,222,179,125,248,182,
227,126,28,217,248,39,204,249,241,11,112,15,118,99,242,147,45,84,26,229,130,158,250,189,225,38,232,162,123,100,179,129,62,224,139,120,0,142,99,201,186,242,89,133,193,103,102,102,1,131,237,64,203,231,18,
112,143,4,92,70,9,77,81,88,129,250,250,58,177,140,91,95,87,75,174,126,68,16,151,17,234,145,91,223,123,13,42,50,240,170,39,95,64,92,126,113,248,55,13,9,241,164,59,55,178,168,214,247,35,13,254,194,117,120,
243,205,55,33,163,142,79,159,158,141,248,121,165,196,203,129,190,223,253,156,154,25,29,84,188,50,194,212,84,234,252,76,95,182,210,41,191,16,13,101,194,16,2,22,153,200,102,138,228,234,220,218,37,80,230,
68,195,71,18,224,32,112,63,197,152,110,233,157,104,32,208,44,46,242,130,204,204,57,88,88,190,72,80,214,147,47,254,28,62,99,139,0,174,137,137,11,63,128,150,128,112,126,200,187,161,28,245,166,40,60,247,
216,3,216,190,125,27,52,148,75,6,118,190,37,206,247,82,62,49,239,123,79,116,88,156,245,185,244,5,71,253,149,128,62,16,234,126,2,98,21,83,54,35,222,206,61,14,125,20,203,185,83,164,100,99,227,52,112,178,
140,42,46,21,202,248,84,1,116,217,178,229,130,169,177,172,92,117,51,220,54,51,106,127,186,17,246,45,207,34,78,238,15,223,215,75,22,227,12,191,171,71,14,13,149,74,165,135,178,114,123,7,212,20,86,236,198,
194,202,111,62,75,60,32,70,132,154,39,184,156,20,249,252,151,11,154,121,163,145,23,231,34,87,113,67,150,158,6,43,23,128,67,223,115,226,227,140,143,137,254,105,23,39,66,227,165,182,47,118,110,57,17,255,
190,25,63,194,245,155,173,221,126,246,12,58,14,126,132,57,177,122,172,90,185,74,40,207,235,241,194,26,147,129,41,121,2,226,212,110,252,241,141,87,113,230,236,89,216,121,149,84,164,246,65,248,106,247,137,
133,3,86,130,195,233,148,214,203,164,177,237,74,64,139,11,185,183,29,27,27,19,177,55,52,52,132,241,241,49,145,121,249,251,144,181,167,93,94,114,115,65,93,71,141,97,192,60,91,71,250,17,75,36,134,155,145,
200,193,201,77,77,137,112,138,74,89,105,89,41,166,166,172,152,162,239,138,230,206,197,226,197,139,81,184,100,141,176,242,48,117,123,239,108,221,34,22,249,57,215,36,20,46,128,123,15,39,47,165,16,6,204,
214,9,26,200,120,41,27,253,23,90,247,230,254,115,147,60,41,11,138,121,149,112,246,157,133,203,52,12,203,136,244,224,204,204,184,33,225,56,102,13,71,38,53,184,188,18,224,240,114,45,119,103,105,48,146,165,
255,227,87,255,134,24,162,166,145,11,14,161,106,241,232,99,143,17,95,110,196,190,189,123,169,230,90,96,240,29,166,224,118,163,229,216,167,152,36,210,97,50,91,68,134,79,155,91,2,235,190,173,162,170,248,
125,50,177,158,198,207,195,245,63,114,153,247,74,55,240,246,203,181,250,213,134,31,189,2,139,215,71,188,154,23,4,169,244,181,127,65,250,164,92,97,146,150,106,216,173,249,71,121,239,121,98,98,2,46,85,12,
80,121,207,244,237,185,229,83,6,48,254,241,75,248,225,143,30,59,111,99,238,253,247,254,32,142,11,168,37,93,184,112,33,214,175,95,79,52,242,140,0,31,160,102,217,64,157,24,247,198,77,167,78,97,212,100,194,
125,63,254,41,78,109,125,69,120,220,24,253,30,55,54,172,108,222,255,10,238,113,117,95,205,6,158,49,224,245,108,176,247,180,195,103,37,45,90,39,185,38,72,219,177,73,116,239,152,52,209,121,121,201,89,156,
20,131,54,183,196,193,17,159,1,196,165,73,91,62,212,24,240,236,153,24,128,127,184,93,36,178,153,46,110,65,107,203,105,64,159,64,250,28,64,75,75,11,182,109,219,38,44,255,253,135,30,66,99,99,19,28,14,59,
236,54,27,242,114,115,49,143,8,142,185,181,94,108,6,112,39,53,66,128,21,228,97,252,57,104,229,55,174,118,3,239,83,145,9,7,78,175,70,114,112,69,196,105,159,118,91,222,177,84,23,5,155,128,224,232,169,37,
179,230,73,235,57,17,141,129,151,64,39,145,197,206,29,92,22,197,88,84,3,20,46,3,122,79,1,205,159,226,243,207,63,23,82,81,81,129,199,31,127,28,205,205,205,194,242,152,24,151,136,19,133,212,20,41,195,41,
129,13,213,230,39,175,213,86,237,1,2,181,129,26,96,45,82,242,103,110,202,49,33,32,134,70,109,144,36,46,27,41,133,196,144,37,109,213,70,142,193,22,120,169,33,224,184,155,36,151,228,146,196,137,141,59,47,
94,38,66,229,93,210,102,190,33,19,40,185,17,200,36,162,50,208,134,1,99,7,89,187,17,37,37,37,88,113,227,141,104,104,168,23,172,144,159,99,96,112,64,88,56,72,164,158,190,156,173,219,47,3,205,252,209,9,187,
169,102,206,188,18,216,60,50,177,74,49,99,112,210,50,15,138,37,33,164,204,37,235,107,194,251,77,225,209,113,20,133,133,115,49,111,94,17,38,77,147,2,40,19,21,6,238,166,2,34,95,253,109,200,201,122,10,2,
196,162,164,158,91,177,96,21,252,198,38,140,247,118,162,171,171,11,102,74,100,188,237,35,74,148,203,73,249,99,82,184,126,144,140,124,243,90,111,202,31,227,237,90,87,255,217,226,178,154,175,99,194,73,84,
80,163,68,162,143,178,102,98,18,220,188,192,144,85,74,205,180,86,242,104,249,57,183,28,33,122,58,222,45,86,75,147,147,83,194,228,132,183,99,143,28,254,28,242,162,229,20,37,203,4,179,82,133,185,61,205,
26,162,151,243,111,130,167,238,99,88,38,39,168,185,137,145,22,30,120,139,156,90,85,99,183,145,151,141,77,193,53,189,203,122,23,69,126,137,231,109,160,248,171,239,255,224,121,220,70,113,22,95,186,10,243,
214,62,128,152,121,75,33,79,166,164,102,30,99,10,119,62,96,246,130,222,6,17,207,76,70,34,71,203,105,105,231,83,179,112,141,104,51,53,33,161,74,160,165,250,205,18,21,151,8,253,234,111,137,243,120,233,137,
105,104,122,98,60,58,58,58,97,179,217,66,139,152,221,184,204,113,169,175,95,112,198,121,103,202,106,169,177,116,214,167,213,172,184,9,39,234,78,160,167,181,9,254,241,97,65,55,103,29,228,214,176,12,83,
121,241,137,23,109,92,193,181,50,222,213,56,116,240,83,17,255,250,91,190,39,74,14,83,76,141,74,9,29,89,89,23,156,181,236,238,126,98,103,181,159,8,78,176,110,213,74,28,252,236,16,218,58,58,66,155,141,31,
253,57,94,180,97,115,29,200,203,203,47,255,206,163,79,161,195,236,197,206,195,39,48,105,181,157,127,230,89,234,184,70,59,196,22,46,47,27,113,61,229,108,205,116,52,180,88,168,72,205,131,54,175,12,134,175,
108,132,154,154,6,22,21,147,28,222,167,18,53,158,226,183,227,11,204,29,62,130,197,75,42,240,214,107,175,160,229,236,89,83,240,13,132,55,112,133,227,74,222,46,98,224,175,71,71,71,223,253,221,141,143,161,
178,250,171,248,226,116,43,62,57,176,31,103,154,106,131,233,111,72,88,56,188,240,31,193,194,184,54,139,140,205,123,224,85,119,33,113,237,119,16,197,11,139,188,201,71,160,181,114,9,124,162,86,133,210,36,
45,230,184,59,209,218,112,28,47,191,244,2,81,225,241,80,12,95,213,59,165,87,243,242,156,120,99,161,162,162,18,143,252,240,17,44,169,172,66,119,223,32,62,222,179,31,135,90,58,81,231,160,44,174,141,230,
21,1,200,198,251,33,243,58,41,243,146,37,115,23,16,161,32,112,5,11,9,28,187,113,80,232,188,5,201,177,40,78,212,163,56,73,143,100,157,2,157,45,71,240,218,43,207,99,239,222,61,161,70,226,193,75,233,151,
175,41,104,222,71,246,122,60,74,143,199,171,161,214,83,191,121,243,230,146,223,253,238,213,103,205,102,243,226,165,85,203,241,237,7,127,128,91,107,238,144,104,9,253,105,27,49,97,192,226,64,219,168,69,
234,203,73,234,6,205,168,152,99,16,76,74,65,37,40,61,90,43,164,36,105,250,173,194,227,71,14,225,143,91,94,197,135,127,250,128,87,103,134,86,172,88,241,204,91,255,245,246,251,42,149,210,70,241,239,80,171,
84,46,34,40,62,222,216,255,31,3,205,59,138,4,84,77,115,140,139,197,229,138,161,227,104,33,46,183,254,165,151,94,92,250,254,251,239,221,71,221,88,33,103,233,219,239,248,42,110,90,185,82,172,137,149,17,
159,22,203,213,129,240,158,90,248,45,162,16,215,25,176,56,209,120,252,48,14,237,221,133,143,118,253,137,234,119,47,244,250,232,201,170,229,85,251,54,111,126,250,61,181,90,61,197,162,209,168,109,52,219,
104,182,208,108,33,208,54,42,111,94,181,216,155,190,134,160,131,175,70,104,157,46,87,188,211,233,138,101,192,211,226,142,14,206,172,140,232,230,230,83,153,239,110,221,178,188,181,165,37,151,226,54,108,
186,178,178,133,212,149,197,133,223,174,138,220,175,56,222,59,9,183,177,41,184,24,168,241,20,21,21,245,82,210,58,123,247,215,238,105,96,144,84,198,108,98,214,104,172,52,91,233,28,171,86,203,199,98,102,
224,38,82,128,59,162,159,190,122,208,188,130,66,160,245,33,208,212,205,72,32,167,129,243,28,77,229,72,40,193,45,121,66,180,209,216,149,68,173,98,58,117,82,9,131,131,3,172,0,249,200,240,176,150,254,39,
202,100,86,118,54,213,57,153,159,90,84,103,98,98,162,189,168,184,100,148,242,67,63,91,82,178,38,129,85,171,173,98,214,136,217,26,2,30,4,108,213,106,180,22,141,86,51,193,238,46,218,218,107,245,198,96,176,
109,179,51,91,242,171,253,193,37,36,201,104,50,153,204,79,226,147,203,101,94,234,111,61,10,133,210,69,26,119,210,53,83,69,69,197,214,252,252,130,33,242,20,45,137,134,56,178,138,154,5,37,93,47,15,46,56,
250,73,232,58,133,155,132,31,218,73,215,58,200,85,237,42,149,218,78,179,228,210,146,18,34,92,59,100,113,181,89,169,82,154,40,55,92,22,224,203,74,100,110,183,135,173,46,39,209,186,61,108,81,79,12,29,243,
172,247,120,220,36,158,40,138,123,29,205,186,16,80,175,207,39,192,250,124,126,85,192,239,87,248,3,126,69,152,188,147,194,228,44,4,92,46,20,166,240,16,104,151,146,20,71,9,75,40,128,192,216,201,138,118,
149,80,132,138,65,91,233,179,149,62,79,209,103,151,62,42,42,240,103,43,89,193,44,46,35,144,74,143,215,171,246,122,61,42,175,152,125,44,12,84,43,89,214,167,36,235,210,28,80,145,133,21,129,128,95,22,8,130,
142,120,187,159,61,197,45,151,43,8,56,41,64,174,160,248,84,56,217,3,8,184,16,149,82,233,33,171,242,236,35,142,30,184,210,172,125,173,94,114,191,224,24,167,22,146,55,247,8,180,140,172,44,243,7,251,238,
80,116,136,69,69,105,165,53,64,165,44,32,237,160,200,97,72,76,196,245,113,125,92,31,87,60,254,91,128,1,0,61,89,190,64,206,170,41,165,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* MainPage::ucSniffdarkgreyicon_png = (const char*) resource_MainComponent_ucSniffdarkgreyicon_png;
const int MainPage::ucSniffdarkgreyicon_pngSize = 5965;

// JUCER_RESOURCE: ucsniffdarkblue_png, 7036, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC-Sniff-Dark-Blue.png"
static const unsigned char resource_MainComponent_ucsniffdarkblue_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,144,0,0,0,34,8,6,0,0,0,176,68,80,198,0,0,0,25,116,69,88,116,83,111,102,
116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,27,30,73,68,65,84,120,218,236,123,123,120,83,231,157,230,123,36,29,221,101,201,146,239,198,182,12,4,99,
140,177,66,32,64,32,32,114,105,155,146,4,167,201,204,102,232,83,162,236,108,218,109,166,125,106,186,59,187,109,103,166,53,211,233,211,118,102,158,214,153,108,119,50,105,155,184,157,77,167,153,166,169,
147,41,233,228,210,32,19,32,92,98,144,13,230,98,99,44,95,228,187,101,93,143,116,116,116,217,223,119,100,27,3,50,183,217,63,182,45,223,243,252,56,32,31,142,206,249,190,247,247,190,239,239,247,29,43,112,
123,220,30,255,129,161,250,127,120,173,109,20,246,217,152,27,30,10,47,69,231,237,169,190,13,160,92,227,41,138,70,57,140,86,152,138,202,96,95,122,7,52,122,3,164,152,128,190,190,30,68,198,125,64,116,38,
64,231,180,81,180,82,180,223,158,246,223,159,193,221,226,255,219,73,209,82,86,94,110,175,175,111,192,133,193,97,56,63,247,23,104,188,111,27,76,26,21,244,20,70,181,18,58,94,129,65,127,24,93,103,206,225,
213,95,254,18,7,127,245,10,16,241,51,86,106,186,13,164,63,76,0,153,103,89,164,241,206,59,215,226,201,63,249,52,84,60,143,116,58,141,183,223,125,23,34,111,132,24,13,225,153,175,126,11,118,187,29,6,94,9,
61,129,104,46,38,66,2,190,217,242,191,240,171,31,124,7,72,196,90,232,58,205,20,193,219,203,240,135,1,160,134,89,240,56,106,107,87,97,231,99,159,130,213,106,131,66,161,64,38,147,65,36,28,134,36,73,72,36,
18,56,53,48,138,39,191,242,29,24,84,74,24,212,12,60,116,228,57,66,159,72,12,165,64,103,255,8,190,240,236,231,209,119,226,48,99,35,231,109,16,253,254,2,200,60,43,55,174,162,162,98,123,73,105,41,236,85,
118,84,84,86,162,180,172,28,74,2,79,246,42,92,246,66,116,148,164,4,78,159,58,133,176,165,2,15,236,254,130,204,66,44,202,226,62,20,232,213,208,234,180,208,104,52,50,216,158,253,252,231,241,206,59,111,47,
6,162,151,41,44,179,223,63,112,123,169,254,255,28,202,235,48,142,187,166,102,101,227,166,77,247,88,234,215,52,160,98,73,37,130,161,0,206,158,57,131,195,135,15,225,98,95,31,86,212,212,200,18,38,71,42,37,
131,200,100,50,193,115,250,52,108,235,31,132,152,162,207,163,51,48,206,12,202,63,75,16,112,24,120,216,120,228,145,71,224,159,153,41,25,30,30,222,24,143,197,220,11,64,244,148,205,102,107,78,165,211,43,
83,169,84,224,182,95,250,221,171,194,24,243,184,239,184,99,133,165,193,113,39,42,43,171,208,217,233,193,129,118,55,68,49,62,127,82,166,170,10,23,46,244,162,138,252,206,155,111,180,97,231,206,199,100,144,
68,163,81,196,150,172,194,112,76,130,81,82,64,51,120,2,33,19,143,100,42,3,173,150,24,72,171,145,89,40,60,57,134,167,62,243,25,168,148,42,231,192,192,128,183,187,251,116,235,228,228,68,179,74,165,106,230,
232,58,164,141,152,149,205,203,198,142,239,254,102,91,215,197,110,231,248,244,4,16,143,210,121,73,232,210,82,54,146,113,228,73,209,214,206,3,111,13,168,84,124,149,166,98,185,75,109,202,7,79,223,167,82,
107,160,84,105,40,109,232,177,21,74,104,192,185,47,188,247,234,213,224,252,236,15,204,170,105,95,163,70,244,55,170,210,146,131,79,75,118,101,58,5,46,149,4,66,147,129,168,32,120,52,177,64,155,38,25,111,
243,13,122,175,201,142,5,95,127,195,60,61,212,239,82,248,7,157,202,148,224,200,75,137,118,99,74,132,38,17,245,10,19,67,222,177,222,83,109,6,131,161,53,16,8,252,135,100,252,233,214,51,85,167,39,199,93,
199,79,28,130,42,54,141,37,137,32,242,132,105,164,35,211,173,167,63,58,120,249,61,62,241,245,6,62,25,110,172,72,80,98,135,39,48,114,190,3,113,33,138,72,36,178,247,178,243,118,125,219,140,120,176,49,63,
25,178,151,210,53,211,83,3,240,158,245,176,53,116,211,253,182,95,11,64,46,122,40,75,125,253,26,84,145,100,245,245,93,192,212,212,36,238,92,187,22,19,227,19,184,120,241,2,204,102,51,156,78,39,246,239,127,
31,187,62,253,25,220,119,223,253,243,204,226,27,30,194,68,84,141,139,4,160,242,232,4,202,135,123,224,47,46,129,190,176,4,74,98,167,137,225,65,232,104,65,149,196,78,163,103,187,48,57,230,147,89,235,147,
159,220,225,34,32,185,78,158,236,144,65,72,126,170,53,151,124,141,250,250,156,98,255,201,102,203,21,159,167,41,36,2,94,38,33,179,217,128,90,205,219,13,197,149,205,74,115,65,22,240,20,201,5,231,243,153,
76,243,149,236,86,255,141,215,191,20,236,62,208,172,139,76,88,174,18,123,242,116,176,150,88,204,86,89,114,157,92,60,210,178,186,98,69,203,233,67,239,236,201,53,137,79,254,176,115,231,225,147,237,173,153,
241,46,139,58,73,160,225,50,224,217,247,146,242,171,180,122,187,190,114,133,189,176,188,218,25,155,30,107,46,18,35,174,158,238,206,55,110,21,64,227,209,136,125,232,248,175,155,173,129,113,249,223,33,138,
24,20,80,79,141,184,231,230,240,129,127,248,208,220,113,182,163,45,51,216,229,84,73,81,249,156,144,46,159,92,237,3,208,79,12,32,226,57,56,15,160,149,223,122,123,167,175,211,221,170,22,70,45,138,116,18,
147,28,65,165,112,25,76,20,202,83,238,102,92,7,64,22,141,70,11,91,65,161,44,77,181,171,234,32,8,2,14,31,58,136,80,40,155,40,45,45,45,114,165,197,226,232,177,227,168,171,171,151,13,52,35,142,179,231,206,
98,250,244,41,164,42,107,80,172,138,32,172,138,35,173,242,99,221,42,7,12,228,129,74,150,174,128,175,151,206,25,244,194,90,93,131,186,213,99,56,122,252,56,244,6,3,138,75,74,240,208,67,59,112,228,200,97,
120,189,222,150,92,55,55,77,149,158,69,145,201,125,231,73,9,66,112,74,254,171,154,24,199,170,228,160,230,210,57,79,77,113,138,203,254,253,248,223,188,254,178,239,104,155,75,79,139,13,197,13,172,154,94,
71,127,232,154,214,221,255,152,227,163,223,254,106,251,21,140,176,173,231,200,155,109,250,254,46,152,9,56,74,6,26,130,176,66,14,200,71,142,5,221,159,161,176,196,146,136,11,109,142,45,37,14,207,193,183,
111,169,233,250,126,239,57,88,137,141,243,23,60,43,93,19,163,148,204,115,227,194,216,96,147,109,240,184,147,75,73,87,61,95,70,171,198,196,236,223,183,124,239,3,243,200,185,15,91,203,66,3,217,36,186,226,
92,41,207,204,178,248,154,18,230,245,251,137,178,136,182,99,49,129,34,134,187,238,90,135,242,242,114,156,60,209,65,140,212,135,214,214,86,153,129,216,96,94,136,1,204,65,114,55,53,53,133,11,189,61,242,
231,129,127,250,43,68,183,109,195,52,153,238,34,199,38,68,56,146,49,41,13,13,73,72,105,237,26,148,46,95,137,225,243,221,232,37,21,40,46,41,149,171,57,150,232,25,66,33,3,36,1,200,145,171,139,157,32,0,217,
144,27,20,9,73,196,248,84,118,42,180,58,29,116,244,240,218,69,0,36,44,152,153,39,191,187,239,27,99,7,127,230,226,19,241,155,94,188,140,74,225,188,243,193,63,250,198,201,119,127,49,159,193,1,111,87,107,
194,123,146,192,131,89,208,204,2,38,147,61,202,124,152,73,103,101,154,142,106,2,82,18,74,198,136,143,221,10,128,226,35,23,161,39,105,52,113,151,18,43,138,20,37,17,205,121,50,171,12,9,95,143,43,63,149,
200,157,76,188,118,254,239,157,163,195,206,138,177,30,139,110,145,57,142,25,204,215,245,64,63,97,61,154,238,51,103,236,117,245,13,242,7,67,62,31,62,58,122,4,211,211,147,36,47,17,98,176,0,154,155,155,217,
34,163,173,173,77,246,71,44,174,28,100,146,137,10,180,216,176,246,94,156,34,31,85,92,90,134,130,162,34,104,21,196,12,4,36,227,138,6,236,254,230,247,241,206,191,180,226,220,225,118,20,218,108,50,136,56,
133,92,215,89,114,58,255,88,68,150,131,92,67,34,246,240,251,179,12,196,107,117,224,153,108,164,147,185,159,82,161,156,247,15,190,119,94,104,78,199,130,139,150,165,209,72,152,188,155,22,228,171,114,150,
178,106,142,103,213,162,12,160,221,47,157,110,240,125,240,47,118,45,45,32,100,192,100,129,146,32,201,22,34,193,150,137,161,62,75,121,133,221,165,213,233,145,161,194,131,37,106,42,153,68,74,165,107,188,
101,19,148,140,33,70,50,99,96,143,37,70,16,156,28,69,34,22,133,209,148,39,39,183,252,184,161,73,187,230,138,100,154,26,238,11,168,120,222,163,209,153,236,115,159,133,125,94,135,57,17,186,28,160,66,152,
174,233,243,24,205,182,0,175,51,221,208,86,134,187,227,163,99,174,252,146,74,228,219,10,161,210,168,113,223,67,143,226,124,87,7,200,239,146,121,238,67,83,83,19,44,22,139,28,242,130,241,252,188,15,146,
111,152,202,124,54,49,124,221,221,216,255,214,155,56,249,163,191,71,229,146,37,88,94,102,196,253,181,22,156,37,108,41,54,236,130,99,211,189,120,244,179,95,194,199,254,196,133,23,254,252,89,228,145,204,
245,246,200,44,214,150,115,221,9,16,170,76,238,236,224,104,65,148,74,37,168,122,163,5,215,81,230,211,121,139,100,93,154,83,103,253,66,95,71,83,120,160,107,209,137,8,134,130,173,61,103,79,181,90,109,5,
142,42,251,242,22,50,249,57,190,88,97,169,222,241,244,182,254,125,47,183,159,243,79,59,53,99,61,224,152,20,210,125,102,88,48,54,79,243,45,93,71,219,101,191,164,228,56,135,213,86,232,96,69,137,24,143,33,
78,1,141,1,171,62,182,171,225,204,59,63,91,84,198,44,249,214,109,49,50,188,121,102,11,219,50,242,12,123,47,102,61,197,235,223,109,39,81,225,124,215,88,80,5,21,24,11,231,141,129,34,52,62,228,154,152,24,
191,220,123,197,131,116,94,234,178,143,232,60,207,197,243,167,239,188,209,42,140,13,135,244,192,211,120,207,115,8,181,133,54,44,93,189,22,234,180,2,119,110,216,140,53,13,13,178,140,181,183,31,144,77,52,
3,146,195,225,32,96,13,64,71,178,193,36,143,141,37,101,229,136,146,100,213,62,254,167,232,248,193,95,225,179,235,153,121,12,97,139,61,130,149,133,211,120,168,144,16,242,155,47,226,163,244,243,88,67,32,
234,57,213,69,6,88,68,56,153,32,102,235,111,93,172,255,195,101,36,112,139,176,138,146,24,135,121,31,230,199,120,13,145,48,93,43,185,8,216,36,46,75,219,67,231,142,53,78,142,14,231,6,25,121,192,100,50,217,
68,17,156,24,31,107,55,24,77,46,66,132,35,215,185,162,101,25,251,188,253,148,111,208,178,86,8,200,160,97,9,196,36,132,37,86,44,33,217,105,126,204,52,63,65,78,165,106,11,134,131,142,52,8,236,10,13,50,122,
98,55,146,134,210,117,159,104,138,223,177,209,21,239,59,193,126,2,29,151,130,145,174,147,140,11,110,46,17,177,75,80,217,227,25,14,34,177,119,138,62,95,85,186,180,229,204,135,239,237,169,120,248,153,109,
54,41,236,86,39,8,20,83,163,242,247,170,169,80,41,92,247,72,179,95,99,106,14,14,158,133,37,25,134,34,115,105,222,212,100,204,202,171,107,218,236,171,214,34,169,212,162,232,193,103,220,61,29,239,58,211,
194,248,101,231,201,101,185,37,223,177,110,243,125,25,165,90,15,129,83,55,159,122,255,245,189,215,5,16,183,113,7,146,59,118,163,231,215,47,98,120,223,107,88,181,229,19,208,25,205,89,202,54,87,160,114,
153,138,74,247,70,84,87,103,205,52,3,208,28,120,140,70,163,76,205,133,155,30,196,177,87,94,64,234,212,126,252,215,47,44,193,217,105,30,43,109,151,88,234,209,245,5,248,230,209,215,17,185,123,51,246,191,
250,19,168,137,197,78,159,238,130,40,138,205,139,118,63,169,92,71,90,202,157,101,196,56,58,98,176,8,73,142,146,116,157,201,109,38,149,27,108,209,124,189,124,60,115,226,3,187,224,189,152,187,207,161,226,
189,137,132,56,95,98,171,180,134,54,82,162,0,72,202,50,10,30,105,90,200,164,66,69,96,36,139,204,41,3,178,71,24,31,242,78,134,34,84,66,251,101,102,137,199,24,195,196,9,212,98,35,1,49,64,139,209,166,214,
234,220,58,163,197,222,221,113,248,178,36,201,60,254,23,13,211,39,126,229,202,235,61,8,91,97,17,242,242,44,228,0,244,100,248,85,78,181,201,76,223,65,223,71,230,95,162,39,21,104,37,162,105,174,169,126,
251,78,36,19,209,54,93,58,6,158,188,148,182,184,108,86,206,147,232,15,135,97,25,239,67,85,220,159,227,217,148,48,229,153,231,253,96,119,143,7,246,233,115,243,21,237,194,97,48,24,231,220,18,21,69,234,27,
98,32,168,8,44,249,182,2,148,125,174,25,218,243,59,112,230,197,189,200,179,22,163,184,118,29,221,186,26,10,125,5,116,6,65,102,29,183,219,45,87,102,123,246,236,145,165,43,159,202,252,184,40,226,228,43,
255,8,229,44,229,191,224,86,35,173,43,64,173,77,177,192,134,40,48,70,101,125,226,208,7,16,70,135,16,38,233,25,29,25,105,189,86,247,89,77,146,148,102,242,144,211,209,166,192,171,179,204,162,228,53,240,
13,246,33,22,9,46,34,97,38,228,125,229,95,183,101,142,252,130,36,193,4,45,201,181,182,180,18,204,94,101,13,47,129,204,123,206,59,182,255,146,146,246,118,123,246,206,121,157,69,135,24,117,247,135,169,242,
236,191,64,82,154,158,109,32,144,196,27,242,80,176,116,37,52,37,213,141,33,163,181,81,176,20,182,148,151,212,121,173,241,153,214,148,74,221,114,230,237,159,5,71,247,62,210,137,245,141,114,219,132,21,50,
70,163,73,174,134,45,249,54,154,111,131,252,76,188,146,135,138,64,171,225,148,48,210,92,139,188,182,137,151,4,119,134,0,148,90,96,226,24,241,198,195,65,202,181,40,133,120,205,91,78,81,226,10,147,35,4,
68,241,250,118,75,115,131,0,178,240,42,44,211,242,40,163,176,173,223,134,173,155,15,224,120,235,223,162,227,253,183,80,120,207,195,132,89,37,49,209,50,76,29,59,44,203,24,51,212,85,85,85,114,175,72,175,
215,131,211,155,160,12,71,230,175,247,213,215,6,209,180,149,50,115,157,93,6,14,27,19,19,2,166,215,61,141,99,47,182,32,66,178,56,53,227,103,146,209,124,205,246,57,77,134,20,207,93,45,49,234,86,105,103,
43,10,154,232,24,157,199,122,74,12,28,182,202,149,208,22,86,33,89,122,7,166,211,74,36,68,170,48,71,6,177,249,83,77,4,58,141,236,155,152,100,177,99,106,182,179,206,165,245,139,89,177,197,199,47,191,53,
160,123,112,119,75,196,219,221,100,90,86,139,234,237,79,32,223,190,134,202,223,18,12,78,140,99,194,63,1,49,64,140,16,38,163,154,210,219,71,211,104,46,144,226,77,21,59,254,139,115,104,223,143,58,243,214,
220,235,89,253,228,23,29,226,249,99,24,60,252,38,250,79,127,72,73,58,66,94,51,31,230,124,43,201,137,149,228,89,75,94,79,13,37,1,137,163,234,54,156,148,28,137,120,24,252,2,127,150,102,189,27,74,168,24,
225,87,75,150,70,65,137,199,225,82,241,33,215,131,74,153,57,17,35,40,36,40,41,195,100,57,148,44,65,175,168,70,21,188,154,253,129,52,1,54,174,224,111,12,64,214,132,128,10,181,10,69,90,53,44,106,37,85,78,
10,124,226,179,95,65,113,121,21,94,251,233,11,80,109,124,132,110,222,128,178,178,149,84,214,255,132,249,22,217,80,179,242,126,100,58,128,154,103,254,18,190,255,182,139,192,146,173,118,44,230,60,40,239,
122,10,223,63,120,8,85,100,247,130,66,18,175,22,63,1,161,161,18,131,29,135,228,68,77,74,146,231,122,123,95,41,50,127,51,129,233,220,4,68,193,204,243,92,149,85,179,251,127,192,180,114,19,226,105,30,67,
195,67,24,152,24,69,146,85,105,81,90,60,214,75,50,88,112,248,232,1,102,130,101,179,139,244,108,121,157,206,86,78,234,241,158,91,42,138,66,239,254,116,143,227,51,127,110,177,44,191,219,213,159,82,163,103,
116,2,241,115,189,0,99,67,2,46,88,71,95,140,201,145,38,153,155,16,5,139,169,192,236,214,239,252,162,61,50,61,26,248,176,35,1,157,181,16,75,254,248,47,81,178,189,31,3,175,253,189,55,16,24,119,232,76,121,
206,65,111,95,155,137,170,171,60,115,62,52,84,201,241,42,13,34,113,209,30,157,28,151,43,197,249,69,39,191,98,52,198,60,17,173,213,169,75,196,28,82,112,188,5,11,252,160,130,36,94,105,171,104,146,212,122,
79,68,165,35,31,164,11,196,173,85,22,213,216,121,151,127,124,196,181,240,121,242,43,86,120,84,150,210,166,56,175,131,164,80,121,111,4,64,30,149,239,130,195,90,95,11,155,70,37,151,221,114,7,149,142,247,
54,238,66,100,124,16,191,62,122,28,88,177,1,125,134,42,172,88,90,35,151,245,204,11,49,67,221,254,220,115,88,187,180,22,59,126,186,31,111,61,117,31,25,91,181,220,189,54,149,45,195,193,116,37,126,123,244,
85,36,62,253,53,152,215,110,69,255,151,30,133,150,216,46,26,147,81,31,184,110,207,35,28,64,104,108,52,231,207,242,87,175,135,54,154,149,172,12,101,231,84,210,128,83,39,58,33,5,24,104,232,115,42,109,73,
211,72,244,233,200,170,71,157,193,155,153,28,98,134,97,182,39,51,215,163,201,134,34,50,115,203,149,181,231,159,255,238,233,229,117,13,173,227,211,193,166,76,65,149,83,99,41,179,136,208,100,129,195,170,
46,6,36,182,21,147,160,32,179,31,14,141,89,184,226,74,151,49,22,68,120,102,6,194,212,8,122,134,250,192,153,44,208,105,77,109,177,152,55,120,177,231,236,27,69,101,149,184,208,123,142,124,137,1,84,201,33,
223,90,64,37,124,158,61,62,61,41,219,135,185,161,201,179,162,160,20,129,190,95,191,216,174,170,170,134,64,140,39,111,17,205,89,1,163,5,182,162,165,158,115,255,254,127,230,187,241,23,40,8,156,78,230,33,
23,14,109,113,117,96,240,221,171,183,125,174,9,160,248,208,121,135,46,243,48,244,172,155,171,96,45,120,14,42,46,123,220,177,251,207,240,254,191,174,131,176,124,83,86,246,75,106,241,243,159,255,92,14,151,
203,37,51,209,43,127,243,44,26,254,211,51,50,43,204,189,246,17,46,221,128,79,45,23,240,181,151,191,129,34,170,240,84,244,64,233,11,157,244,96,217,146,159,181,15,174,183,48,162,192,250,80,151,22,86,91,
90,133,234,199,63,135,138,123,30,69,184,239,44,162,251,254,247,44,128,148,152,238,235,134,148,72,93,2,141,64,204,19,163,201,145,88,217,188,130,202,223,239,12,40,54,252,49,210,76,106,175,0,143,204,64,153,
176,125,33,153,151,222,249,224,247,163,113,137,170,39,142,117,121,200,115,112,100,104,101,199,4,212,174,111,197,47,191,253,147,154,79,236,110,208,136,33,75,98,102,4,67,23,206,64,145,78,183,152,21,82,11,
47,248,160,49,23,53,138,74,83,99,64,140,216,67,254,233,121,127,52,171,191,200,4,102,156,186,164,128,240,68,128,129,155,30,78,79,159,77,66,27,28,9,8,115,139,182,218,73,224,239,39,201,239,64,152,204,255,
248,216,8,140,13,219,229,2,38,153,188,84,48,232,169,178,50,207,202,13,235,72,203,237,132,204,165,239,211,103,148,48,43,213,87,207,47,121,215,185,243,152,15,46,185,231,33,24,212,196,234,93,31,226,102,0,
228,158,57,215,225,138,71,201,217,231,27,8,64,10,138,44,11,49,0,25,9,32,119,111,125,0,110,31,81,124,65,21,198,148,249,176,208,36,50,246,97,224,97,158,104,239,222,189,40,228,36,108,222,180,9,163,163,99,
200,148,215,97,74,212,32,240,193,75,84,196,168,168,228,166,245,220,215,10,61,153,240,49,170,22,148,90,35,82,241,136,231,186,169,157,202,86,96,250,194,37,88,249,196,23,97,88,251,160,188,255,118,160,253,
0,36,202,218,122,149,90,110,76,165,21,42,75,108,124,132,117,1,9,52,161,44,104,46,43,255,179,217,168,143,78,187,35,147,163,206,203,22,115,118,2,69,158,179,215,221,181,185,170,187,227,144,44,171,209,112,
208,21,10,4,115,54,56,81,187,65,222,248,141,167,210,45,195,146,222,153,230,138,32,22,243,224,164,4,68,133,128,209,139,221,244,133,23,88,22,239,169,127,224,137,157,189,177,104,91,124,182,106,189,244,108,
105,139,90,36,128,251,137,21,89,181,163,38,16,233,76,36,85,186,5,167,208,99,212,108,133,130,204,54,55,214,11,105,184,27,25,182,81,204,42,194,5,0,98,32,23,21,217,37,102,133,14,171,8,175,252,249,66,63,51,
55,216,190,164,229,145,38,217,183,165,10,170,49,210,215,139,128,247,196,77,237,198,203,77,60,255,169,15,240,242,151,31,135,40,73,208,198,3,184,107,203,125,184,251,254,71,112,207,131,15,203,82,118,71,221,
26,184,123,223,33,31,97,131,72,153,152,212,89,208,216,216,56,223,92,100,134,122,221,250,117,108,183,29,29,29,29,176,45,41,196,216,201,223,160,179,237,117,4,66,97,216,197,8,101,81,151,92,110,170,139,170,
16,25,238,101,242,117,221,13,69,189,78,27,88,250,228,87,161,172,221,140,115,99,62,8,239,237,35,111,65,255,149,152,137,73,194,96,66,221,92,127,255,227,150,161,161,145,166,180,223,127,5,104,22,140,188,2,
89,46,173,70,109,91,100,44,229,204,201,118,82,6,19,225,68,75,195,246,157,45,254,144,208,232,27,246,229,6,15,49,5,56,165,12,126,69,44,18,136,78,197,178,242,196,100,42,25,71,140,214,169,238,174,123,182,
81,217,46,203,64,82,169,241,170,52,36,103,87,2,136,152,88,33,55,62,153,33,20,179,33,204,64,147,111,88,176,7,67,0,139,14,32,77,204,36,127,111,141,19,210,76,15,138,31,107,130,152,202,32,58,218,15,97,224,
20,20,36,97,115,0,97,29,105,77,245,202,249,45,11,182,145,173,82,105,17,87,94,13,32,61,157,27,43,168,135,247,188,23,169,15,143,145,217,159,129,85,175,184,105,0,49,35,209,202,215,108,116,37,203,106,229,
106,230,224,249,35,56,248,215,95,195,247,246,236,70,253,93,27,49,238,163,164,52,45,203,202,3,1,40,158,86,200,62,104,110,143,140,29,159,127,254,121,212,213,213,97,203,150,45,180,240,122,2,210,27,88,182,
124,57,116,83,126,88,165,8,98,36,51,33,98,31,77,153,220,187,104,185,17,111,17,11,205,184,39,98,84,49,29,119,95,146,38,249,152,101,26,191,24,117,248,7,164,214,107,94,68,75,11,98,43,147,229,210,98,208,180,
6,77,134,230,96,56,154,19,28,147,254,80,99,32,28,107,92,216,101,191,106,216,202,189,120,237,175,229,14,178,65,244,123,56,255,84,35,107,36,206,223,51,97,194,31,77,181,213,110,254,100,171,82,103,12,140,
141,140,184,34,129,28,118,175,168,210,205,205,92,112,94,89,26,104,180,151,0,148,137,16,155,134,38,100,102,146,159,131,230,21,70,13,166,47,246,34,202,27,105,85,243,200,155,110,135,168,225,89,219,62,11,
138,60,250,236,222,93,72,232,11,32,17,35,11,228,5,5,33,8,149,56,147,3,64,22,156,63,242,30,1,135,238,47,60,45,203,190,184,108,197,77,3,136,141,230,240,254,87,92,168,251,120,150,78,137,97,80,125,55,80,185,
22,167,88,82,107,232,15,109,94,214,20,50,122,79,230,216,73,114,124,28,221,26,61,186,95,249,5,138,13,60,30,126,248,97,228,17,194,79,158,60,9,255,249,46,185,92,102,215,245,247,158,8,220,40,128,198,70,71,
58,21,158,119,221,176,86,59,231,64,3,49,186,232,150,69,206,81,82,237,198,107,123,229,5,239,250,237,235,193,226,170,154,230,96,38,221,2,46,119,166,93,19,60,108,127,172,100,105,211,220,182,111,200,63,218,
170,148,226,205,73,165,254,242,215,80,166,2,22,138,166,197,239,201,206,88,172,77,20,130,87,177,33,175,213,47,232,239,208,188,71,88,81,224,103,157,77,90,7,35,184,218,149,36,103,93,89,164,50,35,77,165,121,
202,98,131,176,50,187,151,169,35,195,236,61,248,239,136,38,210,178,97,103,111,45,112,122,3,50,43,86,95,221,103,211,211,154,246,117,102,37,127,86,214,99,139,204,203,245,94,90,24,64,66,104,198,248,57,112,
84,69,97,174,201,196,94,200,98,23,214,16,250,217,11,90,243,187,112,19,114,51,145,177,16,139,246,118,98,235,59,214,131,187,235,147,80,60,245,183,152,180,111,194,143,127,252,99,116,159,233,70,113,73,177,
220,106,15,4,3,180,246,114,213,212,140,155,120,55,58,47,216,239,210,140,157,13,96,122,64,166,248,155,2,79,97,69,0,213,245,151,149,169,227,3,231,159,83,197,38,91,185,69,182,61,174,9,158,218,141,173,120,
251,31,230,165,119,176,175,119,64,25,159,105,81,34,121,227,215,49,178,228,92,211,130,95,52,119,134,103,174,110,81,40,213,151,60,80,102,110,49,217,189,74,36,145,145,9,121,77,50,92,182,145,42,123,68,146,
77,72,113,185,119,147,237,160,155,192,5,198,128,169,65,170,115,233,24,153,70,134,37,93,14,15,196,202,127,249,186,11,60,161,220,83,186,5,6,130,220,117,29,57,219,88,81,191,209,145,200,43,135,137,208,237,
227,212,217,29,222,209,129,133,221,87,84,89,245,242,206,188,108,160,232,200,229,21,64,83,177,146,18,66,41,87,97,156,115,23,196,180,132,253,251,223,193,150,205,91,96,210,235,228,22,127,50,22,97,255,233,
185,155,89,183,64,32,48,80,145,111,115,136,58,125,235,140,144,114,74,201,212,245,255,19,49,33,42,107,220,40,95,225,162,133,186,170,215,148,20,2,79,23,90,242,60,49,133,177,57,18,79,89,174,123,189,130,242,
0,150,174,105,198,91,45,87,221,187,24,139,236,41,41,40,164,204,229,155,130,130,116,237,235,20,87,17,120,234,91,240,111,127,39,111,180,198,216,220,26,243,174,64,208,165,133,78,43,114,228,189,188,192,220,
213,239,9,204,45,60,37,122,230,42,22,89,240,243,203,94,79,81,223,48,215,220,232,47,22,58,7,223,126,217,125,247,227,207,58,162,198,74,72,190,97,226,138,43,246,86,6,79,194,82,85,32,191,39,196,122,65,236,
168,94,115,159,236,123,152,137,230,149,100,14,233,1,210,247,254,17,124,199,247,201,175,182,78,147,1,246,251,229,223,19,115,221,74,159,101,168,191,143,129,96,123,195,230,143,53,248,201,163,8,98,210,73,
165,187,61,33,37,237,18,153,196,12,207,35,163,209,121,37,40,188,48,89,221,36,91,109,120,253,219,215,124,97,107,114,100,240,57,50,187,173,73,78,221,24,138,39,27,227,137,164,61,205,41,28,98,130,40,159,100,
36,201,41,60,146,33,207,11,107,105,27,140,249,109,84,182,47,202,154,99,67,253,123,86,175,223,218,86,12,149,43,34,18,200,51,156,61,69,210,34,81,66,37,232,40,106,13,30,88,203,60,228,99,154,233,58,243,128,
214,40,51,30,181,50,70,254,216,8,181,206,40,55,252,176,160,121,167,177,21,185,141,196,88,81,121,255,77,37,131,32,163,80,121,108,229,21,208,17,13,69,137,85,98,236,77,3,166,16,156,74,54,90,25,37,31,40,44,
43,119,27,50,10,68,169,116,143,114,188,44,125,115,63,191,108,139,71,201,123,203,150,46,119,51,131,29,86,168,233,126,233,92,115,81,206,234,248,102,126,173,71,126,79,250,241,63,253,178,131,43,186,3,191,
61,214,129,153,208,108,179,233,226,81,52,44,49,203,178,229,241,120,228,120,227,141,55,192,151,44,69,217,159,253,0,6,90,72,29,129,136,237,254,22,36,253,176,251,218,209,113,252,40,14,31,190,253,107,61,191,
235,227,150,126,177,112,195,230,109,141,95,254,239,255,19,131,222,139,216,127,232,8,14,12,135,16,145,82,228,219,84,242,187,56,44,212,214,82,152,235,239,133,217,148,15,123,158,1,107,139,205,168,183,170,
208,115,228,223,240,195,23,158,103,239,20,49,195,188,231,246,18,252,129,0,136,237,172,147,95,225,227,177,184,161,241,177,198,93,158,147,39,191,254,241,29,59,139,255,243,51,207,162,110,117,61,70,130,2,
124,161,120,182,85,78,82,197,142,74,58,86,91,244,48,40,129,183,127,243,38,126,250,210,63,162,191,255,226,169,77,155,54,126,239,197,31,254,112,159,86,163,13,147,148,137,6,131,62,115,123,41,126,143,1,20,
141,70,149,66,44,110,163,99,49,153,231,194,104,84,40,16,132,152,237,165,151,126,180,237,224,193,131,27,194,161,96,89,77,77,141,122,235,214,173,217,183,125,103,119,2,216,107,175,93,93,157,248,232,248,113,
105,69,77,205,128,211,185,253,232,99,143,125,234,35,189,65,55,109,208,235,167,245,122,253,36,129,103,156,124,210,132,86,167,141,107,212,234,219,43,242,251,8,160,112,56,162,137,10,66,73,36,18,45,38,16,
21,202,33,8,5,89,32,9,54,10,107,119,247,233,37,227,99,99,214,225,225,33,211,172,101,207,148,151,47,137,22,21,23,7,86,173,170,27,209,233,116,126,2,140,12,28,2,205,148,193,96,152,156,61,142,27,13,6,159,
78,175,11,107,53,154,219,43,242,59,54,110,168,10,83,107,212,98,42,157,242,101,50,153,144,82,169,152,225,121,222,175,209,104,38,245,58,157,37,30,55,90,68,81,52,21,216,108,103,164,164,164,75,38,83,106,58,
79,65,18,150,161,210,61,201,171,84,113,94,205,11,26,181,134,228,74,19,32,201,10,82,248,117,58,237,20,11,173,86,23,80,171,121,241,54,120,126,55,199,255,21,96,0,219,144,75,33,4,238,1,6,0,0,0,0,73,69,78,
68,174,66,96,130,0,0};

const char* MainPage::ucsniffdarkblue_png = (const char*) resource_MainComponent_ucsniffdarkblue_png;
const int MainPage::ucsniffdarkblue_pngSize = 7036;

// JUCER_RESOURCE: ucsniffdarkgrey_png, 5837, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC-Sniff-Dark-Grey.png"
static const unsigned char resource_MainComponent_ucsniffdarkgrey_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,144,0,0,0,34,8,6,0,0,0,176,68,80,198,0,0,0,25,116,69,88,116,83,111,102,
116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,22,111,73,68,65,84,120,218,236,91,9,116,148,215,117,254,52,187,102,70,163,209,142,4,72,3,8,144,64,192,8,
27,108,179,105,48,118,241,18,199,178,29,227,28,59,169,201,214,19,59,137,35,103,57,105,211,52,17,231,36,77,147,166,141,92,59,118,154,186,181,72,27,59,49,1,43,182,193,108,134,145,205,98,196,54,2,137,69,
66,251,190,207,140,52,171,70,163,222,251,102,97,52,26,1,130,180,39,113,244,206,121,250,127,253,255,251,223,123,255,187,223,189,247,187,247,253,35,193,76,153,41,183,80,100,255,7,125,230,80,53,80,53,69,
92,51,83,181,82,173,158,89,242,153,18,171,172,160,250,26,213,33,170,227,92,227,226,226,194,53,116,45,120,255,45,170,15,207,44,217,199,163,196,253,17,128,83,6,169,220,132,84,50,60,14,43,62,121,247,90,124,
189,228,121,36,37,37,65,38,147,138,70,14,135,19,173,173,45,104,184,210,128,99,199,142,225,253,247,15,192,229,114,53,211,173,173,84,43,103,196,240,151,9,160,31,80,45,69,124,34,144,183,17,208,165,2,163,
94,160,163,22,143,222,102,192,136,195,129,135,30,122,24,134,121,243,38,61,232,243,249,176,247,189,61,216,241,230,239,48,56,56,80,17,4,146,109,70,28,127,25,0,34,196,160,156,106,49,116,105,64,254,38,96,
206,34,64,161,10,220,237,109,39,71,213,35,78,83,108,245,248,222,115,127,131,248,248,248,9,29,140,143,143,67,46,151,99,148,0,183,107,231,78,28,216,191,215,18,228,76,127,178,32,74,73,73,41,26,29,245,17,
248,71,89,1,172,94,175,119,134,207,77,19,64,69,84,75,226,116,169,197,227,6,242,92,236,178,184,198,107,137,138,203,25,21,1,154,35,14,244,103,204,7,212,157,193,26,245,32,238,191,103,227,132,1,147,146,147,
33,151,201,16,39,145,192,227,241,224,253,3,251,241,214,91,59,167,2,209,207,169,26,121,236,104,18,254,210,203,191,124,186,246,124,205,214,174,174,78,12,15,15,211,176,126,1,78,191,127,28,237,237,109,104,
104,184,34,6,158,59,55,251,233,236,236,236,173,98,252,184,56,81,67,179,25,25,25,193,217,179,167,55,70,191,236,51,207,126,45,241,220,57,75,137,68,34,41,86,169,84,70,153,84,134,132,132,4,113,175,175,175,
15,30,175,135,221,176,185,169,177,177,220,106,29,218,126,35,11,184,120,113,94,145,205,102,53,141,142,142,194,235,29,21,10,68,231,150,177,177,49,243,255,167,242,100,100,204,202,241,122,61,6,94,123,143,
199,107,29,27,243,197,84,134,244,244,12,106,231,165,118,110,209,206,239,31,171,190,153,40,76,88,156,244,197,183,23,103,222,187,21,182,244,249,232,27,182,193,117,225,24,252,77,167,201,101,93,10,136,226,
129,175,2,74,117,0,67,33,48,45,190,13,199,14,252,39,86,27,251,195,157,197,199,171,8,111,242,8,33,2,107,215,111,224,107,70,178,68,102,171,213,90,18,193,139,138,18,117,186,146,81,114,121,78,167,179,56,26,
64,173,45,173,134,190,254,62,19,247,199,160,140,44,131,67,131,225,115,18,188,97,86,102,150,41,102,24,74,207,70,151,159,191,240,226,211,39,171,170,202,82,83,82,245,145,215,121,193,185,232,116,186,208,37,
83,246,220,108,83,83,83,83,105,93,221,165,98,183,219,29,83,16,191,122,245,181,175,31,61,242,97,41,1,79,175,81,171,39,220,115,144,171,239,237,235,69,115,83,83,25,189,123,41,89,184,155,2,210,142,157,21,
63,216,243,238,59,165,93,93,93,164,64,126,161,76,181,181,181,230,206,206,142,176,114,252,221,119,255,161,232,163,227,199,202,9,8,6,133,66,41,218,113,37,101,51,215,215,215,133,219,125,243,91,223,121,184,
234,196,71,101,10,133,220,32,145,72,195,237,206,156,61,109,182,89,173,27,167,11,160,114,85,222,157,197,115,62,255,99,216,8,137,253,77,53,112,189,251,18,252,189,45,17,2,210,17,160,42,161,88,243,41,120,
14,109,135,124,227,103,133,21,26,39,140,204,207,78,3,105,104,216,117,201,100,105,176,219,109,164,251,65,75,64,117,176,191,15,57,57,6,20,23,63,106,108,108,108,48,159,175,57,111,30,26,28,44,101,142,37,87,
40,224,37,141,13,166,2,38,148,190,222,94,248,2,247,38,149,49,2,93,248,124,108,12,30,183,59,102,59,31,243,182,136,242,247,223,251,254,211,239,31,56,80,78,26,122,195,194,155,53,43,131,22,90,66,66,184,108,
36,64,180,68,222,123,227,119,191,255,249,123,187,119,151,240,26,200,164,210,48,8,195,2,32,75,156,149,153,133,180,212,180,146,211,167,79,153,122,123,123,10,111,6,64,36,112,216,201,10,107,180,218,240,53,
137,84,18,62,255,197,43,191,202,57,87,109,49,39,71,41,26,23,151,219,5,2,144,56,127,229,223,95,45,162,190,42,210,210,211,39,91,18,93,34,8,64,211,178,64,156,211,41,30,47,216,136,33,175,27,206,182,75,112,
31,173,192,120,90,46,176,124,51,112,229,4,208,92,141,249,11,22,224,82,205,33,196,175,123,12,234,7,159,129,223,61,66,214,72,131,49,207,8,250,27,47,194,62,59,48,25,185,92,6,210,82,168,116,122,44,185,109,
53,78,30,218,15,63,113,10,165,82,137,214,230,6,116,118,117,83,244,150,140,7,31,124,200,212,218,210,108,174,173,173,33,193,7,92,69,172,104,173,163,163,93,112,169,152,192,136,0,16,159,83,31,49,219,145,105,
14,159,255,219,75,47,23,237,223,187,183,188,167,167,103,218,2,164,121,232,211,210,210,203,29,142,166,176,134,190,187,103,95,209,239,119,188,89,210,221,221,121,67,125,228,229,229,25,135,135,237,79,211,
92,183,79,119,252,61,187,223,197,130,5,185,19,174,249,73,113,66,165,177,225,74,73,103,103,236,121,140,69,180,171,57,127,110,107,63,185,232,88,37,224,89,166,231,194,56,33,136,5,217,139,144,164,214,162,
110,196,1,205,252,149,176,157,221,71,252,230,40,69,93,110,228,229,47,65,62,85,29,89,161,250,83,111,35,233,222,47,192,175,142,7,209,16,216,46,214,147,121,238,195,9,210,142,69,139,22,33,45,45,13,253,100,
109,214,62,246,36,22,46,93,142,37,171,215,224,204,17,51,142,189,247,46,82,210,50,160,75,76,66,77,237,121,168,53,106,100,102,205,70,54,89,37,214,172,225,186,203,101,177,38,215,75,22,136,251,188,30,128,
88,235,109,182,216,158,193,229,114,134,207,207,156,62,93,126,241,226,133,152,237,152,87,185,221,34,245,0,181,90,109,152,98,129,77,122,189,62,135,92,145,176,66,109,173,173,91,107,107,106,162,0,235,198,
192,192,128,133,231,71,115,55,198,199,79,116,105,164,64,197,46,87,199,180,1,68,209,172,80,190,144,251,226,249,18,127,9,223,63,123,230,140,145,21,53,178,52,53,53,90,57,157,194,207,132,138,229,236,89,131,
54,194,138,113,105,105,105,182,210,124,155,29,142,145,105,3,200,34,94,202,53,132,44,141,22,179,54,60,128,170,202,119,97,203,93,3,44,187,23,178,225,78,116,55,126,4,114,59,162,177,180,241,8,28,114,53,114,
31,248,34,60,78,59,90,14,253,38,108,41,180,26,13,104,113,33,211,36,34,117,97,1,70,124,126,72,201,125,229,223,85,132,101,107,139,112,226,224,94,28,170,216,129,89,25,153,2,233,204,144,152,100,26,141,133,
168,171,187,28,83,96,236,102,156,78,71,204,137,143,70,184,54,182,122,214,40,211,123,21,104,129,118,239,236,222,187,226,91,223,40,49,16,201,141,217,206,233,116,85,208,88,143,144,171,74,212,104,52,22,18,
70,204,57,145,91,102,174,37,0,112,236,216,81,195,80,4,23,227,50,48,208,111,25,26,26,42,12,244,233,56,76,132,214,20,5,66,253,181,4,66,227,231,144,192,217,101,210,185,212,18,226,76,4,152,242,11,23,106,204,
1,98,236,17,10,68,109,172,87,173,117,27,50,51,103,71,112,175,17,94,215,173,212,215,31,38,190,167,3,17,244,84,40,94,172,118,55,10,32,158,92,217,149,67,175,151,100,175,220,36,132,122,231,250,251,49,254,
250,79,208,94,181,23,62,255,24,172,195,125,48,62,82,44,26,219,237,118,92,218,247,95,104,165,58,89,160,94,244,116,119,99,243,83,207,224,228,241,163,144,73,226,80,176,234,46,58,146,249,167,25,175,216,184,
25,203,76,155,177,127,231,111,113,252,157,157,208,17,217,86,40,21,24,13,8,88,31,219,253,120,4,9,189,30,128,24,104,118,187,117,10,179,28,118,1,197,164,145,83,46,4,9,164,44,32,40,191,141,0,89,78,99,151,
198,106,167,80,40,12,87,221,193,121,68,3,146,162,47,61,129,44,135,128,210,66,129,129,153,172,168,73,34,137,11,71,135,116,221,184,253,191,95,63,92,121,248,176,233,194,197,90,225,94,216,21,181,181,183,89,
212,241,106,235,236,57,115,76,114,142,122,227,2,194,109,108,104,40,163,49,158,231,168,74,167,75,52,171,213,99,97,151,68,242,48,63,255,141,111,225,192,254,125,38,230,157,145,107,197,107,151,153,153,85,
193,99,18,152,204,223,252,246,119,76,59,119,188,41,148,45,210,85,113,95,4,242,10,158,35,61,207,65,206,180,73,116,115,207,229,83,56,253,222,107,152,91,180,133,200,169,23,190,156,66,72,201,58,142,201,164,
80,14,247,224,32,133,225,169,83,184,146,80,97,243,57,166,79,199,144,123,20,111,125,247,203,96,50,151,151,147,140,21,41,30,40,244,25,192,170,167,144,187,246,94,172,249,228,22,20,21,111,193,139,37,95,162,
133,243,115,116,130,96,222,41,38,128,34,163,185,104,192,94,21,154,119,2,160,38,106,116,32,91,78,188,199,24,105,242,167,178,198,65,59,83,78,227,154,3,86,64,88,2,72,165,225,243,102,78,13,112,233,234,234,
136,229,58,13,244,108,51,85,234,47,174,130,92,154,145,222,99,66,244,118,226,163,227,63,232,238,233,54,165,167,165,71,144,109,185,49,35,35,99,66,71,76,27,40,82,36,242,125,146,243,82,4,30,205,164,245,57,
114,228,3,16,175,66,16,40,19,73,113,98,98,120,125,142,31,61,42,82,33,209,202,199,133,44,110,184,221,205,132,241,6,242,249,104,122,231,23,232,107,185,136,89,15,61,139,4,195,98,56,136,185,187,237,67,240,
120,199,208,216,211,142,213,119,44,32,159,158,14,47,77,154,93,154,60,94,139,81,87,96,194,25,196,232,249,101,150,62,241,37,92,169,61,135,71,151,41,176,105,121,2,50,146,85,200,75,81,176,225,196,217,115,
255,136,67,158,81,44,41,122,0,214,182,86,12,118,181,11,159,126,229,74,61,131,167,101,106,43,51,126,67,36,58,242,255,80,89,185,242,54,145,49,175,187,124,9,11,23,230,234,127,245,31,175,10,141,227,202,220,
132,73,41,143,65,209,17,254,246,59,223,182,69,128,179,101,170,57,77,228,77,126,51,61,111,138,212,232,172,172,44,228,46,92,136,121,243,230,27,151,46,45,48,210,248,165,199,143,29,181,146,181,42,35,224,149,
145,176,109,149,102,179,37,119,97,46,236,54,123,248,57,150,65,72,192,209,37,57,57,165,132,67,241,104,128,80,184,78,10,216,136,241,241,235,206,19,53,53,231,39,69,136,147,201,182,255,166,0,100,228,80,147,
179,201,238,75,71,112,165,177,26,218,205,95,132,118,217,157,72,32,229,31,35,18,106,189,188,16,187,142,188,141,124,1,6,54,227,74,120,131,224,97,203,163,224,172,179,46,21,117,39,142,96,224,192,175,241,202,
215,12,98,159,172,109,112,8,227,62,5,65,192,143,21,217,114,236,63,246,63,24,94,123,31,118,191,246,75,161,29,20,22,243,162,149,78,53,177,64,130,107,42,11,52,17,48,139,243,242,176,234,246,85,88,190,194,
200,166,27,82,122,167,246,182,54,84,87,91,4,128,56,26,115,56,221,66,1,2,60,162,27,23,46,94,164,255,189,76,34,111,54,79,91,70,64,44,41,44,44,212,47,45,40,192,237,171,86,11,32,12,210,123,51,233,237,232,
232,16,32,209,235,147,244,43,140,198,210,43,245,245,37,20,169,154,134,134,6,172,25,25,235,112,223,253,15,98,128,130,14,114,65,20,124,244,115,208,32,8,111,122,122,186,224,64,17,133,221,162,145,92,226,164,
232,138,184,27,29,125,2,36,44,199,200,136,138,239,115,63,124,206,220,135,45,12,183,139,101,213,227,226,216,186,198,221,20,128,130,254,89,34,180,82,233,31,197,200,174,159,98,56,103,5,84,15,126,5,18,149,
6,202,188,149,112,201,85,232,184,176,23,143,222,123,167,224,66,191,125,35,64,160,147,136,56,243,164,186,107,207,162,251,210,121,113,237,151,135,228,80,106,147,241,245,117,4,158,177,171,168,207,208,201,
209,68,194,106,61,125,84,0,182,181,181,181,252,90,154,206,214,33,50,4,141,214,170,80,249,220,231,191,128,79,62,92,140,206,142,78,180,17,104,234,234,204,24,26,28,20,243,12,145,220,147,39,171,132,187,12,
89,160,80,242,140,181,120,96,96,240,166,224,211,219,219,99,211,233,116,166,81,159,175,220,237,246,24,79,158,168,194,240,200,48,92,36,104,6,41,11,44,68,122,41,194,19,64,32,33,151,251,124,99,37,204,159,
134,8,104,89,179,103,227,241,45,159,198,193,131,251,81,91,91,99,34,87,198,17,145,85,165,82,77,120,87,38,223,209,1,5,203,141,230,176,49,168,212,135,21,10,185,105,34,247,27,55,17,175,169,140,74,71,140,71,
247,65,215,168,157,189,242,102,72,116,4,250,2,136,103,95,159,146,146,2,239,96,35,172,111,148,34,110,211,23,16,151,150,13,204,202,70,127,220,253,56,88,249,38,57,75,23,102,207,158,131,206,206,14,36,104,
19,160,76,76,70,79,132,16,254,105,79,39,238,153,63,136,175,172,90,4,153,50,48,133,190,126,23,118,163,0,109,47,253,12,61,173,173,176,19,143,240,78,65,84,35,1,20,237,175,179,40,252,223,242,196,19,184,124,
249,178,200,141,132,200,172,211,225,20,124,132,205,60,147,73,118,7,35,44,76,87,32,193,216,221,213,101,161,54,166,0,128,124,194,92,7,142,172,197,78,230,32,137,161,136,135,56,65,14,173,137,33,130,247,136,
35,255,47,149,74,155,201,178,132,65,79,32,173,174,58,241,81,225,201,170,19,69,20,133,22,107,181,9,188,53,98,8,145,123,2,86,16,76,30,241,46,244,188,49,33,65,103,228,173,24,78,49,244,245,245,162,137,40,
65,75,171,232,178,133,163,70,190,30,9,22,122,70,8,218,29,149,44,85,42,85,19,56,33,191,79,68,52,7,149,42,254,154,193,71,168,239,200,126,110,193,2,197,137,240,142,59,12,117,26,79,188,162,243,173,159,192,
255,196,182,192,54,134,46,5,141,154,28,60,178,36,30,115,230,204,197,7,31,152,57,23,130,123,127,182,3,245,91,55,193,239,14,152,88,14,233,211,87,125,2,223,222,123,9,73,158,38,72,212,9,248,96,246,131,240,
223,247,105,212,191,184,82,100,167,201,117,88,175,199,51,130,154,39,206,217,69,108,222,124,191,112,85,108,101,216,186,132,74,119,119,143,72,36,6,64,19,0,16,187,63,230,69,161,68,36,181,111,166,16,91,244,
25,0,81,40,159,226,15,154,117,137,49,148,204,36,203,81,74,11,189,53,214,156,18,18,18,24,244,219,8,44,43,8,116,101,44,180,0,137,247,137,177,237,246,225,98,114,223,86,90,207,98,170,37,4,32,3,143,19,41,64,
178,90,198,110,138,88,7,201,74,242,182,9,127,26,19,73,198,139,76,27,145,159,191,20,22,203,25,156,62,117,74,188,75,104,107,98,162,220,48,201,101,77,4,135,100,202,181,189,251,238,77,88,191,97,131,144,243,
79,127,242,227,91,2,144,133,204,184,8,53,165,82,89,24,64,1,16,41,49,151,142,45,123,95,6,54,63,27,232,208,248,87,168,218,251,207,224,108,38,223,103,95,223,93,253,17,30,175,168,198,235,247,204,19,220,35,
137,248,141,54,41,5,87,230,62,139,145,198,125,56,151,85,136,185,79,60,139,75,63,254,42,226,21,10,56,133,38,141,91,174,55,49,94,20,195,188,249,40,42,50,137,69,230,12,114,29,89,30,97,97,236,195,236,6,139,
72,136,150,190,190,158,98,222,112,13,248,248,177,73,202,17,212,80,51,11,105,124,10,198,41,147,197,137,61,58,178,62,137,164,233,166,41,55,14,19,245,205,65,178,219,204,235,70,238,40,232,18,3,160,164,117,
227,77,76,206,169,188,64,150,134,34,57,175,37,134,23,54,48,240,70,70,60,194,98,50,144,34,57,15,43,37,111,43,172,32,62,247,216,167,182,224,28,241,184,223,253,246,117,44,91,182,156,147,131,8,69,129,145,
92,38,152,34,152,100,20,162,203,125,247,221,143,47,63,243,21,52,52,52,136,96,232,210,197,11,136,226,91,211,7,16,47,188,70,163,69,40,108,101,240,4,142,50,161,33,58,226,45,246,174,6,34,60,153,0,113,162,
193,212,197,88,161,83,35,127,206,28,225,202,14,190,248,125,44,162,200,99,66,190,196,248,8,182,230,231,160,244,179,63,194,80,82,39,102,155,62,1,199,7,111,67,66,125,7,9,240,117,1,148,147,99,176,22,22,222,
134,174,206,78,52,92,169,23,11,23,105,101,200,194,112,20,20,51,2,139,216,139,178,176,11,177,90,173,213,244,94,102,18,182,105,138,168,174,152,238,159,165,182,122,106,99,152,98,59,131,133,34,246,236,168,
63,27,147,214,201,99,75,74,9,216,22,178,136,45,100,49,138,163,1,203,107,203,187,243,156,213,14,41,9,243,163,72,33,50,160,40,58,21,95,5,4,211,28,130,102,176,85,186,243,174,53,194,37,50,207,226,172,58,71,
87,34,18,42,92,137,39,159,124,42,28,145,182,182,180,96,215,174,157,152,188,61,212,137,151,127,241,146,24,131,249,33,43,85,40,213,113,179,0,50,135,72,30,105,115,56,39,16,114,105,252,98,28,110,219,91,206,
129,63,46,243,145,155,74,201,89,66,33,64,179,48,127,236,202,148,212,238,232,15,159,195,166,141,119,163,181,163,29,163,234,116,216,165,105,56,248,198,43,24,226,9,38,101,193,94,241,42,241,37,45,122,41,218,
8,44,162,207,124,189,137,145,182,85,144,16,202,88,195,89,83,217,77,113,52,21,109,101,174,85,212,106,77,69,40,193,70,239,82,26,252,180,98,170,61,35,227,181,250,162,32,163,188,173,173,181,37,194,197,242,
199,114,197,81,64,52,18,40,154,25,108,204,99,162,1,196,129,10,185,53,75,244,245,72,107,17,224,114,142,176,123,83,144,213,102,119,180,123,247,59,180,230,10,97,249,153,223,68,230,109,186,136,143,238,218,
249,123,1,158,80,116,22,203,178,112,196,217,220,220,40,214,50,228,226,89,190,183,2,32,214,20,179,221,47,55,249,149,201,232,165,176,83,233,232,224,208,83,76,158,39,43,190,147,113,82,52,229,9,112,28,169,
90,137,15,15,87,162,218,114,54,156,96,156,155,61,151,194,232,219,161,62,31,143,81,185,6,35,117,31,226,195,29,191,22,100,121,217,138,59,48,122,114,191,136,12,2,28,196,199,252,231,15,55,224,194,90,136,108,
178,208,75,199,175,151,236,136,81,120,75,130,8,109,89,4,255,96,142,83,22,252,246,104,90,133,4,111,77,76,76,44,225,140,111,4,128,202,162,1,20,10,175,99,69,143,12,18,90,203,50,10,215,173,215,122,31,190,
199,10,195,0,100,75,145,144,160,13,39,10,35,19,243,188,117,116,117,11,133,168,4,241,170,171,214,82,65,193,208,100,96,112,31,19,231,22,119,77,174,116,163,191,202,40,151,104,147,76,146,141,159,135,151,162,
0,79,95,59,122,234,171,208,211,82,15,169,119,36,144,95,72,201,5,220,14,49,224,48,69,22,249,75,150,224,142,59,238,10,100,86,79,28,199,190,125,251,80,85,85,133,245,235,215,97,225,156,52,88,142,188,142,130,
130,101,104,37,194,155,238,25,66,7,1,103,216,225,8,37,189,202,110,84,112,164,37,219,216,124,211,178,150,222,112,118,134,22,91,171,213,90,72,9,76,100,49,162,83,197,207,211,125,18,32,74,167,74,82,70,247,
197,46,41,37,37,117,107,140,190,42,233,62,147,237,242,235,1,156,53,157,0,88,78,92,141,199,47,138,53,78,52,57,230,62,153,64,51,9,15,125,236,54,245,51,113,147,8,118,44,11,116,163,237,166,11,160,237,222,
246,75,37,56,125,192,136,196,89,220,35,144,153,15,164,206,195,24,91,12,10,219,49,234,226,240,68,52,30,182,17,210,181,81,31,98,229,174,194,80,206,114,188,125,242,3,224,237,119,176,110,221,58,172,91,189,
10,50,66,247,224,229,115,130,251,112,120,74,100,214,58,29,0,5,22,210,191,141,44,97,121,208,18,49,175,208,71,71,36,188,8,44,36,58,86,208,98,87,144,150,111,159,42,179,75,207,111,35,119,93,206,160,140,213,
31,47,50,43,13,185,14,11,89,49,182,24,219,157,206,214,169,250,218,78,218,222,76,103,108,213,138,67,253,132,72,44,211,2,133,66,110,33,64,151,81,248,191,61,242,179,137,80,254,45,16,192,72,195,124,42,240,
30,82,132,246,208,2,220,75,34,188,1,191,103,232,25,118,109,145,223,29,5,238,197,137,35,223,139,197,109,120,62,220,54,148,186,161,185,93,147,3,77,239,147,86,117,162,89,83,244,25,56,108,17,11,239,35,208,
56,73,230,234,164,192,167,173,130,156,156,68,234,104,63,102,19,137,230,151,226,79,50,112,247,86,196,229,173,21,159,177,198,53,87,99,236,192,171,40,88,56,31,249,121,249,98,147,181,179,187,139,88,127,35,
155,125,94,232,23,112,11,133,162,27,241,219,180,200,60,13,91,149,190,190,222,155,250,142,153,132,59,41,239,211,223,223,87,121,115,125,37,20,113,22,254,86,251,249,83,41,211,249,97,97,37,156,182,210,212,
254,139,165,233,119,60,142,241,222,54,56,147,179,208,203,153,220,246,6,76,216,112,25,108,197,163,91,255,90,144,104,230,4,12,32,85,254,26,72,212,193,72,174,96,61,124,116,175,102,231,79,5,217,75,209,39,
6,191,101,22,164,243,133,91,125,169,225,97,251,13,237,85,221,104,161,168,238,143,214,223,200,200,240,199,234,103,76,210,105,182,175,180,117,54,25,214,45,156,107,212,205,91,142,134,134,122,140,246,117,
49,91,140,8,141,78,66,225,28,16,31,157,179,217,181,16,145,182,101,46,133,122,241,106,168,53,9,208,40,84,80,147,85,210,206,154,7,235,225,223,160,96,105,1,252,30,23,78,91,170,45,65,194,233,193,76,249,216,
2,136,203,31,234,206,159,50,204,146,185,140,159,121,252,211,112,83,216,220,209,55,16,204,112,145,135,232,168,65,193,178,229,226,83,3,206,65,112,26,94,74,254,56,117,245,3,208,19,128,146,200,242,36,83,93,
145,164,196,61,139,83,209,211,213,137,131,102,51,135,206,247,97,230,183,97,127,118,229,86,126,88,248,116,102,214,156,178,239,111,251,161,62,39,39,27,251,223,175,196,153,198,118,212,247,219,209,53,226,
185,154,173,206,202,69,194,194,149,200,76,78,197,202,12,61,150,164,104,177,72,47,67,205,241,61,120,229,197,127,225,95,172,110,35,215,85,58,35,138,191,0,0,113,126,192,229,114,73,93,110,183,214,229,114,
39,149,151,151,27,203,95,123,173,196,227,245,22,125,238,75,207,98,205,218,245,88,182,108,153,160,67,237,118,55,186,134,221,72,32,22,159,155,172,17,215,56,199,182,111,79,5,118,237,120,3,67,67,3,151,158,
124,242,169,127,253,218,115,207,29,143,87,169,250,85,241,42,222,97,118,171,227,227,103,164,242,113,5,144,211,233,148,57,28,206,140,17,135,35,195,225,112,164,209,121,42,31,207,156,57,179,224,253,131,7,
111,59,117,234,228,18,138,116,18,55,108,216,16,200,160,140,95,205,164,156,63,87,205,27,126,195,243,230,205,235,122,236,83,91,142,46,95,190,188,81,163,81,247,83,24,76,85,221,71,199,94,173,70,211,65,64,
178,199,171,84,51,146,249,152,2,72,226,116,185,244,4,156,89,116,158,78,199,52,135,211,153,226,116,80,117,185,146,201,58,37,81,213,19,144,50,199,253,126,37,69,86,28,63,115,8,61,90,88,184,178,143,66,122,
91,124,124,188,149,172,204,160,90,173,30,212,168,213,3,106,6,145,90,221,75,199,110,58,246,16,128,220,170,168,95,15,204,148,143,17,7,34,247,197,223,157,72,169,170,221,110,143,206,227,241,36,210,49,145,
220,88,130,215,227,209,142,250,124,26,223,168,79,57,230,31,147,19,128,68,138,88,18,39,25,147,202,164,94,185,76,230,146,43,20,14,165,66,49,66,96,178,43,149,10,27,185,45,114,93,202,17,149,82,229,209,233,
18,198,103,68,242,231,85,254,87,128,1,0,107,116,31,91,237,158,177,233,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* MainPage::ucsniffdarkgrey_png = (const char*) resource_MainComponent_ucsniffdarkgrey_png;
const int MainPage::ucsniffdarkgrey_pngSize = 5837;

// JUCER_RESOURCE: ucsniffdarkred_png, 6807, "usr/src/ucsniff-new-gui/ucsniff-3.09/images/UC-Sniff-Dark-Red.png"
static const unsigned char resource_MainComponent_ucsniffdarkred_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,144,0,0,0,34,8,6,0,0,0,176,68,80,198,0,0,0,25,116,69,88,116,83,111,102,
116,119,97,114,101,0,65,100,111,98,101,32,73,109,97,103,101,82,101,97,100,121,113,201,101,60,0,0,26,57,73,68,65,84,120,218,236,123,121,112,28,215,121,231,175,239,153,158,25,96,8,12,0,18,7,57,34,0,130,
4,73,1,34,45,137,162,66,106,104,69,22,35,89,17,28,91,146,157,181,77,40,107,87,214,222,218,13,156,212,174,83,169,236,10,169,77,85,42,94,71,166,146,56,118,146,114,9,114,100,75,178,68,145,214,97,199,212,
193,129,110,137,150,57,224,1,94,32,56,32,137,115,0,204,125,244,244,181,223,235,1,113,144,32,41,210,251,143,20,190,170,143,221,236,121,253,250,245,123,191,247,251,126,223,247,26,2,174,151,235,229,183,40,
252,255,135,54,218,200,190,71,182,143,204,94,196,226,100,187,201,254,132,108,197,245,33,255,100,21,238,183,184,119,7,89,55,89,16,222,0,224,171,2,180,28,254,249,47,30,198,131,247,223,231,52,108,232,121,
68,14,157,194,129,131,125,232,221,127,8,175,244,190,1,77,211,122,102,238,27,186,62,252,255,49,1,196,88,164,167,182,58,16,210,138,26,166,202,154,128,165,45,4,160,10,32,159,1,134,143,224,51,245,2,10,122,
1,255,237,43,247,226,190,59,55,66,215,45,20,178,6,78,157,24,199,247,159,121,17,63,221,219,11,211,52,25,136,254,234,250,20,124,188,139,112,149,245,239,32,11,151,187,197,213,170,34,98,76,169,3,154,54,17,
164,214,0,158,50,192,79,76,36,169,56,149,145,48,100,85,98,239,107,111,224,63,111,169,129,155,207,64,22,210,64,46,139,91,26,234,176,101,69,29,14,28,27,12,197,139,122,7,181,247,52,153,118,125,42,62,217,
0,98,172,179,211,177,64,131,75,107,104,71,178,229,119,129,198,79,1,53,203,9,52,50,32,74,100,34,80,78,174,172,166,1,168,172,133,86,4,94,120,238,23,248,116,131,31,42,1,200,235,205,67,81,114,248,225,139,
125,24,151,124,168,175,175,95,90,40,228,183,23,10,133,99,87,112,105,229,215,65,246,241,117,97,143,72,110,95,119,237,214,7,128,79,253,30,226,134,141,124,33,7,35,122,8,246,240,137,146,78,246,85,130,91,115,
59,108,219,46,201,102,246,15,59,215,242,192,219,187,241,200,13,192,61,191,227,71,57,145,212,171,3,35,120,225,68,0,75,42,42,156,199,75,4,186,51,103,134,112,232,240,161,112,124,122,154,185,181,222,11,158,
255,184,36,73,157,186,174,51,0,127,235,252,197,15,191,212,210,214,55,170,249,223,27,76,32,201,113,48,92,110,232,178,130,34,47,161,200,9,137,215,35,71,251,88,61,73,146,87,40,138,28,244,168,30,120,93,50,
84,183,11,170,36,194,37,114,180,122,108,22,69,68,94,141,28,77,46,246,226,95,92,238,111,43,8,66,135,198,75,65,147,227,131,228,137,145,177,184,68,50,17,143,12,229,205,136,219,237,14,39,18,137,228,71,29,
236,207,181,214,183,233,162,203,159,231,69,232,16,34,111,68,142,36,63,238,0,18,175,240,251,247,68,127,77,215,167,254,228,159,80,244,44,193,153,100,2,133,253,123,160,191,253,156,35,152,231,23,185,241,38,
160,44,0,62,178,23,102,219,239,58,248,129,75,37,82,178,32,22,44,140,68,167,209,250,25,3,63,125,34,129,218,166,38,7,108,12,189,186,161,99,89,109,29,150,47,95,17,34,32,133,47,0,210,10,143,170,118,242,130,
64,58,74,111,159,255,188,31,31,158,222,121,48,86,8,129,126,179,121,106,169,64,116,167,25,176,57,30,182,164,132,169,202,54,135,58,85,161,179,193,205,117,67,208,8,46,6,213,161,99,145,119,234,153,4,94,81,
85,67,23,130,246,255,110,90,190,227,189,177,108,247,164,97,7,77,139,189,72,113,246,55,47,51,175,171,99,41,157,100,109,62,97,85,52,236,132,162,238,60,112,244,248,162,96,248,193,237,43,118,244,14,37,186,
72,29,182,103,83,25,88,92,14,236,205,139,54,135,27,202,61,209,148,205,247,72,170,103,231,216,216,216,53,131,137,22,211,142,231,251,147,157,253,177,28,116,183,27,69,81,70,220,18,177,255,228,233,109,243,
235,125,35,88,177,99,212,70,71,81,102,32,150,48,158,202,97,50,149,134,229,246,118,77,78,78,246,157,175,247,205,96,197,29,163,166,213,165,41,110,191,38,72,152,204,21,49,49,21,71,209,237,235,153,154,154,
124,226,163,2,136,185,173,46,238,214,251,113,134,87,96,78,199,144,218,253,40,244,201,17,26,193,74,160,130,244,207,232,73,248,124,101,104,111,191,9,251,143,191,3,247,214,47,130,107,104,129,201,149,192,
195,64,178,108,172,31,181,45,141,168,171,214,240,131,61,83,144,203,86,65,43,20,144,206,229,224,247,249,32,43,50,18,137,56,6,7,7,177,98,249,10,220,123,207,103,231,3,9,178,203,133,108,38,195,250,211,51,
191,115,25,147,64,160,40,68,98,37,48,128,88,200,158,49,125,190,143,86,92,176,93,34,93,23,74,245,248,243,245,120,48,96,242,176,22,210,237,237,193,199,247,78,232,157,154,160,92,209,193,123,72,245,1,102,
183,9,163,227,142,246,214,80,111,164,127,22,4,145,255,180,186,252,200,180,190,103,215,64,38,52,45,200,23,221,235,98,3,236,87,131,166,141,238,36,39,116,85,183,52,135,14,30,63,217,119,45,0,58,30,211,131,
239,196,114,33,231,63,249,60,251,7,25,125,161,115,249,206,166,134,199,195,195,233,206,60,91,16,70,214,185,86,67,195,225,43,119,99,140,147,253,147,51,245,254,229,142,224,247,94,138,166,186,210,214,92,91,
204,87,84,44,113,227,20,228,240,212,85,48,80,144,253,179,164,113,3,170,137,254,83,201,73,76,44,187,17,124,101,35,76,157,216,103,240,67,167,82,93,125,61,98,147,49,88,167,222,131,231,150,123,32,52,109,160,
21,102,59,12,84,140,143,225,120,60,142,152,124,6,178,92,141,127,125,147,195,170,118,47,18,185,60,254,242,137,231,209,187,231,103,120,239,197,93,240,123,125,184,229,230,91,112,236,104,63,6,78,13,96,211,
166,219,206,51,18,14,29,58,136,98,177,24,165,71,45,64,126,218,32,224,8,188,3,134,89,240,160,4,12,205,180,231,18,93,164,207,44,65,116,88,202,158,1,219,249,186,12,124,156,61,87,247,31,63,211,252,248,83,
67,122,39,221,116,117,25,50,203,108,135,161,239,57,207,122,172,156,206,152,221,79,12,106,161,188,77,67,44,93,254,118,18,120,126,131,183,217,2,185,233,90,0,244,14,185,113,98,221,5,215,180,124,97,246,252,
201,59,87,182,189,124,46,215,153,99,64,22,46,20,193,60,60,146,219,57,223,181,189,113,197,47,206,229,187,82,220,226,125,246,136,238,171,79,36,174,244,40,88,230,86,81,229,45,199,198,155,183,128,27,31,0,
38,72,239,150,87,131,43,171,194,154,53,173,184,235,174,187,209,249,240,31,161,248,210,99,8,112,22,106,168,126,13,81,41,119,184,228,25,254,252,103,19,248,203,151,162,240,85,215,16,168,243,216,112,223,131,
200,209,36,111,254,253,7,241,237,31,61,131,213,219,182,99,108,106,26,141,205,171,28,32,49,244,157,119,109,119,111,191,7,30,143,103,207,133,253,202,144,32,177,120,193,49,211,57,138,14,80,216,255,11,134,
57,247,130,228,166,152,171,50,137,138,77,170,99,16,19,145,254,64,193,230,81,160,62,240,51,0,218,211,209,114,255,179,35,118,167,206,244,201,53,24,181,21,218,222,222,114,199,12,251,172,120,115,210,234,202,
112,236,217,210,172,89,228,90,10,84,55,111,243,11,174,51,35,38,108,223,214,222,218,118,45,0,154,164,177,176,169,141,249,38,42,115,128,58,50,146,233,24,166,5,231,140,207,60,203,50,21,72,117,61,114,9,45,
191,57,155,109,63,155,183,169,159,210,2,203,80,61,157,198,207,163,72,87,165,129,156,217,207,29,255,0,77,171,63,133,138,229,141,56,24,121,23,205,161,7,49,240,238,75,208,139,57,210,15,94,188,250,202,94,
172,108,108,68,58,149,66,226,220,57,164,30,253,26,234,111,189,23,197,124,26,99,225,103,102,27,123,227,36,135,182,54,15,146,154,142,245,247,126,1,25,18,227,50,79,70,90,128,1,233,142,142,7,103,25,169,140,
132,46,115,109,220,76,42,59,16,168,106,207,102,179,11,58,167,243,165,1,153,99,19,142,156,17,25,211,238,23,2,136,38,205,164,231,24,140,189,45,11,186,105,146,89,80,56,2,144,82,114,47,175,143,235,59,71,179,
230,162,3,193,48,150,215,180,158,84,58,29,173,240,251,187,100,73,244,47,206,68,86,39,27,183,180,102,117,236,79,48,87,58,55,188,28,245,143,238,235,153,152,78,117,79,77,79,119,54,213,45,237,150,132,121,
116,96,51,183,102,177,180,198,85,187,49,195,173,68,221,162,28,214,232,61,167,28,189,50,77,239,229,154,253,61,154,214,72,231,44,100,168,179,169,124,148,0,221,89,173,170,65,85,20,24,195,99,40,145,107,207,
146,214,157,95,198,210,249,4,1,190,179,218,239,241,123,69,62,122,53,0,106,43,119,243,56,250,202,147,88,70,174,201,67,97,121,243,186,155,113,176,151,68,180,64,157,227,138,208,3,65,44,171,92,130,173,91,
157,133,135,87,94,249,85,201,13,189,246,19,242,95,249,5,141,149,149,149,57,236,211,244,192,215,176,251,251,223,69,97,244,12,234,214,220,136,134,85,107,32,209,252,55,173,187,113,1,144,62,252,247,23,28,
32,177,145,37,141,148,184,104,174,216,42,34,54,177,24,128,104,240,9,15,4,16,146,201,132,32,125,158,172,177,105,80,179,132,28,6,24,6,28,131,129,199,48,28,83,5,14,188,79,114,34,186,175,190,49,29,140,23,
172,197,247,123,4,62,60,62,17,123,216,113,55,101,101,209,120,42,211,179,104,72,235,150,28,183,127,152,162,195,209,133,120,135,68,238,214,13,126,231,208,185,97,150,174,248,171,169,172,214,173,200,11,87,
244,18,151,128,127,255,253,166,71,126,121,36,142,49,98,199,162,170,162,72,238,69,227,132,240,72,166,24,76,197,98,29,150,161,251,221,52,46,162,199,27,113,185,221,59,251,142,157,28,210,92,222,176,197,137,
193,184,102,225,196,200,4,201,5,5,1,138,74,223,126,96,213,142,93,125,211,193,179,22,31,98,227,53,191,20,33,68,107,202,188,33,143,36,224,187,107,140,224,203,214,210,206,163,89,227,162,122,182,32,38,42,
61,106,187,79,161,128,131,179,195,87,3,160,208,134,27,188,120,112,163,7,255,227,7,223,194,218,111,254,3,36,98,28,229,134,117,88,33,185,48,124,244,215,48,51,147,24,137,197,240,212,79,159,132,50,143,50,
25,120,212,218,38,228,70,6,74,15,33,22,240,145,206,73,91,28,154,239,121,16,143,110,109,194,109,141,229,164,145,142,225,238,66,14,101,46,30,187,159,211,224,249,163,31,17,72,75,64,138,30,63,10,109,36,138,
129,129,1,36,147,201,157,23,173,58,114,85,69,98,28,211,44,129,134,49,11,59,154,206,113,14,8,236,60,175,21,97,204,128,166,168,207,28,139,20,89,17,37,115,182,7,191,30,53,67,199,198,166,47,57,16,244,110,
179,3,167,186,92,17,134,88,194,158,99,188,147,10,40,185,194,37,50,95,2,208,72,26,137,52,119,81,190,132,192,187,135,218,218,233,243,249,246,8,2,215,173,18,251,121,169,15,10,207,38,194,98,110,44,220,63,
161,133,15,22,103,116,89,154,69,186,57,152,106,25,42,141,28,106,202,24,224,74,160,179,236,124,200,176,197,78,38,222,117,219,240,147,75,239,86,168,31,109,193,250,18,67,51,86,29,53,194,111,11,254,208,98,
193,192,242,10,31,137,110,43,100,232,58,1,94,239,126,89,247,118,67,158,161,252,121,165,194,171,178,119,234,46,82,61,69,22,194,23,230,235,46,7,32,255,230,38,5,95,185,189,156,94,62,137,191,248,238,67,80,
63,251,167,112,53,223,140,20,47,195,211,114,51,180,84,28,133,242,122,164,134,251,240,165,59,111,37,150,41,159,101,33,38,160,103,27,42,47,71,129,34,175,53,95,233,194,145,215,126,129,255,125,87,25,190,241,
233,26,188,119,142,71,91,189,215,169,243,167,53,42,126,240,204,183,144,89,253,42,140,76,2,67,125,191,70,185,199,131,147,39,79,132,23,201,13,129,22,27,49,139,1,115,6,60,38,49,139,233,184,38,19,165,208,
123,6,64,84,39,149,201,130,13,128,78,198,254,111,205,0,204,150,72,68,211,249,243,39,147,126,118,111,185,42,161,109,69,197,69,3,241,214,241,216,44,128,142,15,158,238,187,82,254,108,170,104,68,138,153,2,
245,207,92,44,48,217,153,203,102,119,110,90,234,139,22,56,97,143,206,99,207,190,72,255,236,251,53,52,248,195,134,226,9,45,184,203,52,156,68,173,33,46,100,44,222,212,253,156,161,239,212,68,169,59,111,233,
206,98,154,101,60,81,32,96,100,144,23,60,151,117,127,50,129,247,93,18,225,105,201,127,217,122,46,98,208,37,50,119,85,26,40,36,201,34,152,32,255,50,129,104,75,179,138,255,242,227,239,224,131,253,55,194,
123,247,215,32,249,171,137,146,73,87,16,131,196,251,170,240,252,91,111,162,202,74,194,71,174,138,52,11,38,41,50,115,30,32,137,14,3,77,78,77,226,224,207,127,138,209,200,123,168,219,86,227,252,150,204,155,
11,230,98,181,39,129,247,15,245,225,212,59,175,195,75,225,59,129,7,185,92,174,123,177,206,21,200,37,229,52,195,1,141,65,231,231,93,19,3,136,56,79,91,232,196,52,73,210,103,231,65,179,174,174,28,203,85,
25,238,130,137,58,210,5,195,150,137,207,183,46,195,95,223,223,70,243,68,64,164,73,55,200,7,150,204,116,174,117,156,74,16,99,125,244,68,184,165,200,225,198,37,124,226,192,88,194,153,149,117,245,229,216,
176,172,12,45,94,23,170,137,133,93,132,254,76,186,24,212,140,76,215,176,206,119,213,172,107,140,100,4,190,243,23,125,39,251,82,164,193,239,10,168,161,97,26,183,131,177,185,103,22,169,95,34,233,62,158,
191,128,217,108,59,148,42,24,72,231,181,5,228,81,70,245,226,116,79,142,129,239,50,197,38,26,77,17,56,178,243,162,182,69,221,179,44,64,112,243,87,153,72,164,121,224,68,219,17,168,193,26,25,191,252,179,
229,248,235,167,98,248,225,19,223,66,118,243,87,32,174,191,131,166,159,131,123,221,173,200,186,84,148,157,216,139,123,183,110,69,44,54,129,231,119,61,231,12,122,133,191,2,94,213,227,116,112,252,224,7,
52,0,60,126,24,158,64,190,200,225,207,239,173,91,192,153,83,9,210,85,68,215,7,95,222,5,158,38,246,116,244,244,162,236,227,0,168,168,83,155,26,1,134,1,136,220,146,206,192,84,98,23,213,61,39,32,87,84,170,
248,195,155,91,177,182,66,197,13,228,246,206,12,76,97,106,170,128,28,111,34,70,117,57,122,206,201,129,105,228,207,229,102,211,15,246,188,94,89,116,193,229,82,174,10,64,63,27,156,76,222,119,99,99,168,58,
80,22,238,92,233,247,75,89,29,137,169,60,242,99,5,36,169,189,9,230,86,137,37,153,21,200,124,86,190,221,148,165,112,199,186,96,200,44,230,19,158,68,30,27,73,115,220,25,244,34,98,112,120,129,250,55,48,54,
213,85,85,177,36,88,174,186,187,22,184,87,106,207,46,230,66,73,114,119,194,188,133,227,33,96,40,46,41,82,195,241,72,232,118,251,116,58,187,128,98,26,42,253,81,193,54,163,44,144,48,108,87,180,94,18,195,
20,22,5,135,198,167,130,243,235,213,85,150,39,60,18,31,113,49,151,109,91,137,171,1,80,130,103,249,57,145,225,135,58,201,2,29,90,61,15,108,242,98,203,106,15,254,236,185,167,113,98,244,20,132,173,95,114,
88,132,175,111,194,48,69,102,47,191,244,130,115,115,85,85,149,147,32,12,84,86,98,73,176,25,227,135,15,1,177,209,217,136,228,249,131,25,172,9,76,98,199,38,31,56,90,45,167,71,115,120,58,240,121,156,122,
246,89,140,158,30,40,1,195,48,186,47,213,185,28,69,115,25,138,204,88,61,139,64,96,207,203,231,212,148,171,24,204,149,68,252,150,128,23,173,4,152,248,64,18,111,17,83,101,201,50,52,105,206,209,38,221,65,
10,62,193,132,56,1,242,60,120,172,25,0,177,54,217,81,145,149,171,14,173,95,60,120,170,111,107,91,107,232,251,253,169,238,234,76,178,227,6,85,161,96,129,35,208,88,14,104,10,116,212,28,35,177,204,4,126,
182,224,87,252,106,55,167,40,145,40,45,140,50,122,175,248,105,29,117,4,138,165,70,33,114,184,80,120,236,236,200,40,132,134,250,174,220,60,182,96,226,86,178,12,90,76,121,103,92,231,4,185,136,23,7,71,157,
173,159,165,213,213,251,50,185,252,2,183,232,173,13,244,188,30,57,57,255,107,136,39,150,213,212,60,146,206,100,22,140,185,80,225,139,236,63,113,102,219,181,108,101,68,78,199,180,14,94,162,65,100,64,34,
112,219,4,160,214,86,3,251,159,246,225,203,173,107,240,220,201,3,136,188,78,114,109,243,3,14,75,161,174,25,131,3,85,216,177,185,21,195,20,210,15,159,59,195,114,56,88,249,208,55,80,241,85,31,94,249,230,
253,48,243,89,39,3,189,118,237,90,12,174,255,28,238,123,238,39,176,137,229,244,63,124,148,244,213,122,140,127,113,131,51,145,164,87,46,201,62,37,6,42,178,111,139,230,146,113,164,95,190,180,41,136,135,
214,213,162,255,108,22,255,117,119,41,103,58,22,203,130,215,242,4,154,18,120,152,229,153,49,125,194,49,170,23,145,80,148,68,58,163,205,128,6,37,38,154,57,119,132,179,226,14,158,239,203,167,219,91,87,8,
169,100,39,111,179,164,193,76,30,137,0,204,180,148,204,217,137,221,103,227,143,205,166,46,250,250,153,94,250,156,219,237,46,119,23,185,142,122,175,59,84,45,137,29,50,137,94,139,52,33,115,189,230,60,224,
139,133,98,7,40,186,58,87,72,19,131,240,240,145,142,241,9,38,18,69,123,118,229,215,210,140,157,46,100,48,158,43,109,175,216,30,23,150,250,125,179,250,110,86,251,149,205,37,253,98,83,83,179,46,124,110,
226,47,142,56,39,38,39,157,197,56,203,222,1,15,106,124,50,14,93,227,94,88,244,208,25,138,115,164,153,116,35,129,135,61,243,39,255,226,67,129,148,31,211,83,95,104,89,133,68,223,135,136,86,209,248,46,95,
231,48,145,184,249,243,120,255,253,199,105,114,11,168,173,89,234,0,75,74,140,99,217,166,13,104,125,232,143,113,168,231,81,71,39,53,175,93,143,233,234,91,209,182,113,16,79,62,253,52,182,110,252,29,12,252,
232,111,225,34,49,152,38,61,67,179,215,125,89,157,49,51,238,181,229,46,124,125,99,3,218,124,42,210,99,25,28,122,101,16,7,33,206,174,182,4,13,106,190,80,116,0,83,112,66,249,185,73,147,136,230,57,122,86,
81,114,133,115,122,14,214,60,209,107,159,119,98,116,168,82,149,246,161,153,76,56,87,212,66,70,60,177,104,223,76,145,99,160,127,236,179,235,155,246,9,154,30,138,211,115,167,146,41,98,88,1,121,114,81,7,
207,12,179,149,252,48,69,78,109,130,40,246,216,83,211,11,246,247,24,8,117,90,169,25,186,47,67,172,156,36,0,121,68,150,196,155,155,166,181,196,100,59,86,214,160,159,88,234,72,65,35,55,92,138,114,5,114,
23,243,213,14,63,15,152,2,221,111,154,11,93,176,104,95,12,32,2,58,106,105,252,153,86,219,18,172,64,115,165,7,123,99,69,188,122,141,0,10,31,56,149,199,217,184,133,250,74,193,1,79,114,92,0,175,147,111,157,
137,75,25,233,60,180,122,21,254,246,55,191,4,170,87,58,23,120,111,57,78,78,231,113,203,202,165,228,131,21,172,91,191,30,163,111,191,140,108,239,139,80,139,6,100,89,118,244,196,17,173,6,29,43,42,17,126,
225,36,172,92,26,98,46,133,236,43,79,209,96,243,87,100,31,39,188,116,187,18,183,147,168,251,92,67,5,146,163,5,28,39,13,195,88,38,195,0,98,105,184,179,181,233,17,200,98,36,53,57,213,101,229,52,71,203,92,
36,12,233,218,211,103,147,189,56,155,196,157,117,129,132,165,27,139,134,34,110,163,216,121,123,91,107,143,155,179,163,86,60,222,117,105,201,88,98,10,157,196,55,159,74,66,165,190,200,34,231,172,234,132,
128,89,109,209,27,233,239,187,123,93,99,152,56,100,81,0,149,210,15,54,50,52,94,57,106,203,82,230,166,41,69,109,158,209,138,88,70,174,173,217,231,133,238,145,240,108,188,136,251,86,4,16,75,101,48,77,139,
224,208,84,150,222,109,14,32,23,106,56,198,214,194,34,0,242,170,42,30,44,35,192,210,184,140,79,230,113,46,150,195,180,203,117,205,187,241,67,20,225,68,95,124,179,16,252,227,237,94,82,225,60,202,107,109,
120,21,122,112,145,119,98,39,70,76,53,20,201,212,26,41,140,176,176,93,245,211,224,21,177,180,174,1,75,150,148,17,179,155,136,70,135,208,220,220,228,132,241,19,227,19,112,221,114,11,157,107,88,214,30,194,
238,158,191,199,224,111,126,3,91,113,163,216,187,7,46,219,164,200,44,207,180,71,247,149,52,70,37,135,240,173,62,79,199,137,120,222,1,13,115,77,57,199,76,104,140,193,210,185,110,139,101,171,77,243,146,
109,80,255,103,51,171,134,183,108,39,159,203,47,250,92,177,168,249,237,209,145,136,201,68,161,118,105,49,173,114,112,182,92,226,6,34,30,98,160,5,147,83,200,7,55,175,110,220,77,139,170,71,130,25,52,98,
83,157,139,0,58,81,32,166,151,23,48,173,13,65,20,231,237,1,26,136,18,67,249,8,64,62,145,230,129,0,35,179,109,6,138,22,27,189,42,84,234,227,87,171,151,96,156,162,166,227,19,37,207,183,189,185,6,15,175,
108,70,54,75,204,150,33,163,227,171,154,185,8,128,92,232,75,39,157,52,8,123,46,99,121,153,231,127,171,207,57,118,254,159,93,19,59,119,189,59,93,66,11,161,102,37,239,199,106,57,128,90,159,7,180,184,156,
132,85,123,160,18,35,228,166,32,200,206,214,66,158,4,172,36,85,178,143,113,28,31,208,219,219,139,96,48,72,250,169,213,73,56,70,14,68,144,217,251,119,176,147,73,100,114,57,52,108,190,11,153,222,221,206,
150,68,46,159,191,34,251,176,178,63,22,239,169,129,183,219,197,115,126,166,105,216,254,23,203,7,89,243,137,230,50,224,153,153,240,217,140,114,86,148,119,66,144,58,125,166,30,92,52,140,213,245,203,183,
197,35,250,236,68,214,113,115,227,5,125,79,192,178,187,202,230,133,220,2,177,144,48,62,214,65,254,164,67,191,68,191,20,216,123,226,185,2,106,112,97,22,123,46,186,210,8,64,19,228,190,82,228,218,84,50,69,
18,32,187,36,156,33,150,29,37,150,97,161,190,68,207,149,220,115,48,28,75,22,241,228,59,81,232,140,225,108,219,1,136,88,233,189,120,179,148,152,42,147,92,24,246,43,87,72,3,92,9,64,61,83,233,98,247,216,
167,191,227,31,47,175,69,234,240,219,120,243,236,113,160,255,109,4,236,34,54,86,85,192,69,171,227,20,251,166,170,60,91,250,128,140,73,165,234,85,24,28,60,136,85,171,86,59,141,180,174,93,135,190,200,1,
150,81,70,77,117,53,214,144,0,79,165,82,14,35,121,165,91,96,34,143,100,124,10,217,92,246,35,177,79,41,65,168,39,223,205,153,93,141,130,221,163,216,214,85,71,73,62,1,145,93,147,217,217,40,228,215,71,142,
37,41,12,238,104,85,149,112,64,228,252,87,211,150,75,64,130,152,191,99,150,186,207,157,235,205,185,93,225,53,46,33,116,81,18,248,18,224,41,35,45,79,241,74,247,68,60,217,121,49,128,248,133,25,237,121,238,
205,109,10,144,92,196,224,108,3,218,217,198,97,163,9,103,155,102,246,145,180,168,70,114,11,243,60,1,227,98,96,120,228,139,55,75,57,235,242,139,240,74,187,241,236,251,150,238,83,61,255,11,233,125,187,9,
32,212,216,178,53,192,166,47,98,114,229,22,252,10,85,248,185,230,197,113,67,46,197,251,236,35,179,66,30,185,76,134,232,50,135,99,199,143,98,100,100,24,217,76,22,155,110,219,140,209,177,49,28,233,239,199,
59,239,188,131,126,58,214,214,213,98,195,134,155,80,36,218,228,105,53,37,18,137,143,196,62,231,203,120,50,245,196,132,162,118,22,100,87,226,163,222,195,72,161,92,230,246,120,5,46,116,241,220,154,125,195,
146,43,148,20,229,176,197,93,249,99,77,86,197,37,113,17,191,194,135,158,26,207,46,216,4,213,21,87,199,168,226,9,23,37,249,242,253,161,14,149,43,124,84,21,185,208,83,177,236,208,244,197,219,126,196,42,
220,60,0,217,11,220,27,75,48,242,44,159,117,153,137,151,22,153,101,126,17,70,149,23,171,103,152,191,21,3,177,242,152,149,79,119,240,67,31,134,236,149,155,230,125,196,66,235,196,23,152,137,255,78,17,215,
121,102,66,35,27,198,244,176,3,77,198,64,195,195,231,240,214,91,111,96,229,202,70,180,183,221,196,54,38,113,102,104,8,201,84,28,31,124,176,159,4,158,139,220,154,140,177,241,49,182,63,213,125,181,76,50,
48,58,254,132,111,77,203,30,206,208,58,101,138,144,120,203,12,129,196,240,204,30,189,19,139,179,241,39,139,40,20,37,185,120,244,60,51,20,191,228,142,247,244,244,116,31,57,236,109,109,171,155,239,47,207,
103,59,4,211,8,113,134,17,60,63,113,76,120,11,60,23,229,57,59,226,22,249,158,231,207,76,255,124,209,36,90,34,145,36,40,108,243,172,89,181,131,55,245,14,73,47,134,40,244,247,179,15,216,88,132,196,38,152,
38,54,76,236,213,67,147,180,231,169,51,165,79,99,69,81,136,70,33,134,85,114,245,46,26,23,55,185,40,81,145,35,179,96,112,43,97,54,209,44,209,202,147,123,17,216,118,142,105,69,37,191,47,44,24,46,80,127,
105,210,41,88,49,230,0,194,211,253,98,160,130,34,47,118,143,14,129,192,67,0,138,94,28,4,88,81,62,80,17,102,245,4,167,109,157,218,49,34,151,93,68,31,113,158,216,71,237,97,126,89,75,187,213,64,129,131,56,
147,88,163,144,143,229,64,192,20,190,187,172,180,36,89,57,182,15,91,214,174,44,249,108,250,237,40,177,77,166,172,150,132,181,31,45,62,30,181,181,117,179,251,84,166,179,193,89,36,144,189,25,206,229,114,
219,112,189,124,172,202,71,253,171,12,22,122,188,103,103,166,182,75,254,10,127,227,93,15,97,205,234,245,152,172,172,135,65,34,216,249,150,226,60,120,178,211,240,77,157,192,246,223,187,23,75,151,45,67,
125,125,3,222,127,255,61,136,161,47,3,183,125,1,19,130,31,35,199,250,32,21,82,240,250,124,78,182,247,196,137,227,152,152,152,96,26,98,252,250,148,124,188,202,213,254,97,161,195,68,238,170,186,246,218,
77,247,226,92,154,66,230,226,60,95,106,20,129,195,191,66,64,45,121,70,246,161,217,100,44,134,211,195,163,40,255,250,247,160,84,214,162,86,149,145,239,219,135,19,207,255,3,172,66,150,125,166,193,156,126,
15,174,255,165,234,39,31,64,20,98,115,133,66,65,90,221,210,242,55,20,69,125,189,237,214,173,190,234,117,183,33,207,187,48,50,49,137,220,225,125,72,141,13,161,169,169,121,230,51,14,191,195,46,21,129,42,
180,254,193,127,71,142,126,219,191,119,23,198,7,15,143,182,180,172,254,6,133,247,191,116,185,20,93,85,85,251,250,84,124,130,1,196,18,127,4,28,53,151,207,85,231,114,249,42,210,42,129,215,94,123,109,205,
191,253,248,199,127,112,248,240,161,141,53,53,75,93,183,109,222,140,138,170,165,168,170,191,129,34,19,193,217,228,76,21,116,156,59,217,143,51,231,134,113,232,192,7,44,103,17,219,186,229,142,125,255,243,
219,223,126,137,64,51,165,170,238,73,58,78,168,110,247,132,219,237,202,185,174,144,245,188,94,62,166,0,34,192,8,89,2,78,38,147,89,70,225,121,85,54,155,101,22,160,243,64,150,192,212,219,27,94,221,127,228,
112,195,224,224,169,64,41,158,183,207,183,107,251,253,75,114,117,117,245,211,155,111,191,253,84,115,243,170,179,30,143,103,210,227,81,39,61,42,153,199,19,243,122,61,49,58,142,208,181,9,186,102,93,159,
146,79,32,128,138,164,115,242,249,188,72,230,39,55,22,200,231,11,213,116,94,73,86,145,47,20,252,196,78,229,154,86,244,81,24,238,161,200,202,101,154,166,35,130,4,65,48,68,81,212,36,73,202,186,20,37,77,
12,147,36,139,19,219,196,25,251,184,221,238,152,155,177,15,93,119,171,110,67,145,229,235,51,242,9,23,209,72,167,211,92,65,43,138,154,166,201,204,8,52,46,93,55,92,4,28,153,76,180,76,75,176,216,119,31,78,
210,142,183,121,129,55,25,144,36,81,44,74,178,148,151,101,89,35,43,18,160,52,69,81,140,178,50,223,245,89,248,24,151,255,39,192,0,176,95,186,79,233,202,55,99,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* MainPage::ucsniffdarkred_png = (const char*) resource_MainComponent_ucsniffdarkred_png;
const int MainPage::ucsniffdarkred_pngSize = 6807;
