/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2008 VIPER Lab
    Jason Ostrom <jpo@pobox.com>
    Arjun Sambamoorthy <arjun@sipera.com>

    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TARGET_H
#define TARGET_H

#include <ec.h>
#include <ec_decode.h>
#include <ec_dissect.h>
#include <ec_proto.h>
#include <g722.h>
#include <isac.h>
#include <errno.h>
#define MAX_CALLS 1024

/* Audio Codec Types */
#define COG711MULAW	0x00
#define COG711ALAW 	0x08
#define COG722		0x09	
#define COG729		0x12
#define COG726		0x02
#define COG723		0x04
#define COISAC    0x7c

/* Video Codec Types */
#define COH264		0x60			/* Dynamic payload type, 96 to 127 */ 


#define phtonl(p, v) \
        {                               \
        (p)[0] = (u_int8_t)((v) >> 24);   \
        (p)[1] = (u_int8_t)((v) >> 16);   \
        (p)[2] = (u_int8_t)((v) >> 8);    \
        (p)[3] = (u_int8_t)((v) >> 0);    \
        }

#define phtons(p, v) \
        {                               \
        ((u_int8_t*)(p))[0] = (u_int8_t)((v) >> 8); \
        ((u_int8_t*)(p))[1] = (u_int8_t)((v) >> 0); \
        }

char myIP[MAX_ASCII_ADDR_LEN];

/* RTP Header structure, Payload type is important here, based on the payload the codec is decoded */
struct sniff_rtp

{
        u_char version;               /* Protocol Version */
        u_char payloadType;           /* Payload Type */
        u_short sequence_no;          /* Sequence Number */
        u_int timestamp;             /* TimeStamp */
        u_int ssrc;                  /* Symchronization Source */
};

struct idrPicture{

	u_char idrPictureData[1400];
	int idrPictureLength;
	struct idrPicture *next;

};

struct rtp_data{

/* Old rtp data structure, still used by the skinny code. */

/* Video members */
	int vport;
	FILE *vfwd_fp;
	FILE *vrev_fp;
	int vfwd_count;
	int vrev_count;
	int vrtp_count;
	int vfwd_seqcount;
	int vrev_seqcount;
	char vfwd_file[170];
	char vrev_file[170];
#ifdef HAVE_VIDEO_DECODER	
	char vfwd_mp4[170];
	char vrev_mp4[170];
	char vfwd_avi[170];
	char vrev_avi[170];
#endif

/* Audio members */
    int afwd_data_len;
    int arev_data_len;
	/* RTP members */
	char ip_addr[MAX_ASCII_ADDR_LEN];
	int port;
	char pcap_filename[164];
	char fwd_au[164];
	char rev_au[164];
	char both_au[164];
	char fwd_mp3[164];
	char rev_mp3[164];
	char both_mp3[164];
	int session_active;
	pcap_t *dp;
	pcap_dumper_t *dumper;
	FILE *fwd_fp;
	FILE *rev_fp;
	int rtp_count;
	int fwd_count;
	int rev_count;
	int total_bytes;
	int fwd_seqCount;
	int rev_seqCount;
	unsigned long hDecoderFwd;
	unsigned long hDecoderRev;
	g722_decode_state_t dec_state;
	g722_decode_state_t dec_state1;

	/* Variable for h264 */
	int fwd_fuastart;
	int rev_fuastart;

	/* live monitor variables */
	int liveMonitor;				/* Set to to MONITOR_START if the user wishes to monitor the call */
	int liveMonitorFwdAudioCtr;		/* Live monitor packet counter for fwd audio stream */
	int liveMonitorRevAudioCtr;		/* Live monitor packet counter for rev audio stream */
	int liveMonitorFwdVideoCtr;		/* Live monitor packet counter for fwd video stream */
	int liveMonitorRevVideoCtr;		/* Live monitor packet counter for rev video stream */

#ifdef HAVE_LIB_VLC

	int liveMonitorFwdVlcStarted;   /* Live Monitor fwd direction vlc monitoring started */
	int liveMonitorRevVlcStarted;   /* Live Monitor rev direction vlc monitoring started */

	u_char fwdSPS[64];   /* Fwd h264 Sequence Parameter Set(SPS), remains unchanged for the entire video session */
	u_char fwdPPS[64];  /* Fwd h264 Picture Parameter Set(PPS), remains unchanged for the entire coded picture */

	u_char revSPS[64];
	u_char revPPS[64];

	int savedFwdSPS;	/* Set to 1 after saving the Fwd SPS */
	int savedRevSPS;	/* Set to 1 after saving the Rev SPS */

	/* After decoding IDR(instantaneous decoding refresh) all the following coded pictures in the decoding order can be decoding without any 
	inter prediction from any picture decoded prior to the IDR picture. IDR picture is the first picture of each coded video sequence */

	struct idrPicture *fwdIDRPtr;	/* IDR picture is needed for decoding the subsequent picture frames in the coded video sequence */ 
	struct idrPicture *revIDRPtr;
	struct idrPicture *fwdCurrentIDRPtr;
	struct idrPicture *revCurrentIDRPtr;	
	int fwdIDRCtr;						/* Number of Fwd IDR Packets received */
	int revIDRCtr;						/* Number of Rev IDR Packets received */
	int recvFwdAllIDRPic;				/* Set to 1 if all IDR pictures are received */
	int recvRevAllIDRPic;	
	int currentFwdPPSCtr;				/* This counter is incr after receiving a new PPS */	
	int previousFwdPPSCtr;				/* Incremented after receiving all the IDR coded pictures */ 
	int currentRevPPSCtr;
	int previousRevPPSCtr;
	int liveMonitorSocket;              /* Socket file descriptor for sending RTP to libvlc */

#endif	

/* Set to 1 after the mux thread has copied the media file names for muxing and adding container */
	int copiedFiles;

	/*Codec type*/
	char audioCodecType;
	char videoCodecType;
	
	/* Media payload type */
	char audioPayloadType;
	char videoPayloadType;

	/* RTPMAP/FMTP strings for live monitoring */
	char *audioRTPMAP;
	char *videoRTPMAP;
	char *videoFMTP;
	
	int audioDSPSet;					/* Set to 1 if the audio /dev/dsp device is opened */
	struct rtp_data *next;			
	struct rtp_data *previous;

};

typedef struct rtp_data rtpSKINNY;

/* max number of samples per frame (= 60 ms frame) */
#define MAX_FRAMESAMPLES                                960
#define MIN_FRAMESAMPLES                                480
/* number of samples per 10ms frame */
#define FRAMESAMPLES_10ms                               160
/* number of samples per 30ms frame */
#define FRAMESAMPLES_30ms                               480
/* sampling frequency (Hz) */
#define FS                                                              16000
#define AV_BUF_INC                                      2000000
#define AV_BUF_THRESHOLD                                3000

typedef struct {
        WebRtc_UWord32 send_time;            /* samples */
        WebRtc_UWord32 arrival_time;            /* samples */
        WebRtc_UWord32 sample_count;            /* samples */
        WebRtc_UWord16 rtp_number;
} BottleNeckModel;

struct rtpCalls{

    /* Video members */
    int vport;
    FILE *vfwd_fp;
    FILE *vrev_fp;
    int vfwd_count;
    int vrev_count;
    int vrtp_count;
    int vfwd_seqcount;
    int vrev_seqcount;
	int vfwd_cur_frame_samples;
	int vrev_cur_frame_samples;
	BottleNeckModel vfwd_bottleneck;
	BottleNeckModel vrev_bottleneck;
    char vfwd_file[174];			/* The file format is raw h264 for fwd video stream */
    char vrev_file[174];			/* The file format is raw h264 for rev video stream */
    char vfwd_mp4[174];				/* This file has the avi container for the h264 stream. Video only file with an avi container */
    char vrev_mp4[174];				
    char vfwd_avi[174];				/* This file muxes both the fwd direction audio and video, avi container format */
    char vrev_avi[174];
	/* Variable for h264 */
	int fwd_fuastart;
	int rev_fuastart;

	/* RTP IP and Port number */
	char ip_addr[MAX_ASCII_ADDR_LEN];
    int port;
        
    int session_active;

	/* UCSniff dumps all the RTP packets to a pcap file. The pcap file can be converted to an media file using tools like videosnarf */

	/* RTP packet dump variables */
    pcap_t *dp;
    pcap_dumper_t *dumper;
	char pcap_filename[164];

	/* RTP Media File attributes */
	char fwd_wav[170];					/* Fwd direction audio file, wav format */
    char rev_wav[170];					/* Rev direction audio file, wav format */
    char both_wav[170];					/* Bi-directional audio file, created using sox libraries */
    FILE *fwd_fp;
    FILE *rev_fp;

	/* RTP packet counters */
    int rtp_count;
    int fwd_count;
    int rev_count;
    int total_bytes;
    int fwd_seqCount;
    int rev_seqCount;

	/* Set to 1 after the mux thread has copied the media file names for muxing and adding container */
	int copiedFiles;

	/*Codec type*/
	char audioCodecType;
	char videoCodecType;
	
	/* Media payload type */
	char audioPayloadType;
	char videoPayloadType;

	/* RTPMAP/FMTP strings for live monitoring */
	char *audioRTPMAP;
	char *videoRTPMAP;
	char *videoFMTP;

	/* G722 variables */
    g722_decode_state_t dec_state;
    g722_decode_state_t dec_state1;

	/* Codec handler for G729/G726/G723 */
	unsigned long hDecoderFwd;
	unsigned long hDecoderRev;

	/* G723 Frame Type */
	int fwd723FrameType;		/* 1 if 6.3 kbps or 0 if 5.3 kbps */
	int rev723FrameType;

	/* G726 Sample size */
	int g726SampleSize;			/* 5(bit) for 40kbps, 4(bit) for 32 kbps, 3(bit) for 24 kbps and 2(bit) for 16 kbps */

	/* ISAC variables */
	ISACStruct* fwd_isac_inst;
	ISACStruct* rev_isac_inst;

	/* live monitor variables */
    int liveMonitor;				/* Set to to MONITOR_START if the user wishes to monitor the call */
    int liveMonitorFwdAudioCtr;		/* Live monitor packet counter for fwd audio stream */
    int liveMonitorRevAudioCtr;		/* Live monitor packet counter for rev audio stream */
    int liveMonitorFwdVideoCtr;		/* Live monitor packet counter for fwd video stream */
    int liveMonitorRevVideoCtr;		/* Live monitor packet counter for rev video stream */

	/* audio/video stream buffers */
	u_char* fwdAudioBuf;
	int fwdAudioIdx;
	int fwdAudioBufSize;
	u_char* fwdVideoBuf;
	int fwdVideoIdx;

	u_char* revAudioBuf;
	int revAudioIdx;
	int revAudioBufSize;
	u_char* revVideoBuf;
	int revVideoIdx;
#ifdef HAVE_LIB_VLC

	int liveMonitorFwdVlcStarted;   /* Live Monitor fwd direction vlc monitoring started */
    int liveMonitorRevVlcStarted;   /* Live Monitor rev direction vlc monitoring started */

    u_char fwdSPS[64];   /* Fwd h264 Sequence Parameter Set(SPS), remains unchanged for the entire video session */
    u_char fwdPPS[64];  /* Fwd h264 Picture Parameter Set(PPS), remains unchanged for the entire coded picture */
     
    u_char revSPS[64];
    u_char revPPS[64];

	int savedFwdSPS;	/* Set to 1 after saving the Fwd SPS */
	int savedRevSPS;	/* Set to 1 after saving the Rev SPS */

	/* After decoding IDR(instantaneous decoding refresh) all the following coded pictures in the decoding order can be decoding without any 
	   inter prediction from any picture decoded prior to the IDR picture. IDR picture is the first picture of each coded video sequence */

	struct idrPicture *fwdIDRPtr;	/* IDR picture is needed for decoding the subsequent picture frames in the coded video sequence */ 
	struct idrPicture *revIDRPtr;
	struct idrPicture *fwdCurrentIDRPtr;
	struct idrPicture *revCurrentIDRPtr;	
	int fwdIDRCtr;						/* Number of Fwd IDR Packets received */
	int revIDRCtr;						/* Number of Rev IDR Packets received */
	int recvFwdAllIDRPic;				/* Set to 1 if all IDR pictures are received */
	int recvRevAllIDRPic;	
	int currentFwdPPSCtr;				/* This counter is incr after receiving a new PPS */	
	int previousFwdPPSCtr;				/* Incremented after receiving all the IDR coded pictures */ 
	int currentRevPPSCtr;
	int previousRevPPSCtr;
	int liveMonitorSocket;              /* Socket file descriptor for sending RTP to libvlc */

#endif	
	int audioDSPSet;					/* Set to 1 if the audio /dev/dsp device is opened */

	struct rtpCalls *next;			
	struct rtpCalls *previous;
};

typedef struct rtpCalls rtpSIP;

struct targets{

	char ip[MAX_ASCII_ADDR_LEN];
	u_char mac[MAX_ASCII_ADDR_LEN];
	char extension[64];
	char dirname[64];
	char protocol[11];
	char ua[48];
	char misc[64];
};

struct singletarget{

	char ip[MAX_ASCII_ADDR_LEN];
	char extension[24];
	char dirname[64];
	char protocol[10];
};

struct singlecalltrack{

        /* Video parameters */
        int vport;
        int videoactive;

        int offhook;
        int active;
        char digits[24];
        int digitcount;
        int rtpport;
        char recvdigits[24];
        struct timeval start;
        struct timeval finish;
        char start_time_string[12];

};

struct calls {

	int CallIdentifier;
    int CCallIdentifier;
    char ip1[MAX_ASCII_ADDR_LEN];
    char dirname1[64];
    char dirname2[64];
    char ip2[MAX_ASCII_ADDR_LEN];
    char dialednumber[24];
    char dialingnumber[24];
    int rtpport;
	int call_active;
	int srbc;
	struct timeval start;
    struct timeval finish;
    char start_time_string[12];
};

struct skinnycall {

	/* Member variables for handling video(h.264) */
	int vpassthrupartyid1;
	int vpassthrupartyid2;
	int vrtpport1;
	int vrtpport2;
	int videoactive;

        int CallIdentifier;
        int CCallIdentifier;
        int passthrupartyid1;
        int passthrupartyid2;
        char ip1[MAX_ASCII_ADDR_LEN];
        char ip2[MAX_ASCII_ADDR_LEN];
        int ip1dialing;
        int ip1dialed;
        int ip2dialed;
        int garp1;
        int garp2;
        int offhook1;
        int offhook2;
        char dialingip[MAX_ASCII_ADDR_LEN];
        char dialedip[MAX_ASCII_ADDR_LEN];
        char rtpip[MAX_ASCII_ADDR_LEN];
        char dirname1[64];
        char dirname2[64];
        char dialingnumber[24];
        char dialednumber[24];
        int digitcount;
        int rtpport1;
        int rtpport2;
        int call_active;
        struct timeval start;
        struct timeval finish;
        char start_time_string[12];
        int callnum;
        int wav_set;
        int fwd_set;
        int rev_set;
	int flag;
        struct skinnycall *next_skinnycall;
	
	/* Pointer to RTP call structure */
	rtpSKINNY* myRTPCall;

	struct skinnyCall *next;		/* Pointer to the next Skinny call */
	struct skinnyCall *previous;		/* Pointer to the previous Skinny call */

};
typedef struct skinnycall skinnyDB;

struct sip_uri{

    /* This is a temporary structure, just for getting the values and will eventually assign it to the sip main structure */

    char from_uri[256];
    char to_uri[256];
	char from_tag[64];
	char to_tag[64];
    char src_extension[64];
    char dst_extension[64];
    unsigned char phone_type[64];
    int kpml_enabled;
    int media_src_port;
	int video_port;
    char ip_addr[MAX_ASCII_ADDR_LEN];
	char call_id[256];
	char cseq_method[16];
	int cseq;
	u_char *keypadDigit;
	u_char *ocsIM;
};

struct sip_uri uri_obj;

struct sipDatabase{

        /* IP and Transport information */
	char src_ip[MAX_ASCII_ADDR_LEN];	/* Source IP of the device sending SIP INVITE */
	char dst_ip[MAX_ASCII_ADDR_LEN];	/* Destination IP in the SIP INVITE */
	int src_port;				/* Source TCP/UDP port of the device */
	int dst_port;				/* Destination TCP/UDP port of the SIP INVITE packet */
	
	/* SIP header variables */
	char src_extension[64];			/* Calling party extension, parsed from the from URI of SIP INVITE */
	char dst_extension[64];			/* Called party extension, parsed from the to URI of SIP INVITE */
	char call_id[256];			/* Call ID of the sip call */
	int cseq;				/* Command Sequence of the sip invite request */
	char user_agent[64];			/* User Agent (Phone type) */
	char to_tag[64];			/* TO-Tag */
	char from_tag[64];			/* From-Tag */
	char contact[128];			/* Contact header */
	char remote_extension[64];		/* Remote extension has the actual destination extension, for cisco G series phones that has kpml enabled */

	/* RTP Audio */
	int media_src_port;			/* Media source port */
	int media_dst_port;			/* Media destination port */
	
	/* Media IP */
	char media_src_ip[MAX_ASCII_ADDR_LEN];	/* Source media(audio) IP address */
	char media_dst_ip[MAX_ASCII_ADDR_LEN];  /* Destination media(audio) IP address */
	
	/* Media(Video) port */
	int video_src_port;			/* Video source port */
	int video_dst_port;			/* Video destination port */

	/* Media Codec Type */
	char audioCodecType;
	char videoCodecType;

	/* Media Payload Type */
	char audioPayloadType;
	char videoPayloadType;

	/* G726 Sample size */
    int g726SampleSize;         /* 5(bit) for 40kbps, 4(bit) for 32 kbps, 3(bit) for 24 kbps and 2(bit) for 16 kbps */

	/* RTPMAP/FMTP String for live Monitoring */
	char *audioRTPMAP;
	char *videoRTPMAP;
	char *videoFMTP;

	/* Corporate Directory */
	char src_username[64];                  /* Username of the calling party, matched by extension number, if the corporate download is successful */
    char dst_username[64];                  /* Username of the called party, if the corporate download is successful */

	/* Call number */
	int call_number;			/* SIP call number */
	
	/* Cisco specific variables */
	int kpml_enabled;			/* 0 or 1, if kpml tags are used by cisco ip phones to transfer the dialed digits */
	
	/* Flags */
	int remoteParty;			/* If remote_extension holds a value */
	int invite_active;			/* 0 or 1, if INVITE is received */
	int okay_active;			/* 0 or 1, if 200 OK is received */
	int ack_active;				/* 0 or 1, if ACK is received */
	int session_active;			/* 0 or 1, if SIP session is established */
	int diff_vlan;				/* 0 or 1, 0 if call from same LAN or VLAN, 1 if call from different vlan/GARP diabled on calling party */
	int video_active;			/* Video session is active */
	int server_reinvite;			/* 0 or 1, if reinvite send by server */
	int server_reinviteNoSDP;		/* No SDP in re-Invite, will check ACK for SDP */
	int server_reinviteOkay;		/* Okay for reInvite */
	int server_reinviteAck;			/* Ack for reInvite */
	int server_reinviteCSeq;		/* CSeq number of reInvite */
	int client_reinvite;			/* reInvite send by client */
	int client_reinviteNoSDP;		/* No SDP in re-Invite, will check ACK for SDP */
	int client_reinviteOkay;		/* Okay for reInvite */
	int client_reinviteAck;			/* Ack for reInvite */
	int client_reinviteCSeq;		/* CSeq number of reInvite */

	/* Time structures */
	char start_time_string[12];             /* Time string in hh:mm:ss format */
        char end_time_string[12];               /* Time string in hh:mm:ss format */
	struct timeval start;                   /* Start time of when the call is made */
        struct timeval finish;                  /* End time of the call */
	
	/* Pointer to RTP call structure */
	rtpSIP* myRTPCall;

	/* Self-referential linked list structure */ 
	struct sipDatabase *next;		/* Pointer to the next SIP call */
	struct sipDatabase *previous;		/* Pointer to the previous SIP call */

};

typedef struct sipDatabase sipDB;	


struct digitbuffer{
        char string1[24];
        char string2[24];
};

struct rtp_data rtp[MAX_CALLS];
//int rtp_counter;

/* Checks if the given IP address is in local LAN */
int check_local_ip(char *);

int parse_targets();
void print_targets(void);
void print_targets_summary(void);
void print_targets_count(void);

int okay_targethandler(sipDB*,u_char *);
void updateRemoteExtension(sipDB*,char *);
/* Adds a new entry to the targets.txt */
void sip_targetlookup(sipDB*);
void get_time_string(int mode24,char *);     /* Returns the time as a string in 24 hour format */
int decode_remote_party(unsigned char *);
/* Sip call functions */
void initialize_uri_struct();
sipDB* createNewSipCall(char *, char *, struct packet_object *, u_char *);
void deleteSipCall(sipDB *);
void copyAudioCodecParams(sipDB*, char *, char, char, int);
void copyVideoCodecParams(sipDB*, char *, char *, char, char);
/* SIP header decoders */
int decode_sip_header(unsigned char *);
int decode_from_uri(unsigned char *);
int decode_to_uri(unsigned char *);
int decode_callid(unsigned char *);
int decode_ua(unsigned char *);
int decode_cseq(unsigned char *);
int decode_kpml(unsigned char *);
int decode_media_port(unsigned char *);
int decode_video_port(unsigned char *);
int decode_dialed_ip(unsigned char *);
int getNegotiatedAudioCodecTypeAndRTPMAP(unsigned char*, char **, char *, char *, int *);
int getNegotiatedVideoCodecTypeAndRTPMAP(unsigned char*, char **, char **, char *, char *);
int checkreInviteHold(u_char *);
sipDB* match_bye_request();
sipDB* match_ack_request();
sipDB* match_cancel_request();
sipDB* match_okay_status();
sipDB* matchsipDBInvite(char *,int);
sipDB* matchsipOkay(char *,int);
sipDB* matchsipAck(char *,int);
int check_reinvite_server(sipDB*,u_char *);
int check_serverReInviteOkay(sipDB*,u_char *);
int check_serverReInviteAck(sipDB*,u_char *);
int check_reinvite_client(sipDB*,u_char *);
int check_clientReInviteOkay(sipDB*,u_char *);
int check_clientReInviteAck(sipDB*,u_char *);
void clearServerReInvite(sipDB *);
void clearClientReInvite(sipDB *);
int match_corporate_directory(char *);
/* Add and delete rtp dissector */
void addRTPDissector(int,char *);
void deleteRTPDissector(int,char *);
/* add, initialize, copy, search and delete rtp calls */
void initializeRTPCall(rtpSIP*);
int populateRTPCall(rtpSIP*,sipDB*,int,char*);
void populatevideoRTPCall(rtpSIP*,int,char *,char *,char,char);
rtpSIP* createNewRTPCall(sipDB*,int,char*);
rtpSIP* addNewRTPCall();
rtpSIP* searchRTPCall(int,char *);
void closeSipRTPCall();
void closeRTPCall();
void deleteRTPCall(rtpSIP*);
rtpSIP* checkRTPCall(char *,int,char *,int,int *,int *);
void create_dissect_string(int,char *,char *);
void updateMediaInfoForExistingCall(const char oldIp[], int oldAudioPort, int oldVideoPort, sipDB* currentSipCall,
    const char newIp[], int newAudioPort, int newVideoPort);
/* OCS IM functions */
int check_OCSContentType(u_char *);
int parseOCSIMText(u_char *);
/* Release the audio codecs */
void releaseAudioDecoder(rtpSIP *currentRTPCall);
/* Compression function */
#ifdef COMPRESSION
int compressFileBZ2(char *);
#endif
void createForwardStreamVideoFiles(char *,char *,char *,char *);
void createReverseStreamVideoFiles(char *,char *,char *,char *);
int create_wav_header(FILE *fp, unsigned short, unsigned short, unsigned int, short, off_t);

#endif
