/*
  ==============================================================================

   Demonstration "Hello World" application in JUCE
   Copyright 2008 by Julian Storer.

  ==============================================================================
*/

#include "includes_juce.h"
#include "MainComponent.h"

extern int guiShutdown;

//==============================================================================
/**
    This is the top-level window that we'll pop up. Inside it, we'll create and
    show a component from the MainComponent.cpp file (you can open this file using
    the Jucer to edit it).
*/

extern "C" {
	void *startUCSniffJUCEGUI(void *);
};

extern "C" {
	int startJUCE();
}


class UCSniffWindow  : public DocumentWindow
{
public:
    //==============================================================================
    UCSniffWindow()
        : DocumentWindow (T("UCSniff GUI"),
                          Colours::lightgrey,
                          DocumentWindow::allButtons,
                          true)
    {
        // Create an instance of our main content component, and add it
        // to our window.

        MainComponent* const contentComponent = new MainComponent();

        setContentComponent (contentComponent, true, true);

        centreWithSize (getWidth(), getHeight());

        setVisible (true);
    }

    ~UCSniffWindow()
    {
        // (the content component will be deleted automatically, so no need to do it here)
    }

    //==============================================================================
    void closeButtonPressed()
    {
        // When the user presses the close button, we'll tell the app to quit. This
        // window will be deleted by our UCSniffApplication::shutdown() method
        //
        JUCEApplication::quit();
    }
};

//==============================================================================
/** This is the application object that is started up when Juce starts. It handles
    the initialisation and shutdown of the whole application.
*/
class JUCEUCSniffApplication : public JUCEApplication
{
    /* Important! NEVER embed objects directly inside your JUCEApplication class! Use
       ONLY pointers to objects, which you should create during the initialise() method
       (NOT in the constructor!) and delete in the shutdown() method (NOT in the
       destructor!)

       This is because the application object gets created before Juce has been properly
       initialised, so any embedded objects would also get constructed too soon.
   */
    UCSniffWindow* ucsniffWindow;

public:
    //==============================================================================
    JUCEUCSniffApplication()
        : ucsniffWindow (0)
    {
        // NEVER do anything in here that could involve any Juce function being called
        // - leave all your startup tasks until the initialise() method.
    }

    ~JUCEUCSniffApplication()
    {
        // Your shutdown() method should already have done all the things necessary to
        // clean up this app object, so you should never need to put anything in
        // the destructor.

        // Making any Juce calls in here could be very dangerous...
    }

    //==============================================================================
    void initialise (const String& commandLine)
    {
		
        // just create the main window...
        ucsniffWindow = new UCSniffWindow();

        /*  ..and now return, which will fall into to the main event
            dispatch loop, and this will run until something calls
            JUCEAppliction::quit().

            In this case, JUCEAppliction::quit() will be called by the
            hello world window being clicked.
        */
    }

    void shutdown()
    {
        // clear up..

		if (ucsniffWindow != 0){
            delete ucsniffWindow;
			guiShutdown = 1;
		}
    }

    //==============================================================================
    const String getApplicationName()
    {
        return T("VIPER Lab UCSniff");
    }

    const String getApplicationVersion()
    {
        return T("1.0");
    }

    bool moreThanOneInstanceAllowed()
    {
        return true;
    }

    void anotherInstanceStarted (const String& commandLine)
    {
    }
};


//==============================================================================
// This macro creates the application's main() function..
//START_JUCE_APPLICATION (JUCEUCSniffApplication)

void *startUCSniffJUCEGUI(void *){

	startJUCE();

	return NULL;
}

int startJUCE () 
 { 
	char *argv[] = {"ucsniff.exe"};
	return JUCE_NAMESPACE::JUCEApplication::main (1, argv, new JUCEUCSniffApplication());
 }
