#include <ec.h>
#include <targets.h>
#include <ec_liveMonitor.h>
#include <ec_audioDSPPlayer.h>
#include <ec_threads.h>
#include <errno.h>

int liveMonitorHandler(rtpSIP*, int, char,struct sniff_rtp *);
extern pthread_mutex_t player_mutex;
#define PLAYER_LOCK do{ pthread_mutex_lock(&player_mutex); } while(0)
#define PLAYER_UNLOCK do{ pthread_mutex_unlock(&player_mutex); } while(0)

extern pthread_mutex_t vlcstarted_mutex;
#define VLCSTARTED_LOCK do{ pthread_mutex_lock(&vlcstarted_mutex); } while(0)
#define VLCSTARTED_UNLOCK do{ pthread_mutex_unlock(&vlcstarted_mutex); } while(0)

#ifdef HAVE_LIB_VLC

static void calSeqNoAndTimestamp(struct sniff_rtp*, struct sniff_rtp*, int);
static int sendParameterSet(struct sniff_rtp*, rtpSIP*, int, int);
void sendRTP(u_char *, int, int, int);
static int sendIDRPictures(struct sniff_rtp *, struct idrPicture*, int, int); 

#endif

int liveMonitorHandler(rtpSIP* currentRTPCall, int fileType, char codecType, struct sniff_rtp *rtphdr){

	PLAYER_LOCK;

    if(currentRTPCall->liveMonitor == MONITOR_START){
        if(fileType == FWDAUDIOFT){
            if(currentRTPCall->liveMonitorFwdAudioCtr == 0){
				int samplingRate = 0;
				switch(codecType){
					case COG711MULAW:
					case COG711ALAW:
					case COG722:
					case COG723:
					case COG726:
					case COG729:	
						samplingRate = 8000;				
						break;
					case COISAC:
						samplingRate = 16000;
						break;
					default:
						INSTANT_USER_MSG("Error:  Unknown audio codec.  Aborting live monitor attempt.\n");
						return -1;
				}
				if(samplingRate) {
					if(currentRTPCall->audioDSPSet == 0){
						int retDSP = initDSP(16, 1, samplingRate);
						if(retDSP < 0){
							PLAYER_UNLOCK;
							return -1;
						}
						currentRTPCall->audioDSPSet = 1;
					}
				}
		        currentRTPCall->liveMonitorFwdAudioCtr += 1;
				PLAYER_UNLOCK;
		        return 1;
            }
			else if(currentRTPCall->liveMonitorFwdAudioCtr > 0){
               	currentRTPCall->liveMonitorFwdAudioCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
        }
		else if(fileType == REVAUDIOFT){
            if(currentRTPCall->liveMonitorRevAudioCtr == 0){
				int samplingRate = 0;
				switch(codecType){

					case COG711MULAW:
                    case COG711ALAW:
                    case COG722:
                    case COG723:
                    case COG726:
                    case COG729:
						samplingRate = 8000;
						break;
					case COISAC:
						samplingRate = 16000;
						break;
					default:
						INSTANT_USER_MSG("Error:  Unknown audio codec.  Aborting live monitor attempt.\n");
						return -1;
                }
				if(samplingRate) {
					if(currentRTPCall->audioDSPSet == 0){
						int retDSP = initDSP(16, 1, samplingRate);
						if(retDSP < 0){
							PLAYER_UNLOCK;
							return -1;
						}
						currentRTPCall->audioDSPSet = 1;
					}
				}
                currentRTPCall->liveMonitorRevAudioCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
            else if(currentRTPCall->liveMonitorRevAudioCtr > 0){
                currentRTPCall->liveMonitorRevAudioCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
        }
		else if(fileType == FWDVIDEOFT){
#ifdef HAVE_LIB_VLC
            if(currentRTPCall->liveMonitorFwdVideoCtr == 0 && currentRTPCall->recvFwdAllIDRPic){
				if(!currentRTPCall->liveMonitorSocket){
					currentRTPCall->liveMonitorSocket = socket(AF_INET, SOCK_DGRAM, 0);
					if(currentRTPCall->liveMonitorSocket < 0){
						printf("Error: creating live monitoring socket \n");
						PLAYER_UNLOCK;
						return -1;
					}
				}
				createSDPFile(fileType,currentRTPCall->videoRTPMAP,currentRTPCall->videoFMTP,&currentRTPCall->videoPayloadType);	
				startLiveMonitorPlayer(fileType, &(currentRTPCall->liveMonitorFwdVlcStarted));

				struct sniff_rtp newRTPHeader;

				calSeqNoAndTimestamp(&newRTPHeader, rtphdr, currentRTPCall->fwdIDRCtr);

				 while (1){

				   		VLCSTARTED_LOCK;

						if(currentRTPCall->liveMonitorFwdVlcStarted){
							VLCSTARTED_UNLOCK;
							break;
						}

						VLCSTARTED_UNLOCK;
				}

				int retParamSet = sendParameterSet(&newRTPHeader,currentRTPCall,FWDVIDEOPORT_INT, fileType);
				if(retParamSet < 0){
					/* Stop Currently Monitoring Stream */
					PLAYER_UNLOCK;
					return -1;
				}
				int retIDRPic = sendIDRPictures(&newRTPHeader,currentRTPCall->fwdIDRPtr,FWDVIDEOPORT_INT, currentRTPCall->liveMonitorSocket);
				if(retIDRPic < 0){
					/* Stop currently monitoring stream */
					PLAYER_UNLOCK;
					return -1;
				}
                currentRTPCall->liveMonitorFwdVideoCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
            else if(currentRTPCall->liveMonitorFwdVideoCtr > 0){
                currentRTPCall->liveMonitorFwdVideoCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
#endif			
        }
        else if(fileType == REVVIDEOFT){
#ifdef HAVE_LIB_VLC
			if(!currentRTPCall->liveMonitorSocket){
            	currentRTPCall->liveMonitorSocket = socket(AF_INET, SOCK_DGRAM, 0);
                if(currentRTPCall->liveMonitorSocket < 0){
                	printf("Error: creating live monitoring socket \n");
					PLAYER_UNLOCK;
                    return -1;
                }
            }
            if(currentRTPCall->liveMonitorRevVideoCtr == 0 && currentRTPCall->recvRevAllIDRPic){

				createSDPFile(fileType,currentRTPCall->videoRTPMAP,currentRTPCall->videoFMTP,&currentRTPCall->videoPayloadType);	
                startLiveMonitorPlayer(fileType,&(currentRTPCall->liveMonitorRevVlcStarted));
				struct sniff_rtp newRTPHeader;
				calSeqNoAndTimestamp(&newRTPHeader, rtphdr, currentRTPCall->revIDRCtr);

				while (1){

                        VLCSTARTED_LOCK;

                        if(currentRTPCall->liveMonitorRevVlcStarted){
                            VLCSTARTED_UNLOCK;
                            break;
                        }

                        VLCSTARTED_UNLOCK;
                }

				int retParamSet = sendParameterSet(&newRTPHeader,currentRTPCall,REVVIDEOPORT_INT, fileType);
				if(retParamSet < 0){
					PLAYER_UNLOCK;
					return -1;
				}
				int retIDRPic = sendIDRPictures(&newRTPHeader,currentRTPCall->revIDRPtr,REVVIDEOPORT_INT, currentRTPCall->liveMonitorSocket);
				if(retIDRPic < 0){
					PLAYER_UNLOCK;
					return -1;
				}

                currentRTPCall->liveMonitorRevVideoCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
            else if(currentRTPCall->liveMonitorRevVideoCtr > 0){
                currentRTPCall->liveMonitorRevVideoCtr += 1;
				PLAYER_UNLOCK;
                return 1;
            }
#endif			
        }
    }
	PLAYER_UNLOCK;
    return -1;
}

#ifdef HAVE_LIB_VLC

static int sendIDRPictures(struct sniff_rtp* newRTPHeader,struct idrPicture *idrPicturePtr,int port, int liveMonitorSocket){

	u_char *data = NULL;
	struct idrPicture *tempIDRPic = idrPicturePtr;
	unsigned short tempSequence_no;
	while(tempIDRPic != NULL){
		tempSequence_no = ntohs(newRTPHeader->sequence_no);
		tempSequence_no += 1;
	    newRTPHeader->sequence_no = htons(tempSequence_no);
		data = (u_char *) calloc(sizeof(struct sniff_rtp) + tempIDRPic->idrPictureLength + 1, sizeof(u_char));
		if(data == NULL){
			INSTANT_USER_MSG("Error: Allocating new memory for live monitoring\n");
			return -1;
		}

		/* set the Mark bit for the last IDR Picture */
		if(tempIDRPic->next == NULL)
			newRTPHeader->payloadType |= 0x80;

		memcpy(data,newRTPHeader,sizeof(struct sniff_rtp));
		memcpy(data+sizeof(struct sniff_rtp), tempIDRPic->idrPictureData, tempIDRPic->idrPictureLength);
		sendRTP(data,sizeof(struct sniff_rtp) + tempIDRPic->idrPictureLength, port, liveMonitorSocket);

	    tempIDRPic = tempIDRPic->next;
		free(data);
	}
	return 0;
}

static int sendParameterSet(struct sniff_rtp* newRTPHeader, rtpSIP* currentRTPCall, int port, int fileType){

	if(fileType == FWDVIDEOFT){
		int fwdSPSlen = strlen((const char*)currentRTPCall->fwdSPS);         		
		int fwdPPSlen = strlen((const char*)currentRTPCall->fwdPPS);         		

		u_char *data = (u_char *) calloc(sizeof(struct sniff_rtp) + fwdSPSlen + 1, sizeof(u_char));
		if(data == NULL){
			INSTANT_USER_MSG("Error: Allocating new memory for live monitoring\n");
			return -1;
		}
	 	memcpy(data,newRTPHeader,sizeof(struct sniff_rtp));
	 	memcpy(data+sizeof(struct sniff_rtp),currentRTPCall->fwdSPS,fwdSPSlen);
	 	sendRTP(data,sizeof(struct sniff_rtp) + fwdSPSlen, port, currentRTPCall->liveMonitorSocket);

		unsigned short tempSequence_no = ntohs(newRTPHeader->sequence_no);
		tempSequence_no += 1;
	    newRTPHeader->sequence_no = htons(tempSequence_no);
	 	free(data);
	 	data = (u_char *) calloc(sizeof(struct sniff_rtp) + fwdSPSlen + 1, sizeof(u_char));
	 	if(data == NULL){	
			INSTANT_USER_MSG("Error: Allocating new memory for live monitoring\n");
			return -1;
	 	}	
	 	memcpy(data,newRTPHeader,sizeof(struct sniff_rtp));
     	memcpy(data+sizeof(struct sniff_rtp),currentRTPCall->fwdPPS,fwdPPSlen);
     	sendRTP(data,sizeof(struct sniff_rtp)+fwdPPSlen,port, currentRTPCall->liveMonitorSocket);

	}
	else if(fileType == REVVIDEOFT){
		int revSPSlen = strlen((const char*)currentRTPCall->revSPS);
		int revPPSlen = strlen((const char*)currentRTPCall->revPPS);

		u_char *data = (u_char *) calloc(sizeof(struct sniff_rtp) + revSPSlen + 1, sizeof(u_char));
		if(data == NULL){
			INSTANT_USER_MSG("Error: Allocating new memory for live monitoring\n");
			return -1;
		}
		memcpy(data,newRTPHeader,sizeof(struct sniff_rtp));
		memcpy(data+sizeof(struct sniff_rtp),currentRTPCall->revSPS,revSPSlen);
		sendRTP(data,sizeof(struct sniff_rtp) + revSPSlen,port, currentRTPCall->liveMonitorSocket);
		
		unsigned short tempSequence_no = ntohs(newRTPHeader->sequence_no);
		tempSequence_no += 1;
		newRTPHeader->sequence_no = htons(tempSequence_no);
		free(data);
		data = (u_char *) calloc(sizeof(struct sniff_rtp) + revPPSlen + 1, sizeof(u_char));
		 if(data == NULL){
			INSTANT_USER_MSG("Error: Allocating new memory for live monitoring\n");
			return -1;
		}
		memcpy(data,newRTPHeader,sizeof(struct sniff_rtp));
		memcpy(data+sizeof(struct sniff_rtp),currentRTPCall->revPPS,revPPSlen);
		sendRTP(data,sizeof(struct sniff_rtp)+revPPSlen,port, currentRTPCall->liveMonitorSocket);
	}
	return 0;
}

static void calSeqNoAndTimestamp(struct sniff_rtp* newRTPHeader, struct sniff_rtp* rtphdr, int IDRCtr){

	unsigned short calsequence_no = ntohs(rtphdr->sequence_no) - (IDRCtr + 2); /* 2 - pps and sps */
	unsigned int caltimestamp = ntohl(rtphdr->timestamp) - 3000; /* 3600 for 25 fps */

    newRTPHeader->version = rtphdr->version;
	newRTPHeader->payloadType = (rtphdr->payloadType & 0x7F);
    newRTPHeader->sequence_no = htons(calsequence_no);
	newRTPHeader->timestamp = htonl(caltimestamp);
    newRTPHeader->ssrc = rtphdr->ssrc;

}

void sendRTP(u_char *data, int dataLength, int dstPort, int liveMonitorSocket){

	struct sockaddr_in dstaddr;
	dstaddr.sin_family = AF_INET;
	dstaddr.sin_addr.s_addr = inet_addr(myIP);
	dstaddr.sin_port = htons(dstPort);

	int length = sendto(liveMonitorSocket, (u_char *)data, dataLength, 0, (struct sockaddr*)&dstaddr, sizeof(dstaddr));
	if(length < 0){
		printf("Error: Sending RTP data to live monitor stream : %s\n",strerror(errno));
		return;
	}
}

void deleteIDRPicture(struct idrPicture *idrPicturePtr){

	if(idrPicturePtr == NULL)
		return;

	struct idrPicture *tempIDRPic = NULL;
	tempIDRPic = idrPicturePtr;
	
	while(tempIDRPic != NULL){

		struct idrPicture *deleteIDRPic = tempIDRPic;
		tempIDRPic = tempIDRPic->next;
		free(deleteIDRPic);
	}

	idrPicturePtr = NULL;
}

#endif

