#include <targets.h>
#include <ec_siplog.h>
#include <ec_liveMonitor.h>
#include <ec_audioDSPPlayer.h>

/* add, initialize, copy, search and delete rtp calls */
void initializeRTPCall(rtpSIP*);
int populateRTPCall(rtpSIP*,sipDB*,int,char*);
void populatevideoRTPCall(rtpSIP*,int,char *,char *,char,char);
void closeRTPCall();
rtpSIP* createNewRTPCall(sipDB*,int,char*);
rtpSIP* addNewRTPCall();
rtpSIP* searchRTPCall(int,char *);
void deleteRTPCall(rtpSIP*);
rtpSIP* checkRTPCall(char *,int,char *,int,int *,int *);
int addRTPValues(rtpSIP*,sipDB*,int,char *);
rtpSIP *firstRTPCall = NULL;
rtpSIP *lastRTPCall = NULL;

void closeRTPCall(){

	rtpSIP* currentRTPCall = firstRTPCall;
	while(currentRTPCall != NULL){
		
		rtpSIP* tempRTPCall = currentRTPCall->next;
		free(currentRTPCall);
		currentRTPCall = tempRTPCall;
	}
	firstRTPCall = NULL; lastRTPCall = NULL;
}

rtpSIP* createNewRTPCall(sipDB* currentSipCall,int port, char *ipaddress){

        //SIPLOG("Createing a newRTPcall\n");
        /* Adds a new call, initializes and populates the structure */
        rtpSIP* currentRTPCall = addNewRTPCall();
        if(currentRTPCall == NULL)
                return NULL;
        initializeRTPCall(currentRTPCall);
        int retRTPCall = populateRTPCall(currentRTPCall,currentSipCall,port,ipaddress);
        if(retRTPCall < 0){
                deleteRTPCall(currentRTPCall);
                return NULL;
        }

        return currentRTPCall;

}

rtpSIP* addNewRTPCall(){

        rtpSIP* currentRTPCall = calloc(sizeof(rtpSIP),1);
        if(currentRTPCall == NULL){
                SIPLOG("Error: Failed to allocate memory for rtp call \n");
                INSTANT_USER_MSG("Error: Failed to allocate memory for rtp call \n");
                return NULL;
        }

        if(firstRTPCall == NULL){

                //SIPLOG("Adding first new rtp call\n");
                firstRTPCall = currentRTPCall;
                currentRTPCall->next = NULL;
                currentRTPCall->previous = NULL;
                lastRTPCall = currentRTPCall;
        }
        else{
                //SIPLOG("Adding new rtp call\n");
                lastRTPCall->next = currentRTPCall;
                currentRTPCall->previous = lastRTPCall;
                currentRTPCall->next = NULL;
                lastRTPCall = currentRTPCall;
        }

        return currentRTPCall;
}

void deleteRTPCall(rtpSIP* currentRTPCall){

        if(currentRTPCall == lastRTPCall && currentRTPCall == firstRTPCall){

                //SIPLOG("Deleting one and only RTP call\n");
                firstRTPCall = lastRTPCall = NULL;
				/* Delete the IDR picture if any before deleting the rtp call structure */
#ifdef HAVE_LIB_VLC

				deleteIDRPicture(currentRTPCall->fwdIDRPtr);
				deleteIDRPicture(currentRTPCall->revIDRPtr);
#endif
				releaseAudioDecoder(currentRTPCall);
                free(currentRTPCall);
        }

        /* Deleting the last call */
        else if(currentRTPCall == lastRTPCall){

                //SIPLOG("Deleting last rtp call\n");
                if(currentRTPCall->previous != NULL)
                        (currentRTPCall->previous)->next = NULL;
                lastRTPCall = currentRTPCall->previous;
#ifdef HAVE_LIB_VLC
				deleteIDRPicture(currentRTPCall->fwdIDRPtr);
				deleteIDRPicture(currentRTPCall->revIDRPtr);
#endif
				releaseAudioDecoder(currentRTPCall);
                free(currentRTPCall);
        }
        else if(currentRTPCall == firstRTPCall ){

                //SIPLOG("Deleting first rtp call\n");
                firstRTPCall = currentRTPCall->next;
                if(currentRTPCall->next != NULL)
                        (currentRTPCall->next)->previous = NULL;
#ifdef HAVE_LIB_VLC
				deleteIDRPicture(currentRTPCall->fwdIDRPtr);
				deleteIDRPicture(currentRTPCall->revIDRPtr);
#endif
				releaseAudioDecoder(currentRTPCall);
                free(currentRTPCall);
        }
        else{
                //SIPLOG("Deleting rtp call in the middle \n");
                (currentRTPCall->next)->previous = currentRTPCall->previous;
                (currentRTPCall->previous)->next = currentRTPCall->next;
#ifdef HAVE_LIB_VLC
				deleteIDRPicture(currentRTPCall->fwdIDRPtr);
				deleteIDRPicture(currentRTPCall->revIDRPtr);
#endif
				releaseAudioDecoder(currentRTPCall);
                free(currentRTPCall);
        }
}

void initializeRTPCall(rtpSIP *currentRTPCall){

        //SIPLOG("Initializing new RTP call\n");

        currentRTPCall->vport = 0;
        currentRTPCall->vfwd_fp = NULL;
        currentRTPCall->vrev_fp = NULL;
        currentRTPCall->vfwd_count = 1;
        currentRTPCall->vrev_count = 1;
        currentRTPCall->vrtp_count = 1;
        currentRTPCall->vfwd_seqcount = -1;
        currentRTPCall->vrev_seqcount = -1;
		currentRTPCall->copiedFiles = 0;
		currentRTPCall->vfwd_cur_frame_samples = 0;
		currentRTPCall->vrev_cur_frame_samples = 0;
		currentRTPCall->vfwd_bottleneck.send_time = 0;
		currentRTPCall->vfwd_bottleneck.arrival_time = 0;
		currentRTPCall->vfwd_bottleneck.sample_count = 0;
		currentRTPCall->vfwd_bottleneck.rtp_number = 0;
		currentRTPCall->vrev_bottleneck.send_time = 0;
		currentRTPCall->vrev_bottleneck.arrival_time = 0;
		currentRTPCall->vrev_bottleneck.sample_count = 0;
		currentRTPCall->vrev_bottleneck.rtp_number = 0;

        memset(currentRTPCall->vfwd_file,'\0',sizeof(currentRTPCall->vfwd_file));
        memset(currentRTPCall->vrev_file,'\0',sizeof(currentRTPCall->vrev_file));
        memset(currentRTPCall->vfwd_mp4, '\0',sizeof(currentRTPCall->vfwd_mp4));
        memset(currentRTPCall->vrev_mp4, '\0',sizeof(currentRTPCall->vrev_mp4));
        memset(currentRTPCall->vfwd_avi, '\0',sizeof(currentRTPCall->vfwd_avi));
        memset(currentRTPCall->vrev_avi, '\0',sizeof(currentRTPCall->vrev_avi));

        currentRTPCall->fwd_fuastart = 0;
        currentRTPCall->rev_fuastart = 0;

        memset(currentRTPCall->ip_addr,'\0',sizeof(currentRTPCall->ip_addr));
        currentRTPCall->port = 0;

        currentRTPCall->session_active = 0;

        currentRTPCall->dp = NULL;
        currentRTPCall->dumper = NULL;
        memset(currentRTPCall->pcap_filename, '\0', sizeof(currentRTPCall->pcap_filename));

        memset(currentRTPCall->fwd_wav, '\0', sizeof(currentRTPCall->fwd_wav));
        memset(currentRTPCall->rev_wav, '\0', sizeof(currentRTPCall->rev_wav));
        memset(currentRTPCall->both_wav, '\0', sizeof(currentRTPCall->both_wav));
        currentRTPCall->fwd_fp = NULL;
        currentRTPCall->rev_fp = NULL;
		currentRTPCall->rtp_count = 1;
        currentRTPCall->fwd_count = 1;
        currentRTPCall->rev_count = 1;
        currentRTPCall->fwd_seqCount = -1;
        currentRTPCall->rev_seqCount = -1;
		currentRTPCall->fwd723FrameType = -1;
		currentRTPCall->rev723FrameType = -1;
		currentRTPCall->g726SampleSize = 0;
		currentRTPCall->audioRTPMAP = NULL;
		currentRTPCall->videoRTPMAP = NULL;
		currentRTPCall->videoFMTP = NULL;

        currentRTPCall->fwdAudioBufSize = AV_BUF_INC;
        currentRTPCall->fwdAudioBuf = NULL;
        currentRTPCall->fwdAudioIdx = 0;

        currentRTPCall->revAudioBufSize = AV_BUF_INC;
        currentRTPCall->revAudioBuf = NULL;
        currentRTPCall->revAudioIdx = 0;

		currentRTPCall->liveMonitor = 0;
		currentRTPCall->liveMonitorFwdAudioCtr = 0;
		currentRTPCall->liveMonitorRevAudioCtr = 0;
		currentRTPCall->liveMonitorFwdVideoCtr = 0;
		currentRTPCall->liveMonitorRevVideoCtr = 0;

#ifdef HAVE_LIB_VLC

		currentRTPCall->liveMonitorFwdVlcStarted = 0;
		currentRTPCall->liveMonitorRevVlcStarted = 0;

		currentRTPCall->savedFwdSPS = 0;
		currentRTPCall->savedRevSPS = 0;	
	
		memset(currentRTPCall->fwdSPS, '\0', sizeof(currentRTPCall->fwdSPS));
		memset(currentRTPCall->fwdPPS, '\0', sizeof(currentRTPCall->fwdPPS));
		memset(currentRTPCall->revSPS, '\0', sizeof(currentRTPCall->revSPS));
		memset(currentRTPCall->revPPS, '\0', sizeof(currentRTPCall->revPPS));

		currentRTPCall->fwdIDRPtr = NULL;
		currentRTPCall->revIDRPtr = NULL;
		currentRTPCall->fwdCurrentIDRPtr = NULL;
		currentRTPCall->revCurrentIDRPtr = NULL;

		currentRTPCall->fwdIDRCtr = 0;
		currentRTPCall->revIDRCtr = 0;
		currentRTPCall->recvFwdAllIDRPic = 0;
		currentRTPCall->recvRevAllIDRPic = 0;
		
		currentRTPCall->currentFwdPPSCtr = 0;
		currentRTPCall->previousFwdPPSCtr = 0;
		currentRTPCall->currentRevPPSCtr = 0;
		currentRTPCall->previousRevPPSCtr = 0;

		currentRTPCall->liveMonitorSocket = 0;
#endif

		currentRTPCall->audioDSPSet = 0;

}      

int populateRTPCall(rtpSIP* currentRTPCall, sipDB* currentSipCall,int port, char *ipaddress){

        int retRTPValue = addRTPValues(currentRTPCall, currentSipCall, port, ipaddress);

        return retRTPValue;
}

rtpSIP* checkRTPCall(char *sipaddress, int sport, char *dipaddress, int dport, int *videoYes, int *direction){
        rtpSIP* currentRTPCall = firstRTPCall;
        while(currentRTPCall != NULL){

                if((currentRTPCall->port == sport) && (!strcmp(currentRTPCall->ip_addr,sipaddress))){
                        *videoYes = 0;
						*direction = 1;
                        return currentRTPCall;
                }
                else if((currentRTPCall->vport == sport) && (!strcmp(currentRTPCall->ip_addr,sipaddress))){
                        *videoYes = 1;
						*direction = 1;
                        return currentRTPCall;
                }
				else if((currentRTPCall->port == dport) && (!strcmp(currentRTPCall->ip_addr,dipaddress))){
						*videoYes = 0;
						*direction = 2;
						return currentRTPCall;
				}
				else if((currentRTPCall->vport == dport) && (!strcmp(currentRTPCall->ip_addr,dipaddress))){
						*videoYes = 1;
						*direction = 2;
						return currentRTPCall;
				}
                currentRTPCall = currentRTPCall->next;
        }
        return NULL;
}

/* The actual function that populates the entries in the rtp structure */

int addRTPValues(rtpSIP* currentRTPCall, sipDB *currentSipCall, int port, char *ipaddress){

        //SIPLOG("Adding RTP values for the created RTP call\n");
        char time_string[12];
        char file_name[170] = {'\0'};
        char fwdWav[170] = {'\0'};
        char revWav[170] = {'\0'};
        char bothWav[170] = {'\0'};

        currentRTPCall->port = port;
        memcpy(currentRTPCall->ip_addr, ipaddress, MAX_ASCII_ADDR_LEN);
        currentRTPCall->session_active = 1;
        get_time_string(1,time_string);

		if(currentSipCall->audioRTPMAP != NULL){
			currentRTPCall->audioRTPMAP = (char *) calloc(strlen(currentSipCall->audioRTPMAP) + 1, sizeof(char));
			if(currentRTPCall->audioRTPMAP != NULL){
				strncpy(currentRTPCall->audioRTPMAP, currentSipCall->audioRTPMAP, strlen(currentSipCall->audioRTPMAP));
			}
		}

		currentRTPCall->audioCodecType = currentSipCall->audioCodecType;
		currentRTPCall->audioPayloadType = currentSipCall->audioPayloadType;
		currentRTPCall->g726SampleSize = currentSipCall->g726SampleSize;

        if(strlen(currentSipCall->src_username) == 0){

                char tmp_string[64];
                memset(tmp_string,'\0',sizeof(tmp_string));
                //sprintf(tmp_string,"%lu",sip_db[sip_ptr].sip_info.src_extension);
                strncpy(file_name,currentSipCall->src_extension,sizeof(file_name));
        }
        else{
                strncpy(file_name,currentSipCall->src_username,sizeof(file_name));
        }

        strcat(file_name, "-Calling-");
        if(strlen(currentSipCall->dst_username) == 0){

                char tmp_string[64];
                bzero(tmp_string,sizeof(tmp_string));
                //sprintf(tmp_string,"%lu",sip_db[sip_ptr].sip_info.dst_extension);
                if(currentSipCall->remoteParty)
                        strcat(file_name,currentSipCall->remote_extension);
                else
			strcat(file_name,currentSipCall->dst_extension);
        }
        else{
                strcat(file_name,currentSipCall->dst_username);
        }      

        strcat(file_name,"-");
    strcat(file_name,time_string);
    /* copying the first portion of the file for forward and reverse direction */
        strcat(file_name,"-");
        char stringbuffer[10] = {'\0'};
        snprintf(stringbuffer,sizeof(stringbuffer),"%d",currentSipCall->call_number);
        strcat(file_name,stringbuffer);
        strncpy(fwdWav,file_name,sizeof(fwdWav));
        strncpy(revWav,file_name,sizeof(revWav));
        strncpy(bothWav,file_name,sizeof(bothWav));
        strcat(fwdWav,"-forward");
        strcat(revWav,"-reverse");
        strcat(bothWav,"-both");
        /* end of fwd and reverse file name creation */
        strcat(file_name,".pcap");

        strncpy(currentRTPCall->fwd_wav,fwdWav,sizeof(currentRTPCall->fwd_wav));
        strncpy(currentRTPCall->rev_wav,revWav,sizeof(currentRTPCall->rev_wav));
        strncpy(currentRTPCall->both_wav,bothWav,sizeof(currentRTPCall->both_wav));
        strncpy(currentRTPCall->pcap_filename,file_name,sizeof(currentRTPCall->pcap_filename));

        currentRTPCall->dp = pcap_open_dead(DLT_EN10MB,1024);
        if(currentRTPCall->dp == NULL){
            INSTANT_USER_MSG("Error creating pcap handle, not dumping the rtp message \n");
                        return -1;
        }

        currentRTPCall->dumper = pcap_dump_open(currentRTPCall->dp,currentRTPCall->pcap_filename);
        if(currentRTPCall->dumper == NULL){
                INSTANT_USER_MSG("Error opening pcap dump file \n");
                                return -1;
        }
	return 0;
}

void populatevideoRTPCall(rtpSIP* currentRTPCall, int port, char *videoRTPMAP, char *videoFMTP, char videoCodecType, char videoPayloadType){

        //SIPLOG("Inside populatevideoRTPCall\n");

		if(videoRTPMAP != NULL){

			currentRTPCall->videoRTPMAP = (char *)calloc(strlen(videoRTPMAP) + 1, sizeof(char));
			if(currentRTPCall->videoRTPMAP != NULL){
				strncpy(currentRTPCall->videoRTPMAP,videoRTPMAP,strlen(videoRTPMAP));
			}
		}

		if(videoFMTP != NULL){

			currentRTPCall->videoFMTP = (char *)calloc(strlen(videoFMTP) + 1, sizeof(char));
			if(currentRTPCall->videoFMTP != NULL){
				strncpy(currentRTPCall->videoFMTP,videoFMTP,strlen(videoFMTP));
			}
		}

		currentRTPCall->videoCodecType = videoCodecType;
		currentRTPCall->videoPayloadType = videoPayloadType;

        currentRTPCall->vport = port;

        strncpy(currentRTPCall->vfwd_file,currentRTPCall->fwd_wav,sizeof(currentRTPCall->vfwd_file));
        strcat(currentRTPCall->vfwd_file,"_video");

        strncpy(currentRTPCall->vfwd_mp4,currentRTPCall->fwd_wav,sizeof(currentRTPCall->vfwd_mp4));
        strcat(currentRTPCall->vfwd_mp4,"_video_test.avi");

        strncpy(currentRTPCall->vfwd_avi,currentRTPCall->fwd_wav,sizeof(currentRTPCall->vfwd_avi));
        strcat(currentRTPCall->vfwd_avi,"_video.avi");

        strncpy(currentRTPCall->vrev_file,currentRTPCall->rev_wav,sizeof(currentRTPCall->vrev_file));
        strcat(currentRTPCall->vrev_file,"_video");

        strncpy(currentRTPCall->vrev_mp4,currentRTPCall->rev_wav,sizeof(currentRTPCall->vrev_mp4));
        strcat(currentRTPCall->vrev_mp4,"_video_test.avi");

        strncpy(currentRTPCall->vrev_avi,currentRTPCall->rev_wav,sizeof(currentRTPCall->vrev_avi));
        strcat(currentRTPCall->vrev_avi,"_video.avi");

}

