/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2012 VIPER Lab
    Jason Ostrom <jostrom@viperlab.net>
    Arjun Sambamoorthy <arjun@viperlab.net>
    Tom Mostyn <tmostyn@viperlab.net>

    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <ec.h>
#include <ec_inet.h>
#include <ec_decode.h>
#include <ec_dissect.h>
#include <ec_proto.h>
#include <tftp_client.h>
#include <string.h>
#include <targets.h>
#include <sox.h>
#include <g722.h>
#include <stdlib.h>
#include <stdio.h>
#include <ec_siph264.h>
#include <ec_threads.h>
#include <st_i.h>
#include <g711.h>

#include "avconv.h"
#ifdef HAVE_VIDEO_DECODER
#include "ffmpeg.h"
#endif

#include <ec_voipglobals.h>
#include <ec_voiplogfiles.h>
#ifndef OS_WINDOWS
#include <ec_liveMonitor.h>
#include <ec_audioDSPPlayer.h>
#endif

typedef struct skinnycall SkinnyCall;
typedef SkinnyCall *SkinnyCallPtr;
SkinnyCallPtr temp1_skinnycall;

typedef struct rtp_Data rtpCall;
typedef rtpCall *rtpCallPtr;
rtpCallPtr currentRTPCall;


#ifndef OS_WINDOWS
#ifndef ARCH_X64
extern int initialize_g729_decoder(unsigned long *);
extern int decode_payload_g729(u_char *, int, short **, int *, unsigned long );
#endif
#endif

//from src/vc_garpb.c
extern void sendSpoofedArpReply(char *sourceip, char *targetip, char *targetmac);

//from src/vc_filter.c
extern int func_drop(struct packet_object *);

//from src/vc_dissect.c
extern void dissect_del(char *name);

pthread_mutex_t sccpaccess_mutex = PTHREAD_MUTEX_INITIALIZER;
#define SCCPACCESS_LOCK     do{ pthread_mutex_lock(&sccpaccess_mutex); } while(0)
#define SCCPACCESS_UNLOCK   do{ pthread_mutex_unlock(&sccpaccess_mutex); } while(0)

/* Mutex variable for accessing live monitor */
extern pthread_mutex_t livemonitor_mutex;
#define LIVEMONITOR_LOCK    do{ pthread_mutex_lock(&livemonitor_mutex); } while(0)
#define LIVEMONITOR_UNLOCK  do{ pthread_mutex_unlock(&livemonitor_mutex); } while(0)

/* Function definitions */
extern void getmacfromip( char tempip[MAX_ASCII_ADDR_LEN], char tempmac[MAX_ASCII_ADDR_LEN]);
FUNC_DECODER(dissector_sccp);
FUNC_DECODER(dissector_rtp_skinny);
void sccp_init(void);
char *lookupdialingname (int passthrupartyid, SkinnyCallPtr *sPtr);
char *lookupdialedname (int passthrupartyid, SkinnyCallPtr *sPtr);
char *dialednamelookup (int conferenceid, SkinnyCallPtr *sPtr);
char *dialingnamelookup (int conferenceid, SkinnyCallPtr *sPtr);
void getdialingnumber (int passthrupartyid, SkinnyCallPtr *sPtr, char *msgstring);
//char *getdialingnumber (int passthrupartyid, SkinnyCallPtr *sPtr);
char *getdialingip(int passthrupartyid, SkinnyCallPtr *sPtr);
char *getdialedip(int passthrupartyid, SkinnyCallPtr *sPtr);
//char *getdialednumber (int passthrupartyid, SkinnyCallPtr *sPtr);
void getdialednumber (int passthrupartyid, SkinnyCallPtr *sPtr, char *msgstring);
void gettimestring(int mode24, char *);
void getpartystring(u_char callstring[24], char returnstring[24]);
int populate_rtp_struct(int, char *, int, int);
char *checkmessageid(u_char messageid[4]);
char translate( char digit);
int getcallerdirname(int passthrupartyid, SkinnyCallPtr *sPtr);
int getcalleddirname(int passthrupartyid, SkinnyCallPtr *sPtr);
void printcalltofile(SkinnyCallPtr *sPtr, int conferenceid);
void printcalltofiledetails(SkinnyCallPtr *sPtr, int conferenceid);
void printcallstdout(SkinnyCallPtr *sPtr, int conferenceid);
void printtoskinnyfilestart(SkinnyCallPtr *sPtr, int conferenceid);
char adddigit(int digit);
int checkdigit(char digit);
int mix_rtp_stream(char *, char*);
//int isskinnyempty(struct skinnycall *scall); 
int lookup_garp_yes(char ip[MAX_ASCII_ADDR_LEN]);	
void skinnyfirstcallinit(SkinnyCallPtr *sPtr, char dst_ip[MAX_ASCII_ADDR_LEN], int confid);
void keypadbuttonhandler(SkinnyCallPtr *sPtr, char ip[MAX_ASCII_ADDR_LEN], int keypadbutton);
void backspacereqhandler(SkinnyCallPtr *sPtr, char ip[MAX_ASCII_ADDR_LEN]);
void nodeupdatecallstateoffhook(SkinnyCallPtr *sPtr, char dst_ip[MAX_ASCII_ADDR_LEN], int confid);
void addkeypaddigits(int retval, char ip[MAX_ASCII_ADDR_LEN], struct skinnycall *first_skinnycall);
void handlerdialednumber(SkinnyCallPtr *sPtr, int confid, char calledparty[24]);
//int handlerstationcallinfomessage(SkinnyCallPtr *sPtr, int confid, char *callerid, char ip[MAX_ASCII_ADDR_LEN], int length);
int handlercallinfomessage(SkinnyCallPtr *sPtr, int CallIdentifier, char callingp[24], char calledp[24], char dst_ip[MAX_ASCII_ADDR_LEN], int calltype);	
//void newnodestationcallinfomessage(SkinnyCallPtr *sPtr, int retval, int confid, char *callerid, char ip[MAX_ASCII_ADDR_LEN], int length);
void newnodecallinfomessage(SkinnyCallPtr *sPtr, char callingp[24], char calledp[24], int retval, int confid, char ip[MAX_ASCII_ADDR_LEN]);
void passthrupartyadd(SkinnyCallPtr *sPtr, int confid, int passthrupartyid);
void vpassthrupartyadd(SkinnyCallPtr *sPtr, int confid, int vpassthrupartyid);
int openreceivehandler(SkinnyCallPtr *sPtr, char ip[MAX_ASCII_ADDR_LEN], int rtpport, int passthrupartyid);
int closeaudiochannel(SkinnyCallPtr *sPtr, int passthrupartyid);
void deletecallnode( SkinnyCallPtr *sPtr, int passthrupartyid );
void printList( SkinnyCallPtr currentPtr );
int isEmpty( SkinnyCallPtr sPtr );
int check_fwd_media_garp(SkinnyCallPtr *sPtr, char tempip[MAX_ASCII_ADDR_LEN], int passthrupartyid);
int check_rev_media_garp(SkinnyCallPtr *sPtr, char tempip[MAX_ASCII_ADDR_LEN], int passthrupartyid);
void set_both_good(int passthrupartyid, SkinnyCallPtr *sPtr);
void handlerstartmediatransmission(SkinnyCallPtr *sPtr, int confid, int passthrupartyid, char dst_ip[MAX_ASCII_ADDR_LEN], char remote_ip[MAX_ASCII_ADDR_LEN], int rtpport);
//char *partygetcalling(char remainingStr[24], int splength);
//char *partygetcalled(char remainingStr[24], int splength);
struct digitbuffer partygetstrings(char remainingStr[24], int splength);
//void partygetstrings(char remainingStr[24], int splength, struct digitbuffer retbuffer);
int stypehandler(char *messagetype, u_char *bytes, int length, char src_ip[MAX_ASCII_ADDR_LEN], char dst_ip[MAX_ASCII_ADDR_LEN], int version, struct packet_object *packet);
int populate_rtp_struct_video(int, char *, int, int);
int openmultimediahandler(SkinnyCallPtr *, char *, int, int, int);
int flush;
// TEST
//char callingpStr[24];
/*struct digitbuffer{
	char string1[24];
	char string2[24];
};*/


#define phtonl(p, v) \
	{				\
        (p)[0] = (u_int8_t)((v) >> 24);   \
	(p)[1] = (u_int8_t)((v) >> 16);   \
	(p)[2] = (u_int8_t)((v) >> 8);    \
	(p)[3] = (u_int8_t)((v) >> 0);    \
	}

void __init sccp_init(void) {
   dissect_add("sccp", APP_LAYER_TCP,2000, dissector_sccp);
}

// Initialize this to NULL first
SkinnyCallPtr first_skinnycall = NULL;

struct rtp_data rtp[1024];

struct au_header{

	u_char file_header[4];
	u_char header_size[4];
	u_char file_size[4];
	u_char encoding_type[4];
	u_char sampling_rate[4];
	u_char channels[4];

};

struct offhook_garp_en{
	char ip[MAX_ASCII_ADDR_LEN];
	int garp_yes;
};
struct offhook_garp_en garpenhosts[1024];

int sversion;
int rtp_counter = 0;
int garp_hostcount = 0;
int callcount = 0;
int callnum =  1;
int callnumend =  1;
int ip = 0;
int splength;
int keypadyes = 0;
int callerexistsindir = 0;
int calledexistsindir = 0;
FILE *fp_wav;
int begin_tftp_modify_attack = 1;
int start_tftp_mod_attack = 0;
	
// from targets.tc
extern struct singletarget target1;
struct singlecalltrack track1;
extern FILE *singletarget;
extern int targetlookup(SkinnyCallPtr *sPtr, int passthrupartyid, char ip[MAX_ASCII_ADDR_LEN]);
extern int check_rtp_hdr(char *ip_addr, int media_port, int *video);
extern int check_rtp_entry(char *ip_addr);
extern void printtargetwait(void);

// from directory_parser.c
extern int directory_count;
//extern struct corporate_directory directory[1024];

// from targets.c
extern struct targets targettab[1024];
extern int targetcount;

struct ipcall {
	char ip[MAX_ASCII_ADDR_LEN];
	int lineinstance;
	char digits[24];
	int offhook;
	int digitcount;
};
struct ipcall ipcalltab[1024];

char *checkmessageid(u_char messageid[4]) {

	char *msgstring;

	//Set new messageid like ntohs
	u_char newmessageid[4];

	newmessageid[0] = messageid[3];
	newmessageid[1] = messageid[2];
	newmessageid[2] = messageid[1];
	newmessageid[3] = messageid[0];
	
	//Check skinny message type
	if((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 0)){
		msgstring = "KeepAliveMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 0)){
		msgstring = "KeepAliveAckMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 6)){
		msgstring = "OffHookMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 133)){
		msgstring = "SetRingerMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 136)){
		msgstring = "SetSpeakerModeMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 134)){
		msgstring = "SetLampMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 17)){
		msgstring = "CallStateMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 16)){
		msgstring = "SelectSoftKeysMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 22)){
		msgstring = "ActivateCallPlaneMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 130)){
		msgstring = "StartToneMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 3)){
		msgstring = "KeypadButtonMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 131)){
		msgstring = "StopToneMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 29)){
		msgstring = "DialedNumberMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 5)){
		msgstring = "OpenReceiveChannel";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 34)){
		msgstring = "OpenReceiveChannelAck";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 138)){
		msgstring = "StartMediaTransmission";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 7)){
		msgstring = "OnHookMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 19)){
		msgstring = "ClearPromptStatusMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 6)){
		msgstring = "CloseReceiveChannel";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 139)){
		msgstring = "StopMediaTransmission";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 148)){
		msgstring = "DefineTimeDate";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 74)){
		msgstring = "PartyInfoMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 143)){
		msgstring = "CallInfoMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 38)){
		msgstring = "SoftKeyEventMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 25)){
		msgstring = "BackSpaceReqMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 49)){
		msgstring = "OpenMultiMediaChannelMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 0)&&(newmessageid[3] == 49)){
		msgstring = "OpenMultiMediaReceiveChannelAckMessage";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 50)){
		msgstring = "StartMultiMediaTransmission";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 54)){
		msgstring = "CloseMultiMediaReceiveChannel";
	} else if ((newmessageid[0] == 0)&&(newmessageid[1] == 0)&&(newmessageid[2] == 1)&&(newmessageid[3] == 51)){
		msgstring = "StopMultiMediaTransmission"; 
	} else {
		msgstring = "Unknown";
	}

	return msgstring;
}

char translate( char digit) {

	/* z means it didn't return a numeric digit */
	char returndig = 'z';

 	if (digit == 48) {
		returndig = '0';
	} else if (digit == 49) {
		returndig = '1';
	} else if (digit == 50) {
		returndig = '2';
	} else if (digit == 51) {
		returndig = '3';
	} else if (digit == 52) {
		returndig = '4'; 
	} else if (digit == 53) {
		returndig = '5';
	} else if (digit == 54) {
		returndig = '6';
	} else if (digit == 55) {
		returndig = '7';
	} else if (digit == 56) {
		returndig = '8';
	} else if (digit == 57) {
		returndig = '9';
	}
			
	return returndig;

}
void getdialingnumber (int passthrupartyid, SkinnyCallPtr *sPtr, char *msgstring){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int found = 0;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){
		
			int compare = strcmp(current_skinnycall->dialingnumber,"Unknown");	
			if ( compare == 0 ) {
				strcpy(msgstring,"Unknown");;
			} else {
				strcpy(msgstring,current_skinnycall->dialingnumber);
			}

			found = 1;
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	if (found == 0) {

		strcpy(msgstring,"Unknown");
	}

}
char *getdialingip(int passthrupartyid, SkinnyCallPtr *sPtr){


	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){
			
			return current_skinnycall->dialingip;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	return "IP Unknown";


}
char *getdialedip(int passthrupartyid, SkinnyCallPtr *sPtr){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){

			return current_skinnycall->dialedip;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}


	return "IP Unknown";

}
void getdialednumber (int passthrupartyid, SkinnyCallPtr *sPtr, char *msgstring){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int found = 0;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){
			
			int compare = strcmp(current_skinnycall->dialednumber,"Unknown");
			if ( compare == 0 ) {
				strcpy(msgstring,"Unknown");
			} else {
				strcpy(msgstring,current_skinnycall->dialednumber);
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;
			found = 1;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;
		}

	}

	if (found == 0) {
		printf("found is 0\n");
		strcpy(msgstring,"Unknown");
	}

}
int getcallerdirname(int passthrupartyid, SkinnyCallPtr *sPtr) {

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int retval = 1;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){

			int b;
			for (b = 0; b < directory_count; b++) {

				int  compare = strcmp(directory[b].telephone_number,current_skinnycall->dialingnumber);
				if (compare == 0) {
					strcpy(current_skinnycall->dirname1, directory[b].name);
					//printf("Copying in dirname1 of %s\n",current_skinnycall->dirname1);
					//printf("returning %d\n",retval);
					return retval;
				}

			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}



	}

	retval = 0;
	return retval;

}

void printcalltofile(SkinnyCallPtr *sPtr, int conferenceid) {

	char tmpstring[12];
	gettimestring(1, tmpstring);
	char *null_string = "NULL";

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	int flush;
	while (current_skinnycall != NULL) {

		if ((conferenceid == current_skinnycall->CallIdentifier)&&(current_skinnycall->call_active == 1)) {

			fprintf(ofile, "Call %d: %d: ", callnum, current_skinnycall->CallIdentifier);
			flush = fflush(ofile);

			char *temp1 = lookupdialingname(current_skinnycall->passthrupartyid1, &first_skinnycall);
			int compare = strcmp(temp1, null_string);
			if (compare != 0) {
				fprintf(ofile, "%s ",temp1);
				flush = fflush(ofile);

			}


			fprintf(ofile,"(extension %s,",current_skinnycall->dialingnumber);
			flush = fflush(ofile);
			fprintf(ofile," %s) calling to ",current_skinnycall->dialingip);
			flush = fflush(ofile);

			char *temp2 = lookupdialedname(current_skinnycall->passthrupartyid1, &first_skinnycall);
			compare = strcmp(temp2, null_string);
			if (compare != 0)  {
				fprintf(ofile,"%s ",temp2);
				flush = fflush(ofile);
			}
		
			fprintf(ofile,"(extension %s, %s), RTP Port %d\n",current_skinnycall->dialednumber,current_skinnycall->dialedip,current_skinnycall->rtpport1);
			flush = fflush(ofile);

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else if ((conferenceid == current_skinnycall->CallIdentifier)&&(current_skinnycall->call_active == 0)) {

			gettimestring(1, tmpstring);
			fprintf(ofile,"Call %d: %d: ended at %s.  ",
					current_skinnycall->callnum,
					current_skinnycall->CallIdentifier,
					tmpstring);
			flush = fflush(ofile);
			gettimeofday(&current_skinnycall->finish,0);
			int call_duration = current_skinnycall->finish.tv_sec - current_skinnycall->start.tv_sec;
			fprintf(ofile, "Call duration is %d seconds.\n",call_duration);
			flush = fflush(ofile);

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}


	}
		
	
}

void printtoskinnyfilestart(SkinnyCallPtr *sPtr, int conferenceid) {


	int flush;
	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	
	while (current_skinnycall != NULL) {

		if (conferenceid == current_skinnycall->CallIdentifier) {
			fprintf(sccpfile,"Starting media transmission for call %d:  (%s,%s) calling (%s,%s), rtp port %d \n",current_skinnycall->callnum,current_skinnycall->dialingnumber,current_skinnycall->dialingip,current_skinnycall->dialednumber,current_skinnycall->dialedip,current_skinnycall->rtpport1);
			flush = fflush(sccpfile);
			//callnum++;
			gettimeofday(&current_skinnycall->start,0);
			gettimestring(1,current_skinnycall->start_time_string);
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {
			
			current_skinnycall = current_skinnycall->next_skinnycall;


		}
	}

}

void printcalltofiledetails(SkinnyCallPtr *sPtr, int conferenceid) {

	int flush;
	char *null_string = "NULL";
	char tmpstring[12];
	gettimestring(1, tmpstring);

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if ((conferenceid == current_skinnycall->CallIdentifier)&&(current_skinnycall->call_active == 0)) {

			fprintf(odetailfile, "Call %d: %d:",current_skinnycall->callnum,current_skinnycall->CallIdentifier);
			flush = fflush(odetailfile);
			
			char *temp1 = lookupdialingname(current_skinnycall->passthrupartyid1, &first_skinnycall);	
			int compare = strcmp(temp1,null_string);
			if (compare != 0) {
				fprintf(odetailfile,"%s ",temp1);
				flush = fflush(odetailfile);
			}

			fprintf(odetailfile,"(extension %s,",current_skinnycall->dialingnumber);
			flush = fflush(odetailfile);
			fprintf(odetailfile," %s) calling to ",current_skinnycall->dialingip);
			flush = fflush(odetailfile);
			
			char *temp2 = lookupdialedname(current_skinnycall->passthrupartyid1, &first_skinnycall);
			compare = strcmp(temp2,null_string);
			if (compare != 0) {
				fprintf(odetailfile,"%s ",temp2);
				flush = fflush(odetailfile);
			}

			fprintf(odetailfile,"(extension %s, %s) at %s.",current_skinnycall->dialednumber,current_skinnycall->dialedip,current_skinnycall->start_time_string);
			flush = fflush(odetailfile);

			char tmpstring[12];
			gettimestring(1,tmpstring);
			int call_duration = current_skinnycall->finish.tv_sec - current_skinnycall->start.tv_sec;
			fprintf(odetailfile,"  Call ended at %s.  Duration:  %d seconds\n",tmpstring,call_duration);
			flush = fflush(odetailfile);

			current_skinnycall = current_skinnycall->next_skinnycall;
		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

}	

void printcallstdout(SkinnyCallPtr *sPtr, int conferenceid) {

	char tmpstring[12];
	gettimestring(1,tmpstring);
	char *null_string = "NULL"; 

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if ((conferenceid == current_skinnycall->CallIdentifier)&&(current_skinnycall->call_active == 1)) {
			
			INSTANT_USER_MSG("Call %d (SCCP) in progress at %s. ",callnum,tmpstring);
			if(voipdebug) {
				fprintf(sccpfile,"Call %d (SCCP) in progress at %s. ",callnum,tmpstring);
				flush = fflush(sccpfile);
			}
			current_skinnycall->callnum = callnum;
			printcalltofile(&first_skinnycall, current_skinnycall->CallIdentifier); 
			callnum++;

			char *temp1 = lookupdialingname(current_skinnycall->passthrupartyid1, &first_skinnycall);
                        int compare = strcmp(temp1,null_string);
                        if (compare != 0) {
				INSTANT_USER_MSG("'%s' ",temp1);

                        }
			
			compare = strcmp("IP Unknown",current_skinnycall->dialingip);
			if (compare == 0) {
				INSTANT_USER_MSG("(Number %s) calling ",current_skinnycall->dialingnumber);
			} else {
				INSTANT_USER_MSG("(Number %s, %s) calling ",current_skinnycall->dialingnumber,current_skinnycall->dialingip);
			}

			char *temp2 = lookupdialedname(current_skinnycall->passthrupartyid1, &first_skinnycall);
                        compare = strcmp(temp2,null_string);
                        if (compare != 0) {
				INSTANT_USER_MSG("'%s' ",temp2);
			}
		
			compare = strcmp("IP Unknown",current_skinnycall->dialedip);
			if (compare == 0) {
				INSTANT_USER_MSG("(Number %s).\n",current_skinnycall->dialednumber);
			} else {
				INSTANT_USER_MSG("(Number %s, %s).\n",current_skinnycall->dialednumber,current_skinnycall->dialedip);
			}
			//printf("TEST current_skinnycall->dialednumber is %s\n",current_skinnycall->dialednumber);	

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else if ((conferenceid == current_skinnycall->CallIdentifier)&&(current_skinnycall->call_active == 0)) {

			printcalltofile(&first_skinnycall, current_skinnycall->CallIdentifier);
			printcalltofiledetails(&first_skinnycall, current_skinnycall->CallIdentifier);

			gettimestring(1,tmpstring);
			INSTANT_USER_MSG("Call %d (SCCP) ended at %s.  ",current_skinnycall->callnum,tmpstring);
			if(voipdebug) {
				fprintf(sccpfile,"Call %d (SCCP) ended at %s.  ",current_skinnycall->callnum,tmpstring);
				flush = fflush(sccpfile);
			}
			gettimeofday(&current_skinnycall->finish,0);
			int call_duration = current_skinnycall->finish.tv_sec - current_skinnycall->start.tv_sec;
			INSTANT_USER_MSG("Call duration is %d seconds.\n\n",call_duration);
			if (voipdebug) {
				fprintf(sccpfile,"Call duration is %d seconds.\n\n",call_duration);
				flush = fflush(sccpfile);
			}
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}
                 
}

void gettimestring(int mode24, char *tmp) {

	static char time_str[12] = {0};
	struct tm *now = NULL;
	int hour = 0;
	time_t time_value = 0;

	time_value = time(NULL);
	now = localtime(&time_value);
	hour = now->tm_hour;
	if (!mode24 && hour > 12)
		hour -=12;

	sprintf(time_str,"%d-%d-%d",hour,now->tm_min,now->tm_sec);
 	strncpy(tmp,time_str,sizeof(time_str));
}

int getcalleddirname(int passthrupartyid, SkinnyCallPtr *sPtr) {

	//SkinnyCallPtr prev_skinnycall;
	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int retval = 1;

	while (current_skinnycall != NULL) {

		if ((passthrupartyid == current_skinnycall->passthrupartyid1)||(passthrupartyid == current_skinnycall->passthrupartyid2)){
		
			int b;
			for (b = 0; b < directory_count; b++) {

				int compare = strcmp(directory[b].telephone_number,current_skinnycall->dialednumber);
				if (compare == 0) {
					strcpy(current_skinnycall->dirname2,directory[b].name);
					return retval;
				}
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {
			//prev_skinnycall = first_skinnycall;
			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	retval = 0;
	return retval;

}

char *dialednamelookup (int conferenceid, SkinnyCallPtr *sPtr) {

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	char *msgstring = "NULL";

	while (current_skinnycall != NULL) {
	
		if(conferenceid == current_skinnycall->CallIdentifier) {

			int b;
			for (b = 0; b < directory_count; b++) {
			
				int compare = strcmp(directory[b].telephone_number,current_skinnycall->dialednumber);
				if (compare == 0) {
					if (directory[b].name != NULL) {
						return directory[b].name;
					} else {

					}

				}

			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;
		}
	}

	return msgstring;
}

char *dialingnamelookup (int conferenceid, SkinnyCallPtr *sPtr) {
	
	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	
	char *msgstring = "NULL";

	while (current_skinnycall != NULL) {


		if(conferenceid == current_skinnycall->CallIdentifier) {

			int b;
			for (b = 0; b < directory_count; b++) {

				int compare = strcmp(directory[b].telephone_number,current_skinnycall->dialingnumber);
				if (compare == 0) {
					if (directory[b].name != NULL) {
						return directory[b].name;
					} else {

					}

				}
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {
			current_skinnycall = current_skinnycall->next_skinnycall;

		}
	}
	return msgstring;
}

char *lookupdialingname (int passthrupartyid, SkinnyCallPtr *sPtr) {

	char *msgstring = "NULL";
	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;	

	while (current_skinnycall != NULL) {
	
		if ((current_skinnycall->passthrupartyid1 == passthrupartyid) || (current_skinnycall->passthrupartyid2 == passthrupartyid)) {
	
			int b;
			for (b = 0; b < directory_count; b++) {
				int compare = strcmp(directory[b].telephone_number,current_skinnycall->dialingnumber);
				if (compare == 0) {
				
					if (current_skinnycall->dirname1 != NULL) {
					
						return current_skinnycall->dirname1;

					} else {

					}
				}
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;
			

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	return msgstring;

}

char *lookupdialedname (int passthrupartyid, SkinnyCallPtr *sPtr) {

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	char *msgstring = "NULL";

	while (current_skinnycall != NULL) {
		
		if ((current_skinnycall->passthrupartyid1 == passthrupartyid) || (current_skinnycall->passthrupartyid2 == passthrupartyid)) {
			int b;
			for (b = 0; b < directory_count; b++) {

				int compare = strcmp(directory[b].telephone_number,current_skinnycall->dialednumber);
				if (compare == 0) {
					if (current_skinnycall->dirname2 != NULL) {
						//printf("Directory Name is not NULL, returning %s\n",calltab[a].dirname2);
						return current_skinnycall->dirname2;
					} else {
						//printf("Directory name is NULL, returning NULL\n");
					}
				}
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	return msgstring;
}

char adddigit(int digit){

	/* if it returns z, this means digit was not numeric */
	char retchar = 'z';

	if (digit == 0){
		retchar = '0';
	} else if (digit == 1) {
		retchar = '1';
	} else if (digit == 2) {
		retchar = '2';
	} else if (digit == 3) {
		retchar = '3'; 
	} else if (digit == 4) {
		retchar = '4';
	} else if (digit == 5) {
		retchar = '5'; 
	} else if (digit == 6) {
		retchar = '6';
	} else if (digit == 7) {
		retchar = '7';
	} else if (digit == 8) {
		retchar = '8';
	} else if (digit == 9) {
		retchar = '9';
	}

	return retchar;
}

int checkdigit(char digit){

	int retval = 0;

	if (digit == 48) {
	       retval = 1;
	} else if (digit == 49) {
 		retval = 1;
	} else if (digit == 50) {
		retval = 1;
	} else if (digit == 51) {
		retval = 1;		
	} else if (digit == 52) {
		retval = 1;		
	} else if (digit == 53) {
		retval = 1;		
	} else if (digit == 54) {
		retval = 1;		
	} else if (digit == 55) {
		retval = 1;		
	} else if (digit == 56) {
		retval = 1;		
	} else if (digit == 57) {
		retval = 1;
	}		
	
	return retval;

}

int stypehandler(char *messagetype, u_char *bytes, int length, char src_ip[MAX_ASCII_ADDR_LEN], char dst_ip[MAX_ASCII_ADDR_LEN], int version, struct packet_object *packet) {

	if (strcmp(messagetype,"KeepAliveAckMessage") == 0) {

		if (tftp_modify_yes == 1 && target_yes == 1) {

			if (begin_tftp_modify_attack == 1) {

				INSTANT_USER_MSG("Dropping KeepAliveAckMessage:  %s --> %s\n",src_ip,dst_ip);
				if(voipdebug){
					fprintf(sccpfile,"Dropping KeepAliveAckMessage:  %s --> %s\n",src_ip,dst_ip);
					flush = fflush(sccpfile);
				}
				func_drop(packet);
				start_tftp_mod_attack = 1;

			}

		}
		
	}

	if (strcmp(messagetype,"KeepAliveMessage") == 0) {

		//printf("%s --> %s:  KeepAliveMessage\n",src_ip,dst_ip);

		if (garp_hostcount == 0) {
			strcpy(garpenhosts[0].ip,src_ip);
			garpenhosts[0].garp_yes = 1;
			garp_hostcount++;
			//printf("Setting GARP Enabled for host %s via Skinny KeepAlive Message\n",src_ip);
		} else {
			int i;
			int foundip = 0;
			for (i = 0; i < garp_hostcount; i++) {
				int compare = strcmp(src_ip,garpenhosts[i].ip);
				if (compare == 0) {
					foundip = 1;
				}
			}
			
			if (foundip == 0) {

				strcpy(garpenhosts[garp_hostcount].ip,src_ip);
				garpenhosts[garp_hostcount].garp_yes = 1;
				garp_hostcount++;
				//printf("Setting GARP Enabled for host %s via Skinny KeepAlive Message\n",src_ip);
			}
		}

	}

	if (messagetype == "OpenMultiMediaChannelMessage"){

		INSTANT_USER_MSG("%s --> %s: OpenMultiMediaChannelMessage\n",src_ip,dst_ip);
		struct OpenMultiMediaChannel{
			int confid;
			int passthruid;
		};
		
		struct OpenMultiMediaChannel *openmultimediachannel;
		openmultimediachannel = (struct OpenMultiMediaChannel *) bytes;
		
		//printf("Passthruid is %d\n", openmultimediachannel->passthruid);
		//printf("Conference id is %d\n", openmultimediachannel->confid);
		vpassthrupartyadd(&first_skinnycall, openmultimediachannel->confid, openmultimediachannel->passthruid); 		
		
	}

	if (messagetype == "OpenMultiMediaReceiveChannelAckMessage"){

		INSTANT_USER_MSG("%s --> %s: OpenMultiMediaReceiveChannelAckMessage\n",src_ip,dst_ip);
		struct OpenMultiMediaReceiveChannelAck{
			char status[4];
			struct ip_addr *ip;
			int vport;
			int passthruid;
			int confid;
		};
		
		struct OpenMultiMediaReceiveChannelAck *openmultimediareceivechannelack;
		openmultimediareceivechannelack = (struct OpenMultiMediaReceiveChannelAck *)bytes;

		int retval = openmultimediahandler(&first_skinnycall, src_ip, openmultimediareceivechannelack->confid, openmultimediareceivechannelack->passthruid, openmultimediareceivechannelack -> vport); 		

		//printf("Video port is %d\n", openmultimediareceivechannelack->vport);
	}

	if(messagetype == "StartMultiMediaTransmission"){
		
		//printf("%s --> %s: StartMultiMediaTransmission\n",src_ip,dst_ip);
	}

	if(messagetype == "CloseMultiMediaReceiveChannel"){

		//printf("%s --> %s: CloseMultiMediaReceiveChannel\n",src_ip,dst_ip);
	}
	
	if(messagetype == "StopMultiMediaTransmission"){
		
		//printf("%s --> %s: StopMultiMediaTransmission\n", src_ip,dst_ip);
	}

	if (strcmp(messagetype,"OffHookMessage") == 0) {

		//printf("%s --> %s:  OffHookMessage\n",src_ip,dst_ip);

		if (garp_hostcount == 0) {
			strcpy(garpenhosts[0].ip,src_ip);
			garpenhosts[0].garp_yes = 1;
			garp_hostcount++;
			//printf("Setting GARP Enabled for host %s via OffHook Message\n",src_ip);
		} else {
			int i;
			int foundip = 0;
			for (i = 0; i < garp_hostcount; i++) {
				int compare = strcmp(src_ip,garpenhosts[i].ip);
				if (compare == 0) {
					foundip = 1;
				}

			}

			if (foundip == 0) {

				strcpy(garpenhosts[garp_hostcount].ip,src_ip);
				garpenhosts[garp_hostcount].garp_yes = 1;
				garp_hostcount++;
				//printf("Setting GARP Enabled for host %s via OffHook Message\n",src_ip);
			}

		}

	}
	if (strcmp(messagetype,"CloseReceiveChannel") ==0) {


		#ifndef OS_WINDOWS   
   /* Release any currently monitoring stream */
            LIVEMONITOR_LOCK;
            stopCurrentlyMonitoringStream();
            LIVEMONITOR_UNLOCK;
#endif   
             /* Close lib vlc */



		struct CloseReceiveChannel {
			int confid;
			int passthrupartyid;
		};

		struct CloseReceiveChannel *closereceivechannel;
		closereceivechannel = (struct CloseReceiveChannel*)(bytes);

		//printf("%s --> %s:  %d:  %d:  CloseReceiveChannel message\n",src_ip, dst_ip,closereceivechannel->confid,closereceivechannel->passthrupartyid);
		int retval;	
		retval = closeaudiochannel(&first_skinnycall, closereceivechannel->passthrupartyid);
		#ifdef HAVE_LIB_VLC
               closeLibVlc();
#endif

	}
	if (strcmp(messagetype,"OpenReceiveChannel") == 0) {


		struct OpenReceiveChannel {
			int confid;
			int passthrupartyid;
		};
		
		struct OpenReceiveChannel *openreceivechannel;
		openreceivechannel = (struct OpenReceiveChannel*)(bytes);
		
		//printf("%s --> %s:  %d: %d:  OpenReceiveChannel message\n",src_ip, dst_ip,openreceivechannel->confid,openreceivechannel->passthrupartyid);

		passthrupartyadd(&first_skinnycall, openreceivechannel->confid, openreceivechannel->passthrupartyid);

	}
	if (strcmp(messagetype,"OpenReceiveChannelAck") == 0) {


		if ((version == 7)||(version == 8)){
			struct OpenReceiveChannelAck {

				u_char status[8];
				struct ip_addr *ip;
				u_char misc2[12];
				int rtpport;
				int passthrupartyid;
			};


			struct OpenReceiveChannelAck *openreceivechannelack;
			openreceivechannelack = (struct OpenReceiveChannelAck*)(bytes);
		
			//printf("%s --> %s:  %d:  OpenReceiveChannelAck message\n",src_ip, dst_ip,openreceivechannelack->passthrupartyid);
		
			// Search through node list and find either passthrupartyid1 or 2
			int retval;
			retval = openreceivehandler(&first_skinnycall, src_ip, openreceivechannelack->rtpport, openreceivechannelack->passthrupartyid);
			/* handle target lookup  */
			int targetretval;
			targetretval = targetlookup(&first_skinnycall, openreceivechannelack->passthrupartyid, src_ip);

		} else if (version == 6){
			struct OpenReceiveChannelAck {
				u_char status[4];
				struct ip_addr *ip;
				int rtpport;
				int passthrupartyid;
			};

			struct OpenReceiveChannelAck *openreceivechannelack;
			openreceivechannelack = (struct OpenReceiveChannelAck*)(bytes);

			//printf("%s --> %s:  %d:  OpenReceiveChannelAck message\n",src_ip, dst_ip,openreceivechannelack->passthrupartyid);
			
			// Search through node list and find either passthrupartyid1 or 2
			int retval;
			retval = openreceivehandler(&first_skinnycall, src_ip, openreceivechannelack->rtpport, openreceivechannelack->passthrupartyid);
			
			// handle target lookup
			int targetretval;
			targetretval = targetlookup(&first_skinnycall, openreceivechannelack->passthrupartyid, src_ip);

		}

	}
	if (strcmp(messagetype,"PartyInfoMessage") == 0) {

		int splength = length - 32;

		struct PartyInfoMessage {

			char other4[4];
			int CallIdentifier;
			char other24[24];
			char remaining[splength];
		};

		struct PartyInfoMessage *partyinfomessage;
		partyinfomessage = (struct PartyInfoMessage*)(bytes);
	
		// Print the inbound message details	
		//INSTANT_USER_MSG("%s --> %s:  %d:  PartyInfoMessage\n",src_ip,dst_ip,partyinfomessage->CallIdentifier);
		if(voipdebug){
			fprintf(sccpfile,"%s --> %s:  %d:  PartyInfoMessage\n",src_ip,dst_ip,partyinfomessage->CallIdentifier);
			flush = fflush(sccpfile);
		}
		
		/*   Print some debug on the received bytes	
		INSTANT_USER_MSG("splength:  %d, Printing partyinfo bytes: ", splength);
		int i;
		for (i = 0; i < splength; i++) {
			INSTANT_USER_MSG("%02x ",partyinfomessage->remaining[i]);

		}

		INSTANT_USER_MSG("\n");
		*/
	
		//struct digitbuffer buffer;	
		struct digitbuffer buffer = partygetstrings(partyinfomessage->remaining, splength);
		//partygetstrings(partyinfomessage->remaining, splength, buffer);
		//INSTANT_USER_MSG("After partygetstrings, dialed number is %s\n",buffer.string2);
		if(voipdebug) {
			fprintf(sccpfile,"After partygetstrings(), dialed number is %s\n",buffer.string2);
			flush = fflush(sccpfile);
			//printf("After partygetstrings(), dialed number is %s\n",buffer.string2);
		}
 
		/*
		*  Search through node list for ccallidentifier[24]
		*  If it's there, add in dialingip, ip2, and 
		*  called / calling party digits
		*  Return 1 if it's found, return 0 if it isn't
		*/
		int retval = handlercallinfomessage(&first_skinnycall, partyinfomessage->CallIdentifier, buffer.string1, buffer.string2, dst_ip, 1);

		// If it isn't in there, means I'm only receiving traffic for dialed IP Phone on this VLAN.  Then we must create new node
		newnodecallinfomessage(&first_skinnycall, buffer.string1, buffer.string2, retval, partyinfomessage->CallIdentifier, dst_ip);

	}
	if (strcmp(messagetype,"CallInfoMessage") == 0) {

		struct CallInfoMessage {

			char callingpName[40];
			u_char callingp[24];
			char calledpName[40];
			u_char calledp[24];
			int lineinstance;
			int CallIdentifier;
			int calltype;
		};

		struct CallInfoMessage *callinfomessage;
		callinfomessage = (struct CallInfoMessage*)(bytes);

		u_char callingpstring[24];
		u_char calledpstring[24];

		//printf("%s --> %s: %d: CallInfoMessage message\n",src_ip, dst_ip, callinfomessage->CallIdentifier);

		int i = 0; 
		int retVal = 1;
		while ((i < 24) && (retVal == 1)) {

			retVal = checkdigit(callinfomessage->callingp[i]);

			if (retVal == 1) {

				callingpstring[i] = callinfomessage->callingp[i];

			}

			i++;
		}
		
		callingpstring[i - 1] = '\0';

		i = 0; 
		retVal = 1;
		while ((i < 24) && (retVal == 1)) {
			retVal = checkdigit(callinfomessage->calledp[i]);
			if (retVal == 1) {
				calledpstring[i] = callinfomessage->calledp[i];
			}
			i++;
		}
		calledpstring[i - 1] = '\0';

		// Search through node list for ccallidentifier
		// If it's there, add in dialingip, ip2, and 
		// called / calling party digits
		// Return 1 if it's found, return 0 if it isn't
		int retval = handlercallinfomessage(&first_skinnycall, callinfomessage->CallIdentifier, callingpstring, calledpstring, dst_ip, callinfomessage->calltype);	
		// If it isn't in there, means I'm only receiving traffic for dialed IP Phone
		// on this VLAN.  Then we must create new node
		newnodecallinfomessage(&first_skinnycall, callingpstring, calledpstring, retval, callinfomessage->CallIdentifier, dst_ip);

	}
	if (strcmp(messagetype,"DialedNumberMessage") ==0) {


		struct DialedNumberMessage {
			u_char CalledParty[29];
			unsigned long int CIdentifier;
			u_char Other2[3];
		};


		struct DialedNumberMessage *dialednumbermessage;
		dialednumbermessage = (struct DialedNumberMessage*)(bytes);

		// Set digits correctly for char
		int digitcount = 0;
		int a;
		for (a = 0; a < 24; a++) {

			if (dialednumbermessage->CalledParty[a] != 0) {
				if (dialednumbermessage->CalledParty[a] == 48) {
					dialednumbermessage->CalledParty[a] = '0';
				} else if (dialednumbermessage->CalledParty[a] == 49) {
					dialednumbermessage->CalledParty[a] = '1';
				} else if (dialednumbermessage->CalledParty[a] == 50) {
					dialednumbermessage->CalledParty[a] = '2';
				} else if (dialednumbermessage->CalledParty[a] == 51) {
					dialednumbermessage->CalledParty[a] = '3';
				} else if (dialednumbermessage->CalledParty[a] == 52) {
					dialednumbermessage->CalledParty[a] = '4';
				} else if (dialednumbermessage->CalledParty[a] == 53) {
					dialednumbermessage->CalledParty[a] = '5';
				} else if (dialednumbermessage->CalledParty[a] == 54) {
					dialednumbermessage->CalledParty[a] = '6';
				} else if (dialednumbermessage->CalledParty[a] == 55) {
					dialednumbermessage->CalledParty[a] = '7';
				} else if (dialednumbermessage->CalledParty[a] == 56) {
					dialednumbermessage->CalledParty[a] = '8';
				} else if (dialednumbermessage->CalledParty[a] == 57) {
					dialednumbermessage->CalledParty[a] = '9';
				} else {
				}

				digitcount++;
			}

		}

	}

	if (strcmp(messagetype,"StartMediaTransmission") == 0) {

		if ((version == 8)||(version == 7)) {
                	struct StartMediaTransmission {
                        	int ConferenceID;
                        	int PassThruPartyID;
				u_char other[4];
                        	u_char RemoteIPBytes[4];
				u_char other2[12];
                        	int RemotePort;
                	};
		
			struct StartMediaTransmission *startmediatransmission;
                	startmediatransmission = (struct StartMediaTransmission*)(bytes);

                	// Print some details on inbound message
                	//printf("%s --> %s:  %d:  %d:  StartMediaTransmission\n",src_ip,dst_ip,startmediatransmission->ConferenceID,startmediatransmission->PassThruPartyID);

			/* Disable this 
                	int retval = lookup_garp_yes(dst_ip);
                	if (retval != 1) {

                        	printf("GARP is disabled for IP Phone %s.  Will not record media for this call.\n",dst_ip);
                	}
			*/

                	// Get the remote IP Address
                	struct ip_addr RemoteIP;
                	char remoteipstring[MAX_ASCII_ADDR_LEN];
			int ipretval;
                	ipretval = ip_addr_init(&RemoteIP, AF_INET, startmediatransmission->RemoteIPBytes);
                	ip_addr_ntoa(&RemoteIP, remoteipstring);

                	handlerstartmediatransmission(&first_skinnycall, startmediatransmission->ConferenceID, startmediatransmission->PassThruPartyID,dst_ip, remoteipstring,startmediatransmission->RemotePort);

			if (garpdb_yes == 1) {
				int targetretval;
				targetretval = targetlookup(&first_skinnycall, startmediatransmission->PassThruPartyID, dst_ip);
			}			

		} else if (version == 6) {

			struct StartMediaTransmission {
				int ConferenceID;
				int PassThruPartyID;
				u_char RemoteIPBytes[4];
				int RemotePort;
			};

			struct StartMediaTransmission *startmediatransmission;
			startmediatransmission = (struct StartMediaTransmission*)(bytes);
	
			// Print some details on inbound message	
			//printf("%s --> %s:  %d:  %d:  StartMediaTransmission\n",src_ip,dst_ip,startmediatransmission->ConferenceID,startmediatransmission->PassThruPartyID);
	
			/* Disable this	
			int retval = lookup_garp_yes(dst_ip);
			if (retval != 1) {

				INSTANT_USER_MSG("GARP is disabled for IP Phone %s.  Will not record media for this call.\n",dst_ip);
			}
			*/

			// Get the remote IP Address
			struct ip_addr RemoteIP;
			char remoteipstring[MAX_ASCII_ADDR_LEN];
			int ipretval;
			ipretval = ip_addr_init(&RemoteIP, AF_INET, startmediatransmission->RemoteIPBytes);
			ip_addr_ntoa(&RemoteIP, remoteipstring);
		
			handlerstartmediatransmission(&first_skinnycall, startmediatransmission->ConferenceID, startmediatransmission->PassThruPartyID,dst_ip, remoteipstring,startmediatransmission->RemotePort);

			if(garpdb_yes == 1) {
				int targetretval;
				targetretval = targetlookup(&first_skinnycall, startmediatransmission->PassThruPartyID, dst_ip);
			}
		}

	}
	
	if (strcmp(messagetype,"BackSpaceReqMessage") == 0) {

		struct BackSpaceReqMessage {
			int LineInstance;
			int CallIdentifier;
		};

		struct BackSpaceReqMessage *backspacereqmessage;
		backspacereqmessage = (struct BackSpaceReqMessage*)(bytes);

		/* for target mode */
		if (target_yes == 1) {
			fprintf(singletarget,"Target hit backspace on keypad\n");
			int flush;
			flush = fflush(singletarget);

		}

		backspacereqhandler(&first_skinnycall, src_ip);

	}

	if (strcmp(messagetype,"CallStateMessage") == 0) {
		
		struct CallStateMessage {
			int CallState;
			int LineInstance;
			int CallIdentifier;
		};

		// Get the call state and call identifier
		struct CallStateMessage *callstatemessage;
		callstatemessage = (struct CallStateMessage*)(bytes);

		// If IP Phone goes offhook in callstate message received from UCM
		if (callstatemessage->CallState == 1) {

			// Print some details on the call
			//printf("%s --> %s:  %d:  CallStateMessage Offhook\n",src_ip,dst_ip,callstatemessage->CallIdentifier);

			// Check to see if skinnycall structure is null
			int retval = isEmpty(first_skinnycall);
			if (retval == 1) {

				skinnyfirstcallinit( &first_skinnycall, dst_ip, callstatemessage->CallIdentifier);

				//printf("Test printing after skinnyfirstcallinit\n");
				//printList(first_skinnycall);
				
			} else {

				// We are looking to see if conferenceid is of connected IP Phone
				// or if this is a new call and first time seen callidentifier
				nodeupdatecallstateoffhook(&first_skinnycall, dst_ip, callstatemessage->CallIdentifier);

				//printf("Test printing after nodeupdatecallstateoffhook\n");
				//printList(first_skinnycall);

			}

			/* Handle target code here */
			if (target_yes == 1) {
				if (callstatemessage->CallState == 1) {

					if(target1.dirname != NULL) {

						fprintf(singletarget,"Target %s (Extension %s, IP %s) went offhook\n",target1.dirname,target1.extension,target1.ip);
						int flush;
						flush = fflush(singletarget);
                                	} else {
                                        	fprintf(singletarget,"Target (Extension %s, IP %s) went offhook\n",target1.extension,target1.ip);
						int flush;
                                        	flush = fflush(singletarget);
					}

					track1.offhook = 1;
					track1.digitcount = 0;
                                }

			}

		} else if (callstatemessage->CallState == 2) {
			if (target_yes == 1) {

				if(target1.dirname != NULL) {
					fprintf(singletarget,"Target %s (Extension %s, IP %s) went onhook\n",target1.dirname,target1.extension,target1.ip);
					int flush;
					flush = fflush(singletarget);
				} else {
					fprintf(singletarget,"Target (Extension %s, IP %s) went offhook\n",target1.extension,target1.ip);
					int flush;
					flush = fflush(singletarget);
				}
			}

		}

	}

	if (strcmp(messagetype,"KeypadButtonMessage") == 0) {

		struct KeypadButtonMessage {
			int KeypadButton;
		};

		struct KeypadButtonMessage *keypadbuttonmessage;
		keypadbuttonmessage = (struct KeypadButtonMessage*)(bytes);

		//printf("IP Phone %s keyed button:  %d\n",src_ip,keypadbuttonmessage->KeypadButton);
		keypadbuttonhandler( &first_skinnycall, src_ip, keypadbuttonmessage->KeypadButton);

		/* target mode handler */
		if (target_yes == 1) {
		
			fprintf(singletarget,"IP Phone keyed button:  %d\n",keypadbuttonmessage->KeypadButton);
			int flush;
			flush = fflush(singletarget);

			if (track1.digitcount == 0) {
				track1.digits[0] = adddigit(keypadbuttonmessage->KeypadButton);
                                track1.digitcount++;
                        } else if (track1.digitcount > 0) {

                                int offset = track1.digitcount;
                                track1.digits[offset] = adddigit(keypadbuttonmessage->KeypadButton);
                                track1.digitcount++;
                        }

		}

	}

	if (strcmp(messagetype,"StopMediaTransmission") == 0) {

		struct StopMediaTransmission {
			int ConferenceID;
			int PassThruPartyID;
		};

		struct StopMediaTransmission *stopmediatransmission;
		stopmediatransmission = (struct StopMediaTransmission*)(bytes);
		
		//printf("%s --> %s:  %d:  StopMediaTransmission\n",src_ip,dst_ip,stopmediatransmission->ConferenceID);
	
	}

	return 1;

}


FUNC_DECODER(dissector_sccp){

	DECLARE_DISP_PTR_END(ptr, end);

	if (PACKET->DATA.len == 0) {
		return NULL;
	}

	//Get source IP
	char dst_ip[MAX_ASCII_ADDR_LEN];
	char src_ip[MAX_ASCII_ADDR_LEN];
	u_char src_mac[MAX_ASCII_ADDR_LEN];
	u_char dst_mac[MAX_ASCII_ADDR_LEN];
	ip_addr_ntoa(&PACKET->L3.dst,dst_ip);
	ip_addr_ntoa(&PACKET->L3.src,src_ip);
	mac_addr_ntoa(&PACKET->L2.dst,dst_mac);
	mac_addr_ntoa(&PACKET->L2.src,src_mac);
	//printf("inside main:  dst_ip:  %s  src_ip:  %s\n",dst_ip,src_ip);
	
	// Skinny packet size received inside of SCCP Decoder
	int total_skinny_length = end - ptr;
	int bytes_to_read;
        if (total_skinny_length != 0) {
		bytes_to_read = end - ptr;
	}


	//Structure for sccp packet
	struct SkinnyPacket {
		int datalength;
		u_char reserved[4];
		u_char messageid[4];
		//u_char *sdata;
	};

	// Loop through reading the skinny packet
	// there might be smaller skinny packets inside of the one decoded by the sccp decoder
	if (total_skinny_length != 0) {
		int bytes_read = 0;
		int pcount = 1;
		
		while ( bytes_read < bytes_to_read ) {

			// Print some debug things 
			//printf("Skinny packet number %d, bytes_read: %d, bytes_to_read: %d\n",pcount,bytes_read,bytes_to_read);

			// Create SkinnyPacket structure	
			struct SkinnyPacket *skinnyp;
			skinnyp = (struct SkinnyPacket*)(ptr + bytes_read);
	
			// Print packet length of Skinny packet
			splength = skinnyp->datalength + 8;
			//printf("skinny packet length: %d\n",splength);
			//printf("skinny packet data length: %d\n",skinnyp->datalength);

			// Print reserved bytes
			//printf("Reserved bytes:  %02x %02x %02x %02x \n",skinnyp->reserved[0],skinnyp->reserved[1],skinnyp->reserved[2],skinnyp->reserved[3]);

			// Decode reserved value, to find version of SCCP (there is a difference between 6.1 and 7.0, at the least)
			if (skinnyp->reserved[0] == 17) {
				sversion = 7;
			} else if (skinnyp->reserved[0] == 0) {
				// If true, first network order byte is 0x00, UCM 6.0 SCCP messaging
				sversion = 6;
			} else if (skinnyp->reserved[0] == 19) {
				// If true, first network order byte is 0x13, UCM 8.0 SCCP messaging
				sversion = 8;
			} else {
				//INSTANT_USER_MSG("Unknown UCM Skinny Messaging version\n");
			}
			
	
			// Print messageid
			//printf("Message ID:  %02x %02x %02x %02x \n",skinnyp->messageid[0],skinnyp->messageid[1],skinnyp->messageid[2],skinnyp->messageid[3]);
		
			// Decode messagetype	
			char *messagetype = checkmessageid(skinnyp->messageid);
			//printf("Message type:  %s\n",messagetype);

			/* Here we are printing out all of the bytes below*/
			/*printf("Print all bytes: ");
			int t = 0;
			for (t = 0; t < total_skinny_length; t++) {
				INSTANT_USER_MSG("%02x  ",ptr[t]);
			}*/

                        /*if(strcmp("Unknown",messagetype)==0){
				if(voipdebug){
					fprintf(sccpfile,"Skinny message type is unknown.  Problem decoding Skinny packet.\n");
					flush = fflush(sccpfile);
				}
                                return NULL;
                        } else {*/

				// Do something with packet based on messagetype
				u_char *bytes = ptr + bytes_read + 12;
				int lprintv = skinnyp->datalength - 4;
				//printf("lprintv value:  %d\n",lprintv);
				int retval;
				retval = stypehandler(messagetype, bytes, lprintv, src_ip, dst_ip, sversion, PACKET);

				// Print bytes if any bytes come after the Message ID
 				/*if (skinnyp->datalength > 4) {

					int a;
					int length_to_print = skinnyp->datalength - 4;
					INSTANT_USER_MSG("length of bytes to print: %d\n",length_to_print);
					int offset = bytes_read + 12;
					for (a = 0; a < length_to_print; a++) {

						INSTANT_USER_MSG("%02x ",ptr[offset]);
						offset++;
					}	
					INSTANT_USER_MSG("\n");

				}*/	

				bytes_read = bytes_read + skinnyp->datalength + 8;
				//printf("Bytes_read after: %d\n\n",bytes_read);
				pcount++;

			//}

		}
	}

	return NULL;
}

int populate_rtp_struct_video(int counter, char *tempip, int vpassthrupartyid, int vrtpport){

	rtp[counter].vport = vrtpport;
	rtp[counter].vfwd_count = 1;
	rtp[counter].vrev_count = 1;
	rtp[counter].vrtp_count = 1;
	rtp[counter].vfwd_seqcount = -1;
	rtp[counter].vrev_seqcount = -1;

	memset(rtp[counter].vfwd_file,'\0',sizeof(rtp[counter].vfwd_file));
	strncpy(rtp[counter].vfwd_file,rtp[counter].fwd_au,sizeof(rtp[counter].vfwd_file));
	strcat(rtp[counter].vfwd_file,"_video");
	memset(rtp[counter].vrev_file,'\0',sizeof(rtp[counter].vrev_file));
    strncpy(rtp[counter].vrev_file,rtp[counter].rev_au,sizeof(rtp[counter].vrev_file));
    strcat(rtp[counter].vrev_file,"_video");

#ifdef HAVE_VIDEO_DECODER

	memset(rtp[counter].vfwd_mp4,'\0',sizeof(rtp[counter].vfwd_mp4));
	strncpy(rtp[counter].vfwd_mp4,rtp[counter].fwd_au,sizeof(rtp[counter].vfwd_mp4));
	strcat(rtp[counter].vfwd_mp4,"_video_test.avi");

	memset(rtp[counter].vfwd_avi,'\0',sizeof(rtp[counter].vfwd_avi));
	strncpy(rtp[counter].vfwd_avi,rtp[counter].fwd_au,sizeof(rtp[counter].vfwd_avi));
	strcat(rtp[counter].vfwd_avi,"_video.avi");

	memset(rtp[counter].vrev_mp4,'\0',sizeof(rtp[counter].vrev_mp4));
	strncpy(rtp[counter].vrev_mp4,rtp[counter].rev_au,sizeof(rtp[counter].vrev_mp4));
	strcat(rtp[counter].vrev_mp4,"_video_test.avi");

	memset(rtp[counter].vrev_avi,'\0',sizeof(rtp[counter].vrev_avi));
    strncpy(rtp[counter].vrev_avi,rtp[counter].rev_au,sizeof(rtp[counter].vrev_avi));
    strcat(rtp[counter].vrev_avi,"_video.avi");

#endif

}

int populate_rtp_struct(int counter, char *tempip, int passthrupartyid, int rtpport) {

	char time_string[12];
	char file_name[164];
	char fwd_au[164];
	char rev_au[164];
	char both_au[164];
	//int offset;
	char *null_string = "NULL";


	bzero(file_name, sizeof(file_name));
	bzero(time_string, sizeof(time_string));
	
	if(counter == -1) {

		//printf("populate rtp structure for %s\n",tempip);
	
		// Adding new entries in RTP structure
			
		rtp[rtp_counter].vfwd_count = 1;
      	rtp[rtp_counter].vrev_count = 1;
       	rtp[rtp_counter].vrtp_count = 1;

		rtp[rtp_counter].fwd_count = 1;
		rtp[rtp_counter].fwd_seqCount = -1;
		rtp[rtp_counter].rev_count = 1;
		rtp[rtp_counter].rev_seqCount = -1;
		rtp[rtp_counter].rtp_count = 1;
		bzero(rtp[rtp_counter].ip_addr,MAX_ASCII_ADDR_LEN);
		memcpy(rtp[rtp_counter].ip_addr,tempip,MAX_ASCII_ADDR_LEN);
		rtp[rtp_counter].port = rtpport;
		rtp[rtp_counter].session_active = 1;	
		rtp[rtp_counter].audioDSPSet = 0;	
		rtp[rtp_counter].audioRTPMAP = NULL;
		rtp[rtp_counter].videoRTPMAP = NULL;
		rtp[rtp_counter].videoFMTP = NULL;
		rtp[rtp_counter].audioCodecType = -1;
		rtp[rtp_counter].liveMonitor = 0;
		rtp[rtp_counter].liveMonitorFwdAudioCtr = 0;
		rtp[rtp_counter].liveMonitorRevAudioCtr = 0;
		rtp[rtp_counter].liveMonitorFwdVideoCtr = 0;
		rtp[rtp_counter].liveMonitorRevVideoCtr = 0;
        rtp[rtp_counter].afwd_data_len = 0;
        rtp[rtp_counter].arev_data_len = 0;
#ifdef HAVE_LIB_VLC

		rtp[rtp_counter].liveMonitorFwdVlcStarted = 0;
		rtp[rtp_counter].liveMonitorRevVlcStarted = 0;

		rtp[rtp_counter].savedFwdSPS = 0;
		rtp[rtp_counter].savedRevSPS = 0;	
	
		memset(rtp[rtp_counter].fwdSPS, '\0', sizeof(rtp[rtp_counter].fwdSPS));
		memset(rtp[rtp_counter].fwdPPS, '\0', sizeof(rtp[rtp_counter].fwdPPS));
		memset(rtp[rtp_counter].revSPS, '\0', sizeof(rtp[rtp_counter].revSPS));
		memset(rtp[rtp_counter].revPPS, '\0', sizeof(rtp[rtp_counter].revPPS));

		rtp[rtp_counter].fwdIDRPtr = NULL;
		rtp[rtp_counter].revIDRPtr = NULL;
		rtp[rtp_counter].fwdCurrentIDRPtr = NULL;
		rtp[rtp_counter].revCurrentIDRPtr = NULL;

		rtp[rtp_counter].fwdIDRCtr = 0;
		rtp[rtp_counter].revIDRCtr = 0;
		rtp[rtp_counter].recvFwdAllIDRPic = 0;
		rtp[rtp_counter].recvRevAllIDRPic = 0;
		
		rtp[rtp_counter].currentFwdPPSCtr = 0;
		rtp[rtp_counter].previousFwdPPSCtr = 0;
		rtp[rtp_counter].currentRevPPSCtr = 0;
		rtp[rtp_counter].previousRevPPSCtr = 0;

		rtp[rtp_counter].liveMonitorSocket = 0;

#endif		
		gettimestring(1, time_string);

		// Create base 'file_name'
		char *temp1 = lookupdialingname(passthrupartyid, &first_skinnycall);
		int compare = strcmp(temp1,null_string);
		if(compare == 0) {
			char tmp_string[64];
			bzero(tmp_string,sizeof(tmp_string));
			char DialingStr[64];
			bzero(DialingStr,sizeof(DialingStr));
			getdialingnumber(passthrupartyid, &first_skinnycall, DialingStr);
			int compareval = strcmp(DialingStr, "Unknown");
			if (compareval != 0) {
				sprintf(tmp_string, "%s",DialingStr);
			} else {
				char *DialingIP = getdialingip(passthrupartyid, &first_skinnycall);
				sprintf(tmp_string, "%s", DialingIP);
			}

			strncpy(file_name,tmp_string,sizeof(file_name));

		} else {
			strncpy(file_name,temp1,sizeof(file_name));
		}

		strcat(file_name,"_Calling_");
		char *temp2 = lookupdialedname(passthrupartyid, &first_skinnycall);
		compare = strcmp(temp2,null_string);
		if(compare == 0) {
			char tmp_string[64];
			bzero(tmp_string,sizeof(tmp_string));
			char DialedStr[64];
			bzero(DialedStr,sizeof(DialedStr));
			getdialednumber(passthrupartyid, &first_skinnycall, DialedStr);
			//INSTANT_USER_MSG("After getdialed number, DialedStr is %s\n",DialedStr);
			if(voipdebug) {
				fprintf(sccpfile,"After getdialednumber(), DialedStr is %s\n",DialedStr);
				flush = fflush(sccpfile);
				//printf("After getdialednumber(), DialedStr is %s\n",DialedStr);
			}
			int compareval = strcmp(DialedStr, "Unknown");
			if (compareval != 0) {
				sprintf(tmp_string,"%s",DialedStr);
			} else {
				char *DialedIP = getdialedip(passthrupartyid, &first_skinnycall);
				sprintf(tmp_string, "%s", DialedIP);
			}
			strcat(file_name,tmp_string);

		} else {
			strcat(file_name,temp2);
		}
		strcat(file_name,"_");
		strcat(file_name,time_string);
		// End of create base 'file_name'

		// initialize the forward media file
		int fwd_retval = check_fwd_media_garp(&first_skinnycall, tempip, passthrupartyid);
		if ((fwd_retval == 1)||(monitor_yes == 1)||(target_yes == 1)) {
			//printf("Creating forward media file for IP %s\n",tempip);
			bzero(rtp[rtp_counter].fwd_au, sizeof(rtp[rtp_counter].fwd_au));
			strncpy(fwd_au,file_name,sizeof(fwd_au));
			strcat(fwd_au,"_forward");
			strncpy(rtp[rtp_counter].fwd_au,fwd_au,sizeof(rtp[rtp_counter].fwd_au));
			strncpy(rtp[rtp_counter].fwd_mp3,fwd_au,sizeof(rtp[rtp_counter].fwd_mp3));
			strcat(rtp[rtp_counter].fwd_mp3,".mp3");
		} else {
			INSTANT_USER_MSG("Not creating forward media file for IP %s\n",tempip);
		}
		
		// initialize the reverse media file
		int rev_retval = check_rev_media_garp(&first_skinnycall, tempip, passthrupartyid);
		if ((rev_retval == 1)||(monitor_yes == 1)||(target_yes == 1)) {
			//printf("Creating reverse media file for IP %s\n",tempip);
			bzero(rtp[rtp_counter].rev_au, sizeof(rtp[rtp_counter].rev_au));
			strncpy(rev_au,file_name,sizeof(rev_au));
			strcat(rev_au,"_reverse");
			strncpy(rtp[rtp_counter].rev_au,rev_au,sizeof(rtp[rtp_counter].rev_au));
			strncpy(rtp[rtp_counter].rev_mp3,rev_au,sizeof(rtp[rtp_counter].rev_mp3));
                        strcat(rtp[rtp_counter].rev_mp3,".mp3");
		} else {
			INSTANT_USER_MSG("Not creating reverse media file for IP %s\n",tempip);
		}

		// initialize the bi-directional media file
		//printf("fwd_retval:  %d, rev_retval:  %d\n",fwd_retval,rev_retval);
		if ((fwd_retval == 1)&&(rev_retval == 1)||(monitor_yes == 1)) {
			set_both_good(passthrupartyid, &first_skinnycall);
			bzero(rtp[rtp_counter].both_au, sizeof(rtp[rtp_counter].both_au));
			strncpy(both_au,file_name,sizeof(both_au));
			strcat(both_au,"_both");
			strncpy(rtp[rtp_counter].both_au,both_au,sizeof(rtp[rtp_counter].both_au));
			//printf("Creating both au file of %s\n",rtp[rtp_counter].both_au);
			strncpy(rtp[rtp_counter].both_mp3,both_au,sizeof(rtp[rtp_counter].both_mp3));
                        strcat(rtp[rtp_counter].both_mp3,".mp3");
		} else if ((fwd_retval == 1)&&(rev_retval == 0)) {
			INSTANT_USER_MSG("Only creating forward media file of '%s' for IP Phone %s\n",
					rtp[rtp_counter].fwd_au,
					tempip);
			INSTANT_USER_MSG("Not creating media files because GARP is disabled on reverse direction IP Phone\n");  
		} else if (target_yes == 1) {
			set_both_good(passthrupartyid, &first_skinnycall);
			bzero(rtp[rtp_counter].both_au, sizeof(rtp[rtp_counter].both_au));
			strncpy(both_au,file_name,sizeof(both_au));
			strcat(both_au,"_both");
			strncpy(rtp[rtp_counter].both_au,both_au,sizeof(rtp[rtp_counter].both_au));
			strncpy(rtp[rtp_counter].both_mp3,both_au,sizeof(rtp[rtp_counter].both_mp3));
                        strcat(rtp[rtp_counter].both_mp3,".mp3");
			//printf("Creating both au file of %s\n",rtp[rtp_counter].both_au);
		} else {
			/*printf("Only creating reverse media file of '%s' for IP Phone %s\n",
					rtp[rtp_counter].rev_au,
					tempip);*/
		}


		// initialize the pcap file 
		bzero(rtp[rtp_counter].pcap_filename,sizeof(rtp[rtp_counter].pcap_filename));
		strcat(file_name,".pcap");
		strncpy(rtp[rtp_counter].pcap_filename,file_name,sizeof(rtp[rtp_counter].pcap_filename));
		//printf("init pcap file index %d\n",rtp_counter);
		
		
		// initialize the pcap handle
		rtp[rtp_counter].dp = pcap_open_dead(DLT_EN10MB,8192);
		if(rtp[rtp_counter].dp == NULL) {
			INSTANT_USER_MSG("Error creating pcap handle\n");
			return -1;
		} else {
			//printf("init pcap handle index %d\n",rtp_counter);

		}

		// initialize the pcap dumper
		rtp[rtp_counter].dumper = pcap_dump_open(rtp[rtp_counter].dp,rtp[rtp_counter].pcap_filename);
		if(rtp[rtp_counter].dumper == NULL) {
			INSTANT_USER_MSG("Error opening pcap dump file\n");
			return -1;
		} else {
			//printf("init pcap dumper index %d\n",rtp_counter);

		}

		++rtp_counter;

	} else {
		
		//printf("populate rtp structure for %s\n",tempip);
		// Replace old entries with new entries

		rtp[counter].vfwd_count = 1;
        rtp[counter].vrev_count = 1;
        rtp[counter].vrtp_count = 1;
		rtp[counter].fwd_count = 1;
		rtp[counter].fwd_seqCount = -1;
		rtp[counter].rev_count = 1;
		rtp[counter].rev_seqCount = -1;
		rtp[counter].rtp_count = 1;
		rtp[counter].port = rtpport;
		rtp[counter].session_active = 1;
		rtp[counter].audioDSPSet = 0;	
		rtp[counter].audioRTPMAP = NULL;
		rtp[counter].videoRTPMAP = NULL;
		rtp[counter].videoFMTP = NULL;
		rtp[counter].audioCodecType = -1;

		rtp[counter].liveMonitor = 0;
		rtp[counter].liveMonitorFwdAudioCtr = 0;
		rtp[counter].liveMonitorRevAudioCtr = 0;
		rtp[counter].liveMonitorFwdVideoCtr = 0;
		rtp[counter].liveMonitorRevVideoCtr = 0;

#ifdef HAVE_LIB_VLC

		rtp[counter].liveMonitorFwdVlcStarted = 0;
		rtp[counter].liveMonitorRevVlcStarted = 0;

		rtp[counter].savedFwdSPS = 0;
		rtp[counter].savedRevSPS = 0;	
	
		memset(	rtp[counter].fwdSPS, '\0', sizeof(rtp[rtp_counter].fwdSPS));
		memset(	rtp[counter].fwdPPS, '\0', sizeof(rtp[rtp_counter].fwdPPS));
		memset(	rtp[counter].revSPS, '\0', sizeof(rtp[rtp_counter].revSPS));
		memset(	rtp[counter].revPPS, '\0', sizeof(rtp[rtp_counter].revPPS));

		rtp[counter].fwdIDRPtr = NULL;
		rtp[counter].revIDRPtr = NULL;
		rtp[counter].fwdCurrentIDRPtr = NULL;
		rtp[counter].revCurrentIDRPtr = NULL;

		rtp[counter].fwdIDRCtr = 0;
		rtp[counter].revIDRCtr = 0;
		rtp[counter].recvFwdAllIDRPic = 0;
		rtp[counter].recvRevAllIDRPic = 0;
		
		rtp[counter].currentFwdPPSCtr = 0;
		rtp[counter].previousFwdPPSCtr = 0;
		rtp[counter].currentRevPPSCtr = 0;
		rtp[counter].previousRevPPSCtr = 0;
		rtp[counter].liveMonitorSocket = 0;

#endif
		gettimestring(1,time_string);

		// Create base 'file_name'
		char *temp1 = lookupdialingname(passthrupartyid, &first_skinnycall);
		int compare = strcmp(temp1,null_string);
		if(compare == 0) {
			char tmp_string[64];
			char DialingStr[64];
			bzero(tmp_string,sizeof(tmp_string));
			bzero(DialingStr,sizeof(DialingStr));
			getdialingnumber(passthrupartyid, &first_skinnycall, DialingStr);
			sprintf(tmp_string,"%s",DialingStr);
			strncpy(file_name,tmp_string,sizeof(file_name));
		} else {
			strncpy(file_name,temp1,sizeof(file_name));
		}

		strcat(file_name,"_Calling_");
		char *temp2 = lookupdialedname(passthrupartyid, &first_skinnycall);
		compare = strcmp(temp2,null_string);
		if(compare == 0) {

			char tmp_string[64];
			bzero(tmp_string,sizeof(tmp_string));
			char str[64];
			bzero(str,sizeof(str));
			getdialednumber(passthrupartyid, &first_skinnycall, str);
			sprintf(tmp_string, "%s", str);
			strcat(file_name,tmp_string);
		} else {
			strcat(file_name,temp2);
		}
		strcat(file_name,"-");
		strcat(file_name,time_string);
		// End of create base 'file_name'
		
		// initialize the forward media file
		int fwd_retval = check_fwd_media_garp(&first_skinnycall, tempip, passthrupartyid);
		if ((fwd_retval == 1)||(monitor_yes == 1)||(target_yes == 1)) {
			//printf("Creating forward media file for IP %s\n",tempip);
			bzero(rtp[counter].fwd_au,sizeof(rtp[counter].fwd_au));
			strncpy(fwd_au,file_name,sizeof(fwd_au));
			strcat(fwd_au,"_forward");
			strncpy(rtp[counter].fwd_au,fwd_au,sizeof(rtp[rtp_counter].fwd_au));
			strncpy(rtp[rtp_counter].fwd_mp3,fwd_au,sizeof(rtp[rtp_counter].fwd_mp3));
                        strcat(rtp[rtp_counter].fwd_mp3,".mp3");
		} else {

		}

		// initialize the reverse media file
		int rev_retval = check_rev_media_garp(&first_skinnycall, tempip, passthrupartyid);
		if ((rev_retval == 1)||(monitor_yes == 1)||(target_yes == 1)) {
			bzero(rtp[counter].rev_au,sizeof(rtp[counter].rev_au));
			strncpy(rev_au,file_name,sizeof(fwd_au));
			strcat(rev_au,"_reverse");
			strncpy(rtp[counter].rev_au,rev_au,sizeof(rtp[rtp_counter].rev_au));
			strncpy(rtp[rtp_counter].rev_mp3,rev_au,sizeof(rtp[rtp_counter].rev_mp3));
                        strcat(rtp[rtp_counter].rev_mp3,".mp3");
		} else {
			//printf("Not creating reverse media file for IP %s\n",tempip);
		}
		
		if ((fwd_retval == 1)&&(rev_retval == 1)||(monitor_yes == 1)) {
			set_both_good(passthrupartyid, &first_skinnycall);
			bzero(rtp[counter].both_au,sizeof(rtp[counter].both_au));
			strncpy(both_au,file_name,sizeof(both_au));
			strcat(both_au,"_both");
			strncpy(rtp[counter].both_au,both_au,sizeof(rtp[rtp_counter].both_au));
			strncpy(rtp[rtp_counter].both_mp3,both_au,sizeof(rtp[rtp_counter].both_mp3));
                        strcat(rtp[rtp_counter].both_mp3,".mp3");
			//printf("Creating both au file of %s\n",rtp[counter].both_au);
		} else if ((fwd_retval == 1)&&(rev_retval == 0)) {
			/*printf("Only creating forward media file of '%s' for IP Phone %s\n",
					rtp[counter].fwd_au,
					tempip);*/
			INSTANT_USER_MSG("Not creating media files because GARP is disabled on reverse direction IP Phone\n");  
		} else if (target_yes == 1){
			bzero(rtp[counter].rev_au,sizeof(rtp[counter].rev_au));
			strncpy(rev_au,file_name,sizeof(fwd_au));
			strcat(rev_au,"_reverse");
			strncpy(rtp[counter].rev_au,rev_au,sizeof(rtp[rtp_counter].rev_au));
			strncpy(rtp[rtp_counter].rev_mp3,rev_au,sizeof(rtp[rtp_counter].rev_mp3));
                        strcat(rtp[rtp_counter].rev_mp3,".mp3");
		} else {

		}
		
		// initialize the pcap file 
		bzero(rtp[counter].pcap_filename,sizeof(rtp[counter].pcap_filename));
		strcat(file_name,".pcap");
		strncpy(rtp[counter].pcap_filename,file_name,sizeof(rtp[counter].pcap_filename));
		//printf("init pcap file index %d\n",counter);

		// initialize the pcap handle
		rtp[counter].dp = pcap_open_dead(DLT_EN10MB,8192);
		if(rtp[counter].dp == NULL) {
			INSTANT_USER_MSG("Error creating pcap handle\n");
			return -1;
		} else {
			//printf("init pcap handle index %d\n",counter);

		}

		// initialize the pcap dumper
		rtp[counter].dumper = pcap_dump_open(rtp[counter].dp,rtp[counter].pcap_filename);
		if(rtp[counter].dumper == NULL) {
			INSTANT_USER_MSG("Error opening pcap dump file\n");
			return -1;
		} else {
			//printf("init pcap dumper index %d\n",counter);

		}


	}

	return 0;

}

/*
int check_rtp_hdr(char *ip_addr, int media_port) {

	int i = 0;
	for( i = 0; i < rtp_counter; i++ ) {
		
		if((strcmp(ip_addr,rtp[i].ip_addr) == 0) && (rtp[i].port == media_port)) {
			return i;
		}
	}

	return -1;
}
*/

/*
int check_rtp_entry(char *ip_addr) {

	int i = 0;

	for(i = 0; i < rtp_counter; i++) {
		
		if(strcmp(ip_addr,rtp[i].ip_addr) == 0) {
			return i;
		}
	}
	return -1;
}
*/

FUNC_DECODER(dissector_rtp_skinny){

	DECLARE_DISP_PTR_END(ptr, end);
	char srcip[MAX_ASCII_ADDR_LEN];
	char dstip[MAX_ASCII_ADDR_LEN];
	//char errbuf[PCAP_ERRBUF_SIZE];
	int rtp_ptr = 0;
	//pcap_dumper_t *dumper;
	//pcap_t *dp;
	//struct pcap_pkthdr *h = NULL;
	struct sniff_rtp *rtp_hdr;
	int fwd_direction = 0;
	int rev_direction = 0;
	int vfwd_direction = 0;
	int vrev_direction = 0;
	u_char *sample;
	u_char payload_mask;
	char pd[4];
	int video_yes;
	int payload_type_number = 97;
	char *videoPayloadType = (char)payload_type_number;
	int returnMonitor = 0;

	ip_addr_ntoa(&PACKET->L3.src, srcip);
	ip_addr_ntoa(&PACKET->L3.dst, dstip);

	//printf("%s --> %s RTP Packet:  Inside dissector_rtp\n",srcip,dstip);

	int return_src = check_rtp_hdr(srcip,ntohs(PACKET->L4.src),&video_yes);
	//printf("return_src:  %d for ip %s\n",return_src,srcip);

	if(return_src < 0) {

		int return_dst = check_rtp_hdr(dstip,ntohs(PACKET->L4.dst),&video_yes);
		//printf("return_dst:  %d for ip %s\n",return_dst,dstip);
		if(return_dst < 0) {
			return NULL;
		}
		rtp_ptr = return_dst;
		if(video_yes == 1)
			vrev_direction = 1;
		else							
			rev_direction = 1;

	} else {
		if(video_yes == 1)
			vfwd_direction = 1;
		else
			fwd_direction = 1;
		rtp_ptr = return_src;

	}

	if(rtp[rtp_ptr].session_active != 1){
		return NULL;
	}

	u_char *rtpPayload = PACKET->DATA.data + sizeof(struct sniff_rtp);/* Pointer to RTP Payload */ 
	int payloadLength = PACKET->DATA.len - sizeof(struct sniff_rtp);	/* RTP payload length */
	sample = PACKET->DATA.data;
	rtp_hdr = (struct sniff_rtp *) sample;

	payload_mask = rtp_hdr -> payloadType & 0x7F;
	rtp[rtp_ptr].videoPayloadType = videoPayloadType;

	if(payload_mask == 0x12){
		/* G729 */
#ifndef OS_WINDOWS
#ifndef ARCH_X64

		if (rtp[rtp_ptr].rtp_count == 1) {

			strcat(rtp[rtp_ptr].fwd_au,".wav");
			strcat(rtp[rtp_ptr].rev_au,".wav");
			strcat(rtp[rtp_ptr].both_au,".wav");
			if( rtp[rtp_ptr].audioCodecType == -1)		
				rtp[rtp_ptr].audioCodecType = COG729;

		}
		if(rtp[rtp_ptr].audioCodecType != COG729)
			return -1;
		
		short *g729Synth;
       	int synthLength = 0, g729_ret = 0;

		if(rtp[rtp_ptr].fwd_count == 1 && fwd_direction == 1){

			int g729_init_ret = initialize_g729_decoder(&(rtp[rtp_ptr].hDecoderFwd));
            		if(g729_init_ret < 0){
            			return -1;
            		}

            rtp[rtp_ptr].fwd_fp = fopen(rtp[rtp_ptr].fwd_au,"w+|b");
            if(rtp[rtp_ptr].fwd_fp == NULL) {
                INSTANT_USER_MSG("Error opening forward .au file \n");
                return - 1;
            }


            /* Formula for creating the WAV file header values, the wav file header values have to be in little-endian format
                        unsigned short NumChannels = 1;
                        unsigned short bitsPerSample = 16;
                        unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
            unsigned int sampleRate = 8000;
                        unsigned int avgBytesPerSecond = sampleRate * blockAlign;

            phtonl(pd,0x52494646);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            phtonl(pd,2048);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            phtonl(pd,0x57415645);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            phtonl(pd,0x666d7420);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            int val = 0x00000010;
                        memcpy(pd,&val,sizeof(int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
            val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
            memcpy(pd,&sampleRate,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            memcpy(pd,&blockAlign,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
            memcpy(pd,&bitsPerSample,sizeof(unsigned int));

			fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
            phtonl(pd,0x64617461);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
            */
            create_wav_header(rtp[rtp_ptr].fwd_fp, 1, 16, 8000, 0x0001, 20480000);            
        }

		if(rtp[rtp_ptr].rev_count == 1 && rev_direction == 1){

			int g729_init_ret = initialize_g729_decoder(&(rtp[rtp_ptr].hDecoderRev));
            if(g729_init_ret < 0){
                return -1;
            }

            rtp[rtp_ptr].rev_fp = fopen(rtp[rtp_ptr].rev_au,"w+|b");
            if(rtp[rtp_ptr].rev_fp == NULL) {
                INSTANT_USER_MSG("Error opening reverse .au file\n");
                return -1;
            }

            /* Formula for creating the WAV file header values, the wav file header values have to be in little-endian format
            unsigned short NumChannels = 1;
            unsigned short bitsPerSample = 16;
            unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
            unsigned int sampleRate = 8000;
            unsigned int avgBytesPerSecond = sampleRate * blockAlign;

            phtonl(pd,0x52494646);
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            phtonl(pd,2048);
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            phtonl(pd,0x57415645);
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            phtonl(pd,0x666d7420);
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            int val = 0x00000010;
            memcpy(pd,&val,sizeof(int));
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            val = 0x0001;
            memcpy(pd,&val,sizeof(short));
            fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
            val = 0x0001;
            memcpy(pd,&val,sizeof(short));
            fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
            memcpy(pd,&sampleRate,sizeof(unsigned int));
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
            memcpy(pd,&blockAlign,sizeof(unsigned int));
            fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
            memcpy(pd,&bitsPerSample,sizeof(unsigned int));
            fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
			phtonl(pd,0x64617461);
            fwrite(pd,1,4,rtp[rtp_ptr].rev_fp); */
            create_wav_header(rtp[rtp_ptr].rev_fp, 1, 16, 8000, 0x0001, 20480000);            

        }	

		if(fwd_direction == 1){

			if(rtp[rtp_ptr].fwd_seqCount == rtp_hdr -> sequence_no) {
                //fprintf(stdout, "[+] fwd seq:  %d, rtp_hdr seq: %d:  Skipping duplicate rtp packets ..........\n",rtp[rtp_ptr].fwd_seqCount,rtp_hdr->sequence_no);
                return NULL;
            }
	
			++rtp[rtp_ptr].fwd_count;
	
			g729_ret = decode_payload_g729(PACKET->DATA.data + sizeof(struct sniff_rtp), PACKET->DATA.len - sizeof(struct sniff_rtp), &g729Synth, &synthLength, rtp[rtp_ptr].hDecoderFwd);
            if(g729_ret <0)
            	return NULL;
            fwrite(g729Synth, sizeof(short), synthLength, rtp[rtp_ptr].fwd_fp);
            rtp[rtp_ptr].afwd_data_len += (sizeof(short) * synthLength);
#ifndef OS_WINDOWS
              	returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], FWDAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
  	        if(returnMonitor == 1){
    		        playDSP(g729Synth, synthLength * sizeof(short), FWDAUDIOFT);
       		}
#endif	
            rtp[rtp_ptr].fwd_seqCount = rtp_hdr ->sequence_no;
            fwd_direction = 0;
		}
		else if(rev_direction == 1){

			if(rtp[rtp_ptr].rev_seqCount == rtp_hdr -> sequence_no) {
                //fprintf(stdout, "[+] fwd seq:  %d, rtp_hdr seq: %d:  Skipping duplicate rtp packets ..........\n",rtp[rtp_ptr].fwd_seqCount,rtp_hdr->sequence_no);
                return NULL;
            }

            ++rtp[rtp_ptr].rev_count;
   
            g729_ret = decode_payload_g729(PACKET->DATA.data + sizeof(struct sniff_rtp), PACKET->DATA.len - sizeof(struct sniff_rtp), &g729Synth, &synthLength, rtp[rtp_ptr].hDecoderRev);
            if(g729_ret <0)
                return NULL;
            fwrite(g729Synth, sizeof(short), synthLength, rtp[rtp_ptr].rev_fp);
            rtp[rtp_ptr].arev_data_len += (sizeof(short) * synthLength);
            
#ifndef OS_WINDOWS
		returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], REVAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
	        if(returnMonitor == 1){
		        playDSP(g729Synth, synthLength * sizeof(short), REVAUDIOFT);
       		}
#endif	

            rtp[rtp_ptr].rev_seqCount = rtp_hdr ->sequence_no;
            rev_direction = 0;		

		}			
#endif
#endif

	} // G729
	else if(payload_mask == 0x09) {

		//printf("rtp count is %d\n",rtp[rtp_ptr].rtp_count);
		if (rtp[rtp_ptr].rtp_count == 1) {

			strcat(rtp[rtp_ptr].fwd_au,".wav");
			strcat(rtp[rtp_ptr].rev_au,".wav");
			strcat(rtp[rtp_ptr].both_au,".wav");

			//printf("Created *.wav files\n");
			if( rtp[rtp_ptr].audioCodecType == -1)		
			rtp[rtp_ptr].audioCodecType = COG722;
		
			/* Initialize the G722 decoder */
                        g722_decode_init(&rtp[rtp_ptr].dec_state,64000,G722_PACKED | G722_SAMPLE_RATE_8000);
                        g722_decode_init(&rtp[rtp_ptr].dec_state1,64000,G722_PACKED | G722_SAMPLE_RATE_8000);	
		}
		if(rtp[rtp_ptr].audioCodecType != COG722)
			return -1;
		//++rtp[rtp_ptr].rtp_count;

		if(rtp[rtp_ptr].fwd_count == 1 && fwd_direction == 1){
	
			rtp[rtp_ptr].fwd_fp = fopen(rtp[rtp_ptr].fwd_au,"w+|b");
			if(rtp[rtp_ptr].fwd_fp == NULL) {
				INSTANT_USER_MSG("Error opening forward .au file \n");
				return - 1;
			}


			/* Formula for creating the WAV file header values, the wav file header values have to be in little-endian format
                        unsigned short NumChannels = 1;
                        unsigned short bitsPerSample = 16;
                        unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
			unsigned int sampleRate = 8000;
                        unsigned int avgBytesPerSecond = sampleRate * blockAlign;

			phtonl(pd,0x52494646);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			phtonl(pd,2048);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			phtonl(pd,0x57415645);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			phtonl(pd,0x666d7420);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			int val = 0x00000010;
                        memcpy(pd,&val,sizeof(int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);		
			val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
			val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
			memcpy(pd,&sampleRate,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			memcpy(pd,&blockAlign,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
			memcpy(pd,&bitsPerSample,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
			phtonl(pd,0x64617461);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp); */
            create_wav_header(rtp[rtp_ptr].fwd_fp, 1, 16, 8000, 0x0001, 20480000);                                    
		}
					
		
								
		if(rtp[rtp_ptr].rev_count == 1 && rev_direction == 1){

			rtp[rtp_ptr].rev_fp = fopen(rtp[rtp_ptr].rev_au,"w+|b");
			if(rtp[rtp_ptr].rev_fp == NULL) {
				INSTANT_USER_MSG("Error opening reverse .au file\n");
				return -1;
			}

			/* Formula for creating the WAV file header values, the wav file header values have to be in little-endian format
			unsigned short NumChannels = 1;
			unsigned short bitsPerSample = 16;
			unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
			unsigned int sampleRate = 8000;
			unsigned int avgBytesPerSecond = sampleRate * blockAlign;

			phtonl(pd,0x52494646);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,2048);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,0x57415645);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,0x666d7420);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			int val = 0x00000010;
			memcpy(pd,&val,sizeof(int));
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			val = 0x0001;
			memcpy(pd,&val,sizeof(short));
			fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
			val = 0x0001;
			memcpy(pd,&val,sizeof(short));
			fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
			memcpy(pd,&sampleRate,sizeof(unsigned int));
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int)); 
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			memcpy(pd,&blockAlign,sizeof(unsigned int));
			fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
			memcpy(pd,&bitsPerSample,sizeof(unsigned int));
			fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
			phtonl(pd,0x64617461); 
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp); */
            create_wav_header(rtp[rtp_ptr].rev_fp, 1, 16, 8000, 0x0001, 20480000);			
		}


		if (fwd_direction == 1) {

			/* Decodes the received encoded payload and writes it to the wav file,
			 * the decoder is obtained from libspandsp (http://soft-switch.org/spandsp-doc/ 
			 */

			if(rtp[rtp_ptr].fwd_seqCount == rtp_hdr -> sequence_no) {
				//fprintf(stdout, "[+] fwd seq:  %d, rtp_hdr seq: %d:  Skipping duplicate rtp packets ..........\n",rtp[rtp_ptr].fwd_seqCount,rtp_hdr->sequence_no);
				return NULL;
			}
			
			int16_t *outdata;

			outdata = malloc((PACKET->DATA.len - sizeof(struct sniff_rtp)) * sizeof(int16_t));
			//int len1 = g722_decode(&rtp[rtp_ptr].dec_state, outdata, (const uint8_t *)PACKET->packet + 54, (PACKET->DATA.len - sizeof(struct sniff_rtp)));
			int len1;
			len1 = g722_decode(&rtp[rtp_ptr].dec_state, outdata, (const uint8_t *)PACKET->DATA.data + sizeof(struct sniff_rtp), (PACKET->DATA.len - sizeof(struct sniff_rtp)));
			fwrite(outdata,sizeof(int16_t),(PACKET->DATA.len - sizeof(struct sniff_rtp)),rtp[rtp_ptr].fwd_fp);
            rtp[rtp_ptr].afwd_data_len += (sizeof(int16_t) * (PACKET->DATA.len - sizeof(struct sniff_rtp)) );			
#ifndef OS_WINDOWS
			//printf("Just before  entering livemonitor handler \n");
                       	returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], FWDAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
                       	if(returnMonitor == 1){
                           	playDSP(outdata, (payloadLength * sizeof(int16_t)), FWDAUDIOFT);
                       	}
#endif	
			free(outdata);

			rtp[rtp_ptr].fwd_seqCount = rtp_hdr ->sequence_no;
			++rtp[rtp_ptr].fwd_count;
			fwd_direction = 0;

		} else if (rev_direction == 1) {

			if(rtp[rtp_ptr].rev_seqCount == rtp_hdr -> sequence_no) {
				//fprintf(stdout, "[+] rev seq:  %d, rtp_hdr seq: %d:  Skipping duplicate rtp packets ..........\n",rtp[rtp_ptr].rev_seqCount,rtp_hdr->sequence_no);
				return NULL;
			}

			int16_t *outdata;

			outdata = malloc((PACKET->DATA.len - sizeof(struct sniff_rtp)) * sizeof(int16_t));
			//int len1 = g722_decode(&rtp[rtp_ptr].dec_state1, outdata, (const uint8_t *)PACKET->packet + 54, (PACKET->DATA.len - sizeof(struct sniff_rtp)));
			int len1;
			len1 = g722_decode(&rtp[rtp_ptr].dec_state1, outdata, (const uint8_t *)PACKET->DATA.data + sizeof(struct sniff_rtp), (PACKET->DATA.len - sizeof(struct sniff_rtp)));
			fwrite(outdata,sizeof(int16_t),(PACKET->DATA.len - sizeof(struct sniff_rtp)),rtp[rtp_ptr].rev_fp);
            rtp[rtp_ptr].arev_data_len += (sizeof(int16_t) * (PACKET->DATA.len - sizeof(struct sniff_rtp)) );			
			
#ifndef OS_WINDOWS
                       	returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], REVAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
                       	if(returnMonitor == 1){
                           	playDSP(outdata, (payloadLength * sizeof(int16_t)), REVAUDIOFT);
                       	}
#endif	
			rtp[rtp_ptr].rev_seqCount = rtp_hdr->sequence_no;
			++rtp[rtp_ptr].rev_count;

			rev_direction = 0;

		}
		++rtp[rtp_ptr].rtp_count;

		//printf("Incremented rtp_count to %d\n",rtp[rtp_ptr].rtp_count);

	} else if (payload_mask == 0x00) {


		//printf("G.711 packet\n");
		
		if(rtp[rtp_ptr].rtp_count == 1){

			strcat(rtp[rtp_ptr].fwd_au,".wav");
                        strcat(rtp[rtp_ptr].rev_au,".wav");
                        strcat(rtp[rtp_ptr].both_au,".wav");
			if( rtp[rtp_ptr].audioCodecType == -1)		
				rtp[rtp_ptr].audioCodecType = COG711MULAW;
		}
		if(rtp[rtp_ptr].audioCodecType != COG711MULAW)
			return -1;
		//printf("Testing fwd_count is %d, fwd_direction is %d\n",rtp[rtp_ptr].fwd_count,fwd_direction);
		if(rtp[rtp_ptr].fwd_count == 1 && fwd_direction == 1){

			rtp[rtp_ptr].fwd_fp = fopen(rtp[rtp_ptr].fwd_au,"w+|b");
                        if(rtp[rtp_ptr].fwd_fp == NULL) {
                        	INSTANT_USER_MSG("Error opening forward .au file \n");
                                return -1;
                        } else {
                               //printf("Created forward au file\n");
                        }
/*
			unsigned short NumChannels = 1;
                        unsigned short bitsPerSample = 8;
                        unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
                        unsigned int sampleRate = 8000;
                        unsigned int avgBytesPerSecond = sampleRate * blockAlign;

                        phtonl(pd,0x52494646);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,2048);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,0x57415645);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,0x666d7420);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        int val = 0x00000010;
                        memcpy(pd,&val,sizeof(int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        val = 0x0007;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
                        val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
                        memcpy(pd,&sampleRate,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        memcpy(pd,&blockAlign,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
                        memcpy(pd,&bitsPerSample,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,0x64617461);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
 */
            create_wav_header(rtp[rtp_ptr].fwd_fp, 1, 16, 8000, 0x0001, 20480000);
			/*
		        phtonl(pd,0x2e736e64);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,24);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,-1);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,1);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,8000);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
                        phtonl(pd,1);
                        fwrite(pd,1,4,rtp[rtp_ptr].fwd_fp);
			*/
		}

		if(rtp[rtp_ptr].rev_count == 1 && rev_direction == 1){

			rtp[rtp_ptr].rev_fp = fopen(rtp[rtp_ptr].rev_au,"w+|b");
			if(rtp[rtp_ptr].rev_fp == NULL) {
				INSTANT_USER_MSG("Error opening reverse .au file \n");
				return -1;
			} else {
				//printf("Created reverse au file\n");
			}
/*
			unsigned short NumChannels = 1;
                        unsigned short bitsPerSample = 8;
                        unsigned short blockAlign = bitsPerSample / 8 * NumChannels;
                        unsigned int sampleRate = 8000;
                        unsigned int avgBytesPerSecond = sampleRate * blockAlign;

                        phtonl(pd,0x52494646);
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        phtonl(pd,2048);
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        phtonl(pd,0x57415645);
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        phtonl(pd,0x666d7420);
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        int val = 0x00000010;
                        memcpy(pd,&val,sizeof(int));
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        val = 0x0007;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
                        val = 0x0001;
                        memcpy(pd,&val,sizeof(short));
                        fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
                        memcpy(pd,&sampleRate,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
                        memcpy(pd,&blockAlign,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
                        memcpy(pd,&bitsPerSample,sizeof(unsigned int));
                        fwrite(pd,1,2,rtp[rtp_ptr].rev_fp);
                        phtonl(pd,0x64617461);
                        fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);		
 */
            create_wav_header(rtp[rtp_ptr].rev_fp, 1, 16, 8000, 0x0001, 20480000);            
 
			/*
			phtonl(pd,0x2e736e64);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,24);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,-1);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,1);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,8000);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			phtonl(pd,1);
			fwrite(pd,1,4,rtp[rtp_ptr].rev_fp);
			*/
		}
	
		if (fwd_direction == 1) {


			if(rtp[rtp_ptr].fwd_seqCount == rtp_hdr -> sequence_no) {
				return NULL;
			}
			int16_t *pcm711mulaw = (int16_t *) calloc(payloadLength, sizeof(int16_t));
							if(pcm711mulaw == NULL){
								INSTANT_USER_MSG("Error: Allocating memory for mulaw to pcm conversion \n");
								//SIPRTPLOG("Error: Allocating memory for mulaw to pcm conversion \n");
								return -1;
							}
							u_char *rtpData = rtpPayload;
							unsigned int i = 0;
							for(i = 0; i < payloadLength; i++){
								//pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
								pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
							}
		
#ifndef OS_WINDOWS
			//printf("Just before  entering livemonitor handler \n");
                     returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], FWDAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
                    	if(returnMonitor == 1){
 	                      	//playDSP((PACKET->DATA.data + sizeof(struct sniff_rtp)), ((PACKET->DATA.len - sizeof(struct sniff_rtp)) * sizeof(u_char)), FWDAUDIOFT);
 	                      	playDSP(pcm711mulaw, (payloadLength * sizeof(int16_t)), FWDAUDIOFT);
                 	}
#endif	

			//fwrite(PACKET->packet + 54, sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].fwd_fp);
			fwrite(PACKET->DATA.data + sizeof(struct sniff_rtp), sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].fwd_fp);
            rtp[rtp_ptr].afwd_data_len += (sizeof(u_char) * (PACKET->DATA.len - sizeof(struct sniff_rtp)) );						
			++rtp[rtp_ptr].fwd_count;
			fwd_direction = 0;
			rtp[rtp_ptr].fwd_seqCount = rtp_hdr -> sequence_no;
			++rtp[rtp_ptr].fwd_count;
			free(pcm711mulaw);

		} else if (rev_direction == 1) {

			if (rtp[rtp_ptr].rev_seqCount == rtp_hdr -> sequence_no){
				return NULL;
			}

int16_t *pcm711mulaw = (int16_t *) calloc(payloadLength, sizeof(int16_t));
                            if(pcm711mulaw == NULL){
                                INSTANT_USER_MSG("Error: Allocating memory for mulaw to pcm conversion \n");
                                //SIPRTPLOG("Error: Allocating memory for mulaw to pcm conversion \n");
                                return -1;
                            }
                            u_char *rtpData = rtpPayload;
                            int i = 0;
                            for(i = 0; i < payloadLength; i++){
                                pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
							}
	
 #ifndef OS_WINDOWS
			//printf("Just before  entering livemonitor handler \n");
                    	returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], REVAUDIOFT, rtp[rtp_ptr].audioCodecType,rtp_hdr);
                    	if(returnMonitor == 1){
        	              	//playDSP((PACKET->DATA.data+ sizeof(struct sniff_rtp)), ((PACKET->DATA.len - sizeof(struct sniff_rtp)) * sizeof(u_char)), REVAUDIOFT);
        	              	 playDSP(pcm711mulaw, (payloadLength * sizeof(int16_t)), REVAUDIOFT);
                	}
#endif 
			//fwrite(PACKET->packet + 54, sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].rev_fp);
			fwrite(PACKET->DATA.data + sizeof(struct sniff_rtp), sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].rev_fp);
            rtp[rtp_ptr].arev_data_len += (sizeof(u_char) * (PACKET->DATA.len - sizeof(struct sniff_rtp)) );			
			++rtp[rtp_ptr].rev_count;
			rev_direction = 0;
			rtp[rtp_ptr].rev_seqCount = rtp_hdr -> sequence_no;
			++rtp[rtp_ptr].rev_count;
			free(pcm711mulaw);

		}
		++rtp[rtp_ptr].rtp_count;

	}
	else if(payload_mask == 0x61) {
 		
		/* Payload type is h.264 video */
		if(rtp[rtp_ptr].vrtp_count == 1){
		      printf(".264 files are getting created\n");
			strcat(rtp[rtp_ptr].vfwd_file,".264");
			strcat(rtp[rtp_ptr].vrev_file,".264");
		}

   		if(vfwd_direction == 1 && rtp[rtp_ptr].vfwd_count == 1){
            rtp[rtp_ptr].vfwd_fp = fopen(rtp[rtp_ptr].vfwd_file,"w+|b"); 
            
   			if(rtp[rtp_ptr].vfwd_fp == NULL){
   				INSTANT_USER_MSG("Error: Opening H264 forward file\n");
   				return -1;
   			}
   		}
   		else if(vrev_direction == 1 && rtp[rtp_ptr].vrev_count == 1){

   			rtp[rtp_ptr].vrev_fp = fopen(rtp[rtp_ptr].vrev_file,"w+|b");
   			
   			if(rtp[rtp_ptr].vrev_fp == NULL){
   				INSTANT_USER_MSG("Error: Creating H264 reverse file\n");
   				return -1;
   			}
   			
   		}

		if(vfwd_direction == 1){

			//printf("Inside the vfwd direction 1\n");

			if(rtp[rtp_ptr].vfwd_seqcount == rtp_hdr -> sequence_no) 
			{
                        return NULL;
                  }

/* Anil Mahale: Newly Added code for livemonitoring */
        
         struct naluHeader naluHeaderValue;
         int writeNALPrefixCode = 1;
         int writePacket = 1;
         int offset = 0;



         u_char *value = rtpPayload;

         naluHeaderValue.forbidden = (*value & 0x80) >> 7;
         naluHeaderValue.nri = (*value & 0x60) >> 5;
         naluHeaderValue.type = *value & 0x1F;

		   if(naluHeaderValue.type == 5){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
#ifdef HAVE_LIB_VLC
			   /* Coded slice IDR Picture */
		
			   /* Get the RTP marker bit */ 
			   u_char markerBit = rtp_hdr -> payloadType & 0x80;

			   if(rtp[rtp_ptr].currentFwdPPSCtr != rtp[rtp_ptr].previousFwdPPSCtr ){
				   if(rtp[rtp_ptr].recvFwdAllIDRPic == 1){
					
					   /* Delete the current IDR pictures */ 
					   deleteIDRPicture(rtp[rtp_ptr].fwdIDRPtr);
					   rtp[rtp_ptr].recvFwdAllIDRPic = 0;
					   rtp[rtp_ptr].fwdIDRPtr = NULL;
					   rtp[rtp_ptr].fwdIDRCtr = 0;
				   }

					if(rtp[rtp_ptr].fwdIDRPtr == NULL){

						/* First IDR coded picture slice in the coded video sequence */

						rtp[rtp_ptr].fwdIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
						if(rtp[rtp_ptr].fwdIDRPtr != NULL){

							rtp[rtp_ptr].fwdCurrentIDRPtr = rtp[rtp_ptr].fwdIDRPtr;

							memset(rtp[rtp_ptr].fwdIDRPtr->idrPictureData,'\0',sizeof(rtp[rtp_ptr].fwdIDRPtr->idrPictureData));
							rtp[rtp_ptr].fwdIDRPtr->idrPictureLength =  (PACKET->DATA.len - sizeof(struct sniff_rtp));
							memcpy(rtp[rtp_ptr].fwdIDRPtr->idrPictureData, value, (PACKET->DATA.len - sizeof(struct sniff_rtp)));
							rtp[rtp_ptr].fwdIDRPtr->next = NULL;
							rtp[rtp_ptr].fwdIDRCtr += 1;

							if(markerBit == 0x80){

								/* The marker bit is set to 1 */
								/* All IDR codec pic for this coded video seq are received */
								rtp[rtp_ptr].recvFwdAllIDRPic = 1;
								/* Incr the PPS ctr */
								rtp[rtp_ptr].previousFwdPPSCtr += 1;
							}
						}

			      }
				else {
							
						struct idrPicture *tempIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
                                    if(tempIDRPtr != NULL){

						            rtp[rtp_ptr].fwdCurrentIDRPtr->next = tempIDRPtr;
						            rtp[rtp_ptr].fwdCurrentIDRPtr = tempIDRPtr;
						
						            memset(rtp[rtp_ptr].fwdCurrentIDRPtr->idrPictureData, '\0', sizeof(rtp[rtp_ptr].fwdCurrentIDRPtr->idrPictureData));
						            rtp[rtp_ptr].fwdCurrentIDRPtr->idrPictureLength = (PACKET->DATA.len - sizeof(struct sniff_rtp));
						            memcpy(rtp[rtp_ptr].fwdCurrentIDRPtr->idrPictureData, value, rtp[rtp_ptr].fwdCurrentIDRPtr->idrPictureLength);
						            rtp[rtp_ptr].fwdCurrentIDRPtr->next = NULL;
						            rtp[rtp_ptr].fwdIDRCtr += 1;

							      if(markerBit == 0x80){

                                                     	/* The marker bit is set to 1 */
                                                     	/* All IDR codec pic for this coded video seq are received */
                                                     	rtp[rtp_ptr].recvFwdAllIDRPic = 1;
                                                     	/* Incr the PPS ctr */
                                                     	rtp[rtp_ptr].previousFwdPPSCtr += 1;
                                                }
                                    }
				}
			}
#endif
		}
				else if(naluHeaderValue.type == 7){
#ifdef HAVE_LIB_VLC
                                          printf("Alu Header value type is %d ", naluHeaderValue.type);
					/* Saving the h264 parameter sets for live video monitoring, the video decoder needs the paramter sets for decoding the raw h264 video */
					/* Sequence Parameter Set */ 
					//if(!currentRTPCall->savedFwdSPS){
						memset(rtp[rtp_ptr].fwdSPS,'\0',sizeof(rtp[rtp_ptr].fwdSPS));
						memcpy(rtp[rtp_ptr].fwdSPS,value,sizeof(rtp[rtp_ptr].fwdSPS));
					//	currentRTPCall->savedFwdSPS = 1;
					//}
#endif						
				}
				else if(naluHeaderValue.type == 8){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
#ifdef HAVE_LIB_VLC
					/* Picture Parameter Set */ 
					memset(rtp[rtp_ptr].fwdPPS,'\0',sizeof(rtp[rtp_ptr].fwdPPS));
					int PPSLength = payloadLength;
					if(payloadLength > sizeof(rtp[rtp_ptr].fwdPPS))
						PPSLength = sizeof(rtp[rtp_ptr].fwdPPS);
					memcpy(rtp[rtp_ptr].fwdPPS,value,PPSLength);
					rtp[rtp_ptr].currentFwdPPSCtr += 1;
#endif							
				}
                        else if(naluHeaderValue.type == 28){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
                                parseH264FUANAL(&naluHeaderValue, &value, &offset, &writeNALPrefixCode, &writePacket, &rtp[rtp_ptr].fwd_fuastart);
                                /* This is FU-A NAL unit */

                                /* Thanks to Manuel Koschuch for fixing FU-A parsing bugs */
                        }
                        else if(naluHeaderValue.type == 24){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
                                /* STAP-A NAL aggregate packets */
                                parseH264STAPANALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vfwd_file, rtp[rtp_ptr].vfwd_fp, &writeNALPrefixCode, &writePacket, 2, rtp_hdr);
                        }
                        else if(naluHeaderValue.type == 25){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
                                /* STAP-B NAL aggregate packets */
                                parseH264STAPBNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vfwd_file, rtp[rtp_ptr].vfwd_fp, &writeNALPrefixCode, &writePacket, 2, rtp_hdr);
                        }

			      else if(naluHeaderValue.type == 26){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
                                /* MTAP-16 Aggregate NAL Units */
                                parseH264MTAPNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vfwd_file, rtp[rtp_ptr].vfwd_fp, &writeNALPrefixCode, &writePacket, 16, 2, rtp_hdr);
                        }

                  else if(naluHeaderValue.type == 27){
                  printf("Alu Header value type is %d ", naluHeaderValue.type);
                                /* MTAP-32 NAL Aggregate Unit */
                                parseH264MTAPNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vfwd_file, rtp[rtp_ptr].vfwd_fp, &writeNALPrefixCode, &writePacket, 24, 2, rtp_hdr);

                        }
                  if(writeNALPrefixCode) {
			        phtonl(pd,0x00000001);
			        fwrite(pd,1,4,rtp[rtp_ptr].vfwd_fp);
			      }
#ifdef HAVE_LIB_VLC
                        int returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], FWDVIDEOFT, rtp[rtp_ptr].videoPayloadType, rtp_hdr);
                       // printf("parameter returnMonitor = %d \n",returnMonitor);

                        if(returnMonitor == 1){
                             // printf("sending rtp from ec_sccp \n");
                              sendRTPSKINNY(sample, PACKET->DATA.len, FWDVIDEOPORT_INT_SKINNY,rtp[rtp_ptr].liveMonitorSocket);
                        }
                        #endif						


//			phtonl(pd,0x00000001);
//			fwrite(pd,1,4,rtp[rtp_ptr].vfwd_fp);
                        //fwrite(PACKET->packet + 54, sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].fwd_fp);
                        fwrite(PACKET->DATA.data + sizeof(struct sniff_rtp), sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].vfwd_fp);
                        ++rtp[rtp_ptr].vfwd_count;
                        vfwd_direction = 0;
                        rtp[rtp_ptr].vfwd_seqcount = rtp_hdr -> sequence_no;
                        ++rtp[rtp_ptr].vfwd_count;
                        ++rtp[rtp_ptr].vrtp_count;
                        //func_drop(PACKET);
		}
		else if(vrev_direction == 1){
			
			if(rtp[rtp_ptr].vrev_seqcount == rtp_hdr -> sequence_no){
				return NULL;
			}
			
			 struct naluHeader naluHeaderValue;
                        int writeNALPrefixCode = 1;
                        int writePacket = 1;
                        int offset = 0;

                        u_char *value = rtpPayload;

                        naluHeaderValue.forbidden = (*value & 0x80) >> 7;
                        naluHeaderValue.nri = (*value & 0x60) >> 5;
                        naluHeaderValue.type = *value & 0x1F;

						if(naluHeaderValue.type == 5){

#ifdef HAVE_LIB_VLC
                            /* Coded slice IDR Picture */

                            /* Get the RTP marker bit */
                            u_char markerBit = rtp_hdr -> payloadType & 0x80;

                            if(rtp[rtp_ptr].currentRevPPSCtr != rtp[rtp_ptr].previousRevPPSCtr ){

                                if(rtp[rtp_ptr].recvRevAllIDRPic == 1){

                        	        /* Delete the current IDR pictures */
									deleteIDRPicture(rtp[rtp_ptr].revIDRPtr);
                                    rtp[rtp_ptr].recvRevAllIDRPic = 0;
									rtp[rtp_ptr].revIDRPtr = NULL;
									rtp[rtp_ptr].revIDRCtr = 0;
                                }

                                if(rtp[rtp_ptr].revIDRPtr == NULL){

                                    /* First IDR coded picture slice in the coded video sequence */

                                    rtp[rtp_ptr].revIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
                                    if(rtp[rtp_ptr].revIDRPtr != NULL){

                                        rtp[rtp_ptr].revCurrentIDRPtr = rtp[rtp_ptr].revIDRPtr;

                                        memset(rtp[rtp_ptr].revIDRPtr->idrPictureData,'\0',sizeof(rtp[rtp_ptr].revIDRPtr->idrPictureData));
                                        rtp[rtp_ptr].revIDRPtr->idrPictureLength =  (PACKET->DATA.len - sizeof(struct sniff_rtp));
                                        memcpy(rtp[rtp_ptr].revIDRPtr->idrPictureData, value, (PACKET->DATA.len - sizeof(struct sniff_rtp)));
                                        rtp[rtp_ptr].revIDRPtr->next = NULL;
										rtp[rtp_ptr].revIDRCtr += 1;

                                        if(markerBit == 0x80){

                                            /* The marker bit is set to 1 */
                                            /* All IDR codec pic for this coded video seq are received */
                                            rtp[rtp_ptr].recvRevAllIDRPic = 1;
                                            /* Incr the PPS ctr */
                                            rtp[rtp_ptr].previousRevPPSCtr += 1;
                                        }
                                    }

                                }
								else {
                                        struct idrPicture *tempIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
                                        if(tempIDRPtr != NULL){

                                            rtp[rtp_ptr].revCurrentIDRPtr->next = tempIDRPtr;
                                            rtp[rtp_ptr].revCurrentIDRPtr = tempIDRPtr;

                                            memset(rtp[rtp_ptr].revCurrentIDRPtr->idrPictureData, '\0', sizeof(rtp[rtp_ptr].revCurrentIDRPtr->idrPictureData));
                                            rtp[rtp_ptr].revCurrentIDRPtr->idrPictureLength = (PACKET->DATA.len - sizeof(struct sniff_rtp));
                                            memcpy(rtp[rtp_ptr].revCurrentIDRPtr->idrPictureData, value, rtp[rtp_ptr].revCurrentIDRPtr->idrPictureLength);
                                            rtp[rtp_ptr].revCurrentIDRPtr->next = NULL;
							rtp[rtp_ptr].revIDRCtr += 1;

                                            if(markerBit == 0x80){

                                                /* The marker bit is set to 1 */
                                                /* All IDR codec pic for this coded video seq are received */
                                                rtp[rtp_ptr].recvRevAllIDRPic = 1;
                                                /* Incr the PPS ctr */
                                                rtp[rtp_ptr].previousRevPPSCtr += 1;
                                            }
                                        }
                                }
                            }
#endif
                        }
						else if(naluHeaderValue.type == 7){

#ifdef HAVE_LIB_VLC
                            /* Saving the h264 parameter sets for live video monitoring, the video decoder needs the paramter sets for decoding the raw h264 video */
                            /* Sequence Parameter Set */
                            //if(!currentRTPCall->savedRevSPS){
								memset(rtp[rtp_ptr].revSPS,'\0',sizeof(rtp[rtp_ptr].revSPS));
                                memcpy(rtp[rtp_ptr].revSPS,value,sizeof(rtp[rtp_ptr].revSPS));
                                rtp[rtp_ptr].savedRevSPS = 1;
                            //}
#endif
                        }
                        else if(naluHeaderValue.type == 8){

#ifdef HAVE_LIB_VLC
                            /* Picture Parameter Set */ 
							int PPSLength = payloadLength;
							if(payloadLength > sizeof(rtp[rtp_ptr].revPPS))
								PPSLength = sizeof(rtp[rtp_ptr].revPPS);
                            memset(rtp[rtp_ptr].revPPS,'\0',sizeof(rtp[rtp_ptr].revPPS));
                            memcpy(rtp[rtp_ptr].revPPS,value,PPSLength); 
							rtp[rtp_ptr].currentRevPPSCtr += 1;
#endif

                        }
                        else if(naluHeaderValue.type == 28){

                                parseH264FUANAL(&naluHeaderValue, &value, &offset, &writeNALPrefixCode, &writePacket, &rtp[rtp_ptr].rev_fuastart);
                                /* This is FU-A NAL unit */

                                /* Thanks to Manuel Koschuch for fixing FU-A parsing bugs */
                        }
                        else if(naluHeaderValue.type == 24){

                                /* STAP-A NAL aggregate packets */
                                parseH264STAPANALSkinny(&value, currentRTPCall, PACKET->DATA.len,rtp[rtp_ptr].vrev_file, rtp[rtp_ptr].vrev_fp, &writeNALPrefixCode, &writePacket, 3, rtp_hdr);
                        }
						else if(naluHeaderValue.type == 25){

                                /* STAP-B NAL aggregate packets */
                                parseH264STAPBNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vrev_file, rtp[rtp_ptr].vrev_fp, &writeNALPrefixCode, &writePacket, 3, rtp_hdr);
                        }
                        else if(naluHeaderValue.type == 26){

                                /* MTAP-16 Aggregate NAL Units */
                                parseH264MTAPNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vrev_file, rtp[rtp_ptr].vrev_fp, &writeNALPrefixCode, &writePacket, 16, 3, rtp_hdr);
                        }

                        else if(naluHeaderValue.type == 27){

                                /* MTAP-32 NAL Aggregate Unit */
                                parseH264MTAPNALSkinny(&value, currentRTPCall, PACKET->DATA.len, rtp[rtp_ptr].vrev_file, rtp[rtp_ptr].vrev_fp, &writeNALPrefixCode, &writePacket, 24, 3, rtp_hdr);

                        }

			
                        if(writeNALPrefixCode) {
			                phtonl(pd,0x00000001);
			                fwrite(pd,1,4,rtp[rtp_ptr].vrev_fp);
			            }
			#ifdef HAVE_LIB_VLC
                        int returnMonitor = liveMonitorHandlerSkinny(&rtp[rtp_ptr], REVVIDEOFT, rtp[rtp_ptr].videoPayloadType, rtp_hdr);
                       // printf("parameter returnMonitor = %d \n",returnMonitor);

                        if(returnMonitor == 1){
                              //printf("sending rtp from ec_sccp \n");
                              sendRTPSKINNY(sample, PACKET->DATA.len, REVVIDEOPORT_INT_SKINNY,rtp[rtp_ptr].liveMonitorSocket);
                        }
                        #endif		
         //fwrite(PACKET->packet + 54, sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].fwd_fp);
			 fwrite(PACKET->DATA.data + sizeof(struct sniff_rtp), sizeof(u_char), (PACKET->DATA.len - sizeof(struct sniff_rtp)), rtp[rtp_ptr].vrev_fp);
			 ++rtp[rtp_ptr].vrev_count;
			 vrev_direction = 0;
			 rtp[rtp_ptr].vrev_seqcount = rtp_hdr -> sequence_no;
			 ++rtp[rtp_ptr].vrev_count;
			 ++rtp[rtp_ptr].vrtp_count;
		}

	}
	else {
		// Unknown codec.  Payload not handled, just dumping to pcap file
	}

	pcap_dump((u_char *)rtp[rtp_ptr].dumper,dump_pkthdr,PACKET->packet);
	//++rtp[rtp_ptr].rtp_count;

        return NULL;
}

// Return 1 if the list is empty, 0 otherwise
int isskinnyempty(struct skinnycall *scall){

	if (scall == NULL) {
		return 1;
	} else {
		return 0;
	}

}

int lookup_garp_yes(char ip[MAX_ASCII_ADDR_LEN]){

	int i;
	for (i = 0; i < garp_hostcount; i++) {
		int compare = strcmp(ip,garpenhosts[i].ip);
		if (compare == 0) {
			return 1;
		}
	}

	return 0;

}

void skinnyfirstcallinit(SkinnyCallPtr *sPtr, char dst_ip[MAX_ASCII_ADDR_LEN], int confid){

	SCCPACCESS_LOCK;

	SkinnyCallPtr new_skinnycall;
	SkinnyCallPtr prev_skinnycall;
	SkinnyCallPtr current_skinnycall;

	//new_skinnycall = malloc(sizeof(struct skinnycall));
	new_skinnycall = (skinnyDB*) calloc(sizeof(skinnyDB),1);

	if (new_skinnycall != NULL) {

		new_skinnycall->CallIdentifier = confid;
		new_skinnycall->CCallIdentifier = confid + 1;
		strcpy(new_skinnycall->ip1,dst_ip);
		strcpy(new_skinnycall->dialingip,dst_ip);
		//printf("Setting dialingip to %s via skinnyfirstcallinit\n",new_skinnycall->dialingip);
		new_skinnycall->ip1dialing = 1;
		new_skinnycall->offhook1 = 1;
		new_skinnycall->digitcount = 0;
		// initialize dialing and dialed number to NULL 
		strcpy(new_skinnycall->dialingnumber,"Unknown");
		//printf("Setting dialing number to %s via skinnyfirstcallinit\n",new_skinnycall->dialingnumber);
		strcpy(new_skinnycall->dialednumber,"Unknown");
		//printf("Setting dialed number to %s via skinnyfirstcallinit\n",new_skinnycall->dialednumber);
		// init the dialed IP
		strcpy(new_skinnycall->dialedip,"IP Unknown");
		strcpy(new_skinnycall->ip2,"IP Unknown");
		new_skinnycall->next_skinnycall = NULL;

		prev_skinnycall = NULL;
		current_skinnycall = *sPtr;
            new_skinnycall -> myRTPCall = NULL;
		// Loop to find the correct location in the list
		while ( current_skinnycall != NULL ) {

			prev_skinnycall  = current_skinnycall;
			current_skinnycall = current_skinnycall->next_skinnycall;

		}

		// Insert new node at beginning of the list
		if (prev_skinnycall == NULL) {
			new_skinnycall->next_skinnycall = *sPtr;
			*sPtr = new_skinnycall;
			
		} else {
			// insert new node between previous and current
			prev_skinnycall->next_skinnycall = new_skinnycall;
			new_skinnycall->next_skinnycall = current_skinnycall;
		}


	} else {
		INSTANT_USER_MSG("skinnycall node not created.  No memory available.\n");
	}

	SCCPACCESS_UNLOCK;

}


void nodeupdatecallstateoffhook(SkinnyCallPtr *sPtr, char dst_ip[MAX_ASCII_ADDR_LEN], int confid){

	SkinnyCallPtr new_skinnycall;
	SkinnyCallPtr prev_skinnycall;
	SkinnyCallPtr current_skinnycall;
	SkinnyCallPtr first_skinnycall;

	current_skinnycall = *sPtr;
	first_skinnycall = *sPtr;

	int foundcconfid = 0;
	int foundconfid = 0;
	while ( current_skinnycall != NULL ) {

		// Check to see if confid is of the 2nd, dialed IP phone
		if (current_skinnycall->CCallIdentifier == confid) {
			strcpy(current_skinnycall->ip2,dst_ip);
			strcpy(current_skinnycall->dialedip,dst_ip);
			current_skinnycall->offhook2 = 1;
			current_skinnycall->ip2dialed = 1;
			foundcconfid = 1;
			current_skinnycall = current_skinnycall->next_skinnycall;
		// Check to see if confid is already in node list
		} else if (current_skinnycall->CallIdentifier == confid) {
			foundconfid = 1;
			current_skinnycall = current_skinnycall->next_skinnycall;
		} else {
			prev_skinnycall = current_skinnycall;
			current_skinnycall = current_skinnycall->next_skinnycall;
		}

	}	

	if ((foundcconfid == 0)&&(foundconfid == 0)) {

		// new call, first IP phone went offhook
//		new_skinnycall = malloc(sizeof(struct skinnycall));
		new_skinnycall = (skinnyDB*) calloc(sizeof(skinnyDB),1);
		
		if (new_skinnycall != NULL) {

			new_skinnycall->CallIdentifier = confid;
			new_skinnycall->CCallIdentifier = confid + 1;
			strcpy(new_skinnycall->ip1,dst_ip);
			strcpy(new_skinnycall->dialingip,dst_ip);
			//printf("dialingip %s set via nodeupdatecallstateoffhook\n", new_skinnycall->dialingip); 
			new_skinnycall->ip1dialing = 1;
			new_skinnycall->offhook1 = 1;
			strcpy(new_skinnycall->dialingnumber,"Unknown");
			strcpy(new_skinnycall->dialednumber,"Unknown");
			new_skinnycall->digitcount = 0;
		
			prev_skinnycall = NULL;
			current_skinnycall = first_skinnycall;
		
			// Loop to find the correct location in the list
			while ( current_skinnycall != NULL ) {
				prev_skinnycall  = current_skinnycall;
				current_skinnycall = current_skinnycall->next_skinnycall;
			}
		
			// Insert new node at beginning of the list
			if (prev_skinnycall == NULL) {
				new_skinnycall->next_skinnycall = first_skinnycall;
				first_skinnycall = new_skinnycall;
			} else {
				// insert new node between previous and current
				prev_skinnycall->next_skinnycall = new_skinnycall;
				new_skinnycall->next_skinnycall = current_skinnycall;
			}

		} else {

			INSTANT_USER_MSG("skinnycall node not created.  No memory available.\n");

		}


	}


}

void backspacereqhandler(SkinnyCallPtr *sPtr, char ip[MAX_ASCII_ADDR_LEN]){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		if (strcmp(ip,current_skinnycall->ip1) == 0) {

			if(current_skinnycall->offhook1 == 1) {
				int offset = current_skinnycall->digitcount - 1;
				current_skinnycall->dialednumber[offset] = '\0';
				current_skinnycall->digitcount = current_skinnycall->digitcount - 1;

			}

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}


}
void keypadbuttonhandler(SkinnyCallPtr *sPtr, char ip[MAX_ASCII_ADDR_LEN], int keypadbutton){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {
	
		if (strcmp(ip,current_skinnycall->ip1) == 0) {

			if(current_skinnycall->offhook1 == 1) {

				if (current_skinnycall->digitcount == 0) {
					current_skinnycall->dialednumber[0] = adddigit(keypadbutton);
					current_skinnycall->digitcount++;
					int digitcount = current_skinnycall->digitcount;
					current_skinnycall->dialednumber[digitcount] = '\0';
				} else {
					int offset = current_skinnycall->digitcount;
					current_skinnycall->dialednumber[offset] = adddigit(keypadbutton);
					current_skinnycall->digitcount++;
					int digitcount = current_skinnycall->digitcount;
					current_skinnycall->dialednumber[digitcount] = '\0';
				}

			}

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}


}
void handlerdialednumber(SkinnyCallPtr *sPtr, int confid, char calledparty[24]){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		//printf("Comparing confid to cidentifier:  %d to %d\n",confid,current_skinnycall->CallIdentifier);
		if (confid == current_skinnycall->CallIdentifier) {

			strcpy(current_skinnycall->dialednumber,calledparty);
			//printf("Setting dialed number to %s via handlerdialednumber\n",current_skinnycall->dialednumber);
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

}
/*int handlerstationcallinfomessage(SkinnyCallPtr *sPtr, int confid, char *callerid, char ip[MAX_ASCII_ADDR_LEN], int length){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		if (confid == current_skinnycall->CCallIdentifier) {
			strcpy(current_skinnycall->ip2,ip);
			strcpy(current_skinnycall->dialedip,ip);

			int i = 0;
			int retval = 1;
			while ((i < length) && (retval == 1)) {

				retval = checkdigit(callerid[i]);

				if (retval == 1) {
					current_skinnycall->dialingnumber[i] = callerid[i];

				}

				i++;
			}
			int offset = i - 1;
			current_skinnycall->dialingnumber[offset] = '\0';
			return 1;

		} else {
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		}


	}

	return 0;

}*/
void newnodecallinfomessage(SkinnyCallPtr *sPtr, char callingp[24], char calledp[24], int retval, int confid, char ip[MAX_ASCII_ADDR_LEN]){

        SkinnyCallPtr new_skinnycall;
        SkinnyCallPtr prev_skinnycall;
        SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int foundconfid = 0;

	if (retval == 0) {

		// Check to see if conf is is in there
		while ( current_skinnycall != NULL ) {

			if (current_skinnycall->CallIdentifier == confid) {
				foundconfid = 1;
				current_skinnycall = current_skinnycall->next_skinnycall;
			} else {
				prev_skinnycall = current_skinnycall;
				current_skinnycall = current_skinnycall->next_skinnycall;
			}

		}

		if (foundconfid == 0) {

			new_skinnycall = malloc(sizeof(struct skinnycall));
			if (new_skinnycall != NULL) {

				new_skinnycall->CallIdentifier = confid;
				new_skinnycall->CCallIdentifier = confid - 1;
				strcpy(new_skinnycall->ip1,ip);
				if(strcmp("Unknown",calledp) != 0) {
					strcpy(new_skinnycall->dialednumber,calledp);
				}
				strcpy(new_skinnycall->dialingnumber,callingp);
				strcpy(new_skinnycall->dialedip,ip);
				new_skinnycall->ip1dialed = 1;
				
				new_skinnycall->next_skinnycall = NULL;
				prev_skinnycall = NULL;

				// Loop to find the correct location in the list
				while ( current_skinnycall != NULL ) {

					prev_skinnycall = current_skinnycall;
					current_skinnycall = current_skinnycall->next_skinnycall;

				}
				
				// Insert new node at beginning of the list
				if (prev_skinnycall == NULL) {

					new_skinnycall->next_skinnycall = *sPtr;
					*sPtr = new_skinnycall;

				} else {
					// insert new node between previous and current
					prev_skinnycall->next_skinnycall = new_skinnycall;
					new_skinnycall->next_skinnycall = current_skinnycall;
				}
				

			} else {
  				
				INSTANT_USER_MSG("skinnycall node not created.  No memory available.\n");

			}

		}


	}
	
}
void passthrupartyadd(SkinnyCallPtr *sPtr, int confid, int passthrupartyid){

	//SkinnyCallPtr prev_skinnycall;
	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		if (current_skinnycall->CallIdentifier == confid) {

			current_skinnycall->passthrupartyid1 = passthrupartyid;
			//printf("Added passthrupartyid1 of %d to CallIdentifier of %d\n",current_skinnycall->passthrupartyid1,current_skinnycall->CallIdentifier);
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else if (current_skinnycall->CCallIdentifier == confid) {

			current_skinnycall->passthrupartyid2 = passthrupartyid;
			//printf("Added passthrupartyid2 of %d to CallIdentifier of %d\n",current_skinnycall->passthrupartyid2,current_skinnycall->CallIdentifier);
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			//prev_skinnycall = first_skinnycall;
			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

}
void vpassthrupartyadd(SkinnyCallPtr *sPtr, int confid, int vpassthrupartyid){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		if(current_skinnycall->CallIdentifier == confid){

			current_skinnycall -> vpassthrupartyid1 = vpassthrupartyid;
			current_skinnycall = current_skinnycall -> next_skinnycall;
		}
		else if(current_skinnycall -> CCallIdentifier == confid){

			current_skinnycall -> vpassthrupartyid2 = vpassthrupartyid;
			current_skinnycall = current_skinnycall -> next_skinnycall;
		}
		else {
			current_skinnycall = current_skinnycall->next_skinnycall;
		}
	}
}

int openmultimediahandler(SkinnyCallPtr *sPtr, char *ip, int confid, int vpassthrupartyid, int vrtpport){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while( current_skinnycall != NULL ){

		//if ((current_skinnycall->vpassthrupartyid1 == vpassthrupartyid) || (current_skinnycall->passthrupartyid2 == vpassthrupartyid)) {

		if ((current_skinnycall->vpassthrupartyid1 == vpassthrupartyid)) {

			if((current_skinnycall->videoactive != 1) && (current_skinnycall->call_active == 1)) {
			
				char *dissect_name;
				dissect_name = (char *) malloc ((6 + MAX_ASCII_ADDR_LEN + 3) * sizeof(char));
				memset(dissect_name,'\0',(6+MAX_ASCII_ADDR_LEN + 3));
				create_dissect_string( vrtpport, ip, dissect_name);
				dissect_add(dissect_name, APP_LAYER_UDP, vrtpport, dissector_rtp_skinny);						
				free(dissect_name);

				//Adding videoport value to the call structure
				current_skinnycall->vrtpport1 = vrtpport;

				int return_rtp = check_rtp_entry(ip);					
				if(return_rtp < 0){
					INSTANT_USER_MSG("Audio path should be setup before a video call\n");
				}
				else{
					int return_value = populate_rtp_struct_video(return_rtp, ip, vpassthrupartyid, vrtpport);
					if(return_value == -1)
						return NULL;
				}

				//Activates video call
				current_skinnycall->videoactive = 1;

				return 1;
			}
			current_skinnycall = current_skinnycall->next_skinnycall;
		}	
		else{
			current_skinnycall = current_skinnycall->next_skinnycall;
		}
	}					
}

int openreceivehandler(SkinnyCallPtr *sPtr, char ip [MAX_ASCII_ADDR_LEN], int rtpport, int passthrupartyid){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ) {

		if ((current_skinnycall->passthrupartyid1 == passthrupartyid)||(current_skinnycall->passthrupartyid2 == passthrupartyid)) {
			
		        // Check to see if call isn't already active
			// Only proceed if call inactive	
			if (current_skinnycall->call_active != 1) {

				// Add Directory entries for the users into node, if the user has a directory entry 
				callerexistsindir = getcallerdirname(passthrupartyid, &first_skinnycall);

				calledexistsindir = getcalleddirname(passthrupartyid, &first_skinnycall);

				//printf("Adding RTP Dissector for port %d, passthrupartyid %d, IP %s\n",rtpport,passthrupartyid,ip);
				current_skinnycall->rtpport1 = rtpport;

				char *dissect_name;
                                dissect_name = (char *) malloc ((6 + MAX_ASCII_ADDR_LEN + 3) * sizeof(char));
                                memset(dissect_name,'\0',(6+MAX_ASCII_ADDR_LEN+3));
				create_dissect_string(rtpport, ip, dissect_name);
				//printf("TEST adding rtp dissector for ip %s and port %d\n",ip,rtpport);
				dissect_add(dissect_name,APP_LAYER_UDP,rtpport,dissector_rtp_skinny);
				free(dissect_name);

				/* populate values in rtp structure
				 * check to see if the ip has been populated for the rtp struct entry
				 * if it hasn't, call populate rtp_struct
				 * return_rtp returns 'i' i rtp[i] for the ip passed to it.
				 * It basically finds the i entry for the IP address in rtp[] */
				int return_rtp = check_rtp_entry(ip);
				if (return_rtp < 0) { 
					int return_value = populate_rtp_struct(return_rtp, ip, passthrupartyid, rtpport);
					if(return_value == -1) {
						return NULL;
					}
				} else {
					int return_value = populate_rtp_struct(return_rtp, ip, passthrupartyid, rtpport);
					if (return_value == -1) {
						return NULL;
					}
				}

				// Set the IP monitored for RTP
				strcpy(current_skinnycall->rtpip,ip);
				//printf("Setting the monitored RTP IP Address to %s\n",current_skinnycall->rtpip);

				// Set call to active
				current_skinnycall->call_active = 1;
				//printf("This is in openreceivehandler function \n Value of rtp_counter is %d\n",rtp_counter);
				//printf("Value of return_rtp is %d\n",return_rtp);
				if(return_rtp== -1)
				current_skinnycall->myRTPCall = &rtp[rtp_counter-1];
				else
				current_skinnycall->myRTPCall = &rtp[return_rtp];


				// Set call start time
				gettimeofday(&current_skinnycall->start,0);

				// Set start_time_string
				gettimestring(1, &current_skinnycall->start_time_string);

				printcallstdout(&first_skinnycall, current_skinnycall->CallIdentifier);

				printtoskinnyfilestart(&first_skinnycall, current_skinnycall->CallIdentifier);

				return 1;

			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	return 1;

}
int closeaudiochannel(SkinnyCallPtr *sPtr, int passthrupartyid){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while ( current_skinnycall != NULL ){ 
		
		if (( current_skinnycall->passthrupartyid1 == passthrupartyid) || (current_skinnycall->passthrupartyid2 == passthrupartyid)) {

			int return_rtp = check_rtp_entry(current_skinnycall->rtpip);
			if(return_rtp < 0) {

			} else {
				rtp[return_rtp].session_active = 0;
				rtp[return_rtp].vrtp_count = 1;   /* Added for video */
				pcap_dump_close(rtp[return_rtp].dumper);
				pcap_close(rtp[return_rtp].dp);
	
				if(rtp[return_rtp].vfwd_count > 1){
					fclose(rtp[return_rtp].vfwd_fp);
				}
				
				if(rtp[return_rtp].vrev_count > 1 ){
					fclose(rtp[return_rtp].vrev_fp);
				}

				if (rtp[return_rtp].fwd_count > 1) {
				    // write wav header with correct length
				    fseek(rtp[return_rtp].fwd_fp, 0L, SEEK_SET);
                    create_wav_header(rtp[return_rtp].fwd_fp, 1, 16, 8000, 0x0001, rtp[return_rtp].afwd_data_len); 				    
				    fseek(rtp[return_rtp].fwd_fp, 0L, SEEK_END);
					fclose(rtp[return_rtp].fwd_fp);
				}
				
				if (rtp[return_rtp].rev_count > 1) {
				    // write wav header with correct length
				    fseek(rtp[return_rtp].rev_fp, 0L, SEEK_SET);
                    create_wav_header(rtp[return_rtp].rev_fp, 1, 16, 8000, 0x0001, rtp[return_rtp].arev_data_len); 				    
				    fseek(rtp[return_rtp].rev_fp, 0L, SEEK_END);
					fclose(rtp[return_rtp].rev_fp);
				}

				if ((rtp[return_rtp].fwd_count > 1) && (rtp[return_rtp].rev_count > 1)) {
					int mix_return;
					mix_return = mix_files(rtp[return_rtp].fwd_au,rtp[return_rtp].rev_au,rtp[return_rtp].both_au);
					if(current_skinnycall->videoactive == 1 && mixAv_modify_yes){
#ifdef HAVE_VIDEO_DECODER
						createForwardStreamVideoFiles(	rtp[return_rtp].fwd_au, 
														rtp[return_rtp].vfwd_file,
														rtp[return_rtp].vfwd_mp4,
														rtp[return_rtp].vfwd_avi );						
						createReverseStreamVideoFiles(	rtp[return_rtp].rev_au, 
														rtp[return_rtp].vrev_file,
														rtp[return_rtp].vrev_mp4,
														rtp[return_rtp].vrev_avi );						

/*
<<<<<<<<<<<<<<<<<<<<
						int mp4_ret = create_mp4_header(rtp[return_rtp].vfwd_file,rtp[return_rtp].vfwd_mp4);
						if(mp4_ret == 1){
							INSTANT_USER_MSG("Error: Creating forward avi file, try playing the .264 file using vlc \n");
						}	
						int mp4_ret1 = create_mp4_header(rtp[return_rtp].vrev_file,rtp[return_rtp].vrev_mp4);
						if(mp4_ret1 == 1){
							INSTANT_USER_MSG("Error: Creating reverse avi file, try playing the .264 file using vlc \n");
						}
						if(mp4_ret != 1 && mp4_ret1 != 1){ 
							int avi_ret = mix_audio_video(rtp[return_rtp].fwd_au,rtp[return_rtp].vfwd_mp4,rtp[return_rtp].vfwd_avi);
							if(avi_ret == 1){
								INSTANT_USER_MSG("Error: Mixing forward audio and video stream \n");
								INSTANT_USER_MSG("Saving forward video conversation with no audio to file, '");
								INSTANT_USER_MSG("%s'\n",rtp[return_rtp].vfwd_mp4);
								//remove(rtp[return_rtp].vfwd_file);
                                                        	remove(rtp[return_rtp].fwd_au);
							}
							else{
								INSTANT_USER_MSG("Saving forward video conversation to file, '");
								INSTANT_USER_MSG("%s'\n",rtp[return_rtp].vfwd_avi);
								remove(rtp[return_rtp].vfwd_mp4);
                                                        	//remove(rtp[return_rtp].vfwd_file);
                                                        	remove(rtp[return_rtp].fwd_au);
							}
							int avi_ret1 = mix_audio_video(rtp[return_rtp].rev_au,rtp[return_rtp].vrev_mp4,rtp[return_rtp].vrev_avi);
							if(avi_ret1 == 1){
								INSTANT_USER_MSG("Error: Mixing reverse audio and video stream \n");
								INSTANT_USER_MSG("Saving reverse video conversation with no audio to file, '");
								INSTANT_USER_MSG("%s'\n",rtp[return_rtp].vrev_mp4);
								//remove(rtp[return_rtp].vrev_file);
                                                        	remove(rtp[return_rtp].rev_au); 
							}
							else{
								INSTANT_USER_MSG("Saving reverse video conversation to file, '");
								INSTANT_USER_MSG("%s'\n", rtp[return_rtp].vrev_avi);
								remove(rtp[return_rtp].vrev_mp4);
								//remove(rtp[return_rtp].vrev_file);
                                                        	remove(rtp[return_rtp].rev_au);
							}
						}
 */
#else
						INSTANT_USER_MSG("Saving forward video conversation to file, '");
						INSTANT_USER_MSG("%s'\n",rtp[return_rtp].vfwd_file);
						INSTANT_USER_MSG("Saving reverse video conversation to file, '");                   
                        INSTANT_USER_MSG("%s'\n",rtp[return_rtp].vrev_file);
#endif //end HAVE_VIDEO_DECODER
					}
					 

					INSTANT_USER_MSG("Saving audio conversation to file, '");
					INSTANT_USER_MSG("%s'\n",rtp[return_rtp].both_au);
					if(voipdebug){
						fprintf(sccpfile,"Saving audio conversation to file, '");
						flush = fflush(sccpfile);
						fprintf(sccpfile,"%s'\n",rtp[return_rtp].both_au);
						flush = fflush(sccpfile);
					}
					//remove(rtp[return_rtp].fwd_au);
                    //remove(rtp[return_rtp].rev_au);
				} else {
					INSTANT_USER_MSG("Not saving conversation media file because either forward or reverse media not received.\n");
					if(voipdebug) {
						fprintf(sccpfile,"Not saving conversation media file because either forward or reverse media not received.\n");
						flush = fflush(sccpfile);
					}
					if (garpdb_yes != 1) {
						INSTANT_USER_MSG("Try running UCSniff with the --garpdb option, which enables the GARP Disablement Bypass feature\n");
						if(voipdebug){
							fprintf(sccpfile,"Try running UCSniff with the --garpdb option, which enables the GARP Disablement Bypass feature\n");				
							flush = fflush(sccpfile);
						}
					}
					//printf("rtp fwd count is %d, rtp rev count is %d\n",rtp[return_rtp].fwd_count,rtp[return_rtp].rev_count);

				}

				if (current_skinnycall->call_active == 1) {
					char *dissect_name;
                                	dissect_name = (char *) malloc ((6 + 3 + MAX_ASCII_ADDR_LEN) * sizeof(char));
                                	memset(dissect_name,'\0',(6+3+MAX_ASCII_ADDR_LEN));
					create_dissect_string(current_skinnycall->rtpport1, current_skinnycall->rtpip,dissect_name);
					//printf("TEST deleting dissector with ip %s and port %d\n",current_skinnycall->rtpip,current_skinnycall->rtpport1);
					dissect_del(dissect_name);
					free(dissect_name);

					if (target_yes == 1) {
						printtargetwait();
					}
					if(current_skinnycall->videoactive == 1){
						current_skinnycall->videoactive = 0;
						char *dissect_vname;
                                		dissect_vname = (char *) malloc ((6 + 3 +  MAX_ASCII_ADDR_LEN) * sizeof(char));
                                		memset(dissect_vname,'\0',(6+MAX_ASCII_ADDR_LEN));
						create_dissect_string(current_skinnycall->vrtpport1, current_skinnycall->rtpip,dissect_vname);
						dissect_del(dissect_vname);
						free(dissect_vname);
					}
					current_skinnycall->call_active = 0;
					printcallstdout(&first_skinnycall, current_skinnycall->CallIdentifier);

					SCCPACCESS_LOCK;

					deletecallnode(&first_skinnycall, passthrupartyid);

					SCCPACCESS_UNLOCK;
				}

			}

			return 1;

		} else {
			
			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}
	
	return 1;

}
void deletecallnode( SkinnyCallPtr *sPtr, int passthrupartyid ){

	SkinnyCallPtr temp_skinnycall;
	SkinnyCallPtr previous_skinnycall;
	SkinnyCallPtr current_skinnycall;

	if ((passthrupartyid == ( *sPtr )->passthrupartyid1) || (passthrupartyid == ( *sPtr )->passthrupartyid2 )) {
		temp_skinnycall = *sPtr;
		*sPtr = ( *sPtr )->next_skinnycall;
		free(temp_skinnycall);
	} else {
		previous_skinnycall = *sPtr;
		current_skinnycall = ( *sPtr )->next_skinnycall;

		while (current_skinnycall != NULL && current_skinnycall->passthrupartyid1 != passthrupartyid && current_skinnycall->passthrupartyid2 != passthrupartyid) {
			previous_skinnycall = current_skinnycall;
			current_skinnycall = current_skinnycall->next_skinnycall;
		}

		if (current_skinnycall != NULL) {
			temp_skinnycall = current_skinnycall;
			previous_skinnycall->next_skinnycall = current_skinnycall->next_skinnycall;
			free (temp_skinnycall);
		}
	}

}
void printNode( int confid ){

	SkinnyCallPtr prev_skinnycall;

	while ( first_skinnycall != NULL ) {

		if ( first_skinnycall->CallIdentifier == confid ) {

			INSTANT_USER_MSG("Found call for confid %d\n",confid);

			INSTANT_USER_MSG("Dialing IP %s\n",first_skinnycall->dialingip);


		} else {
			prev_skinnycall = first_skinnycall;
			first_skinnycall = first_skinnycall->next_skinnycall;

		}

	}

}
/* Return 1 if the list is empty, 0 otherwise */
int isEmpty(SkinnyCallPtr sPtr){

	return sPtr == NULL;

}

void printList( SkinnyCallPtr currentPtr ){


	if (currentPtr == NULL) {
		INSTANT_USER_MSG("List is empty.\n\n");
	} else {

		INSTANT_USER_MSG("The list is: \n");

		while (currentPtr != NULL) {

			INSTANT_USER_MSG("%d --> ",currentPtr->CallIdentifier);
			currentPtr = currentPtr->next_skinnycall;

		}

		INSTANT_USER_MSG("NULL\n\n");
	}

}	
int check_fwd_media_garp(SkinnyCallPtr *sPtr, char tempip[MAX_ASCII_ADDR_LEN], int passthrupartyid){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int dialing;

	if (target_yes == 1){
		//printf("Target_yes == 1, retuning 1\n");
		return 1;
	}

	while (current_skinnycall != NULL) {

		if ((current_skinnycall->passthrupartyid1 == passthrupartyid)||(current_skinnycall->passthrupartyid2 == passthrupartyid)) {

			int compare = strcmp(tempip, current_skinnycall->dialingip);
			if (compare == 0) {
				dialing = 1;
				int garp_retval = lookup_garp_yes(tempip);
				if (garp_retval == 1) {
					current_skinnycall->fwd_set = 1;
					return 1;
				} else {
					if (garpdb_yes == 1) {
						return 1;
					} else {
						INSTANT_USER_MSG("Gratuitous ARP disabled for %s ~ try running with --garpdb\n",tempip);
						current_skinnycall->fwd_set = 0;
						return 0;
					}
				}

			} else {
				dialing = 0;
				int garp_retval = lookup_garp_yes(tempip);
				if (garp_retval == 1) {
					current_skinnycall->fwd_set = 1;
					return 1;
				} else {
					// Only do this if not running in -G mode
					if (garpdb_yes == 1) {
						return 1;
					} else {
						INSTANT_USER_MSG("Gratuitous ARP disabled for %s ~ try running with --garpdb\n",tempip);
						current_skinnycall->fwd_set = 0;
						return 0;
					}
				}
			}

			current_skinnycall = current_skinnycall->next_skinnycall;
		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}
	}

	return 1;

}
int check_rev_media_garp(SkinnyCallPtr *sPtr, char tempip[MAX_ASCII_ADDR_LEN], int passthrupartyid){
		
	/* If it's an IP Address in remote VLAN, then we are Ok
	*  This can be found by checking vettercap hosts List
	*	
	*  If It's an IP Address in source VLAN, then we need
	*  to make sure GARP is enabled; otherwise the rev 
	*  file should not be created
	*	
	*  First we need to find out if this IP address is dialing or dialed
	*/

	if (target_yes == 1) {
		return 1;
	}

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;
	int dialing;
	int remote_dialing;
	char remote_ip[MAX_ASCII_ADDR_LEN];
	int iplocal = 0;

	while (current_skinnycall != NULL) {

		if ((current_skinnycall->passthrupartyid1 == passthrupartyid)||(current_skinnycall->passthrupartyid2 == passthrupartyid)) {

			int compare = strcmp(tempip, current_skinnycall->dialingip);

			if (compare == 0) {

				dialing = 1;
				remote_dialing = 0;
				strcpy(remote_ip,current_skinnycall->dialedip);

				struct hosts_list *hl;
				char ip[MAX_ASCII_ADDR_LEN];
	
				LIST_FOREACH(hl, &GBL_HOSTLIST, next) {
		
					ip_addr_ntoa(&hl->ip, ip);
		
					int compare = strcmp(ip, remote_ip);
					if (compare == 0) {
						iplocal = 1;
			
						// check to see if GARP is enabled for this host
						int retval = lookup_garp_yes(remote_ip);
						if (retval == 1){
							current_skinnycall->rev_set = 1;
							return 1;
						} else {
							if (garpdb_yes == 1) {
								return 1;
							} else {
								//printf("Gratuitous ARP disabled for Dialing IP Phone %s located in source VLAN\n",remote_ip);
								current_skinnycall->rev_set = 0;
								return 0;
							}

						}

					}


				}
	
				if (iplocal != 1) {
					if (monitor_yes != 1) {
						current_skinnycall->rev_set = 1;
						return 1;
					}
				}

			} else {
				dialing = 0;
				remote_dialing = 1;
				strcpy(remote_ip,current_skinnycall->dialingip);
				//printf("Copying current_skinnycall->dialingip to remote_ip value of %s\n",remote_ip);

				struct hosts_list *hl;
				char ip[MAX_ASCII_ADDR_LEN];

				LIST_FOREACH(hl, &GBL_HOSTLIST, next) {
					
					ip_addr_ntoa(&hl->ip, ip);
					
					int compare = strcmp(ip, remote_ip);
					if (compare == 0) {
						
						iplocal = 1;

						// check to see if GARP is enabled for this host
						int retval = lookup_garp_yes(remote_ip);
						if (retval == 1){
							current_skinnycall->rev_set = 1;
							return 1;
						} else {
							if (garpdb_yes == 1) {
								return 1;
							} else {
								//printf("Gratuitous ARP disabled for Dialed IP Phone %s located in source VLAN\n",remote_ip);
								current_skinnycall->rev_set = 0;
								return 0;
							}
						}

					}

				}
				
				if (iplocal != 1) {
					if (monitor_yes != 1) {

						current_skinnycall->rev_set = 1;
						return 1;

					} else {
						// Monitor Mode Enabled
						return 1;
					}
				}

			}

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}
	return 1;
}
void set_both_good(int passthrupartyid, SkinnyCallPtr *sPtr) {

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if ((current_skinnycall->passthrupartyid1 == passthrupartyid)||(current_skinnycall->passthrupartyid2 == passthrupartyid)) {

			current_skinnycall->wav_set = 1;
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

}
void handlerstartmediatransmission(SkinnyCallPtr *sPtr, int confid, int passthrupartyid, char dst_ip[MAX_ASCII_ADDR_LEN], char remote_ip[MAX_ASCII_ADDR_LEN], int rtpport){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	/* add arp reply poisoning */
	/* Send spoofed arp reply packet to the MAC address of dst_ip
	telling the remote_ip's mac address is this station
	Sender MAC Address:  <UCSniff station's MAC address>
	Sender IP Address:  <remote_ip>
	Target MAC Address: <remote_ip>'s MAC address
	Target IP Address: <dst_ip> 
	*/
	if (garpdb_yes == 1) {
		INSTANT_USER_MSG("UCSniff running GARP Disablement bypass flood for IP Phone %s\n",dst_ip);
		if(voipdebug){
			fprintf(sccpfile,"UCSniff running GARP Disablement bypass flood for IP Phone %s\n",dst_ip);
			flush = fflush(sccpfile);
		}
        	char target_mac1[MAX_ASCII_ADDR_LEN];
        	getmacfromip(dst_ip,target_mac1);
		sendSpoofedArpReply(remote_ip,dst_ip,target_mac1);

		if(target_yes == 1){
			INSTANT_USER_MSG("UCSniff running GARP Disablement bypass flood for IP Phone %s\n",dst_ip);
			if(voipdebug){
				fprintf(sccpfile,"UCSniff running GARP Disablement bypass flood for IP Phone %s\n",dst_ip);
				flush = fflush(sccpfile);
			}
        		char target_mac2[MAX_ASCII_ADDR_LEN];
        		getmacfromip(remote_ip,target_mac2);
			sendSpoofedArpReply(dst_ip,remote_ip,target_mac2);
		}

	}
	
	while (current_skinnycall != NULL) {

		if (current_skinnycall->CallIdentifier == confid) {

			int compare = strcmp(current_skinnycall->dialingip, dst_ip);
			if (compare == 0) {
				strcpy(current_skinnycall->dialedip, remote_ip);
				//printf("Setting dialed IP to %s\n", current_skinnycall->dialedip);
			}


			int compare2 = strcmp(current_skinnycall->dialedip, dst_ip);
			if (compare2 == 0) {
				strcpy(current_skinnycall->dialingip, remote_ip);
				//printf("Setting dialing IP to %s vi startmediatransmission\n", current_skinnycall->dialingip);

			}
			
			/* Do some things here to add an RTP Dissector in case GARP is Disabled*/
			if (current_skinnycall->call_active != 1) {
			
				// Add directory entries for the users into node, if the user has a directory entry
				callerexistsindir = getcallerdirname(passthrupartyid, &first_skinnycall);

				calledexistsindir = getcalleddirname(passthrupartyid, &first_skinnycall);
		
				current_skinnycall->rtpport1 = rtpport;

				//printf("Adding rtp dissector via startmultimediatransmission, IP %s, port %d\n",remote_ip, rtpport);
				char *dissect_name;
				dissect_name = (char *) malloc((6 + MAX_ASCII_ADDR_LEN + 3) * sizeof(char));
				memset(dissect_name,'\0',(6+MAX_ASCII_ADDR_LEN+3));
				create_dissect_string(rtpport, remote_ip, dissect_name);
				dissect_add(dissect_name,APP_LAYER_UDP,rtpport,dissector_rtp_skinny);
				free(dissect_name);

				/* populate values in rtp structure */
				int return_rtp = check_rtp_entry(remote_ip);
				if (return_rtp < 0) {
					int return_value = populate_rtp_struct(return_rtp, remote_ip, current_skinnycall->passthrupartyid1, rtpport);
					if(return_value == -1) {
						return NULL;
					}
				} else {
					int return_value = populate_rtp_struct(return_rtp, remote_ip, current_skinnycall->passthrupartyid1, rtpport);
					if(return_value == -1) {
						return NULL;
					}
				}

				// Set the  IP monitored for RTP
				strcpy(current_skinnycall->rtpip,remote_ip);

				// Set call to active
				current_skinnycall->call_active = 1;
				
				printf("This is in handlerstatedmediatransmission \n Value of rtp_counter is %d\n",rtp_counter);
				printf("Value of return_rtp is %d\n",return_rtp);
				if(return_rtp== -1)
				current_skinnycall->myRTPCall = &rtp[rtp_counter-1];
				else
				current_skinnycall->myRTPCall = &rtp[return_rtp];

				// Set call start time
				gettimeofday(&current_skinnycall->start,0);

				// Set start_time_string
				gettimestring(1, &current_skinnycall->start_time_string);

				printcallstdout(&first_skinnycall, current_skinnycall->CallIdentifier);

				printtoskinnyfilestart(&first_skinnycall, current_skinnycall->CallIdentifier);


			} else {
				//printf("DEBUG call is already active!\n");
			}
			/*     end of doing things */

	
			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;
		
		}


	}


}
int handlercallinfomessage(SkinnyCallPtr *sPtr, int CallIdentifier, char callingp[24], char calledp[24], char dst_ip[MAX_ASCII_ADDR_LEN], int calltype){

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = *sPtr;

	while (current_skinnycall != NULL) {

		if (CallIdentifier == current_skinnycall->CCallIdentifier) {
			
			strcpy(current_skinnycall->ip2,dst_ip);
			strcpy(current_skinnycall->dialedip,dst_ip);

			// Set dialed and dialing number
			strcpy(current_skinnycall->dialingnumber,callingp);
			if(voipdebug){
				fprintf(sccpfile,"Setting calling party to %s\n",current_skinnycall->dialingnumber);
				flush = fflush(sccpfile);
			}
			if(strcmp("Unknown",calledp) != 0) {
				strcpy(current_skinnycall->dialednumber,calledp);
				//INSTANT_USER_MSG("Setting called party to %s\n",current_skinnycall->dialednumber);
				if(voipdebug){
					fprintf(sccpfile,"Setting calling party to %s\n",current_skinnycall->dialingnumber);
					flush = fflush(sccpfile);
				}
			}else{
				strcpy(current_skinnycall->dialednumber,"Unknown");
				if(voipdebug){
					fprintf(sccpfile,"Setting calling party to %s\n",current_skinnycall->dialingnumber);
					flush = fflush(sccpfile);
				}
			}
		
			current_skinnycall = current_skinnycall->next_skinnycall;

			return 1;

		} else if (CallIdentifier == current_skinnycall->CallIdentifier) {

			// If dialing phone is only phone in VLAN
			if(strcmp("Unknown",calledp) != 0) {
				strcpy(current_skinnycall->dialednumber,calledp);
				//printf("Setting called party to %s\n",current_skinnycall->dialednumber);
			}else{
				strcpy(current_skinnycall->dialednumber,"Unknown");
			}
			strcpy(current_skinnycall->dialingnumber,callingp);

			current_skinnycall = current_skinnycall->next_skinnycall;

		} else {

			current_skinnycall = current_skinnycall->next_skinnycall;

		}

	}

	return 0;

}
void getpartystring(u_char callstring[24], char returnstring[24]){

	int i;
	int retval = 1;

	i = 0;
	while (( i < 24) && (retval == 1)) {

		retval = checkdigit(callstring[i]);

		if (retval == 1) {
			
			returnstring[i] = callstring[i];

		}

		i++;

	}

	returnstring[i - 1] = '\0';
}
struct digitbuffer partygetstrings(char remainingStr[24], int splength){
//void partygetstrings(char remainingStr[24], int splength, struct digitbuffer retbuffer){

	int i;
	int retval = 1;
	char returnstring1[24];
	char returnstring2[24];

	struct digitbuffer retbuffer;

	//printf("splength is %d, printing all digits\n",splength);
	/*int k;
	for (k = 0; k < splength; k++){
		printf("%02X ", remainingStr[k]);
	}
	printf("\n");*/ 
	

	i = 0;
	//printf("returnstring1: \n");
	while (( i < splength) && (retval == 1)) {

		retval = checkdigit(remainingStr[i]);

		if (retval == 1) {
			returnstring1[i] = remainingStr[i];
		}

		i++;

	}

	returnstring1[i - 1] = '\0';
	strcpy(retbuffer.string1,returnstring1);
	//printf("returnstring1:  %s\n",retbuffer.string1);

	int test;
	test = checkdigit(remainingStr[i]);

	// Clear out if there is still an extra 0x00 byte
	while (checkdigit(remainingStr[i]) == 0) {
		i++;
		if (i > splength) {
			strcpy(retbuffer.string2,"Unknown");
			return retbuffer;
		}
	}

	retval = 1;
	int a = 0;

	while ((i < splength) && (retval == 1)){

		retval = checkdigit(remainingStr[i]);

		if (retval == 1) {

			returnstring2[a] = remainingStr[i];

		}
		i++;
		a++;
	}

	returnstring2[a - 1] = '\0';
	if(strcmp(returnstring1,returnstring2) == 0){
		strcpy(retbuffer.string2,"Unknown");
		//memset(retbuffer.string2,'\0',sizeof(retbuffer.string2));

	} else {
		strcpy(retbuffer.string2,returnstring2);
		//printf("returnstring2:  %s\n",returnstring2);
	}
	return retbuffer;

}
