#include <targets.h>
#include <ec_siplog.h>
#include <tftp_client.h>

sipDB *firstSipCall = NULL;
sipDB *lastSipCall = NULL;

/* Call counter */
int call_number = 0;

/* Add, copy and delete a sip call */
sipDB* createNewSipCall(char *, char *, struct packet_object *, u_char *);
static sipDB* addNewSipCall();
static void copySipDB(sipDB *, char *, char *, struct packet_object *, u_char *);
void deleteSipCall(sipDB *);
void initialize_uri_struct();
void copyAudioCodecParams(sipDB*, char *, char, char, int);
void copyVideoCodecParams(sipDB*, char *, char *, char, char);
static void initialize_sipdb(sipDB *);
void closeSipRTPCall();

void initialize_uri_struct(){

        memset(uri_obj.from_uri,'\0',sizeof(uri_obj.from_uri));
        memset(uri_obj.to_uri,'\0',sizeof(uri_obj.to_uri));
        memset(uri_obj.src_extension,'\0',sizeof(uri_obj.src_extension));
        memset(uri_obj.dst_extension,'\0',sizeof(uri_obj.dst_extension)) ;
        memset(uri_obj.phone_type,'\0',sizeof(uri_obj.phone_type));
		memset(uri_obj.from_tag,'\0',sizeof(uri_obj.from_tag));
		memset(uri_obj.to_tag, '\0', sizeof(uri_obj.to_tag));
        uri_obj.kpml_enabled = 0;
        uri_obj.media_src_port = 0;
        uri_obj.video_port = 0;
        uri_obj.cseq = 0;
        memset(uri_obj.call_id,'\0',sizeof(uri_obj.call_id));
        memset(uri_obj.ip_addr,'\0',sizeof(uri_obj.ip_addr));
        memset(uri_obj.cseq_method,'\0',sizeof(uri_obj.cseq_method));
        uri_obj.ocsIM = NULL;
}

static void initialize_sipdb(sipDB *currentSipCall){

        SIPLOG("Initializing the new sip call structure\n");

        /* IP and Transport information */
        memset(currentSipCall -> src_ip, '\0', sizeof(currentSipCall->src_ip));
        memset(currentSipCall -> dst_ip, '\0', sizeof(currentSipCall->dst_ip));
        currentSipCall -> src_port = 0;
        currentSipCall -> dst_port = 0;

        /* SIP header variables */
        memset(currentSipCall -> src_extension, '\0', sizeof(currentSipCall->src_extension));
        memset(currentSipCall -> dst_extension, '\0', sizeof(currentSipCall->dst_extension));
        memset(currentSipCall -> call_id, '\0', sizeof(currentSipCall->call_id));
        currentSipCall -> cseq = 0;
        memset(currentSipCall -> user_agent, '\0', sizeof(currentSipCall->user_agent));
        memset(currentSipCall -> to_tag, '\0', sizeof(currentSipCall->to_tag));
        memset(currentSipCall -> from_tag, '\0', sizeof(currentSipCall->from_tag));
        memset(currentSipCall -> contact, '\0', sizeof(currentSipCall->contact));
        memset(currentSipCall -> remote_extension, '\0', sizeof(currentSipCall->remote_extension));

        /* RTP Audio */
        currentSipCall -> media_src_port = 0;
        currentSipCall -> media_dst_port = 0;

        /* Media IP */
        memset(currentSipCall -> media_src_ip, '\0', sizeof(currentSipCall->media_src_ip));
        memset(currentSipCall -> media_dst_ip, '\0', sizeof(currentSipCall->media_dst_ip));

        /* Media(video) port */
        currentSipCall -> video_src_port = 0;
        currentSipCall -> video_dst_port = 0;

        /* Call Number */
        currentSipCall -> call_number = 0;

        /* Cisco specific variables */
        currentSipCall -> kpml_enabled = 0;

		/* Codec and Payload types */
		currentSipCall->audioCodecType = -1;
		currentSipCall->audioPayloadType = -1;
		currentSipCall->videoCodecType = -1;
		currentSipCall->videoPayloadType = -1;
		currentSipCall->g726SampleSize = 0;

		/*RTPMAP/FMTP strings */
		currentSipCall->audioRTPMAP = NULL;
		currentSipCall->videoRTPMAP = NULL;
		currentSipCall->videoFMTP = NULL;

	 /* Flags */
        currentSipCall -> remoteParty = 0;
        currentSipCall -> invite_active = 0;
        currentSipCall -> okay_active = 0;
        currentSipCall -> ack_active = 0;
        currentSipCall -> session_active = 0;
        currentSipCall -> diff_vlan = 0;
        currentSipCall -> server_reinvite = 0;
        currentSipCall -> server_reinviteNoSDP = 0;
        currentSipCall -> server_reinviteOkay = 0;
        currentSipCall -> server_reinviteAck = 0;
        currentSipCall -> server_reinviteCSeq = 0;
        currentSipCall -> client_reinvite = 0;
        currentSipCall -> client_reinviteNoSDP = 0;
        currentSipCall -> client_reinviteOkay = 0;
        currentSipCall -> client_reinviteAck = 0;
        currentSipCall -> client_reinviteCSeq = 0;

        currentSipCall -> myRTPCall = NULL;

        /* Self-referential linked list structure */
        //currentSipCall -> next = NULL;
        //currentSipCall -> previous = NULL;
}

static void copySipDB(sipDB *currentSipCall, char *temp, char *temp1, struct packet_object *packet,u_char *ptr){

	SIPLOG("Copying the parsed values to the allocated new sip call structure\n");

        memcpy(currentSipCall -> src_ip, temp, MAX_ASCII_ADDR_LEN);
        memcpy(currentSipCall -> dst_ip, temp1, MAX_ASCII_ADDR_LEN);
        currentSipCall -> src_port = ntohs(packet->L4.src);
        currentSipCall -> dst_port = ntohs(packet->L4.dst);
        strncpy(currentSipCall -> src_extension, uri_obj.src_extension, sizeof(currentSipCall -> src_extension));
        strncpy(currentSipCall -> dst_extension, uri_obj.dst_extension, sizeof(currentSipCall -> dst_extension));
        memcpy(currentSipCall -> call_id, uri_obj.call_id, sizeof(currentSipCall -> call_id));
        memcpy(currentSipCall -> user_agent, uri_obj.phone_type, sizeof(uri_obj.phone_type));
        currentSipCall -> kpml_enabled = uri_obj.kpml_enabled;
        currentSipCall -> media_src_port = uri_obj.media_src_port;
		memcpy(currentSipCall->from_tag,uri_obj.from_tag,sizeof(currentSipCall->from_tag));
		memcpy(currentSipCall->to_tag,uri_obj.to_tag,sizeof(currentSipCall->to_tag));
        currentSipCall -> video_src_port = uri_obj.video_port;
        currentSipCall -> cseq = uri_obj.cseq;

        /* Check if the source IP address is from the same LAN or VLAN */
        int return_local = check_local_ip(currentSipCall -> src_ip);
        if(return_local == 1)
                currentSipCall -> diff_vlan = 0;
        else
                currentSipCall -> diff_vlan = 1;

        /* Looks up the corporate directory structure for the username of the extension */
        int return_val = match_corporate_directory(uri_obj.src_extension);
        if(return_val < 0){
        	/* Parse from-uri display name */
        }
        else{
		memcpy(currentSipCall->src_username,directory[return_val].name,sizeof(currentSipCall->src_username));
        }

        int dst_directory = match_corporate_directory(uri_obj.dst_extension);
        if(dst_directory < 0){
        	/* Parse to-uri display name */
        }
	
	else{
                memcpy(currentSipCall->dst_username,directory[dst_directory].name,sizeof(currentSipCall->dst_username));
        }

        /* If there is a sdp message body in the sip invite, update the media src ip */
        int return_ip = decode_dialed_ip(ptr);
        if(return_ip >= 0)
        	memcpy(currentSipCall -> media_src_ip, uri_obj.ip_addr, MAX_ASCII_ADDR_LEN);

        gettimeofday(&(currentSipCall->start),0);
        get_time_string(1,currentSipCall->start_time_string);

        call_number = call_number + 1;
        currentSipCall -> call_number = call_number;
        currentSipCall -> invite_active = 1;
}
		
static sipDB * addNewSipCall(){

        //SIPLOG("Inside addNewSipCall\n");
        sipDB *currentSipCall = NULL;
        currentSipCall = (sipDB*) calloc(sizeof(sipDB),1);
        if(currentSipCall == NULL){
                SIPLOG("Error: Failed to allocate memory for the new SIP call\n");
                INSTANT_USER_MSG("Error: Failed to allocate memory for the new SIP call\n");
                return NULL;
        }

        /* first sip call */
        if(firstSipCall == NULL){
                //SIPLOG("Adding the firstSipCall\n");
                firstSipCall = currentSipCall;
                currentSipCall -> next = NULL;
                currentSipCall -> previous = NULL;
                lastSipCall = currentSipCall;
        }
        /* Other sip calls are added to the end of the list */
        else{
                //SIPLOG("Adding other sip calls to the end of the linked list\n");
                lastSipCall -> next = currentSipCall;
                currentSipCall -> previous = lastSipCall;
                currentSipCall -> next = NULL;
                lastSipCall = currentSipCall;
        }

        return currentSipCall;
}

sipDB* createNewSipCall(char *temp, char *temp1, struct packet_object *packet, u_char *ptr){

        //SIPLOG("Inside createNewSipCall\n");
        sipDB* currentSipCall = NULL;
        /* Adds a new call to the sip structure */
        currentSipCall = addNewSipCall();
        if(currentSipCall == NULL)
                return NULL;

        /* Initialize the sip structure */
        initialize_sipdb(currentSipCall);

        /* Copy the parsed values to the allocated call structure */
        copySipDB(currentSipCall, temp, temp1, packet, ptr);

        return currentSipCall;
}

void deleteSipCall(sipDB *currentSipCall){

        if(currentSipCall == lastSipCall && currentSipCall == firstSipCall){

                //SIPLOG("Deleting one and only sip call\n");
                lastSipCall=firstSipCall = NULL;
                free(currentSipCall);

        }
        else if(currentSipCall == lastSipCall){

                //SIPLOG("Deleting the last sip call\n");
                if(currentSipCall->previous != NULL)
                        (currentSipCall->previous)->next = NULL;
                lastSipCall = currentSipCall->previous;
                free(currentSipCall);
        }
        else if(currentSipCall == firstSipCall ){

                //SIPLOG("Deleting the first sip call\n");
                firstSipCall = currentSipCall->next;
                if(currentSipCall->next != NULL)
                        (currentSipCall->next)->previous = NULL;
                free(currentSipCall);
        }
        else{
                //SIPLOG("Deleting the sip call in the middle\n");
                (currentSipCall->next)->previous = currentSipCall->previous;
                (currentSipCall->previous)->next = currentSipCall->next;
                free(currentSipCall);
        }      
}

void closeSipRTPCall(){

	sipDB* currentSipCall = firstSipCall;
	while(currentSipCall != NULL){

		sipDB* tempSipCall = currentSipCall->next;
		free(currentSipCall);
		currentSipCall = tempSipCall;
	}
	firstSipCall = NULL;lastSipCall = NULL;
	closeRTPCall();
}

void copyAudioCodecParams(sipDB *currentSipCall,char *rtpmap, char audioCodecType, char audioPayloadType, int g726SampleSize){

	if(currentSipCall->audioRTPMAP != NULL){

		currentSipCall->audioRTPMAP = (char *)calloc(strlen(rtpmap) + 1, sizeof(char));
		if(currentSipCall->audioRTPMAP == NULL){

			INSTANT_USER_MSG("Error: Allocating memory for rtpmap \n");
			return;
		}
	
		strncpy(currentSipCall->audioRTPMAP, rtpmap, strlen(rtpmap));
	}

	currentSipCall->g726SampleSize = g726SampleSize;
	currentSipCall->audioCodecType = audioCodecType;
	currentSipCall->audioPayloadType = audioPayloadType;

}

void copyVideoCodecParams(sipDB* currentSipCall, char *vrtpmap, char *vfmtp, char videoCodecType, char videoPayloadType){

	currentSipCall->videoRTPMAP = (char *)calloc(strlen(vrtpmap) + 1, sizeof(char));
	if(currentSipCall->videoRTPMAP == NULL){

		INSTANT_USER_MSG("Error: Allocating memory for rtpmap\n");
		return;
	}

	strncpy(currentSipCall->videoRTPMAP, vrtpmap, strlen(vrtpmap));

	if(vfmtp != NULL){

		currentSipCall->videoFMTP = (char*)calloc(strlen(vfmtp) + 1, sizeof(char));
		if(currentSipCall->videoFMTP == NULL){

			INSTANT_USER_MSG("Error: Allocating memory for fmtp\n");
			return;
		}

		strncpy(currentSipCall->videoFMTP, vfmtp, strlen(vfmtp));
	}

	currentSipCall->videoCodecType = videoCodecType;
	currentSipCall->videoPayloadType = videoPayloadType;
}
	
