#include <targets.h>
#include <ec_siplog.h>
#include <tftp_client.h>	/* For the corporate directory */

int check_reinvite_server(sipDB*,u_char *);
int check_serverReInviteOkay(sipDB*,u_char *);
int check_serverReInviteAck(sipDB*,u_char *);
int check_reinvite_client(sipDB*,u_char *);
int check_clientReInviteOkay(sipDB*,u_char *);
int check_clientReInviteAck(sipDB*,u_char *);
void clearServerReInvite(sipDB *);
void clearClientReInvite(sipDB *);

sipDB* match_bye_request();
sipDB* match_cancel_request();
sipDB* matchsipDBInvite(char *,int);
sipDB* matchsipOkay(char *,int);
sipDB* matchsipAck(char *,int);

extern sipDB *firstSipCall;
extern sipDB *lastSipCall;

/* Searches the corporate directory for the given user and returns the index */
int match_corporate_directory(char *);

//from somewhere else
extern int structure_size;

static void handle_changed_media(char curIp[], int* curAudioPort, int* curVideoPort, sipDB* currentSipCall)
{
    char oldIp[MAX_ASCII_ADDR_LEN+1] = { '\0' };
    int oldVideoPort = 0;
    int oldAudioPort = 0;

    if(strcmp(uri_obj.ip_addr,curIp) != 0){
        SIPLOG("Updating the media ip from %s to %s \n", curIp, uri_obj.ip_addr);
        strncpy(oldIp, curIp, MAX_ASCII_ADDR_LEN);
        memset(curIp,'\0',MAX_ASCII_ADDR_LEN);
        strncpy(curIp,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
        sip_targetlookup(currentSipCall);
    }

    if(*curAudioPort != uri_obj.media_src_port){
        SIPLOG("Updating the media port from %d to %d \n", *curAudioPort, uri_obj.media_src_port);
        oldAudioPort = *curAudioPort;
        *curAudioPort = uri_obj.media_src_port;
    }
    if(*curVideoPort != uri_obj.video_port){
        SIPLOG("Updating the video port from %d to %d \n", *curVideoPort, uri_obj.media_src_port);
        oldVideoPort = *curVideoPort;
        *curVideoPort = uri_obj.video_port;
    }
    if(currentSipCall -> diff_vlan == 1){
        // Have to update the dissector table to sniff for the updated ip address and port
        SIPLOG("Different VLAN detected.  Updating dissectors.\n");
        if(strlen(oldIp) || oldAudioPort || oldVideoPort) {
            updateMediaInfoForExistingCall(oldIp, oldAudioPort, oldVideoPort, currentSipCall, 
                curIp, *curAudioPort, *curVideoPort);
        }
    }
}

int check_reinvite_server(sipDB* currentSipCall,u_char *msg){

    //SIPLOG("Inside check_reinvite_server\n");
    if(currentSipCall == NULL)
        return -1;

    if(currentSipCall->server_reinvite == 1 && currentSipCall->server_reinviteAck == 1 && currentSipCall->server_reinviteCSeq < uri_obj.cseq){
        SIPLOG("A new server reinvite\n");
        clearServerReInvite(currentSipCall);
    }
    else if(currentSipCall->server_reinvite == 1 && currentSipCall->server_reinviteCSeq == uri_obj.cseq){
        SIPLOG("Skipping duplicate server reinvite\n");
        return 1;
    }

    currentSipCall->server_reinvite = 1;
    currentSipCall->server_reinviteCSeq = uri_obj.cseq;

    int return_media = decode_dialed_ip(msg);
    if(return_media < 0){
        currentSipCall->server_reinviteNoSDP = 1;
        SIPLOG("Failed: parsing decode_dialed_ip\n");
        return 1;
    }

    int return_port = decode_media_port(msg);
    if(return_port < 0){
        currentSipCall->server_reinviteNoSDP = 1;
        SIPLOG("Failed: parsing decode_media_port\n");
        return 1;
    }

    int return_vport = decode_video_port(msg);
    if(return_vport < 0) {
        currentSipCall->server_reinviteNoSDP = 1;
        SIPLOG("Failed: parsing decode_video_port\n");
        return 1;
    }

    if((!strcmp(uri_obj.ip_addr,currentSipCall->media_dst_ip)) && (currentSipCall->media_dst_port == uri_obj.media_src_port)){
        SIPLOG("Media port and Media IP address has not changed in server reinvite\n");
        return 1;
    }
    else{
	int retHold = checkreInviteHold(msg);
        if(retHold == 1){
            SIPLOG("Server reinvite requesting a call hold\n");
            return 1;
        }

        SIPLOG("SIP Call(%d): The server has sent a re-Invite and the called party IP address is %s,"
            " media port is %d, and video port is %d\n",
            currentSipCall->call_number, 
            uri_obj.ip_addr,
            uri_obj.media_src_port,
            uri_obj.video_port);
        //printf("SIP Call(%d): The server has sent a re-Invite and the called party IP address is %s and media port is %d \n",currentSipCall->call_number, uri_obj.ip_addr,uri_obj.media_src_port);
        handle_changed_media(currentSipCall->media_dst_ip, &currentSipCall->media_dst_port, &currentSipCall->video_dst_port,
            currentSipCall);
/*        if(strcmp(uri_obj.ip_addr,currentSipCall->media_dst_ip) != 0){
            SIPLOG("Updating the media_dst_ip from %s to %s \n",currentSipCall->media_dst_ip,uri_obj.ip_addr);
            strncpy(oldIp, currentSipCall->media_dst_ip, MAX_ASCII_ADDR_LEN);
            memset(currentSipCall->media_dst_ip,'\0',MAX_ASCII_ADDR_LEN);
            strncpy(currentSipCall->media_dst_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
            sip_targetlookup(currentSipCall);
        }
        if(currentSipCall->media_dst_port != uri_obj.media_src_port){
            SIPLOG("Updating the media_dst_port from %d to %d \n", currentSipCall->media_dst_port,uri_obj.media_src_port);
            oldAudioPort = currentSipCall->media_dst_port;
            currentSipCall->media_dst_port = uri_obj.media_src_port;
        }
        if(currentSipCall->video_dst_port != uri_obj.video_port){
            SIPLOG("Updating the video_dst_port from %d to %d \n", currentSipCall->video_dst_port,uri_obj.media_src_port);
            oldVideoPort = currentSipCall->video_dst_port;
            currentSipCall->video_dst_port = uri_obj.video_port;
        }
        if(currentSipCall -> diff_vlan == 1){
                    // Have to update the dissector table to sniff for the updated ip address and port
            SIPLOG("Different VLAN detected.  Updating dissectors.\n");
            if(strlen(oldIp) || oldAudioPort || oldVideoPort) {
                //updateMediaInfoForExistingCall(oldIp, oldAudioPort, oldVideoPort, currentSipCall);
            }
        }
 */
        printtoSipLog(currentSipCall,NULL,"re-Invite");
        return 1;
    }
    return 1;
}

int check_serverReInviteOkay(sipDB* currentSipCall,u_char *msg){

        //SIPLOG("Inside check_serverReInviteOkay\n");

        if(currentSipCall == NULL)
                return -1;

        currentSipCall->server_reinviteOkay = 1;

        int return_ip = decode_dialed_ip(msg);
        if(return_ip < 0){
                //SIPLOG("Failed: parsing decode_dialed_ip\n");
                return 1;
        }

        int return_port = decode_media_port(msg);
        if(return_port < 0){
                //SIPLOG("Failed: parsing decode_media_port\n");
                return 1;
        }

        if((!strcmp(currentSipCall->media_src_ip,uri_obj.ip_addr)) && currentSipCall->media_src_port == uri_obj.media_src_port){
                SIPLOG("200 OK, serverReInvite media_src_ip and media_src_port has not changed\n");
                return 1;
        }
        else{
                char oldIp[MAX_ASCII_ADDR_LEN+1] = { '\0' };
                int oldVideoPort = 0;
                int oldAudioPort = 0;

                SIPLOG("SIP Call(%d): Server has sent a re-Invite and client has updated its media IP address %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                // printf("SIP Call(%d): Server has sent a re-Invite and client has updated its media IP address %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                handle_changed_media(currentSipCall->media_src_ip, &currentSipCall->media_src_port, &currentSipCall->video_src_port,
                    currentSipCall);
/*

                if(strcmp(currentSipCall->media_src_ip,uri_obj.ip_addr) != 0){
                        SIPLOG("Updating media_src_ip from %s to %s\n",currentSipCall->media_src_ip,uri_obj.ip_addr);
                        
                        memset(currentSipCall->media_src_ip,'\0',sizeof(currentSipCall->media_src_ip));
                        strncpy(currentSipCall->media_src_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
                        sip_targetlookup(currentSipCall);
                }
                if(currentSipCall->media_src_port != uri_obj.media_src_port){

			SIPLOG("Updating media_src_port from %d to %d\n",currentSipCall->media_src_port,uri_obj.media_src_port);
                         currentSipCall->media_src_port = uri_obj.media_src_port;
                }

                if(currentSipCall->diff_vlan == 0){
                        // Update the dissector table and RTP structure
                }
 */               
        }
        printtoSipLog(currentSipCall,NULL,"re-Invite 200 Okay");
        return 1;
}

int check_serverReInviteAck(sipDB* currentSipCall,u_char *msg){

        //SIPLOG("Inside check_serverReinviteAck\n");
        if(currentSipCall == NULL)
                return -1;

        if(currentSipCall->server_reinviteOkay != 1){
                SIPLOG("server reinvite ACK received not 200 OK, clearing serverReInvite\n");
                clearServerReInvite(currentSipCall);
                return 1;
        }

        currentSipCall->server_reinviteAck = 1;

        if(currentSipCall->server_reinviteNoSDP == 1){

                int return_media = decode_dialed_ip(msg);
                if(return_media < 0){
                        //SIPLOG("Failed: parsing decode_dialed_ip\n");
                        return 1;
                }

                int return_port = decode_media_port(msg);
                if(return_port < 0){
                        //SIPLOG("Failed: parsing decode_media_port\n");
                        return 1;
                }

                if((!strcmp(uri_obj.ip_addr,currentSipCall->media_dst_ip)) && (uri_obj.media_src_port == currentSipCall->media_dst_port)){
                        SIPLOG("Ack serverreInvite, media_dst_ip and media_dst_port has not changed\n");
                        return 1;
                }

                handle_changed_media(currentSipCall->media_dst_ip, &currentSipCall->media_dst_port, &currentSipCall->video_dst_port,
                    currentSipCall);

/*
                if(strcmp(uri_obj.ip_addr,currentSipCall->media_dst_ip) != 0){

                        int retHold = checkreInviteHold(msg);
                        if(retHold == 1){
                                SIPLOG("serverreInvite Ack call is put on hold\n");
				SIPLOG("serverreInvite Ack call is put on hold\n");
                                return 1;
                        }

                        SIPLOG("SIP Call(%d): The server has sent a re-Invite and the called party IP address is %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                        //printf("SIP Call(%d): The server has sent a re-Invite and the called party IP address is %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);

                        SIPLOG("Updating media_dst_ip from %s to %s\n",currentSipCall->media_dst_ip,uri_obj.ip_addr);
                        memset(currentSipCall->media_dst_ip,'\0',MAX_ASCII_ADDR_LEN);
                        memcpy(currentSipCall->media_dst_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
                        sip_targetlookup(currentSipCall);
                }

                if(currentSipCall->media_dst_port != uri_obj.media_src_port){
                        SIPLOG("Updating media_dst_port from %d to %d\n",currentSipCall->media_dst_port,uri_obj.media_src_port);
                        currentSipCall->media_dst_port = uri_obj.media_src_port;
                }

                if(currentSipCall -> diff_vlan == 1){
                        // Have to update the dissector table to sniff for the updated ip address and port
                }
 */
        }
        printtoSipLog(currentSipCall,NULL,"re-Invite ACK");
        return 1;
}

int check_reinvite_client(sipDB* currentSipCall,u_char *msg){

    //SIPLOG("Inside check_reinvite_client\n");

    if(currentSipCall->cseq == uri_obj.cseq){
            SIPLOG("Duplicate Invite sent by client \n");
            return 1;
    }
    else if(currentSipCall->client_reinvite == 1 && currentSipCall->client_reinviteAck == 1 && currentSipCall->client_reinviteCSeq < uri_obj.cseq){
            SIPLOG("New re-Invite sent by client, erasing old client reInvite\n");
            clearClientReInvite(currentSipCall);
    }
    else if(currentSipCall->client_reinvite == 1 && currentSipCall->client_reinviteCSeq == uri_obj.cseq){
            SIPLOG("Duplicate client reInvite \n");
            return 1;
    }

    currentSipCall->client_reinvite = 1;
    currentSipCall->client_reinviteCSeq = uri_obj.cseq;

    int return_media = decode_dialed_ip(msg);
    if(return_media < 0){
            SIPLOG("Failed: Parsing decode_dialed_ip\n");
            currentSipCall->client_reinviteNoSDP = 1;
            return 1;
    }

    int return_port = decode_media_port(msg);
    if(return_port < 0){
             SIPLOG("Failed: Parsing decode_media_port\n");
             currentSipCall->client_reinviteNoSDP = 1;
             return 1;
    }
    int return_vport = decode_video_port(msg);
    if(return_vport < 0){
             SIPLOG("Failed: Parsing decode_video_port\n");
             currentSipCall->client_reinviteNoSDP = 1;
             return 1;
    }


    if((!strcmp(uri_obj.ip_addr,currentSipCall->media_src_ip)) && (uri_obj.media_src_port == currentSipCall->media_src_port)){
            //SIPLOG("Client-reinvite, media IP address and port number has not changed \n");
            return 1;
    }
    else{
	/* Skipping re-Invite for hold */
        int retHold = checkreInviteHold(msg);
        if(retHold == 1){
                SIPLOG("Client has sent a re-Invite by putting the call on hold\n");
                return 1;
        }
        
        handle_changed_media(currentSipCall->media_src_ip, &currentSipCall->media_src_port, &currentSipCall->video_src_port,
            currentSipCall);
/*
        char oldIp[MAX_ASCII_ADDR_LEN+1] = { '\0' };
        int oldVideoPort = 0;
        int oldAudioPort = 0;
        
        SIPLOG("SIP Call(%d): The client has sent a re-Invite and the calling party IP address is %s, "
           "media port is %d, and video port is %d\n",
           currentSipCall->call_number,
           uri_obj.ip_addr,
           uri_obj.media_src_port,
           uri_obj.video_port);
        if(strcmp(uri_obj.ip_addr,currentSipCall->media_src_ip) != 0){
            SIPLOG("Updating media_src_ip from %s to %s\n",currentSipCall->media_src_ip,uri_obj.ip_addr);
            strncpy(oldIp, currentSipCall->media_src_ip, MAX_ASCII_ADDR_LEN);
            memset(currentSipCall->media_src_ip,'\0',MAX_ASCII_ADDR_LEN);
            strncpy(currentSipCall->media_src_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
            sip_targetlookup(currentSipCall);
        }
            
        if(currentSipCall->media_src_port != uri_obj.media_src_port){
            SIPLOG("Updating media_src_port from %d to %d\n",currentSipCall->media_src_port,uri_obj.media_src_port);
            oldAudioPort = currentSipCall->media_src_port;
            currentSipCall->media_src_port = uri_obj.media_src_port;
        }

        if(currentSipCall->video_src_port != uri_obj.video_port){
            SIPLOG("Updating video_src_port from %d to %d\n",currentSipCall->video_src_port,uri_obj.video_port);
            oldVideoPort = currentSipCall->video_src_port;
            currentSipCall->video_src_port = uri_obj.video_port;
        }

        if(currentSipCall -> diff_vlan == 1){
            // Have to update the dissector table to sniff for the updated ip address and port
            SIPLOG("Different VLAN detected.  Updating dissectors.\n");
            if(strlen(oldIp) || oldAudioPort || oldVideoPort) {
                updateMediaInfoForExistingCall(oldIp, oldAudioPort, oldVideoPort, currentSipCall);
            }
        }
 */
        printtoSipLog(currentSipCall,NULL,"re-Invite");
        return 1;
    }
    return 1;
}

int check_clientReInviteOkay(sipDB* currentSipCall,u_char *msg){

        //SIPLOG("Inside check_clientReInviteOkay\n");

        if(currentSipCall == NULL)
                return -1;

        currentSipCall->client_reinviteOkay = 1;

        int return_ip = decode_dialed_ip(msg);
        if(return_ip < 0){
                //SIPLOG("Failed: parsing decode_dialed_ip\n");
                return 1;
        }

        int return_port = decode_media_port(msg);
        if(return_port < 0){
                SIPLOG("Failed: parsing decode_media_port\n");
                return 1;
        }

        if((!strcmp(currentSipCall->media_dst_ip,uri_obj.ip_addr)) && currentSipCall->media_dst_port == uri_obj.media_src_port){
                SIPLOG("200 OK, clientReInviteOkay media_dst_ip and media_dst_port has not changed\n");
                return 1;
        }
        else{
                SIPLOG("SIP Call(%d): Server has sent a re-Invite and client has "
                    "updated its media IP address %s and media port is %d \n",
                    currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);

                handle_changed_media(currentSipCall->media_dst_ip, &currentSipCall->media_dst_port, &currentSipCall->video_dst_port,
                    currentSipCall);
/*
                //printf("SIP Call(%d): Server has sent a re-Invite and client has updated its media IP address %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                if(strcmp(currentSipCall->media_dst_ip,uri_obj.ip_addr) != 0){
                        SIPLOG("Updating media_dst_ip from %s to %s\n",currentSipCall->media_dst_ip,uri_obj.ip_addr);
                        memset(currentSipCall->media_dst_ip,'\0',sizeof(currentSipCall->media_dst_ip));
                        strncpy(currentSipCall->media_dst_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
                        sip_targetlookup(currentSipCall);
                }
		if(currentSipCall->media_dst_port != uri_obj.media_src_port){
                        SIPLOG("Updating media_dst_port from %d to %d\n",currentSipCall->media_dst_port,uri_obj.media_src_port);
                        currentSipCall->media_dst_port = uri_obj.media_src_port;
                }

                if(currentSipCall->diff_vlan == 1){
                        // Update the dissector table and RTP structure
                }
 */
        }
        printtoSipLog(currentSipCall,NULL,"re-Invite 200 Okay");
        return 1;
}

int check_clientReInviteAck(sipDB* currentSipCall,u_char *msg){

        //SIPLOG("Inside check_clientReInviteAck\n");

        if(currentSipCall == NULL)
                return -1;

        if(currentSipCall->client_reinviteOkay != 1){
                SIPLOG("client reInviteAck received but not an 200 OK\n");
                clearClientReInvite(currentSipCall);
        }

        if(currentSipCall->client_reinviteNoSDP == 1){

                int return_media = decode_dialed_ip(msg);
                if(return_media < 0){
                        //SIPLOG("Failed: parsing decode_dialed_ip\n");
                        return 1;
                }

                int return_port = decode_media_port(msg);
                if(return_port < 0){
                        //SIPLOG("Failed: parsing decode_media_port\n");
                        return 1;
                }

                if((!strcmp(uri_obj.ip_addr,currentSipCall->media_src_ip)) && (currentSipCall->media_src_port == uri_obj.media_src_port)){
                        SIPLOG("Ack, media_src_ip and media_src_port has not changed\n");
                        return 1;
                }
                SIPLOG("SIP Call(%d): Server has sent a re-Invite ACK and client has "
                    "updated its media IP address %s and media port is %d \n",
                    currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                handle_changed_media(currentSipCall->media_src_ip, &currentSipCall->media_src_port, &currentSipCall->video_src_port,
                    currentSipCall);
/*
		if(strcmp(uri_obj.ip_addr,currentSipCall->media_src_ip) != 0){

                        int retHold = checkreInviteHold(msg);
                        if(retHold == 1){
                                SIPLOG("Client sending reinviteAck after putting the call on hold\n");
                                return 1;
                        }
                        SIPLOG("SIP Call(%d): The client has sent a re-Invite and the calling party IP address is %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);
                        //printf("SIP Call(%d): The client has sent a re-Invite and the calling party IP address is %s and media port is %d \n",currentSipCall->call_number,uri_obj.ip_addr,uri_obj.media_src_port);

                        SIPLOG("Updating media_src_ip from %s to %s\n",currentSipCall->media_src_ip,uri_obj.ip_addr);
                        memset(currentSipCall->media_src_ip,'\0',MAX_ASCII_ADDR_LEN);
                        strncpy(currentSipCall->media_src_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
                        sip_targetlookup(currentSipCall);
                }

                if(currentSipCall->media_src_port != uri_obj.media_src_port){
                        SIPLOG("Updating media_src_port from %d to %d\n",currentSipCall->media_src_port,uri_obj.media_src_port);
                        currentSipCall->media_src_port = uri_obj.media_src_port;
                }

                if(currentSipCall -> diff_vlan == 0){
                        // Have to update the dissector table to sniff for the updated ip address and port
                }
 */
        }
        printtoSipLog(currentSipCall,NULL,"re-Invite ACK\n");
        return 1;
}
	
void clearServerReInvite(sipDB *currentSipCall){

        //SIPLOG("Clearing serverReInvite values\n");
        currentSipCall->server_reinvite = 0;
        currentSipCall->server_reinviteOkay = 0;
        currentSipCall->server_reinviteAck = 0;
        currentSipCall->server_reinviteCSeq = 0;
        currentSipCall->server_reinviteNoSDP = 0;
}

void clearClientReInvite(sipDB *currentSipCall){

        //SIPLOG("Clearing clientReInvite values\n");
        currentSipCall->client_reinvite = 0;
        currentSipCall->client_reinviteOkay = 0;
        currentSipCall->client_reinviteAck = 0;
        currentSipCall->client_reinviteCSeq = 0;
        currentSipCall->client_reinviteNoSDP = 0;
}

sipDB* matchsipOkay(char *buffer,int length){

        sipDB* currentSipCall = firstSipCall;

        while(currentSipCall != NULL){

                if((!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->invite_active == 1) && (currentSipCall->cseq == uri_obj.cseq)){
                        return currentSipCall;
                }
                else if((!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->client_reinvite == 1) && (currentSipCall->client_reinviteCSeq == uri_obj.cseq) && (currentSipCall->session_active == 1)){
                        strncpy(buffer,"client",length);
                        return currentSipCall;
                }
                else if((!strcmp(currentSipCall->from_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->server_reinvite == 1) && (currentSipCall->server_reinviteCSeq == uri_obj.cseq)&& (currentSipCall->session_active == 1)){
                        strncpy(buffer,"server",length);
                        return currentSipCall;
                }
                currentSipCall = currentSipCall->next;
        }
        return NULL;
}

sipDB* matchsipAck(char *buffer,int length){

        sipDB* currentSipCall = firstSipCall;
        while(currentSipCall != NULL){
				
				/* As we are not paring 4xx response. Nothing checking for to-tag just dst_extension */
SIPLOG("Test #1: currentSipCall->from_tag(%s) == uri_obj.from_tag(%s) AND\n"
	   "         currentSipCall->call_id(%s) == uri_obj.call_id(%s) AND\n"
       "         currentSipCall->invite_active(%d) == 1\n"
       "         currentSipCall->cseq(%d) == uri_obj.cseq(%d)\n"
       "         currentSipCall->dst_extension(%s) to uri_obj.dst_extension(%s)\n", 
	currentSipCall->from_tag, uri_obj.from_tag,
	currentSipCall->call_id, uri_obj.call_id,
	currentSipCall->invite_active,
	currentSipCall->cseq, uri_obj.cseq,
	currentSipCall->dst_extension, uri_obj.dst_extension);

SIPLOG("Test #2: currentSipCall->from_tag(%s) == uri_obj.from_tag(%s) AND\n"
	   "         currentSipCall->to_tag(%s) == uri_obj.to_tag(%s) AND\n"
	   "         currentSipCall->call_id(%s) == uri_obj.call_id(%s) AND\n"
       "         currentSipCall->session_active(%d) == 1\n"
       "         currentSipCall->client_reinvite(%d) == 1\n"
       "         currentSipCall->client_reinviteCSeq(%d) == uri_obj.cseq(%d)\n",
	currentSipCall->from_tag, uri_obj.from_tag,
	currentSipCall->to_tag, uri_obj.to_tag,
	currentSipCall->call_id, uri_obj.call_id,
	currentSipCall->session_active,
	currentSipCall->client_reinvite,
	currentSipCall->client_reinviteCSeq, uri_obj.cseq);

SIPLOG("Test #3: currentSipCall->from_tag(%s) == uri_obj.to_tag(%s) AND\n"
	   "         currentSipCall->to_tag(%s) == uri_obj.from_tag(%s) AND\n"
	   "         currentSipCall->call_id(%s) == uri_obj.call_id(%s) AND\n"
       "         currentSipCall->session_active(%d) == 1\n"
       "         currentSipCall->server_reinvite(%d) == 1\n"
       "         currentSipCall->server_reinviteCSeq(%d) == uri_obj.cseq(%d)\n",
	currentSipCall->from_tag, uri_obj.to_tag,
	currentSipCall->to_tag, uri_obj.to_tag,
	currentSipCall->call_id, uri_obj.call_id,
	currentSipCall->session_active,
	currentSipCall->server_reinvite,
	currentSipCall->server_reinviteCSeq, uri_obj.cseq);


                if((!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->invite_active == 1) && (currentSipCall->cseq == uri_obj.cseq) && (!strcmp(currentSipCall->dst_extension,uri_obj.dst_extension))){
						SIPLOG("Test #1 is true\n");
                        return currentSipCall;
                }
                else if((!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->session_active == 1) && (currentSipCall->client_reinvite == 1) && (currentSipCall->client_reinviteCSeq == uri_obj.cseq)){
                        strncpy(buffer,"client",length);
						SIPLOG("Test #2 is true\n");
                        return currentSipCall;
                }
                else if((!strcmp(currentSipCall->from_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->session_active == 1) && (currentSipCall->server_reinvite == 1) && (currentSipCall->server_reinviteCSeq == uri_obj.cseq)){
                        strncpy(buffer,"server",length);
						SIPLOG("Test #3 is true\n");
                        return currentSipCall;
                }
                currentSipCall = currentSipCall->next;
        }
SIPLOG("All tests failed. No match to existing SIP call\n");
        return NULL;
}

sipDB* match_cancel_request(){

        sipDB *currentSipCall = firstSipCall;

        while(currentSipCall != NULL){

                if((!strcmp(currentSipCall->src_extension,uri_obj.src_extension)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->invite_active == 1) && (currentSipCall->cseq == uri_obj.cseq) && (!strcmp(currentSipCall->dst_extension,uri_obj.dst_extension)))
                        return currentSipCall;

                currentSipCall = currentSipCall -> next;

        }
        return NULL;
}

sipDB* match_bye_request(){

        sipDB* currentSipCall = firstSipCall;

        while(currentSipCall != NULL){

                if((!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->session_active == 1) && ((!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.to_tag)))){
                        return currentSipCall;
                }
                else if((!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->session_active == 1) && ((!strcmp(currentSipCall->from_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->to_tag,uri_obj.from_tag)))){
                        return currentSipCall;
                }

                currentSipCall = currentSipCall->next;
        }
        return NULL;
}

sipDB* matchsipDBInvite(char *buffer,int length){

        sipDB *currentSipCall = NULL;
        currentSipCall = firstSipCall;

        SIPLOG("Inisde matchsipDBInvite\n");
        while(currentSipCall != NULL){
SIPLOG("----------------------------------------------------------------------"
    "-------\n");
SIPLOG("currentSipCall->invite_active = %d\n", currentSipCall->invite_active);
SIPLOG("currentSipCall->from_tag = %s, uri_obj.from_tag = %s\n", 
    currentSipCall->from_tag, uri_obj.from_tag);            
SIPLOG("currentSipCall->to_tag = %s, uri_obj.to_tag = %s\n", 
    currentSipCall->to_tag, uri_obj.to_tag);
SIPLOG("currentSipCall->call_id = %s, uri_obj.call_id = %s\n", 
    currentSipCall->call_id, uri_obj.call_id);            
SIPLOG("currentSipCall->src_extension = %s, uri_obj.src_extension = %s\n", 
    currentSipCall->src_extension, uri_obj.src_extension);            
SIPLOG("currentSipCall->dst_extension = %s, uri_obj.dst_extension = %s\n", 
    currentSipCall->dst_extension, uri_obj.dst_extension);            
SIPLOG("currentSipCall->cseq = %d, uri_obj.cseq = %d\n", 
    currentSipCall->cseq, uri_obj.cseq);      
SIPLOG("currentSipCall->src_ip = %s, currentSipCall->dst_ip = %s, uri_obj.ip_addr = %s\n", 
    currentSipCall->src_ip, currentSipCall->dst_ip, uri_obj.ip_addr);                     
SIPLOG("----------------------------------------------------------------------"
    "-------\n");
            
            if((currentSipCall->session_active == 1) && (!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && 
                    (!strcmp(currentSipCall->to_tag,uri_obj.to_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id))){

                strncpy(buffer,"client",length);
SIPLOG("Returning 1\n");
                return currentSipCall;
            }
            else if((currentSipCall->session_active == 1) && (!strcmp(currentSipCall->from_tag,uri_obj.to_tag)) && 
                    (!strcmp(currentSipCall->to_tag,uri_obj.from_tag)) && (!strcmp(currentSipCall->call_id,uri_obj.call_id))){

                strncpy(buffer,"server",length);
SIPLOG("Returning 2\n");
                return currentSipCall;
            }
            else if((currentSipCall->invite_active == 1) && (!strcmp(currentSipCall->from_tag,uri_obj.from_tag)) && 
                    (!strcmp(currentSipCall->dst_extension,uri_obj.dst_extension)) && 
                    (!strcmp(currentSipCall->call_id,uri_obj.call_id)) && (currentSipCall->cseq == uri_obj.cseq) && 
                    (currentSipCall->session_active != 1)){
                /* Duplicate Invite */
SIPLOG("Returning 3\n");
                return NULL;
            }
            else if((currentSipCall->invite_active == 1) && (!strcmp(currentSipCall->src_extension,uri_obj.src_extension)) &&       
                    (currentSipCall->cseq == uri_obj.cseq) ) {
SIPLOG("Returning 4\n");
                return currentSipCall;
            }
            else if((currentSipCall->invite_active == 1) && (!strcmp(currentSipCall->src_extension,uri_obj.src_extension)) &&       
                    (!strcmp(currentSipCall->dst_extension,uri_obj.dst_extension)) ) {
SIPLOG("Returning 5\n");
                return currentSipCall;
            }
            currentSipCall = currentSipCall->next;
        }
        strncpy(buffer,"newcall",length);
SIPLOG("Returning 6\n");
        return NULL;
}

int match_corporate_directory(char *src_extn){

        int i = 0;

        /*linear search for username in corporate directory */
        for(i = 0; i < structure_size; i++){

                if(strcmp(directory[i].telephone_number, src_extn) == 0){
                        return i;
                }
        }
        return -1;
}


