#include <targets.h>
#include <ec_siplog.h>
#include <ec_siph264.h>
#include <ec_threads.h>
#include <st_i.h>
#include <g711.h>
#include <EasyG7231_macros.h>
#include <isac.h>

#ifndef OS_WINDOWS
#include <ec_liveMonitor.h>
#include <ec_audioDSPPlayer.h>
#endif

FUNC_DECODER(dissector_rtp_sip);
/* Add and delete rtp dissector */
void addRTPDissector(int,char *);
void deleteRTPDissector(int,char *);
void releaseAudioDecoder(rtpSIP *currentRTPCall);
static int processRTP(rtpSIP*,struct packet_object*,int ,int,u_char*);
static int fileWrite(char *, int, int, FILE *);
extern void dissect_del(char* name);

#ifndef OS_WINDOWS
#ifndef ARCH_X64
extern int initialize_g726_decoder(unsigned long *);
extern int decode_payload_g726(u_char*, int, short *, int *, unsigned long, int);
extern void release_g726_decoder(unsigned long);
extern int initialize_g723_decoder(unsigned long *, int);
extern int decode_payload_g723(u_char *, int ,short *, int *, unsigned long, int);
extern void release_g723_decoder(unsigned long);
extern int initialize_g729_decoder(unsigned long *);
extern int decode_payload_g729(u_char *, int, short *, int *, unsigned long );
extern void release_g729_decoder(unsigned long);
#endif
#endif 

/* Mutex variable for accessing RTP call structure */
extern pthread_mutex_t rtpaccess_mutex;				/* Extern from ec_sip.c */
#define RTPACCESS_LOCK      do{ pthread_mutex_lock(&rtpaccess_mutex); } while(0)
#define RTPACCESS_UNLOCK    do{ pthread_mutex_unlock(&rtpaccess_mutex); } while(0)

void get_arrival_time(int current_framesamples,   /* samples */
                                          int packet_size,            /* bytes */
                                          int bottleneck,             /* excluding headers; bits/s */
                                          BottleNeckModel *BN_data)
{
        const int HeaderSize = 35;
        int HeaderRate;

        HeaderRate = HeaderSize * 8 * FS / current_framesamples;     /* bits/s */

        /* everything in samples */
        BN_data->sample_count = BN_data->sample_count + current_framesamples;

        BN_data->arrival_time += ((packet_size + HeaderSize) * 8 * FS) / (bottleneck + HeaderRate);
        BN_data->send_time += current_framesamples;

        if (BN_data->arrival_time < BN_data->sample_count)
                BN_data->arrival_time = BN_data->sample_count;

        BN_data->rtp_number++;
}

int alloc_new_avbuffer(u_char **audioBuf, int audioBufSize) 
{
    int ret = 0;
	*audioBuf = (u_char*)malloc(audioBufSize);
	if(*audioBuf == NULL) {
	    SIPLOG("Error #%d allocating audio buffer:  %s\n", errno, strerror(errno) );
		ret = -1;
	}
	bzero(*audioBuf, audioBufSize);
	
	return ret;
}

int realloc_avbuffer(u_char** audioBuf, int* audioBufSize)
{
    int ret = 0;
    int oldSize = *audioBufSize;
    u_char* oldBuf = *audioBuf;
    
    *audioBufSize *= 2;
    if(alloc_new_avbuffer(audioBuf, *audioBufSize) < 0) {
	    SIPLOG("Error #%d re-allocating audio buffer:  %s\n", errno, strerror(errno) );
        return -1;
    }
    memcpy(*audioBuf, oldBuf, oldSize);
    free(oldBuf);
    
    return ret;
}

int initializeAudioForCall(rtpSIP* currentRTPCall, int audioCodecType)
{
    int ret = 0;
        //SIPRTPLOG("Initialzing the file names \n");
    strcat(currentRTPCall->fwd_wav,".wav");
   	strcat(currentRTPCall->rev_wav,".wav");
   	strcat(currentRTPCall->both_wav,".wav");
	/* Something went wrong in the SDP codec negotiation, adding G711MULAW as the codec */
	if(currentRTPCall->audioCodecType == -1)
		currentRTPCall->audioCodecType = audioCodecType;
	if(alloc_new_avbuffer(&currentRTPCall->fwdAudioBuf, currentRTPCall->fwdAudioBufSize)  < 0) 
	    ret = -1;
    
	if(alloc_new_avbuffer(&currentRTPCall->revAudioBuf, currentRTPCall->revAudioBufSize) < 0)
	    ret = -1;
	return ret;
}


FUNC_DECODER(dissector_rtp_sip){

        DECLARE_DISP_PTR_END(ptr, end);
        char temp[MAX_ASCII_ADDR_LEN] = {'\0'};
        char temp1[MAX_ASCII_ADDR_LEN] = {'\0'};
        int direction = 0;
		int mdirection = 0;
        int vdirection = 0;
        int video_yes = 0;

        ip_addr_ntoa(&PACKET->L3.src, temp);
        ip_addr_ntoa(&PACKET->L3.dst, temp1);

        int bytes = end - ptr;
        if(bytes == 0)
        	return NULL;

        rtpSIP* currentRTPCall = NULL;

        //SIPRTPLOG("*************************************************************\n");
        //SIPRTPLOG("Logging new RTP message \n");
        //SIPRTPLOG("Source IP address = %s, Source Port = %d and Destination IP address = %s, Destination port = %d \n",temp,ntohs(PACKET->L4.src),temp1,ntohs(PACKET->L4.dst));

		RTPACCESS_LOCK;

        currentRTPCall = checkRTPCall(temp,ntohs(PACKET->L4.src),temp1,ntohs(PACKET->L4.dst),&video_yes,&direction);
/* SIPLOG("Incoming packet from %s:%d to %s:%d\n",
    temp, ntohs(PACKET->L4.src), temp1, ntohs(PACKET->L4.dst) ); */

		RTPACCESS_UNLOCK;

        if(currentRTPCall != NULL){

        	if(video_yes == 1 && direction == 1){
                vdirection = 1;
			}
            else if(video_yes == 0 && direction == 1){
                mdirection = 1;
			}
			else if(video_yes == 1 && direction == 2){
				vdirection = 2;
			}
			else if(video_yes == 0 && direction == 2){
				mdirection =2 ;
			}
			else {
			    /* SIPLOG("No direction for video set for current RTP call for incoming packet from %s:%d to %s:%d\n",
                    temp, ntohs(PACKET->L4.src), temp1, ntohs(PACKET->L4.dst) ); */
				return NULL;
			}
            //SIPRTPLOG("Processing the forward direction media, direction = %d and vdirection = %d\n",direction,vdirection);
            processRTP(currentRTPCall,PACKET,mdirection,vdirection,end);
        }
        /* else {
		    SIPLOG("No current RTP call for incoming packet from %s:%d to %s:%d\n",
                temp, ntohs(PACKET->L4.src), temp1, ntohs(PACKET->L4.dst) );
			return NULL;
		} */

        return NULL;
}

int processRTP(rtpSIP* currentRTPCall, struct packet_object *packet, int direction, int vdirection, u_char *end){

		struct sniff_rtp *rtp_hdr;
    	u_char *sample;                 /* for g722 */
    	u_char payload_mask = 0xFF;
        char pd[4];

        /* If the phone session is not active it should be the remaining rtp packets in the buffer after the phone has sent a bye */
        /* The application layer data is in PACKET->DATA.data and the DATA length can be found in PACKET->DATA.len */

        if(currentRTPCall->session_active != 1){
            //SIPRTPLOG("Session is not active of the currentRTPCall, returning\n");
            return -1;
        }

        sample = packet->DATA.data;                     /* for g722 */
        rtp_hdr = (struct sniff_rtp *) sample;          /* for g722 */

        //SIPRTPLOG("RTP header info, version = %02X, payload type = %02X, sequence number = %d, timestamp = %lu, ssrc = %lu\n",rtp_hdr->version,rtp_hdr->payloadType,ntohs(rtp_hdr->sequence_no),ntohl(rtp_hdr->timestamp),ntohl(rtp_hdr->ssrc));

        /* Masking the payload to get the actual payload type, because the msb bit is the marker it can be either 0 or 1 and the rest 7 bits are the payload type. So if the payload type is 09 it is G.722 and for 00 it is G.711 alaw */

		u_char *rtpPayload = packet->DATA.data + sizeof(struct sniff_rtp);	/* Pointer to RTP Payload */ 
		int payloadLength = packet->DATA.len - sizeof(struct sniff_rtp);	/* RTP payload length */
        payload_mask = rtp_hdr -> payloadType & 0x7F;

		if(direction > 0){

			/* Audio RTP Payload */
			//size_t ret;
			int returnMonitor = 0;

			switch(payload_mask){

				case COG722:

					if(currentRTPCall->rtp_count == 1){
					    if(initializeAudioForCall(currentRTPCall, COG722) < 0) {
					        SIPLOG("Failed to initialize audio for RTP call.\n");
					        return -1;
					    }
                        g722_decode_init(&currentRTPCall->dec_state,64000,G722_PACKED | G722_SAMPLE_RATE_8000);
                        g722_decode_init(&currentRTPCall->dec_state1,64000,G722_PACKED | G722_SAMPLE_RATE_8000);
                	}

					if(currentRTPCall->audioCodecType != COG722)
						return -1;
						
					++currentRTPCall->rtp_count;
				
					int16_t *outdata;

					switch (direction){

						case 1:
							/* Forward direction packets */

						    /* Decodes the received encoded payload and writes it to the wav file, the decoder is got from libspandsp(http://www.soft-switch.org/spandsp-doc/ */
                            if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                            	//SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall fwdseqcount = %d\n",ntohs(currentRTPCall->fwd_seqCount));
                                return -1;
                            }
     
                            ++currentRTPCall->fwd_count;

                            outdata = NULL;

						    if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						        if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						            return -1;
						    }

                            outdata = (int16_t *)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];// malloc(payloadLength * sizeof(int16_t));
                            g722_decode(&currentRTPCall->dec_state, outdata, (const uint8_t *)(rtpPayload),payloadLength);
#ifndef OS_WINDOWS						
                            returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                            if(returnMonitor == 1){
                                playDSP(outdata, (payloadLength * sizeof(int16_t)), FWDAUDIOFT);
                            }
#endif						
                            currentRTPCall->fwdAudioIdx += (payloadLength * sizeof(int16_t));

                        /* Getting the Sequence number of the packet, if the next packet has the same sequence 
                           number it must be a duplicate */               
                        currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;

						/* Dump the payload to the pcap file */
						break;		
					case 2:
						/* Reverse direction packets */
						if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                               //SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall revseqcount = %d\n",ntohs(currentRTPCall->rev_seqCount));
                               return -1;
                       	}

                       	++currentRTPCall->rev_count;

					    if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
					        if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
					            return -1;
					    }  

                       	outdata = (int16_t *)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];//malloc(payloadLength * sizeof(int16_t));
                    	g722_decode(&currentRTPCall->dec_state1, outdata, (const uint8_t *) (rtpPayload), (payloadLength));

#ifndef OS_WINDOWS
                       	returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                       	if(returnMonitor == 1){
                           	playDSP(outdata, (payloadLength * sizeof(int16_t)), REVAUDIOFT);
                       	}
#endif						
          	            // free(outdata);
                        currentRTPCall->fwdAudioIdx += (payloadLength * sizeof(int16_t));

               	        currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
						break;
					}
					break;

				case COG711MULAW:

					if(currentRTPCall->rtp_count == 1){
					    if(initializeAudioForCall(currentRTPCall, COG711MULAW) < 0) {
					        SIPLOG("Failed to initialize audio for RTP call.\n");
					        return -1;
					    }
               		}

               		++currentRTPCall->rtp_count;						
				
					if(currentRTPCall->audioCodecType != COG711MULAW)
						return -1;

					switch (direction){

						case 1:
							/* Fwd direction RTP packets */
							++currentRTPCall->fwd_count;
							
							if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                               	return -1;
                        	}
                        	
						    if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						        if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						            return -1;
						    }  

							int16_t *pcm711mulaw = (int16_t *)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];

							u_char *rtpData = rtpPayload;
							unsigned int i = 0;
							for(i = 0; i < payloadLength; i++){
								//pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
							    pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
							}
#ifndef OS_WINDOWS
      			            returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                	        if(returnMonitor == 1){
                    			playDSP(pcm711mulaw, (payloadLength * sizeof(int16_t)), FWDAUDIOFT);
                        	}
#endif							
		                    /* Getting the Sequence number of the packet, if the next packet has the same 
		                       sequence number it must be a duplicate */
        		            currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                		   //SIPRTPLOG("Successfully decoded/dumped G711 forwared direction media payload\n");
							currentRTPCall->fwdAudioIdx += (payloadLength * sizeof(int16_t));
							/* Dump to pcap file */
							break;
						case 2:
							/* Reverse direction RTP packets */
							++currentRTPCall->rev_count;

							if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                        	       return -1;
                        	}
						    if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
						        if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
						            return -1;
						    }                          	
							pcm711mulaw = (int16_t *)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];
                            rtpData = rtpPayload;
                            i = 0;
                            for(i = 0; i < payloadLength; i++){
                                pcm711mulaw[i] = st_ulaw2linear16(rtpData[i]);
							}
#ifndef OS_WINDOWS
	                        returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
    	                    if(returnMonitor == 1){
        	                    playDSP(pcm711mulaw, (payloadLength * sizeof(int16_t)), REVAUDIOFT);
            	            }
#endif							
							//free(pcm711mulaw);
                   			currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
                   			currentRTPCall->revAudioIdx += (payloadLength * sizeof(int16_t));
							break;
           				}
						break;

					case COG726:
#ifndef ARCH_X64
						if(currentRTPCall->rtp_count == 1){
					        if(initializeAudioForCall(currentRTPCall, COG726) < 0) {
					            SIPLOG("Failed to initialize audio for RTP call.\n");
					            return -1;
					        }
                        	if(currentRTPCall->audioCodecType == -1)
                            	currentRTPCall->audioCodecType = COG726;
                        }

                        ++currentRTPCall->rtp_count;

                        if(currentRTPCall->audioCodecType != COG726)
                            return -1;

                        short *g726Synth;
                        int g726SynthLength = 0, g726_ret = 0;
	
						switch(direction){

							case 1:

								/* Fwd direction RTP packets */
                                if(currentRTPCall->fwd_count == 1 && direction == 1){

                                    int g726_init_ret = initialize_g726_decoder(&(currentRTPCall->hDecoderFwd));
                                    if(g726_init_ret < 0){
                                        INSTANT_USER_MSG("Error initializing G726 decoder %s \n", currentRTPCall->fwd_wav);
                                        SIPRTPLOG("Error initializing G726 decoder %s \n", currentRTPCall->fwd_wav);
                                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                                        deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                                        return -1;
                                    }

                               }

                                ++currentRTPCall->fwd_count;

								if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                                    return -1;
                                }

						        if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						                return -1;
						        }  
                                
                                g726Synth = (short*)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];
                                g726_ret = decode_payload_g726(rtpPayload, payloadLength, g726Synth, &g726SynthLength, 
                                    currentRTPCall->hDecoderFwd, currentRTPCall->g726SampleSize);
                                if(g726_ret <0)
                                    return -1;
#ifndef OS_WINDOWS								
      			                returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, 
      			                    currentRTPCall->audioCodecType,rtp_hdr);
                        		if(returnMonitor == 1){
                            		playDSP(g726Synth, g726SynthLength * sizeof(short), FWDAUDIOFT);
                        		}
#endif
                                currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->fwdAudioIdx += (sizeof(short) * g726SynthLength);
							
							break;
							case 2:
							
								if(currentRTPCall->rev_count == 1){

                                    //SIPRTPLOG("Creating reverse direction wav file header for file %s\n",currentRTPCall->rev_wav);

                                    int g726_init_ret = initialize_g726_decoder(&(currentRTPCall->hDecoderRev));
                                    if(g726_init_ret < 0){
                                        INSTANT_USER_MSG("Error initializing G726 decoder %s \n", currentRTPCall->rev_wav);
                                        SIPRTPLOG("Error initializing G726 decoder %s \n", currentRTPCall->rev_wav);
                                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                                        deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                                        return -1;
                                    }
                               }

                                ++currentRTPCall->rev_count;	
								
								if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                                    return -1;
                                }
                                
						        if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
						                return -1;
						        }  
                                
                                g726Synth = (short*)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];   
                                g726_ret = decode_payload_g726(rtpPayload, payloadLength, g726Synth, &g726SynthLength, 
                                    currentRTPCall->hDecoderRev, currentRTPCall->g726SampleSize);
                                if(g726_ret <0)
                                    return -1;
#ifndef OS_WINDOWS								
                                returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                                if(returnMonitor == 1){
                                    playDSP(g726Synth, g726SynthLength * sizeof(short), REVAUDIOFT);
                                }
#endif
                                currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->revAudioIdx += (sizeof(short) * g726SynthLength);
							break;
						}
#endif						
					break;

					case COG723:

#ifndef OS_WINDOWS					
#ifndef ARCH_X64
					   if(currentRTPCall->rtp_count == 1){
					        if(initializeAudioForCall(currentRTPCall, COG723) < 0) {
					            SIPLOG("Failed to initialize audio for RTP call.\n");
					            return -1;
					        }
							/* Something went wrong in the SDP codec negotiation, adding G723 as the codec */
                        	if(currentRTPCall->audioCodecType == -1)
                            	currentRTPCall->audioCodecType = COG723;
                        }

                        ++currentRTPCall->rtp_count;

                        if(currentRTPCall->audioCodecType != COG723)
                            return -1;

                        short *g723Synth;
                        int g723SynthLength = 0, g723_ret = 0;

						switch(direction){

							case 1:

								/* Fwd direction RTP packets */
                                if(currentRTPCall->fwd_count == 1 && direction == 1){

									if(!((packet->DATA.len - sizeof(struct sniff_rtp)) % L_G7231_FRAME_COMPRESSED_63))
										currentRTPCall->fwd723FrameType = 1;
									else if(!((packet->DATA.len - sizeof(struct sniff_rtp)) % L_G7231_FRAME_COMPRESSED_53))
										currentRTPCall->fwd723FrameType = 0;

                                	int g723_init_ret = initialize_g723_decoder(&(currentRTPCall->hDecoderFwd), currentRTPCall->fwd723FrameType);
                                    if(g723_init_ret < 0){
                                    	INSTANT_USER_MSG("Error initializing G723 decoder %s \n", currentRTPCall->fwd_wav);
                                        SIPRTPLOG("Error initializing G723 decoder %s \n", currentRTPCall->fwd_wav);
                                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                                        deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                                        return -1;
                                    }

                                }

                                ++currentRTPCall->fwd_count;	

                                if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                                    return -1;
                                }
                                
						        if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						                return -1;
						        }  

                                g723Synth = (short*)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];
                                								
                                g723_ret = decode_payload_g723(rtpPayload, payloadLength, g723Synth, &g723SynthLength, currentRTPCall->hDecoderFwd, currentRTPCall->fwd723FrameType);
                                if(g723_ret <0)
                                    return -1;

#ifndef OS_WINDOWS								
                                returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, 
                                    currentRTPCall->audioCodecType,rtp_hdr);
                                if(returnMonitor == 1){
                                    playDSP(g723Synth, g723SynthLength * sizeof(short), FWDAUDIOFT);
                                }
#endif								
                                currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->fwdAudioIdx += (sizeof(short) * g723SynthLength);

							break;

							case 2:

								if(currentRTPCall->rev_count == 1){

                                    //SIPRTPLOG("Creating reverse direction wav file header for file %s\n",currentRTPCall->rev_wav);

									if(!((packet->DATA.len - sizeof(struct sniff_rtp)) % L_G7231_FRAME_COMPRESSED_63))
                                        currentRTPCall->rev723FrameType = 1;
                                    else if(!((packet->DATA.len - sizeof(struct sniff_rtp)) % L_G7231_FRAME_COMPRESSED_53))
                                        currentRTPCall->rev723FrameType = 0;

                                    int g723_init_ret = initialize_g723_decoder(&(currentRTPCall->hDecoderRev),currentRTPCall->rev723FrameType);
                                    if(g723_init_ret < 0){
                                    	INSTANT_USER_MSG("Error initializing G723 decoder %s \n", currentRTPCall->rev_wav);
                                        SIPRTPLOG("Error initializing G723 decoder %s \n", currentRTPCall->rev_wav);
                                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                                        deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                                        return -1;
                                    }

                                }

                                ++currentRTPCall->rev_count;

                                if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                                    //SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall revseqcount = %d\n",ntohs(currentRTPCall->rev_seqCount));
                                    return -1;
                                }

						        if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
						                return -1;
						        }  
                                
                                g723Synth = (short*)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];
								
							 	g723_ret = decode_payload_g723(rtpPayload, payloadLength, g723Synth, &g723SynthLength, currentRTPCall->hDecoderRev, currentRTPCall->rev723FrameType);
                                if(g723_ret <0)
                                    return -1;

#ifndef OS_WINDOWS								
                                returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                                if(returnMonitor == 1){
                                    playDSP(g723Synth, g723SynthLength * sizeof(short), FWDAUDIOFT);
                                }
#endif								
                                currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->revAudioIdx += (sizeof(short) * g723SynthLength);
							break;

						}
#endif
#endif						
					break;

					case COG729:
#ifndef OS_WINDOWS							
#ifndef ARCH_X64					
						if(currentRTPCall->rtp_count == 1){
					        if(initializeAudioForCall(currentRTPCall, COG729) < 0) {
					            SIPLOG("Failed to initialize audio for RTP call.\n");
					            return -1;
					        }
							/* Something went wrong in the SDP codec negotiation, adding G729 as the codec */
                        	if(currentRTPCall->audioCodecType == -1)
                            	currentRTPCall->audioCodecType = COG729;
                        }

                        ++currentRTPCall->rtp_count;

                        if(currentRTPCall->audioCodecType != COG729)
                            return -1;	

						short *g729Synth;
                        int synthLength = 0, g729_ret = 0;

						switch(direction){

							case 1:
							
								/* Fwd direction RTP packets */
                                if(currentRTPCall->fwd_count == 1 && direction == 1){
									int g729_init_ret = initialize_g729_decoder(&(currentRTPCall->hDecoderFwd));
									if(g729_init_ret < 0){
										INSTANT_USER_MSG("Error initializing G729 decoder %s \n", currentRTPCall->fwd_wav);
        	                            SIPRTPLOG("Error initializing G729 decoder %s \n", currentRTPCall->fwd_wav);
            	                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                	                    deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                    	                return -1;
                        	        }
                                }

								++currentRTPCall->fwd_count;

                                if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                                    //SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall fwdseqcount = %d\n",ntohs(currentRTPCall->fwd_seqCount));
                                    return -1;
                                }
                                
						        if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						                return -1;
						        }  
                                
                                g729Synth = (short*)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];
								g729_ret = decode_payload_g729(rtpPayload, payloadLength, g729Synth, &synthLength, 
								    currentRTPCall->hDecoderFwd);
								if(g729_ret <0)
									return -1;
#ifndef OS_WINDOWS								
      			                returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                		        if(returnMonitor == 1){
                        	   		playDSP(g729Synth, synthLength * sizeof(short), FWDAUDIOFT);
                        		}
#endif								
								currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->fwdAudioIdx += (sizeof(short) * synthLength);
								
								break;
							
							case 2:
		
								if(currentRTPCall->rev_count == 1){

                                    //SIPRTPLOG("Creating reverse direction wav file header for file %s\n",currentRTPCall->rev_wav);

                                    int g729_init_ret = initialize_g729_decoder(&(currentRTPCall->hDecoderRev));
                                    if(g729_init_ret < 0){
                                        INSTANT_USER_MSG("Error initializing G729 decoder %s \n", currentRTPCall->rev_wav);
                                        SIPRTPLOG("Error initializing G729 decoder %s \n", currentRTPCall->rev_wav);
                                        INSTANT_USER_MSG("Stopped recording the conversation\n");
                                        deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                                        return -1;
                                    }
                                }

                                ++currentRTPCall->rev_count;

                                if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                                    //SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall revseqcount = %d\n",ntohs(currentRTPCall->rev_seqCount));
                                    return -1;
                                }
                                
						        if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
						                return -1;
						        }  
                                	
                                g729Synth = (short*)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];					
								g729_ret = decode_payload_g729(rtpPayload, payloadLength, g729Synth, &synthLength, 
								    currentRTPCall->hDecoderRev);
                                if(g729_ret <0)
                                    return -1;

#ifndef OS_WINDOWS
      		                  	returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
            		            if(returnMonitor == 1){
                    		        playDSP(g729Synth, synthLength * sizeof(short), REVAUDIOFT);
                        		}
#endif								
                                currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
                                currentRTPCall->revAudioIdx += (sizeof(short) * synthLength);
								
								break;
							}
#endif
#endif						
						break;

					case COG711ALAW:								
						if(currentRTPCall->rtp_count == 1){
					        if(initializeAudioForCall(currentRTPCall, COG711ALAW) < 0) {
					            SIPLOG("Failed to initialize audio for RTP call.\n");
					            return -1;
					        }
							/* Something went wrong in the SDP codec negotiation, adding G711MULAW as the codec */
                        	if(currentRTPCall->audioCodecType == -1)
                            	currentRTPCall->audioCodecType = COG711ALAW;
               			}

               			++currentRTPCall->rtp_count;

               			if(currentRTPCall->audioCodecType != COG711ALAW)
                   			return -1;		

						switch(direction){

							case 1:
									
								/* Fwd direction RTP packets */

                              	++currentRTPCall->fwd_count;

                       			if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                           			return -1;
                       			}

						        if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						                return -1;
						        }  

								int16_t *pcm711alaw = (int16_t *)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx];
                            	u_char *rtpData = rtpPayload;

                            	int i = 0;
                            	for(i = 0; i < payloadLength; i++){
                                	pcm711alaw[i] = st_alaw2linear16(rtpData[i]);
                            	}
#ifndef OS_WINDOWS
                            	returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                            	if(returnMonitor == 1){
                                	playDSP(pcm711alaw, (payloadLength * sizeof(int16_t)), FWDAUDIOFT);
                            	}
#endif
                                currentRTPCall->fwdAudioIdx += (payloadLength * sizeof(int16_t));
                       			/* Getting the Sequence number of the packet, if the next packet has the same sequence number it must be a duplicate */
                       			currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                       			//SIPRTPLOG("Successfully decoded/dumped G711 forwared direction media payload\n");

                       			/* Dump to pcap file */
                       			break;

							case 2:
									
								/* Reverse direction RTP packets */
                               	++currentRTPCall->rev_count;

                               	if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                                    return -1;
                              	}

						        if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
						            if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
						                return -1;
						        }  

								pcm711alaw = (int16_t *)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx];
                                rtpData = rtpPayload;
                                i = 0;
                                for(i = 0; i < payloadLength; i++){
                                    pcm711alaw[i] = st_alaw2linear16(rtpData[i]);
                                }
#ifndef OS_WINDOWS
                                returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                                if(returnMonitor == 1){
                                    playDSP(pcm711alaw, (payloadLength * sizeof(int16_t)), REVAUDIOFT);
                                }
#endif
                                currentRTPCall->revAudioIdx += (payloadLength * sizeof(int16_t));
                               	currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
                               	//SIPRTPLOG("Successfully decoded/dumped G711 reverse direction media payload\n");
                               	break;
                           	}
                          	break;
				case COISAC:
					if(currentRTPCall->rtp_count == 1){
					    if(initializeAudioForCall(currentRTPCall, COISAC) < 0) {
					        SIPLOG("Failed to initialize audio for RTP call.\n");
					        return -1;
					    }
						WebRtcIsac_Create(&currentRTPCall->fwd_isac_inst);
						WebRtcIsac_DecoderInit(currentRTPCall->fwd_isac_inst);
						WebRtcIsac_Create(&currentRTPCall->rev_isac_inst);
						WebRtcIsac_DecoderInit(currentRTPCall->rev_isac_inst);
                	}

					if(currentRTPCall->audioCodecType != COISAC) {
						return -1;
					}
 
					++currentRTPCall->rtp_count;
				
					WebRtc_Word16 isac_length = 0;

					switch (direction){

						case 1:
							/* Forward direction packets */

						    /* Decodes the received encoded payload and writes it to the wav file, the decoder is got from libspandsp(http://www.soft-switch.org/spandsp-doc/ */
                            if(currentRTPCall->fwd_seqCount == rtp_hdr->sequence_no){
                                return -1;
                            }

                            ++currentRTPCall->fwd_count;

						    currentRTPCall->vfwd_cur_frame_samples += FRAMESAMPLES_30ms;

						    get_arrival_time(currentRTPCall->vfwd_cur_frame_samples, payloadLength, 25000, &currentRTPCall->vfwd_bottleneck);

						    int err = WebRtcIsac_UpdateBwEstimate(currentRTPCall->fwd_isac_inst,
							    (WebRtc_UWord16 *)rtpPayload,
							    payloadLength,
							    currentRTPCall->vfwd_bottleneck.rtp_number,
							    currentRTPCall->vfwd_bottleneck.send_time,
							    currentRTPCall->vfwd_bottleneck.arrival_time);

						    if (err < 0) {
							    // exit if returned with error
							    int errtype=WebRtcIsac_GetErrorCode(currentRTPCall->fwd_isac_inst);
							    SIPLOG("\n\nError in ISAC decoder: %d.\n\n", errtype);
							    //exit(EXIT_FAILURE);
						    }
						    /* iSAC decoding */
						    if( (currentRTPCall->fwdAudioBufSize - currentRTPCall->fwdAudioIdx) < AV_BUF_THRESHOLD) {
						        if(realloc_avbuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioBufSize) < 0)
						            return -1;
						    }  
						    WebRtc_Word16 speechType;
						    isac_length = WebRtcIsac_Decode(currentRTPCall->fwd_isac_inst,
							    (WebRtc_UWord16 *)rtpPayload,
							    payloadLength,
							    (WebRtc_Word16 *)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx],//isac_outdata,
							    &speechType);
     
						    if (isac_length <= 0) {
							    // exit if returned with error
							    int errtype=WebRtcIsac_GetErrorCode(currentRTPCall->fwd_isac_inst);
							    printf("\n\nError in ISAC decoder: %d.\n\n", errtype);
							    //exit(EXIT_FAILURE);
						    }

#ifndef OS_WINDOWS		
                        returnMonitor = liveMonitorHandler(currentRTPCall, FWDAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                        if(returnMonitor == 1){
                            playDSP((short int *)&currentRTPCall->fwdAudioBuf[currentRTPCall->fwdAudioIdx], isac_length * sizeof(WebRtc_Word16),
                                FWDAUDIOFT);
                        }
#endif						
						currentRTPCall->fwdAudioIdx += (isac_length * sizeof(WebRtc_Word16) );

                        /* Getting the Sequence number of the packet, if the next packet has the same
                           sequence number it must be a duplicate */               
                        currentRTPCall->fwd_seqCount = rtp_hdr->sequence_no;
                        //SIPRTPLOG("Successfully decoded and dumped the fwd direction g722 payload\n");

						/* Dump the payload to the pcap file */
						break;		
					case 2:
						/* Reverse direction packets */

						if(currentRTPCall->rev_seqCount == rtp_hdr->sequence_no){
                               return -1;
                       	}

                       	++currentRTPCall->rev_count;
						currentRTPCall->vrev_cur_frame_samples += FRAMESAMPLES_30ms;

						get_arrival_time(currentRTPCall->vrev_cur_frame_samples, payloadLength, 25000, &currentRTPCall->vrev_bottleneck);

						err = WebRtcIsac_UpdateBwEstimate(currentRTPCall->rev_isac_inst,
							(WebRtc_UWord16 *)rtpPayload,
							payloadLength,
							currentRTPCall->vrev_bottleneck.rtp_number,
							currentRTPCall->vrev_bottleneck.send_time,
							currentRTPCall->vrev_bottleneck.arrival_time);

						if (err < 0) {
							// exit if returned with error
							int errtype=WebRtcIsac_GetErrorCode(currentRTPCall->rev_isac_inst);
							SIPLOG("\n\nError in ISAC decoder: %d.\n\n", errtype);
							//exit(EXIT_FAILURE);
						}
						/* iSAC decoding */

					    if( (currentRTPCall->revAudioBufSize - currentRTPCall->revAudioIdx) < AV_BUF_THRESHOLD) {
					        if(realloc_avbuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioBufSize) < 0)
					            return -1;
					    }  

						isac_length = WebRtcIsac_Decode(currentRTPCall->rev_isac_inst,
							(WebRtc_UWord16 *)rtpPayload,
							payloadLength,
							(WebRtc_Word16 *)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx],
							&speechType);
						if (isac_length <= 0) {
							// exit if returned with error
							int errtype=WebRtcIsac_GetErrorCode(currentRTPCall->rev_isac_inst);
							printf("\n\nError in decoder: %d.\n\n", errtype);
							//exit(EXIT_FAILURE);
						}
 
#ifndef OS_WINDOWS
                       	returnMonitor = liveMonitorHandler(currentRTPCall, REVAUDIOFT, currentRTPCall->audioCodecType,rtp_hdr);
                       	if(returnMonitor == 1){
                           	playDSP((short int *)&currentRTPCall->revAudioBuf[currentRTPCall->revAudioIdx], isac_length * sizeof(WebRtc_Word16),
                                REVAUDIOFT);
                       	}
#endif						
						currentRTPCall->revAudioIdx += (isac_length * sizeof(WebRtc_Word16) );
               	        currentRTPCall->rev_seqCount = rtp_hdr->sequence_no;
						break;
					}
					break;
				}                          		
		}
		else if(vdirection > 0){

			/* Video RTP Payload */
			if(payload_mask >= 0x60 && payload_mask <= 0x7F){

                /* Payload type is h.264 video, the payload type will be 
                   negotiated in sdp. UCSniff is not handling the negotiation.
                   It is hard coded. */

                //SIPRTPLOG("RTP payload type is H264, video codec \n");

                if(currentRTPCall->vrtp_count == 1){

                	//SIPRTPLOG("Initializing the h264 file names\n");
                    strcat(currentRTPCall->vfwd_file,".264");
                    strcat(currentRTPCall->vrev_file,".264");
                }
                ++currentRTPCall->vrtp_count;
                if(vdirection == 1 && currentRTPCall->vfwd_count == 1){

                	//SIPRTPLOG("Opening the forward direction file %s\n",currentRTPCall->vfwd_file);

                    currentRTPCall->vfwd_fp = fopen(currentRTPCall->vfwd_file,"w+|b");
                    if(currentRTPCall->vfwd_fp == NULL){
                    	INSTANT_USER_MSG("Error: Opening file %s\n",currentRTPCall->vfwd_file);
                        SIPRTPLOG("Error: Opening file %s\n",currentRTPCall->vfwd_file);
                        INSTANT_USER_MSG("Deleting the dissector\n");
                        deleteRTPDissector(currentRTPCall->vport,currentRTPCall->ip_addr);
                        return -1;
                    }
                }
		 		else if(vdirection == 2 && currentRTPCall->vrev_count == 1){

                	//SIPRTPLOG("Opening the reverse direction file %s\n",currentRTPCall->vrev_file);

                    currentRTPCall->vrev_fp = fopen(currentRTPCall->vrev_file,"w+|b");
                    if(currentRTPCall->vrev_fp == NULL){
                    	INSTANT_USER_MSG("Error: Creating file %s\n",currentRTPCall->vrev_file);
                        SIPRTPLOG("Error: Creating file %s\n",currentRTPCall->vrev_file);
                        INSTANT_USER_MSG("Deleting the dissector\n");
                        deleteRTPDissector(currentRTPCall->vport,currentRTPCall->ip_addr);
                        return -1;
                    }  
                }

				if(vdirection == 1){

                	/* Skipping duplicate rtp packets */
                    if(currentRTPCall->vfwd_seqcount == rtp_hdr -> sequence_no) {
                    	//SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall vfwd_seqcount = %d\n",ntohs(currentRTPCall->vfwd_seqcount));
                        return -1;
                    }

                    struct naluHeader naluHeaderValue;
                    int writeNALPrefixCode = 1;
                    int writePacket = 1;
                    int offset = 0;

                    u_char *value = rtpPayload;

                    naluHeaderValue.forbidden = (*value & 0x80) >> 7;
                    naluHeaderValue.nri = (*value & 0x60) >> 5;
                    naluHeaderValue.type = *value & 0x1F;
//printf("%s:%d %s naluHeaderValue.type == %d\n", __FILE__, __LINE__, __func__, naluHeaderValue.type);
					if(naluHeaderValue.type == 5){

#ifdef HAVE_LIB_VLC
						/* Coded slice IDR Picture */
					
						/* Get the RTP marker bit */ 
						u_char markerBit = rtp_hdr -> payloadType & 0x80;

						if(currentRTPCall->currentFwdPPSCtr != currentRTPCall->previousFwdPPSCtr ){

							if(currentRTPCall->recvFwdAllIDRPic == 1){
								
								/* Delete the current IDR pictures */ 
								deleteIDRPicture(currentRTPCall->fwdIDRPtr);
								currentRTPCall->recvFwdAllIDRPic = 0;
								currentRTPCall->fwdIDRPtr = NULL;
								currentRTPCall->fwdIDRCtr = 0;
							}

							if(currentRTPCall->fwdIDRPtr == NULL){

								/* First IDR coded picture slice in the coded video sequence */

								currentRTPCall->fwdIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
								if(currentRTPCall->fwdIDRPtr != NULL){

									currentRTPCall->fwdCurrentIDRPtr = currentRTPCall->fwdIDRPtr;

									memset(currentRTPCall->fwdIDRPtr->idrPictureData,'\0',sizeof(currentRTPCall->fwdIDRPtr->idrPictureData));
									currentRTPCall->fwdIDRPtr->idrPictureLength =  (packet->DATA.len - sizeof(struct sniff_rtp));
									memcpy(currentRTPCall->fwdIDRPtr->idrPictureData, value, (packet->DATA.len - sizeof(struct sniff_rtp)));
									currentRTPCall->fwdIDRPtr->next = NULL;
									currentRTPCall->fwdIDRCtr += 1;

									if(markerBit == 0x80){

										/* The marker bit is set to 1 */
										/* All IDR codec pic for this coded video seq are received */
										currentRTPCall->recvFwdAllIDRPic = 1;
										/* Incr the PPS ctr */
										currentRTPCall->previousFwdPPSCtr += 1;
									}
								}

							}
							else {
									struct idrPicture *tempIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
									if(tempIDRPtr != NULL){

										currentRTPCall->fwdCurrentIDRPtr->next = tempIDRPtr;
										currentRTPCall->fwdCurrentIDRPtr = tempIDRPtr;
										
										memset(currentRTPCall->fwdCurrentIDRPtr->idrPictureData, '\0', sizeof(currentRTPCall->fwdCurrentIDRPtr->idrPictureData));
										currentRTPCall->fwdCurrentIDRPtr->idrPictureLength = (packet->DATA.len - sizeof(struct sniff_rtp));
										memcpy(currentRTPCall->fwdCurrentIDRPtr->idrPictureData, value, currentRTPCall->fwdCurrentIDRPtr->idrPictureLength);
										currentRTPCall->fwdCurrentIDRPtr->next = NULL;
										currentRTPCall->fwdIDRCtr += 1;

										if(markerBit == 0x80){

                                        	/* The marker bit is set to 1 */
                                        	/* All IDR codec pic for this coded video seq are received */
                                        	currentRTPCall->recvFwdAllIDRPic = 1;
                                        	/* Incr the PPS ctr */
                                        	currentRTPCall->previousFwdPPSCtr += 1;
                                    	}
									}
								}
							}
#endif
						}
						else if(naluHeaderValue.type == 7){
#ifdef HAVE_LIB_VLC

							/* Saving the h264 parameter sets for live video monitoring, the video decoder needs the paramter sets for decoding the raw h264 video */
							/* Sequence Parameter Set */ 
							//if(!currentRTPCall->savedFwdSPS){
								memset(currentRTPCall->fwdSPS,'\0',sizeof(currentRTPCall->fwdSPS));
								memcpy(currentRTPCall->fwdSPS,value,sizeof(currentRTPCall->fwdSPS));
							//	currentRTPCall->savedFwdSPS = 1;
							//}
#endif						
						}
						else if(naluHeaderValue.type == 8){

#ifdef HAVE_LIB_VLC
							/* Picture Parameter Set */ 
							memset(currentRTPCall->fwdPPS,'\0',sizeof(currentRTPCall->fwdPPS));
							int PPSLength = payloadLength;
							if(payloadLength > sizeof(currentRTPCall->fwdPPS))
								PPSLength = sizeof(currentRTPCall->fwdPPS);
							memcpy(currentRTPCall->fwdPPS,value,PPSLength);
							currentRTPCall->currentFwdPPSCtr += 1;
#endif							
						}
                        else if(naluHeaderValue.type == 28){

                                parseH264FUANAL(&naluHeaderValue, &value, &offset, &writeNALPrefixCode, &writePacket, &currentRTPCall->fwd_fuastart);
                                /* This is FU-A NAL unit */

                                /* Thanks to Manuel Koschuch for fixing FU-A parsing bugs */
                        }
                        else if(naluHeaderValue.type == 24){

                                /* STAP-A NAL aggregate packets */
                                parseH264STAPANAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vfwd_file, currentRTPCall->vfwd_fp, &writeNALPrefixCode, &writePacket, 2, rtp_hdr);
                        }
                        else if(naluHeaderValue.type == 25){

                                /* STAP-B NAL aggregate packets */
                                parseH264STAPBNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vfwd_file, currentRTPCall->vfwd_fp, &writeNALPrefixCode, &writePacket, 2, rtp_hdr);
                        }

			else if(naluHeaderValue.type == 26){

                                /* MTAP-16 Aggregate NAL Units */
                                parseH264MTAPNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vfwd_file, currentRTPCall->vfwd_fp, &writeNALPrefixCode, &writePacket, 16, 2, rtp_hdr);
                        }

                        else if(naluHeaderValue.type == 27){

                                /* MTAP-32 NAL Aggregate Unit */
                                parseH264MTAPNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vfwd_file, currentRTPCall->vfwd_fp, &writeNALPrefixCode, &writePacket, 24, 2, rtp_hdr);

                        }

#ifdef HAVE_LIB_VLC
                        int returnMonitor = liveMonitorHandler(currentRTPCall, FWDVIDEOFT, currentRTPCall->videoPayloadType, rtp_hdr);
                        if(returnMonitor == 1){
       		                sendRTP(sample, packet->DATA.len, FWDVIDEOPORT_INT,currentRTPCall->liveMonitorSocket);
                        }
#endif						

                        if(writeNALPrefixCode){
                                phtonl(pd,0x00000001);
                                fwrite(pd,1,4,currentRTPCall->vfwd_fp);
                        }

                        if(writePacket){

                        	size_t ret = fwrite(value, sizeof(u_char), (payloadLength - offset), currentRTPCall->vfwd_fp);
                            if(ret < (payloadLength - offset)){
                            	INSTANT_USER_MSG("Error: Writing data to file %s:%s \n",currentRTPCall->vfwd_file,strerror(errno));
                                SIPRTPLOG("Error: Writing data to file %s:%s \n",currentRTPCall->vfwd_file,strerror(errno));
                                INSTANT_USER_MSG("Stopped eavesdropping on this conversation, deleting RTP dissector\n");
                                SIPRTPLOG("Stopped eavesdropping on this conversation, deleting RTP dissector\n");
                                deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                            }
                        }

						++currentRTPCall->vfwd_count;
                        currentRTPCall->vfwd_seqcount = rtp_hdr -> sequence_no;
                        ++currentRTPCall->vfwd_count;
                        ++currentRTPCall->vrtp_count;
                        //SIPRTPLOG("Successfully decoded and dumped the forward direction video payload \n");
                }
                else if(vdirection == 2){

                        if(currentRTPCall->vrev_seqcount == rtp_hdr -> sequence_no){
                                //SIPRTPLOG("Probably a duplicate RTP packet, currentRTPCall vrev_seqcount = %d\n",ntohs(currentRTPCall->vrev_seqcount));
                                return -1;
                        }

                        struct naluHeader naluHeaderValue;
                        int writeNALPrefixCode = 1;
                        int writePacket = 1;
                        int offset = 0;

                        u_char *value = rtpPayload;

                        naluHeaderValue.forbidden = (*value & 0x80) >> 7;
                        naluHeaderValue.nri = (*value & 0x60) >> 5;
                        naluHeaderValue.type = *value & 0x1F;

						if(naluHeaderValue.type == 5){

#ifdef HAVE_LIB_VLC
                            /* Coded slice IDR Picture */

                            /* Get the RTP marker bit */
                            u_char markerBit = rtp_hdr -> payloadType & 0x80;

                            if(currentRTPCall->currentRevPPSCtr != currentRTPCall->previousRevPPSCtr ){

                                if(currentRTPCall->recvRevAllIDRPic == 1){

                        	        /* Delete the current IDR pictures */
									deleteIDRPicture(currentRTPCall->revIDRPtr);
                                    currentRTPCall->recvRevAllIDRPic = 0;
									currentRTPCall->revIDRPtr = NULL;
									currentRTPCall->revIDRCtr = 0;
                                }

                                if(currentRTPCall->revIDRPtr == NULL){

                                    /* First IDR coded picture slice in the coded video sequence */

                                    currentRTPCall->revIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
                                    if(currentRTPCall->revIDRPtr != NULL){

                                        currentRTPCall->revCurrentIDRPtr = currentRTPCall->revIDRPtr;

                                        memset(currentRTPCall->revIDRPtr->idrPictureData,'\0',sizeof(currentRTPCall->revIDRPtr->idrPictureData));
                                        currentRTPCall->revIDRPtr->idrPictureLength =  (packet->DATA.len - sizeof(struct sniff_rtp));
                                        memcpy(currentRTPCall->revIDRPtr->idrPictureData, value, (packet->DATA.len - sizeof(struct sniff_rtp)));
                                        currentRTPCall->revIDRPtr->next = NULL;
										currentRTPCall->revIDRCtr += 1;

                                        if(markerBit == 0x80){

                                            /* The marker bit is set to 1 */
                                            /* All IDR codec pic for this coded video seq are received */
                                            currentRTPCall->recvRevAllIDRPic = 1;
                                            /* Incr the PPS ctr */
                                            currentRTPCall->previousRevPPSCtr += 1;
                                        }
                                    }

                                }
								else {
                                        struct idrPicture *tempIDRPtr = (struct idrPicture *) malloc(1 * sizeof(struct idrPicture));
                                        if(tempIDRPtr != NULL){

                                            currentRTPCall->revCurrentIDRPtr->next = tempIDRPtr;
                                            currentRTPCall->revCurrentIDRPtr = tempIDRPtr;

                                            memset(currentRTPCall->revCurrentIDRPtr->idrPictureData, '\0', sizeof(currentRTPCall->revCurrentIDRPtr->idrPictureData));
                                            currentRTPCall->revCurrentIDRPtr->idrPictureLength = (packet->DATA.len - sizeof(struct sniff_rtp));
                                            memcpy(currentRTPCall->revCurrentIDRPtr->idrPictureData, value, currentRTPCall->revCurrentIDRPtr->idrPictureLength);
                                            currentRTPCall->revCurrentIDRPtr->next = NULL;
											currentRTPCall->revIDRCtr += 1;

                                            if(markerBit == 0x80){

                                                /* The marker bit is set to 1 */
                                                /* All IDR codec pic for this coded video seq are received */
                                                currentRTPCall->recvRevAllIDRPic = 1;
                                                /* Incr the PPS ctr */
                                                currentRTPCall->previousRevPPSCtr += 1;
                                            }
                                        }
                                }
                            }
#endif
                        }
						else if(naluHeaderValue.type == 7){

#ifdef HAVE_LIB_VLC
                            /* Saving the h264 parameter sets for live video monitoring, the video decoder needs the paramter sets for decoding the raw h264 video */
                            /* Sequence Parameter Set */
                            //if(!currentRTPCall->savedRevSPS){
								memset(currentRTPCall->revSPS,'\0',sizeof(currentRTPCall->revSPS));
                                memcpy(currentRTPCall->revSPS,value,sizeof(currentRTPCall->revSPS));
                                currentRTPCall->savedRevSPS = 1;
                            //}
#endif
                        }
                        else if(naluHeaderValue.type == 8){

#ifdef HAVE_LIB_VLC
                            /* Picture Parameter Set */ 
							int PPSLength = payloadLength;
							if(payloadLength > sizeof(currentRTPCall->revPPS))
								PPSLength = sizeof(currentRTPCall->revPPS);
                            memset(currentRTPCall->revPPS,'\0',sizeof(currentRTPCall->revPPS));
                            memcpy(currentRTPCall->revPPS,value,PPSLength); 
							currentRTPCall->currentRevPPSCtr += 1;
#endif

                        }
                        else if(naluHeaderValue.type == 28){

                                parseH264FUANAL(&naluHeaderValue, &value, &offset, &writeNALPrefixCode, &writePacket, &currentRTPCall->rev_fuastart);
                                /* This is FU-A NAL unit */

                                /* Thanks to Manuel Koschuch for fixing FU-A parsing bugs */
                        }
                        else if(naluHeaderValue.type == 24){

                                /* STAP-A NAL aggregate packets */
                                parseH264STAPANAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vrev_file, currentRTPCall->vrev_fp, &writeNALPrefixCode, &writePacket, 3, rtp_hdr);
                        }
						else if(naluHeaderValue.type == 25){

                                /* STAP-B NAL aggregate packets */
                                parseH264STAPBNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vrev_file, currentRTPCall->vrev_fp, &writeNALPrefixCode, &writePacket, 3, rtp_hdr);
                        }
                        else if(naluHeaderValue.type == 26){

                                /* MTAP-16 Aggregate NAL Units */
                                parseH264MTAPNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vrev_file, currentRTPCall->vrev_fp, &writeNALPrefixCode, &writePacket, 16, 3, rtp_hdr);
                        }

                        else if(naluHeaderValue.type == 27){

                                /* MTAP-32 NAL Aggregate Unit */
                                parseH264MTAPNAL(&value, currentRTPCall, packet->DATA.len, currentRTPCall->vrev_file, currentRTPCall->vrev_fp, &writeNALPrefixCode, &writePacket, 24, 3, rtp_hdr);

                        }

#ifdef HAVE_LIB_VLC
                        int returnMonitor = liveMonitorHandler(currentRTPCall, REVVIDEOFT, currentRTPCall->videoPayloadType, rtp_hdr);
                        if(returnMonitor == 1){
                           sendRTP(sample, packet->DATA.len, REVVIDEOPORT_INT,currentRTPCall->liveMonitorSocket);
                        }
#endif

                        if(writeNALPrefixCode){
                                phtonl(pd,0x00000001);
                                fwrite(pd,1,4,currentRTPCall->vrev_fp);
                        }

						if(writePacket){

                        	size_t ret = fwrite(value, sizeof(u_char), (payloadLength - offset), currentRTPCall->vrev_fp);
                            if(ret < (payloadLength - offset)){
                            	INSTANT_USER_MSG("Error: Writing data to file %s:%s \n",currentRTPCall->vrev_file,strerror(errno));
                                SIPRTPLOG("Error: Writing data to file %s:%s \n",currentRTPCall->vrev_file,strerror(errno));
                                INSTANT_USER_MSG("Stopped eavesdropping on this conversation, deleting RTP dissector\n");
                                SIPRTPLOG("Stopped eavesdropping on this conversation, deleting RTP dissector\n");
                                deleteRTPDissector(currentRTPCall->port,currentRTPCall->ip_addr);
                            }
                        }

                        ++currentRTPCall->vrev_count;
                        currentRTPCall->vrev_seqcount = rtp_hdr -> sequence_no;
                        ++currentRTPCall->vrev_count;
                        ++currentRTPCall->vrtp_count;
                        //SIPRTPLOG("Successfully decoded and dumped the reverse direction video payload \n");
                }
        }
	}
    /* dump_pkthdr is a global variable in ec_decode.h, and it gets the values assigned in ec_decode.c
    	 dump_pkthdr = pkthdr */

    if(currentRTPCall->dumper != NULL)
    	pcap_dump((u_char *)currentRTPCall->dumper,dump_pkthdr,packet->packet);

    return -1;
}

int create_wav_header(FILE *fp, unsigned short numChannels, unsigned short bitsPerSample, unsigned int sampleRate, short audio_format, off_t length){

        //SIPRTPLOG("Inside create_wav_header\n");

        if(fp == NULL){
                INSTANT_USER_MSG("Error: Null file pointer at create_wav_header\n");
                SIPRTPLOG("Error: Null file pointer at create_wav_header\n");
                return -1;
        }

        unsigned short blockAlign = bitsPerSample / 8 * numChannels;
        unsigned int avgBytesPerSecond = sampleRate * blockAlign;
        char pd[4];
        int ret = 0;

        phtonl(pd,0x52494646);                           /*RIFF header value */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
		off_t chunkSize = length - 8;                     /* Chunk size */
		memcpy(pd, &chunkSize, sizeof(chunkSize) );
        //phtonl(pd,length-8);                                 
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        phtonl(pd,0x57415645);                          /*WAVE header value */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        phtonl(pd,0x666d7420);                          /*fmt header value */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
		int val = 0x00000010;                           /*Length of the subchunk 1 */
        memcpy(pd,&val,sizeof(int));
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        val = audio_format;                                     /*Audio Format, Using Linear PCM */
        memcpy(pd,&val,sizeof(short));
        if((ret = fileWrite(pd,1,2,fp)) < 0){
                return ret;
        }
        val = 0x0001;                                   /*Number of channels, 1 = Mono and 2 = Stereo */
        memcpy(pd,&val,sizeof(short));
        if((ret = fileWrite(pd,1,2,fp)) < 0){
                return ret;
        }
        memcpy(pd,&sampleRate,sizeof(unsigned int));                    /* Sampling Rate */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        memcpy(pd,&avgBytesPerSecond,sizeof(unsigned int));             /*Average Bytes per Second,calculated above */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        memcpy(pd,&blockAlign,sizeof(unsigned int));                    /*Block Align*/
        if((ret = fileWrite(pd,1,2,fp)) < 0){
                return ret;
        }
        memcpy(pd,&bitsPerSample,sizeof(unsigned int));                 /*Bits per sample */
        if((ret = fileWrite(pd,1,2,fp)) < 0){
                return ret;
        }
        phtonl(pd,0x64617461);                                          /*Data, indicates the following are Data chunks */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }

		off_t subChunk2Size = length - 44;
		memcpy(pd, &subChunk2Size, sizeof(subChunk2Size) );
        //phtonl(pd,length-44);                                 /* Length of the subchunk 2 */
        if((ret = fileWrite(pd,1,4,fp)) < 0){
                return ret;
        }
        return 0;
}

int fileWrite(char *buffer, int size, int nitems, FILE *fp){

        size_t ret = fwrite(buffer, size, nitems, fp);
        if(ret < nitems){
                INSTANT_USER_MSG("Error writing data at filwWrite:%s \n",strerror(errno));
                SIPRTPLOG("Error writing data at filwWrite:%s \n",strerror(errno));
                return -1;
        }
        return 0;

}

void releaseAudioDecoder(rtpSIP *currentRTPCall){

	switch(currentRTPCall->audioCodecType){

		case COG722:
			/*
			if(currentRTPCall->rtp_count > 1){
				g722_decode_release(&currentRTPCall->dec_state);
				g722_decode_release(&currentRTPCall->dec_state1);
			}
			*/
		break;
		case COG723:
#ifndef OS_WINDOWS
#ifndef ARCH_X64		
			if(currentRTPCall->hDecoderFwd >= 0)
				release_g723_decoder(currentRTPCall->hDecoderFwd);

			if(currentRTPCall->hDecoderRev >= 0)
				release_g723_decoder(currentRTPCall->hDecoderRev);
#endif
#endif			
		break;
		case COG726:
#ifndef ARCH_X64
			if(currentRTPCall->hDecoderFwd >= 0)
                release_g726_decoder(currentRTPCall->hDecoderFwd);

            if(currentRTPCall->hDecoderRev >= 0)
                release_g726_decoder(currentRTPCall->hDecoderRev);	
#endif			
		break;
		case COG729:
#ifndef OS_WINDOWS
#ifndef ARCH_X64		
			if(currentRTPCall->hDecoderFwd >= 0)
                release_g729_decoder(currentRTPCall->hDecoderFwd);

            if(currentRTPCall->hDecoderRev >= 0)
                release_g729_decoder(currentRTPCall->hDecoderRev);
#endif
#endif			
		break;
		case COISAC:
			if(currentRTPCall->rtp_count >= 1) {
				WebRtcIsac_Free(currentRTPCall->fwd_isac_inst);
				WebRtcIsac_Free(currentRTPCall->rev_isac_inst);
			}
		break;
	}
}

void addRTPDissector(int port, char *ipaddress){

         char *dissect_name;
         dissect_name = (char *) malloc((6 + 3 +  MAX_ASCII_ADDR_LEN) * sizeof(char));
         memset(dissect_name,'\0',(6 + 3 + MAX_ASCII_ADDR_LEN));
         create_dissect_string(port,ipaddress,dissect_name);
         SIPLOG("Adding RTP dissector value is %s\n",dissect_name);
         dissect_add(dissect_name, APP_LAYER_UDP,port,dissector_rtp_sip);
         free(dissect_name);
}

void deleteRTPDissector(int port, char *ipaddress){

         char *dissect_name;
         dissect_name = (char *) malloc((6 + 3 + MAX_ASCII_ADDR_LEN) * sizeof(char));
         memset(dissect_name,'\0',(6 + 3 + MAX_ASCII_ADDR_LEN));
         create_dissect_string(port,ipaddress,dissect_name);
         SIPLOG("Deleting RTP dissector value is %s\n",dissect_name);
         dissect_del(dissect_name);
         free(dissect_name);
}
 
