#include <sox.h>
#include <targets.h>
#include <ec_siplog.h>
#include <ec_siprtpmuxer.h>

extern pthread_mutex_t muxing_mutex;
void *muxMediaFiles(void *);

void writeWavFile(char* filename, u_char* audioBuf, int len, int audioCodecType)
{
    int sampleRate = 0;

    if(audioCodecType == COISAC)
        sampleRate = 16000;
    else
        sampleRate = 8000;
	SIPRTPLOG("Opening the fwd direction media file %s\n", filename);
	FILE* fp = fopen(filename,"w+|b");
	if(fp == NULL ){
       INSTANT_USER_MSG("Error opening media file %s\n",filename);
	   SIPRTPLOG("Error opening media file %s\n",filename);
        return;
	}
	if(create_wav_header(fp, 1, 16, sampleRate, 1, len) < 0){
        INSTANT_USER_MSG("Error creating wav file,%s\n",filename);
        SIPRTPLOG("Error creating wav file,%s\n",filename);
        return;
	}
    if(fwrite(audioBuf, sizeof(u_char), len, fp) < 0) {
        INSTANT_USER_MSG("Error #%d writing wav file, %s: %s\n", errno, filename, strerror(errno) );
        SIPRTPLOG("Error #%d writing wav file, %s: %s\n", errno, filename, strerror(errno) );
    }
    if(fclose(fp) < 0) {
        INSTANT_USER_MSG("Error #%d closing wav file, %s: %s\n", errno, filename, strerror(errno) );
        SIPRTPLOG("Error #%d closing wav file, %s: %s\n", errno, filename, strerror(errno) );
    }
}

int updateWavHeader(FILE* fp, int sampleRate)
{
	int ret = -1;
	struct stat s;
	if(fstat(fileno(fp), &s) == 0) {
		rewind(fp);
		if(create_wav_header(fp, 1, 16, sampleRate, 1, s.st_size) == 0) {
			if(fseek(fp, 0L, SEEK_END) == 0) {
				ret = 0;
			}
		}
	}
	return ret;
}

int moveAudioBuffer(u_char** oldAudioBuffer, int* oldAudioIdx, u_char** audioBuffer)
{
	*audioBuffer = (u_char*)malloc(*oldAudioIdx + 1);
	bzero(*audioBuffer, *oldAudioIdx + 1);
	memcpy(*audioBuffer, *oldAudioBuffer, *oldAudioIdx);
    free(*oldAudioBuffer);
    *oldAudioBuffer = NULL;
	int audioIdx = *oldAudioIdx;
	*oldAudioIdx = 0;
	return audioIdx;
}

void *muxMediaFiles(void *argPtr){
//printf("%s:%d - %s in thread to create media files\n", __FILE__, __LINE__, __func__);

	ec_thread_init();

	struct muxVariable{
            rtpSIP *currentRTPCall;
            int video_active;
    };
	
	struct muxVariable *muxV = (struct muxVariable *)argPtr;
	
	rtpSIP *currentRTPCall = muxV->currentRTPCall;
	int video_active = muxV->video_active;

	char fwd_wav[170] = {'\0'};
	char rev_wav[170] = {'\0'};
	char both_wav[170] = {'\0'};
	char vfwd_file[174] = {'\0'};
	char vrev_file[174] = {'\0'};
	char vfwd_mp4[174] = {'\0'};
	char vrev_mp4[174] = {'\0'};
	char vfwd_avi[174] = {'\0'};
	char vrev_avi[174] = {'\0'};

	//MUXING_LOCK;
	pthread_mutex_lock(&muxing_mutex);

	int fwd_count = currentRTPCall->fwd_count;
	int rev_count = currentRTPCall->rev_count;
	int vfwd_count = currentRTPCall->vfwd_count;
	int vrev_count = currentRTPCall->vrev_count;
	int audioCodecType = currentRTPCall->audioCodecType;
	FILE *fwd_fp = currentRTPCall->fwd_fp;
	FILE *rev_fp = currentRTPCall->rev_fp;
	FILE *vfwd_fp = currentRTPCall->vfwd_fp;
	FILE *vrev_fp = currentRTPCall->vrev_fp;
	/* Copy the file name from the RTP struct, because the RTP structure will be deleted */
	strncpy(fwd_wav,currentRTPCall->fwd_wav,sizeof(fwd_wav));
	strncpy(rev_wav,currentRTPCall->rev_wav,sizeof(rev_wav));
	strncpy(both_wav,currentRTPCall->both_wav,sizeof(both_wav));
	strncpy(vfwd_file,currentRTPCall->vfwd_file,sizeof(vfwd_file));
	strncpy(vrev_file,currentRTPCall->vrev_file,sizeof(vrev_file));
	strncpy(vfwd_mp4,currentRTPCall->vfwd_mp4,sizeof(vfwd_mp4));
	strncpy(vrev_mp4,currentRTPCall->vrev_mp4,sizeof(vrev_mp4));
	strncpy(vfwd_avi,currentRTPCall->vfwd_avi,sizeof(vfwd_avi));
	strncpy(vrev_avi,currentRTPCall->vrev_avi,sizeof(vrev_avi));

    u_char* fwdAudioBuf;
    int fwdAudioIdx = moveAudioBuffer(&currentRTPCall->fwdAudioBuf, &currentRTPCall->fwdAudioIdx, &fwdAudioBuf);
	
	u_char* revAudioBuf;
	int revAudioIdx  = moveAudioBuffer(&currentRTPCall->revAudioBuf, &currentRTPCall->revAudioIdx, &revAudioBuf);

	currentRTPCall->copiedFiles = 1;
	
	pthread_mutex_unlock(&muxing_mutex);
	//MUXING_UNLOCK;

	/* Close the opened media file descriptors */
	if(fwd_count > 1) {
		writeWavFile(fwd_wav, fwdAudioBuf, fwdAudioIdx, audioCodecType);
		free(fwdAudioBuf);
/*
		if(updateWavHeader(fwd_fp) < 0)
			SIPLOG("Error #%d updating WAV header in %s:  %s\n", errno, fwd_wav, strerror(errno) );
		if(fclose(fwd_fp) < 0)
			SIPLOG("Error #%d closing forward audio file pointer:  %s\n", errno, strerror(errno) );
 */
	}
	if(rev_count > 1){
		writeWavFile(rev_wav, revAudioBuf, revAudioIdx, audioCodecType);
		free(revAudioBuf);
/*
		SIPLOG("Closing reverse audio file\n");
		if(updateWavHeader(rev_fp) < 0)
			SIPLOG("Error #%d updating WAV header in %s:  %s\n", errno, rev_wav, strerror(errno) );
		if(fclose(rev_fp) < 0)
			SIPLOG("Error #%d closing reverse audio file pointer:  %s\n", errno, strerror(errno) );
*/
	}

	if(video_active == 1){

		if(vrev_count > 1){
			SIPLOG("Closing reverse video file\n");
			if(fsync(fileno(vrev_fp)) < 0) {
				SIPLOG("Error #%d syncing reverse video file pointer:  %s\n", errno, strerror(errno) );
			}
        	if(fclose(vrev_fp) < 0) {
				SIPLOG("Error #%d closing reverse video file pointer:  %s\n", errno, strerror(errno) );
			}
			if(rev_count > 1){
				struct stat s;
				stat(rev_wav, &s);
				SIPLOG("%s is %d bytes\n", rev_wav, s.st_size);
				if(mixAv_modify_yes) {
				    createReverseStreamVideoFiles(rev_wav, vrev_file, vrev_mp4, vrev_avi);
				}
			}
			INSTANT_USER_MSG("SIP Call ended. Reverse direction video conversation saved to file %s\n",vrev_file);
			SIPLOG("SIP Call ended. Reverse direction video conversation saved to file %s\n",vrev_file);
		}

		if(vfwd_count > 1){
			SIPLOG("Closing forward video file\n");
			if(fsync(fileno(vfwd_fp)) < 0) {
				SIPLOG("Error #%d syncing forward video file pointer:  %s\n", errno, strerror(errno) );
			}
        	if(fclose(vfwd_fp) < 0)
				SIPLOG("Error #%d closing forward video file pointer:  %s\n", errno, strerror(errno) );
			if(fwd_count > 1){
				if(mixAv_modify_yes) {
    				createForwardStreamVideoFiles(fwd_wav, vfwd_file, vfwd_mp4, vfwd_avi);
                }
			}
			INSTANT_USER_MSG("SIP Call ended. Forward direction video conversation saved to file %s\n",vfwd_file);
			SIPLOG("SIP Call ended. Forward direction video conversation saved to file %s\n",vfwd_file);
		}
	}

	if(fwd_count > 1 && rev_count > 1){	
		struct stat s, t;
		stat(rev_wav, &s);
		SIPLOG("%s is %d bytes\n", rev_wav, s.st_size);
		stat(fwd_wav, &t);
		SIPLOG("%s is %d bytes\n", fwd_wav, t.st_size);

		int mix_return = mix_files(fwd_wav,rev_wav,both_wav);
    	if(mix_return == -1){
    		int compressValue = 0, compressValue1 = 0;
#ifdef COMPRESSION
        	compressValue = compressFileBZ2(fwd_wav);
#endif
        	if(compressValue == 1){
        		SIPLOG("SIP Call ended. Forward conversation recorded in file '%s.bz2'\n",fwd_wav);
            	INSTANT_USER_MSG("SIP Call ended. Forward conversation recoded in files '%s.bz2'\n",fwd_wav);
            	remove(fwd_wav);
        	}
        	else{
        		SIPLOG("SIP Call ended. Forward conversation recoded in file '%s'\n",fwd_wav);
            	INSTANT_USER_MSG("SIP Call ended. Forward conversation recoded in file '%s'\n",fwd_wav);
        	}
#ifdef COMPRESSION
        	compressValue1 = compressFileBZ2(rev_wav);
#endif
        	if(compressValue1 == 1){
        		SIPLOG("SIP Call ended. Reverse conversation recorded in file '%s.bz2'\n",rev_wav);
            	INSTANT_USER_MSG("SIP Call ended. Reverse conversation recorded in file '%s.bz2'\n",rev_wav);
            	remove(rev_wav);
        	}
        	else{
        		SIPLOG("SIP Call ended. Reverse conversation recorded in file '%s'\n",rev_wav);
            	INSTANT_USER_MSG("SIP Call ended. Reverse conversation recorded in file '%s'\n",rev_wav);
        	}
		}
		else{
        	int compressValue = 0;
#ifdef COMPRESSION
            compressValue = compressFileBZ2(both_wav);
#endif
            if(compressValue == 1){
            	SIPLOG("SIP Call ended. Conversation recorded and compressed in file '%s.bz2' \n",both_wav);
                INSTANT_USER_MSG("SIP Call ended. Conversation recorded and compressed in file '%s.bz2'\n",both_wav);
                remove(both_wav);
                remove(fwd_wav);
                remove(rev_wav);
            }
            else{
            	SIPLOG("SIP Call ended. Conversation recorded in file '%s' \n",both_wav);
                INSTANT_USER_MSG("SIP Call ended. Conversation recorded in file '%s' \n",both_wav);
                //remove(currentRTPCall->fwd_wav);
                //remove(currentRTPCall->rev_wav);
            }
		}
	}
	else if((fwd_count == 1) && (rev_count == 1)){
    	SIPLOG("SIP Call ended. No conversation saved.Probably G-ARP disabled on both forward and reverse direction phones or if the SIP server is a b2bua like Asterisk the same call will be detected twice or codec not supported \n");
    	INSTANT_USER_MSG("SIP Call ended. No conversation saved.Probably G-ARP disabled on both forward and reverse direction phones or if the SIP server is a b2bua like Asterisk the same call will be detected twice or codec not supported\n");
    }
	else if(fwd_count == 1 && rev_count > 1){
    	SIPLOG("G-ARP disabled on the forward direction phone \n");
        INSTANT_USER_MSG("G-ARP disabled on the forward direction phone \n");
        int compressValue = 0;
#ifdef COMPRESSION
        compressValue = compressFileBZ2(rev_wav);
#endif
        if(compressValue == 1){

        	SIPLOG("SIP Call ended. Conversation recorded in file '%s.bz2' \n", rev_wav);
            INSTANT_USER_MSG("SIP Call ended. Conversation recorded in file '%s.bz2' \n", rev_wav);
            remove(rev_wav);
        }
        else{
        	SIPLOG("SIP Call ended. Conversation recorded in file '%s' \n", rev_wav);
            INSTANT_USER_MSG("SIP Call ended. Conversation recorded in file '%s' \n", rev_wav);
        }
	}
	else if(rev_count == 1 && fwd_count > 1){
    	SIPLOG("G-ARP disabled on the reverse direction phone \n");
        INSTANT_USER_MSG("G-ARP disabled on the reverse direction phone \n");
        int compressValue = 0;
#ifdef COMPRESSION
        compressValue = compressFileBZ2(fwd_wav);
#endif
        if(compressValue == 1){

        	SIPLOG("SIP Call ended. Conversation recorded in file '%s.bz2' \n", fwd_wav);
            INSTANT_USER_MSG("SIP Call ended. Conversation recorded in file '%s.bz2' \n", fwd_wav);
            remove(fwd_wav);
        }
        else{
        	SIPLOG("SIP Call ended. Conversation recorded in file '%s' \n", fwd_wav);
            INSTANT_USER_MSG("SIP Call ended. Conversation recorded in file '%s' \n", fwd_wav);
        }
    }

//printf("%s:%d - %s calling pthread_exit\n", __FILE__, __LINE__, __func__);
//	pthread_exit(NULL);
	return NULL;
}
