#include <ec.h>
#include <targets.h>
#include <ec_siplog.h>
#include "avconv.h"

#ifdef COMPRESSION

#include <bzlib.h>


const int BUFSIZE = 65536; // 64 KB
void catchError(const int, FILE *);
int compressFileBZ2(char *);

#endif

#ifdef HAVE_VIDEO_DECODER
#include "ffmpeg.h"
#endif

const int MAX_ARGS = 100;
const char* AV_DEFAULT_COMMAND = "avconv";
const char* AV_DEFAULT_AUDIOIN_OPTIONS = "-y -b:a 128k";
const char* AV_DEFAULT_VIDEOIN_OPTIONS = "-r 6";
const char* AV_DEFAULT_AVOUT_OPTIONS = "-s 352x288 -b:a 128k -r 25";
const char* AV_DEFAULT_INPUT_OPTION = "-i";

int buildArgArray(char *argArray[], const char *args, int argIdx)
{
	char* tmpArgs = (char*)malloc(strlen(args) + 1);
	bzero(tmpArgs, strlen(args) + 1);
	if(tmpArgs) 
	{
		strncpy(tmpArgs, args, strlen(args) );
		char *savePtr, *argPtr = NULL;
		argPtr = strtok_r(tmpArgs, " ", &savePtr);
		while( argPtr != NULL && argIdx < MAX_ARGS )
		{
			argArray[argIdx] = (char*)malloc(strlen(argPtr)+1);
			bzero(argArray[argIdx], strlen(argPtr)+1);
			strncpy(argArray[argIdx], argPtr, strlen(argPtr) );
			++argIdx;
		   	argPtr = strtok_r(NULL, " ", &savePtr);
		}
	}
	free(tmpArgs);
	return argIdx;
}

const char* get_av_options(char* options, const char* defaultOptions)
{
	if(options)
	{
		return options;
	}
	else
	{
		return defaultOptions;
	}
}

char* add_av_arg(const char* avArg)
{
	char* arg = (char*)malloc(strlen(avArg) + 1);
	bzero(arg, strlen(avArg) + 1);
	strncpy(arg, avArg, strlen(avArg) );
	return arg;
}

void createForwardStreamVideoFiles(char *fwd_wav, char *vfwd_file, char *vfwd_mp4, char *vfwd_avi){

#ifdef HAVE_VIDEO_DECODER
	char* argArray[MAX_ARGS];

	const char* avCommand = get_av_options(GBL_CONF->av_command, AV_DEFAULT_COMMAND);
	const char* audioInOptions = get_av_options(GBL_CONF->av_audio_in_opts, AV_DEFAULT_AUDIOIN_OPTIONS);
	const char* videoInOptions = get_av_options(GBL_CONF->av_video_in_opts, AV_DEFAULT_VIDEOIN_OPTIONS);
	const char* avOutOptions   = get_av_options(GBL_CONF->av_out_opts, AV_DEFAULT_AVOUT_OPTIONS);
	const char* avInputOption  = get_av_options(GBL_CONF->av_input_opt, AV_DEFAULT_INPUT_OPTION);  

    if(strncmp(avCommand, AV_DEFAULT_COMMAND, strlen(avCommand) ) == 0) {
	    argArray[0] = (char*)malloc(strlen(avCommand) + 1);
	    strncpy(argArray[0], avCommand, strlen(avCommand) );
	    int argIdx = 1;


	    argIdx = buildArgArray(argArray, audioInOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(avInputOption);
	    argArray[argIdx++] = add_av_arg(fwd_wav);

	    argIdx = buildArgArray(argArray, videoInOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(avInputOption);
	    argArray[argIdx++] = add_av_arg(vfwd_file);

	    argIdx = buildArgArray(argArray, avOutOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(vfwd_avi);
	    avconv_main(argIdx, argArray);

	    int i = 0;
	    for(; i < argIdx; ++i)
	    {
		    if(argArray[i])
                free(argArray[i]);
	    }
	}
	else {
	    char command[1024] = { '\0' };
	    snprintf(command, sizeof(command), "%s %s %s %s %s %s %s %s %s",
	        avCommand,
	        audioInOptions,
	        avInputOption, 
	        fwd_wav,
	        videoInOptions,
	        avInputOption,
	        vfwd_file,
	        avOutOptions,
	        vfwd_avi);
	    
	    if(system(command) != 0) {
	        INSTANT_USER_MSG("'%s' return non-zero.\n", command);
	    }
	}
/*	int mp4_ret = create_mp4_header(vfwd_file,vfwd_mp4);
    if(mp4_ret == 1){
    	SIPLOG("Error: Creating forward avi file, try playing the %s file using vlc \n",vfwd_file);
        INSTANT_USER_MSG("Error: Creating forward avi file, try playing the %s file using vlc \n",vfwd_file);
    }
    if(mp4_ret != 1){
    	int avi_ret = mix_audio_video(fwd_wav,vfwd_mp4,vfwd_avi);
        if(avi_ret == 1){
        	SIPLOG("Error: Mixing forward audio and video stream \n");
            INSTANT_USER_MSG("Error: Mixing forward audio and video stream \n");
            SIPLOG("Saving forward video conversation with no audio to file, '");
            INSTANT_USER_MSG("Saving forward video conversation with no audio to file, '");
            int compressValue = 0;
#ifdef COMPRESSION
            compressValue = compressFileBZ2(vfwd_mp4);
#endif
            if(compressValue == 1){

            	SIPLOG("%s.bz2'\n",vfwd_mp4);
                INSTANT_USER_MSG("%s.bz2'\n",vfwd_mp4);
                remove(vfwd_file);
                remove(vfwd_mp4);
            }
			else{

            	SIPLOG("%s'\n",vfwd_mp4);
                INSTANT_USER_MSG("%s'\n",vfwd_mp4);
                remove(vfwd_file);
                //remove(currentRTPCall->fwd_wav);
            }
		}
    	else{
    		SIPLOG("Saving forward video conversation to file, '");
        	INSTANT_USER_MSG("Saving forward video conversation to file, '");
        	int compressValue = 0;
#ifdef COMPRESSION
        	compressValue = compressFileBZ2(vfwd_avi);
#endif
        	if(compressValue == 1){

        		SIPLOG("%s.bz2'\n",vfwd_avi);
            	INSTANT_USER_MSG("%s.bz2'\n",vfwd_avi);
            	remove(vfwd_mp4);
            	remove(vfwd_file);
            	remove(vfwd_avi);
        	}
       	 	else{
        		SIPLOG("%s'\n",vfwd_avi);
            	INSTANT_USER_MSG("%s'\n",vfwd_avi);
            	remove(vfwd_mp4);
            	//remove(currentRTPCall->vfwd_file);
            	//remove(currentRTPCall->fwd_wav);
        	}
		}
	}
#else
    int compressValue = 0;
#ifdef COMPRESSION
    compressValue = compressFileBZ2(vfwd_file);
#endif
    if(compressValue == 1){

    	INSTANT_USER_MSG("SIP Call ended. Forward direction video conversation saved to file %s.bz2\n",vfwd_file);
        SIPLOG("SIP Call ended. Forward direction video conversation saved to file %s.bz2\n",vfwd_file);
        remove(vfwd_file);
    }
    else{
    	INSTANT_USER_MSG("SIP Call ended. Forward direction video conversation saved to file %s\n",vfwd_file);
        SIPLOG("SIP Call ended. Forward direction video conversation saved to file %s\n",vfwd_file);
    }
*/
#endif // end HAVE_VIDEO_ENCODER
}

void createReverseStreamVideoFiles(char *rev_wav, char *vrev_file, char *vrev_mp4, char *vrev_avi){

#ifdef HAVE_VIDEO_DECODER
	char* argArray[MAX_ARGS];

	const char* avCommand = get_av_options(GBL_CONF->av_command, AV_DEFAULT_COMMAND);
	const char* audioInOptions = get_av_options(GBL_CONF->av_audio_in_opts, AV_DEFAULT_AUDIOIN_OPTIONS);
	const char* videoInOptions = get_av_options(GBL_CONF->av_video_in_opts, AV_DEFAULT_VIDEOIN_OPTIONS);
	const char* avOutOptions   = get_av_options(GBL_CONF->av_out_opts, AV_DEFAULT_AVOUT_OPTIONS);
	const char* avInputOption  = get_av_options(GBL_CONF->av_input_opt, AV_DEFAULT_INPUT_OPTION);  

    if(strncmp(avCommand, AV_DEFAULT_COMMAND, strlen(avCommand) ) == 0) {
	    argArray[0] = (char*)malloc(strlen(avCommand) + 1);
	    strncpy(argArray[0], avCommand, strlen(avCommand) );
	    int argIdx = 1;

	    argIdx = buildArgArray(argArray, audioInOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(avInputOption);
	    argArray[argIdx++] = add_av_arg(rev_wav);

	    argIdx = buildArgArray(argArray, videoInOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(avInputOption);
	    argArray[argIdx++] = add_av_arg(vrev_file);

	    argIdx = buildArgArray(argArray, avOutOptions, argIdx);
	    argArray[argIdx++] = add_av_arg(vrev_avi);

	    avconv_main(argIdx, argArray);

	    int i = 0;
	    for(; i < argIdx; ++i)
	    {
		    if(argArray[i])
                free(argArray[i]);
	    }
    }	
    else {
	    char command[1024] = { '\0' };
	    snprintf(command, sizeof(command), "%s %s %s %s %s %s %s %s %s",
	        avCommand,
	        audioInOptions,
	        avInputOption, 
	        rev_wav,
	        videoInOptions,
	        avInputOption,
	        vrev_file,
	        avOutOptions,
	        vrev_avi);
	    
	    if(system(command) != 0) {
	        INSTANT_USER_MSG("'%s' return non-zero.\n", command);
	    }
	}

/*
	int mp4_ret1 = create_mp4_header(vrev_file,vrev_mp4);
    if(mp4_ret1 == 1){
    	SIPLOG("Error: Creating reverse avi file, try playing the %s file using vlc \n",vrev_file);
        INSTANT_USER_MSG("Error: Creating reverse avi file, try playing the %s file using vlc \n",vrev_file);
    }
    if(mp4_ret1 != 1){

    	int avi_ret1 = mix_audio_video(rev_wav,vrev_mp4,vrev_avi);
        if(avi_ret1 == 1){
        	SIPLOG("Error: Mixing reverse audio and video stream \n");
            INSTANT_USER_MSG("Error: Mixing reverse audio and video stream \n");
            SIPLOG("Saving reverse video conversation with no audio to file, '");
            INSTANT_USER_MSG("Saving reverse video conversation with no audio to file, '");
            int compressValue = 0;
#ifdef COMPRESSION
            compressValue = compressFileBZ2(vrev_mp4);
#endif
            if(compressValue == 1){

            	SIPLOG("%s.bz2'\n",vrev_mp4);
                INSTANT_USER_MSG("%s.bz2'\n",vrev_mp4);
                remove(vrev_file);
                remove(vrev_mp4);
            }
			else{
            	SIPLOG("%s'\n",vrev_mp4);
                INSTANT_USER_MSG("%s'\n",vrev_mp4);
                remove(vrev_file);
                //remove(currentRTPCall->rev_wav);
            }
		}
        else{
        	SIPLOG("Saving reverse video conversation to file, '");
            INSTANT_USER_MSG("Saving reverse video conversation to file, '");
            int compressValue = 0;
#ifdef COMPRESSION
            compressValue = compressFileBZ2(vrev_avi);
#endif
            if(compressValue == 1){

            	SIPLOG("%s.bz2'\n", vrev_avi);
                INSTANT_USER_MSG("%s.bz2'\n", vrev_avi);
                remove(vrev_mp4);
                remove(vrev_file);
                remove(vrev_avi);
            }
            else{
            	SIPLOG("%s'\n", vrev_avi);
                INSTANT_USER_MSG("%s'\n", vrev_avi);
                remove(vrev_mp4);
                //remove(currentRTPCall->vrev_file);
                //remove(currentRTPCall->rev_wav);
            }
		}
	}
#else
    int compressValue = 0;
#ifdef COMPRESSION
    compressValue = compressFileBZ2(vrev_file);
#endif
	if(compressValue == 1){

    	INSTANT_USER_MSG("SIP Call ended. Reverse direction video conversation saved to file %s.bz2\n",vrev_file);
        SIPLOG("SIP Call ended. Reverse direction video conversation saved to file %s.bz2\n",vrev_file);
        remove(vrev_file);
	}
    else{
    	INSTANT_USER_MSG("SIP Call ended. Reverse direction video conversation saved to file %s\n",vrev_file);
        SIPLOG("SIP Call ended. Reverse direction video conversation saved to file %s\n",vrev_file);
    }
    */
#endif // end HAVE_VIDEO_DENCODER
}

#ifdef COMPRESSION
void catchError(const int bzerror,FILE* f) {
    // takes the enumerated bzerror as input and displays the reason
    //  and closes the output file
    switch (bzerror) {
    case BZ_CONFIG_ERROR:
         INSTANT_USER_MSG("reports that the library has been improperly compiled on your platform\n");
         break;
    case BZ_SEQUENCE_ERROR:
        INSTANT_USER_MSG("reports that the function sequence in code is incorrect\n");
        break;
    case BZ_PARAM_ERROR:
        INSTANT_USER_MSG(" reports that a parameter is out of range or otherwise manifestly incorrect\n");
        break;
    case BZ_MEM_ERROR:
        INSTANT_USER_MSG(" reports that a request to allocate memory failed\n");
        break;
    case BZ_DATA_ERROR:
        INSTANT_USER_MSG(" reports that a data integrity error is detected during decompression\n");
        break;
    case BZ_DATA_ERROR_MAGIC:
        INSTANT_USER_MSG("reports that the compressed stream does not start with the correct magic bytes\n");
        break;
    case BZ_IO_ERROR:
        INSTANT_USER_MSG(" reports that there was an IO error\n");
        break;
    case BZ_UNEXPECTED_EOF:
        INSTANT_USER_MSG(" reports that the compressed file finished before the logical end of stream was detected\n");
        break;
    case BZ_OUTBUFF_FULL:
        INSTANT_USER_MSG(" reports that that the output data will not fit into the output buffer provided\n");
        break;
    default:
        INSTANT_USER_MSG(" reports a non-enumerated general error\n");
        break;

    }    // end of switch
    if (f)
    {
        // file was fopened so lets fclose it
        if(fclose(f) < 0) 
		{
			INSTANT_USER_MSG("Error closing file: %s", strerror(errno) );
			SIPLOG("%s:%d - Error closing file: %s", __FILE__, __LINE__, strerror(errno) );
		}
    }

}

int compressFileBZ2(char *inputFile){

        int bzerror;
        int nBuf;
        char buf[BUFSIZE];
        unsigned int nBytesRaw, nBytesCompressed;

        FILE *unf,*bzipf = NULL;
        char *bzipFilename = NULL;

        bzipFilename = (char *) calloc((strlen(inputFile) + 5),sizeof(char));
        if(bzipFilename == NULL){
                INSTANT_USER_MSG("Error allocating memory for compress filename\n");
                return -1;
        }

        strncpy(bzipFilename,inputFile,strlen(inputFile));
        strcat(bzipFilename,".bz2");

        unf = fopen(inputFile,"rb");
        if(!unf){
                INSTANT_USER_MSG("Error opening file %s\n",inputFile);
                return -1;
        }

        bzipf = fopen(bzipFilename,"wb");
        if(bzipf == NULL){
                INSTANT_USER_MSG("Error opening file %s\n",bzipFilename);
                return -1;
        }
	BZFILE *b = BZ2_bzWriteOpen(&bzerror, bzipf, 9, 0, 30);
        if(bzerror != BZ_OK){

                BZ2_bzWriteClose(&bzerror,b,0,NULL,NULL);
                catchError(bzerror,bzipf);
                if(fclose(unf) < 0) {
					INSTANT_USER_MSG("Error closing file: %s", strerror(errno) );
					SIPLOG("%s:%d - Error closing file: %s", __FILE__, __LINE__, strerror(errno) );
				}
                return -1;
        }

        while(!feof(unf)){

                nBuf = fread(buf,1,BUFSIZE,unf);

                BZ2_bzWrite(&bzerror,b,buf,nBuf);
                if(bzerror != BZ_OK){

                        BZ2_bzWriteClose(&bzerror,b,0,NULL,NULL);
                        catchError(bzerror,bzipf);
				        if(fclose(unf) < 0) {
							INSTANT_USER_MSG("Error closing file: %s", strerror(errno) );
							SIPLOG("%s:%d - Error closing file: %s", __FILE__, __LINE__, strerror(errno) );
						}
                        return -1;
                }
        }
        if(fclose(unf) < 0) {
			INSTANT_USER_MSG("Error closing file: %s", strerror(errno) );
			SIPLOG("%s:%d - Error closing file: %s", __FILE__, __LINE__, strerror(errno) );
		}

        BZ2_bzWriteClose(&bzerror,b,0,&nBytesRaw,&nBytesCompressed);
        if(bzerror != BZ_OK){
                catchError(bzerror,bzipf);
                return -1;
        }

        if(fclose(bzipf) < 0) {
			INSTANT_USER_MSG("Error closing file: %s", strerror(errno) );
			SIPLOG("%s:%d - Error closing file: %s", __FILE__, __LINE__, strerror(errno) );
		}

        return 1;
}
#endif
