#include <ec.h>
#include <ec_inet.h>
#include <targets.h>
#include <dirent.h>
#include <ec_liveMonitor.h>
#include <ec_threads.h>

struct filelist{
	char fileName[256];	/* 265 from bits/dirent.h */
	int count;
	int type;
	struct filelist *next;
};

struct filelist *headFileList = NULL;
struct filelist *previousFileList = NULL;

extern int numHosts;
extern sipDB *firstSipCall;
extern sipDB *lastSipCall;

extern int targetcount;
extern struct targets targettab[1024];
extern struct singletarget target1;

typedef struct skinnycall SkinnyCall;
typedef SkinnyCall *SkinnyCallPtr;
extern SkinnyCallPtr first_skinnycall;

extern int isEmpty( SkinnyCallPtr sPtr);
extern char *lookupdialingname (int conferenceid, SkinnyCallPtr *sPtr);
extern char *lookupdialedname (int conferenceid, SkinnyCallPtr *sPtr);

char gblip[MAX_ASCII_ADDR_LEN];
char gblmac[MAX_ASCII_ADDR_LEN];
char tmpBuffer[256] = {'\0'};

/* Mutex variable for accessing SIP call structure */
extern pthread_mutex_t sipaccess_mutex;     /* Extern from text/interface */
#define SIPACCESS_LOCK     do{ pthread_mutex_lock(&sipaccess_mutex); } while(0)
#define SIPACCESS_UNLOCK   do{ pthread_mutex_unlock(&sipaccess_mutex); } while(0)

extern pthread_mutex_t sccpaccess_mutex;
#define SCCPACCESS_LOCK     do{ pthread_mutex_lock(&sccpaccess_mutex); } while(0)
#define SCCPACCESS_UNLOCK   do{ pthread_mutex_unlock(&sccpaccess_mutex); } while(0)

int getNumberOfHosts();
char* getHostsList(int,int);
int getNumberOfActiveCalls();
char* getActiveCalls(int,int);
char* getTargetTable(int,int);
int getNumberOfFiles();
char* getMediaFilesTable(int,int);
int checkValidFileType(char *);
void copyTargets(int);
void *getActiveCallPtr(int,int,int*);

int getNumberOfFiles(){

	struct dirent *dirptr;
	DIR *dirdp;
	int count = 0;
	int type = 0;

	headFileList = NULL;
    previousFileList = NULL;
	
	/* File Types : 
		1 = .h264/264, raw h264 file format 
		2 = .avi, avi file format
		3 = .pcap, pcap file format
	*/

	char buf[256] = { '\0'};
	size_t size_buf = 255;	

	char *getcwdReturn = getcwd(buf,size_buf);
	if(getcwdReturn == NULL){
		printf("Error: Getting the current working directory name\n");
	}

	dirdp = opendir(buf);
	if(dirdp == NULL){
		printf("Error: Opening the current directory\n");
		return -1;
	}

	while((dirptr = readdir(dirdp)) != NULL){

		type = checkValidFileType(dirptr->d_name);
		
		if(type > 3)
			continue;

		struct filelist *currentFileList = (struct filelist *) calloc(1,sizeof(struct filelist));
		if(currentFileList == NULL){
			printf("Error: Allocating memory while reading files in the current directory\n");
			return -1;
		}

		if(headFileList == NULL){
			previousFileList = currentFileList;
			headFileList = currentFileList;
			currentFileList->next = NULL;
		}
		else{
			currentFileList->next = NULL;
			previousFileList->next = currentFileList;
			previousFileList = currentFileList;
		}
		
		memset(currentFileList->fileName, '\0', sizeof(currentFileList->fileName));
		memcpy(currentFileList->fileName, dirptr->d_name, sizeof(currentFileList->fileName) - 1);
	
		count = count + 1;
		currentFileList->count = count;
		currentFileList->type = type;
	}

	return count;
}

char *getMediaFilesTable(int rowNumber, int columnID){

	int i = 0;
	int count = getNumberOfFiles();
	if(count == 0){

		return NULL;
	}

	struct filelist *newFileList = headFileList;
	if(newFileList == NULL){
		return NULL;
	}
	while(newFileList != NULL){
		
		if( i == rowNumber){
			memset(tmpBuffer,'\0',sizeof(tmpBuffer));
			strncpy(tmpBuffer,newFileList->fileName,sizeof(tmpBuffer));
			return tmpBuffer;
		}
		newFileList = newFileList->next;
		i += 1;
	}
	return NULL;
}

int checkValidFileType(char *fileName){

	char *returnptr = NULL;
	
	if((returnptr = strstr(fileName,".h264")) != NULL){
		return 1;
	}
	else if((returnptr = strstr(fileName,".264")) != NULL){
		return 1;
	}
	else if((returnptr = strstr(fileName,".avi")) != NULL){
		return 2;
	}
	else if((returnptr = strstr(fileName,".wav")) != NULL){
		return 3;
	}
	else{
		return 4;
	}
}



char* getTargetTable(int rowNumber, int columnID){

	int i = 0;

	for(i = 0; i < targetcount; i++){

		if( i == rowNumber ){
			
			switch(columnID){

				case 1:

					memset(tmpBuffer,'\0',sizeof(tmpBuffer));
					strncpy(tmpBuffer,targettab[i].extension,sizeof(tmpBuffer));
					return tmpBuffer;
					break;

				case 2:
					
					memset(tmpBuffer,'\0',sizeof(tmpBuffer));
					strncpy(tmpBuffer,targettab[i].dirname,sizeof(tmpBuffer));
					return tmpBuffer;
					break;

				case 3:

					memset(tmpBuffer,'\0',sizeof(tmpBuffer));
					strncpy(tmpBuffer,targettab[i].ip,sizeof(tmpBuffer));
					return tmpBuffer;
					break;

				case 4:

					memset(tmpBuffer,'\0',sizeof(tmpBuffer));
					strncpy(tmpBuffer,targettab[i].protocol,sizeof(tmpBuffer));
					return tmpBuffer;
					break;
			}
		}
	}
	return NULL;
}

void copyTargets(int rowNumber){

	int index = 0;

	for(index = 0; index < targetcount; index++){

		if(rowNumber == index){

			memset(target1.ip,'\0',sizeof(target1.ip));
			memset(target1.extension,'\0',sizeof(target1.extension));
			memset(target1.dirname,'\0',sizeof(target1.dirname));
			memset(target1.protocol,'\0',sizeof(target1.protocol));

			strcpy(target1.ip,targettab[index].ip);
			strcpy(target1.extension,targettab[index].extension);
			strcpy(target1.dirname,targettab[index].dirname);
			strcpy(target1.protocol,targettab[index].protocol);

			break;
		}
	}
}


int getNumberOfHosts(){

	return numHosts;
}

char* getHostsList(int rowNumber, int columnID){

	struct hosts_list *hl;
	char ip[MAX_ASCII_ADDR_LEN];
	char mac[MAX_ASCII_ADDR_LEN];
	int i = 0;

	LIST_FOREACH(hl,&GBL_HOSTLIST, next){

		if( i == rowNumber){

			if(columnID == 1){

				memset(gblip,'\0',MAX_ASCII_ADDR_LEN);
				ip_addr_ntoa(&hl->ip, ip);
				memcpy(gblip,ip,MAX_ASCII_ADDR_LEN);
				return gblip;
			}

			else if(columnID == 2){

				memset(gblmac,'\0',MAX_ASCII_ADDR_LEN);
				mac_addr_ntoa(hl->mac, mac);
				memcpy(gblmac,mac,MAX_ASCII_ADDR_LEN);
				return gblmac;
			}
		}
		i += 1;
	}
	return NULL;
}

int getNumberOfActiveCalls(){

	int sip_callcount = 0;
	int sccp_callcount = 0;

	SCCPACCESS_LOCK;

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = first_skinnycall;

	int retval = isEmpty(first_skinnycall);
	if (retval == 1) {
		sccp_callcount = 0;

	}
	else {

		while(current_skinnycall != NULL){

			if (current_skinnycall->call_active == 1) {

				sccp_callcount += 1;
			}
			current_skinnycall = current_skinnycall->next_skinnycall;
		}
	}

	SCCPACCESS_UNLOCK;

	SIPACCESS_LOCK;

	sipDB* currentSipCall = firstSipCall;

	while(currentSipCall != NULL){

		if(currentSipCall->session_active == 1){

			sip_callcount += 1;
		}
		currentSipCall = currentSipCall->next;
	}
	
	SIPACCESS_UNLOCK;

	return (sip_callcount + sccp_callcount);
}

void* getActiveCallPtr(int rowNumber, int columnID, int *callType){

	SCCPACCESS_LOCK;
		
	int i = 0;
	SkinnyCallPtr current_skinnycall;
    current_skinnycall = first_skinnycall;
	
	int retval = isEmpty(first_skinnycall);
    if (retval == 1) {

    }
	else {

        while(current_skinnycall != NULL){

            if(current_skinnycall->call_active == 1){

                if(i == rowNumber){

					*callType = SKINNYCALL;
					SCCPACCESS_UNLOCK;
					return current_skinnycall;
				}
				i += 1;
			}
			
			current_skinnycall = current_skinnycall->next_skinnycall;
		}
	}

	SCCPACCESS_UNLOCK;

	SIPACCESS_LOCK;

	sipDB* currentSipCall = firstSipCall;
	while(currentSipCall != NULL){

        if(currentSipCall->session_active == 1){

            if(i == rowNumber){
				
				*callType = SIPCALL;
				SIPACCESS_UNLOCK;
				return currentSipCall;
			}
			
			i += 1;
		}

		currentSipCall = currentSipCall->next;
	}

	SIPACCESS_UNLOCK;
	
	return NULL;

}	

char* getActiveCalls(int rowNumber, int columnID){

	int i = 0;
	int dlen1 = 0;
	int extenlen1 = 0;
	struct timeval curtime;
	char *null_string = "NULL";
	char *temp1 = NULL;
	int compare = 0;

	SCCPACCESS_LOCK;

	SkinnyCallPtr current_skinnycall;
	current_skinnycall = first_skinnycall;


	int retval = isEmpty(first_skinnycall);
	if (retval == 1) {

	}
	else {

		while(current_skinnycall != NULL){

			if(current_skinnycall->call_active == 1){

				if(i == rowNumber){

					switch(columnID){

						case 1:

							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							temp1 = lookupdialingname(current_skinnycall->passthrupartyid1, &first_skinnycall);
							compare = strcmp(temp1, null_string);
							if (compare == 0) {
								strcpy(current_skinnycall->dirname1,"Unknown");
								strcpy(tmpBuffer,"Unknown");
							}
							else{
								strncpy(tmpBuffer,current_skinnycall->dirname1,sizeof(tmpBuffer));
							}
							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;
						
						case 2:
						
							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							strncpy(tmpBuffer,current_skinnycall->dialingnumber,sizeof(tmpBuffer));
							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;

						case 3:

							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							temp1 = lookupdialingname(current_skinnycall->passthrupartyid1, &first_skinnycall);
							compare = strcmp(temp1, null_string);
							if (compare == 0) {
								strcpy(current_skinnycall->dirname2,"Unknown");
								strcpy(tmpBuffer,"Unknown");
							}
							else{
								strncpy(tmpBuffer,current_skinnycall->dirname2,sizeof(tmpBuffer));
							}
							
							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;

						case 4:

							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							strncpy(tmpBuffer,current_skinnycall->dialednumber,sizeof(tmpBuffer));

							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;

						case 5:

							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							strncpy(tmpBuffer,current_skinnycall->start_time_string,sizeof(tmpBuffer));

							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;
						
						case 6:


							gettimeofday(&curtime,0);
							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							int call_duration = curtime.tv_sec - current_skinnycall->start.tv_sec;
							snprintf(tmpBuffer, sizeof(tmpBuffer), "%d", call_duration);

							SCCPACCESS_UNLOCK;
							return tmpBuffer;
							break;
					}
				}
				i += 1;
			}

			current_skinnycall = current_skinnycall->next_skinnycall;
		}
		//return NULL;
	}

	SCCPACCESS_UNLOCK;

	SIPACCESS_LOCK;

	sipDB* currentSipCall = firstSipCall;

	while(currentSipCall != NULL){

		if(currentSipCall->session_active == 1){

			if(i == rowNumber){

				switch(columnID){

					case 1:

						dlen1 = strlen(currentSipCall->src_username);
						if(dlen1 == 0){
							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							strncpy(tmpBuffer,"unknown",sizeof(tmpBuffer));
						}
						else{
							memset(tmpBuffer,'\0',sizeof(tmpBuffer));
							strncpy(tmpBuffer,currentSipCall->src_username,sizeof(tmpBuffer));		
						}
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					case 2:

						memset(tmpBuffer,'\0',sizeof(tmpBuffer));
						extenlen1 = strlen(currentSipCall->src_extension);
						if(extenlen1 == 0){
							strncpy(tmpBuffer,"<invalid>",sizeof(tmpBuffer));
						}
						else{
							strncpy(tmpBuffer,currentSipCall->src_extension,sizeof(tmpBuffer));
						}
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					case 3:
					
						memset(tmpBuffer,'\0',sizeof(tmpBuffer));
						int dlen2 = strlen(currentSipCall->dst_username);
                        if(dlen2 == 0){
                                strncpy(tmpBuffer,"unknown",sizeof(tmpBuffer));
						}
						else{
                                strncpy(tmpBuffer,currentSipCall->dst_username,sizeof(tmpBuffer));
						}
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					case 4:
						
						memset(tmpBuffer,'\0',sizeof(tmpBuffer));

						if(currentSipCall->remoteParty){

							extenlen1 = strlen(currentSipCall->remote_extension);
							if(extenlen1 == 0){
								strncpy(tmpBuffer,"<invalid>",sizeof(tmpBuffer));
							}
							else{
								strncpy(tmpBuffer,currentSipCall->remote_extension,sizeof(tmpBuffer));	
							}
						}else {

							extenlen1 = strlen(currentSipCall->dst_extension);
							if(extenlen1 == 0){
								strncpy(tmpBuffer,"<invalid>",sizeof(tmpBuffer));
							}
							else{
								strncpy(tmpBuffer,currentSipCall->dst_extension,sizeof(tmpBuffer));
							}
						}
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					case 5:
						
						memset(tmpBuffer,'\0',sizeof(tmpBuffer));
						strncpy(tmpBuffer,currentSipCall->start_time_string,sizeof(tmpBuffer));
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					case 6:

                        gettimeofday(&curtime,0);
						memset(tmpBuffer,'\0',sizeof(tmpBuffer));
                        int call_duration = curtime.tv_sec - currentSipCall->start.tv_sec;
						snprintf(tmpBuffer, sizeof(tmpBuffer), "%d", call_duration);
						SIPACCESS_UNLOCK;
						return tmpBuffer;
						break;

					default:
						break;
				}
			}
			i += 1;
		}

		currentSipCall = currentSipCall->next;
	}
	SIPACCESS_UNLOCK;
	return NULL;
}
                        




                       

						














