/* Uses libvlc as a media player plugin for live video monitoring the calls. 
   Supports video monitoring, of one phone call at a time. See usage for more details on how to monitor live */

#include <ec.h>
#include <ec_ui.h>
#include <ec_liveMonitor.h>
#include <targets.h>
#include <ec_threads.h>
#ifdef HAVE_LIB_VLC
#include <vlc/vlc.h>
#endif
#include <ec_audioDSPPlayer.h>
#include <ec_siplog.h>

/* Mutual exclusion between RTP threads and user-interface/sip threads*/
pthread_mutex_t player_mutex = PTHREAD_MUTEX_INITIALIZER;
#define PLAYER_LOCK do{ pthread_mutex_lock(&player_mutex); } while(0)
#define PLAYER_UNLOCK do{ pthread_mutex_unlock(&player_mutex); } while(0)

/* Mutual exclusion between user-interface and vlc threads */
static pthread_mutex_t playerstop_mutex = PTHREAD_MUTEX_INITIALIZER;
#define PLAYERSTOP_LOCK do{ pthread_mutex_lock(&playerstop_mutex); } while(0)
#define PLAYERSTOP_UNLOCK do{ pthread_mutex_unlock(&playerstop_mutex); } while(0)

/* Mutual exclusion between multiple usually 2 vlc threads */
static pthread_mutex_t vlccounter_mutex = PTHREAD_MUTEX_INITIALIZER;
#define VLCCOUNTER_LOCK do{ pthread_mutex_lock(&vlccounter_mutex); } while(0)
#define VLCCOUNTER_UNLOCK do{ pthread_mutex_unlock(&vlccounter_mutex); } while(0)

/* Mutual exclusion between vlc and RTP threads */
pthread_mutex_t vlcstarted_mutex = PTHREAD_MUTEX_INITIALIZER;
#define VLCSTARTED_LOCK	do{ pthread_mutex_lock(&vlcstarted_mutex); } while(0)
#define VLCSTARTED_UNLOCK do{ pthread_mutex_unlock(&vlcstarted_mutex); } while(0)



/* Mutual exclusion between RTP threads and user-interface/sip threads*/
pthread_mutex_t player_mutex_skinny = PTHREAD_MUTEX_INITIALIZER;
#define PLAYER_SKINNY_LOCK do{ pthread_mutex_lock(&player_mutex_skinny); } while(0)
#define PLAYER_SKINNY_UNLOCK do{ pthread_mutex_unlock(&player_mutex_skinny); } while(0)

/* Mutual exclusion between user-interface and vlc threads */
static pthread_mutex_t playerstop_mutex_skinny = PTHREAD_MUTEX_INITIALIZER;
#define PLAYERSTOP_SKINNY_LOCK do{ pthread_mutex_lock(&playerstop_mutex_skinny); } while(0)
#define PLAYERSTOP_SKINNY_UNLOCK do{ pthread_mutex_unlock(&playerstop_mutex_skinny); } while(0)

/* Mutual exclusion between multiple usually 2 vlc threads */
static pthread_mutex_t vlccounter_mutex_skinny = PTHREAD_MUTEX_INITIALIZER;
#define VLCCOUNTER_SKINNY_LOCK do{ pthread_mutex_lock(&vlccounter_mutex_skinny); } while(0)
#define VLCCOUNTER_SKINNY_UNLOCK do{ pthread_mutex_unlock(&vlccounter_mutex_skinny); } while(0)

/* Mutual exclusion between vlc and RTP threads */
pthread_mutex_t vlcstarted_mutex_skinny = PTHREAD_MUTEX_INITIALIZER;
#define VLCSTARTED_SKINNY_LOCK	do{ pthread_mutex_lock(&vlcstarted_mutex_skinny); } while(0)
#define VLCSTARTED_SKINNY_UNLOCK do{ pthread_mutex_unlock(&vlcstarted_mutex_skinny); } while(0)


sipDB *currentlyMonitoringSipCall = NULL;
typedef struct skinnycall SkinnyCallPtr;
SkinnyCallPtr *currentlyMonitoringSkinnyCall = NULL;
int currentCallType = 0;
int stopPlayers = 0;
int stopPlayers_Skinny = 0;
void reInitLiveMonitor();
void stopCurrentlyMonitoringStream();
void stopMonitoringCall(void *, int);

struct vlcStartStruct{

	int fileType;
	int *vlcStartFlag;
};

#ifdef HAVE_LIB_VLC
struct vlcStartStruct vlcStartObj[2];
FILE *monitorFP[NUM_MONITOR_CALLS];
FILE *monitorFPSkinny[NUM_MONITOR_CALLS];
pthread_t threads[NUM_MONITOR_CALLS]; /* Two threads,for fwd video, rev video */
char *monitorFileName[] = {"FwdVideo.sdp", "RevVideo.sdp" };
char *monitorFileNameSkinny[] = {"FwdSkinny.sdp", "RevSkinny.sdp" };
int openMonitorStreamFiles();
int openMonitorStreamFilesSkinny();
int createSDPFile(int fileType, char *rtpmap, char *fmtp, char *payloadType);
int createSDPFileSkinny(int fileType, char *payloadType);
void *startVlcPlayer(void *args);
void *startVlcPlayerSkinny(void *args);
void closeLibVlc();
int threadCounter = 0;
int threadCounter_Skinny = 0;
int startVlcCounter = 0;
int startVlcCounter_Skinny = 0;
//libvlc_exception_t ex;
libvlc_instance_t * inst_Skinny;
libvlc_instance_t * inst;
#endif 

void stopMonitoringCall(void *callPtr, int callType){

	if(callType == SIPCALL){

		if(currentlyMonitoringSipCall != NULL){

			sipDB *tempPtr = (sipDB*) callPtr;

			if(tempPtr == currentlyMonitoringSipCall){

				stopCurrentlyMonitoringStream();
			}
		}
	}

	if(callType == SKINNYCALL){

		if(currentlyMonitoringSkinnyCall != NULL){

			SkinnyCallPtr *tempPtr = (SkinnyCallPtr*) callPtr;

			if(tempPtr == currentlyMonitoringSkinnyCall){

				stopCurrentlyMonitoringStream();
			}
		}
	}
}

/* This function sets the monitoring variables for call, opens the monitoring stream file, closes the previously monitoring call players and files */

void newLiveMonitorCall(void *callPtr, int callType){

	if(callType == SIPCALL){
		//if(currentlyMonitoringSipCall != NULL || currentlyMonitoringSkinnyCall != NULL){
		if(currentlyMonitoringSipCall != NULL){
			if(currentCallType == SIPCALL){
				sipDB *tempPtr = (sipDB*) callPtr;
				if(tempPtr == currentlyMonitoringSipCall){
					/* The new monitor request and the currently monitoring call are the same */
					return;	
				}
			}
			
        	stopCurrentlyMonitoringStream();

    	}

		currentlyMonitoringSipCall = (sipDB *) callPtr;
		currentCallType = SIPCALL;

#ifdef HAVE_LIB_VLC
		int returnValue = openMonitorStreamFiles();
		if(returnValue < 0){
			/* Some monitor files cannot be opened */
			currentlyMonitoringSipCall = NULL;
			currentCallType = 0;
			return;
		}
#endif	
		PLAYER_LOCK;
		PLAYERSTOP_LOCK;
		stopPlayers = 0;
		PLAYERSTOP_UNLOCK;

		if(currentlyMonitoringSipCall->remoteParty)
			SIPLOG("[UCSniff-monitor-debugs]:Monitoring call between %s and %s \n", currentlyMonitoringSipCall->src_extension,currentlyMonitoringSipCall->remote_extension);
		else
			SIPLOG("[UCSniff-monitor-debugs]:Monitoring call between %s and %s \n", currentlyMonitoringSipCall->src_extension,currentlyMonitoringSipCall->dst_extension);

		currentlyMonitoringSipCall->myRTPCall->liveMonitor = MONITOR_START;
		PLAYER_UNLOCK;
	}
	else if(callType == SKINNYCALL){

		/* Have to change the skinny and rtp data struct, have to link the two structures. For now works only for SIP */
//if(currentlyMonitoringSipCall != NULL || currentlyMonitoringSkinnyCall != NULL){
		if(currentlyMonitoringSkinnyCall != NULL){

			if(currentCallType == SKINNYCALL){

				SkinnyCallPtr *tempPtr = (SkinnyCallPtr*) callPtr;
				if(tempPtr == currentlyMonitoringSkinnyCall){

					/* The new monitor request and the currently monitoring call are the same */
					return;	
				}
			}
			
        		stopCurrentlyMonitoringStream();

    		}	
            //printf("The currentcalltype is skinny call\n");
		currentlyMonitoringSkinnyCall = (SkinnyCallPtr *) callPtr;
		currentCallType = SKINNYCALL;
#ifdef HAVE_LIB_VLC
		int returnValue = openMonitorStreamFilesSkinny();
		if(returnValue < 0){
			/* Some monitor files cannot be opened */
			currentlyMonitoringSkinnyCall = NULL;
			currentCallType = 0;
			return;
		}
#endif	
		PLAYER_SKINNY_LOCK;
		PLAYERSTOP_SKINNY_LOCK;
		stopPlayers_Skinny = 0;
		PLAYERSTOP_SKINNY_UNLOCK;				
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitor = MONITOR_START;
		currentlyMonitoringSkinnyCall->flag = 1;
		//printf("New Live Monitor function is working");
		PLAYER_SKINNY_UNLOCK;
	}

}

void stopCurrentlyMonitoringStream(){

	if(currentCallType == SIPCALL){

		PLAYER_LOCK;
		PLAYERSTOP_LOCK;
		stopPlayers = 1;	
		PLAYERSTOP_UNLOCK;
		currentlyMonitoringSipCall->myRTPCall->liveMonitor = MONITOR_STOP;
		currentlyMonitoringSipCall->myRTPCall->audioDSPSet = 0;
		killDSP();
		/* Make sure libvlc has stopped */
#ifdef HAVE_LIB_VLC		
		while(1){
			
			VLCCOUNTER_LOCK;
			if(threadCounter == 0){
				VLCCOUNTER_UNLOCK;
				break;
			}
			VLCCOUNTER_UNLOCK;
		}

		/* Close the live monitoring socket */
		if(currentlyMonitoringSipCall->myRTPCall->liveMonitorSocket)
			close(currentlyMonitoringSipCall->myRTPCall->liveMonitorSocket);

		currentlyMonitoringSipCall->myRTPCall->liveMonitorSocket = 0;
#endif
		reInitLiveMonitor();	
		currentlyMonitoringSipCall = NULL;
		currentCallType = 0;
		SIPLOG("[UCSniff-monitor-debugs]:Stopping currently monitoring sip stream \n");
		PLAYER_UNLOCK;
	}
	else if(currentCallType == SKINNYCALL){

		/* Not supported */
		PLAYER_SKINNY_LOCK;
		PLAYERSTOP_SKINNY_LOCK;
		stopPlayers_Skinny = 1;	
		PLAYERSTOP_SKINNY_UNLOCK;
		if(currentlyMonitoringSkinnyCall->myRTPCall->audioDSPSet != 0)
		killDSP();
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitor = MONITOR_STOP;
		currentlyMonitoringSkinnyCall->myRTPCall->audioDSPSet = 0;
		
		/* Make sure libvlc has stopped */
#ifdef HAVE_LIB_VLC		
		while(1){
			
			VLCCOUNTER_SKINNY_LOCK;
			if(threadCounter_Skinny == 0){
				VLCCOUNTER_SKINNY_UNLOCK;
				break;
			}
			VLCCOUNTER_SKINNY_UNLOCK;
		}

		/* Close the live monitoring socket */
		if(currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorSocket)
			close(currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorSocket);

		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorSocket = 0;
#endif
		reInitLiveMonitor();	
		currentlyMonitoringSkinnyCall = NULL;
		currentCallType = 0;
		printf("[UCSniff-monitor-debugs]:Stopping currently monitoring sip stream \n");
		PLAYER_SKINNY_UNLOCK;
	}
}

void reInitLiveMonitor(){

	if(currentCallType == SIPCALL){

		currentlyMonitoringSipCall->myRTPCall->liveMonitor = 0;
		currentlyMonitoringSipCall->myRTPCall->liveMonitorFwdAudioCtr = 0;
		currentlyMonitoringSipCall->myRTPCall->liveMonitorRevAudioCtr = 0;
		currentlyMonitoringSipCall->myRTPCall->liveMonitorFwdVideoCtr = 0;
		currentlyMonitoringSipCall->myRTPCall->liveMonitorRevVideoCtr = 0;
#ifdef HAVE_LIB_VLC
		currentlyMonitoringSipCall->myRTPCall->liveMonitorFwdVlcStarted = 0;
		currentlyMonitoringSipCall->myRTPCall->liveMonitorRevVlcStarted = 0;
#endif
	}
	else if(currentCallType == SKINNYCALL){
		currentlyMonitoringSkinnyCall->flag = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitor = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorFwdAudioCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorRevAudioCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorFwdVideoCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorRevVideoCtr = 0;
#ifdef HAVE_LIB_VLC
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorFwdVlcStarted = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->liveMonitorRevVlcStarted = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->savedFwdSPS = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->savedRevSPS = 0;	
		currentlyMonitoringSkinnyCall->myRTPCall->fwdIDRPtr = NULL;
		currentlyMonitoringSkinnyCall->myRTPCall->revIDRPtr = NULL;
		currentlyMonitoringSkinnyCall->myRTPCall->fwdCurrentIDRPtr = NULL;
		currentlyMonitoringSkinnyCall->myRTPCall->revCurrentIDRPtr = NULL;

		currentlyMonitoringSkinnyCall->myRTPCall->fwdIDRCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->revIDRCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->recvFwdAllIDRPic = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->recvRevAllIDRPic = 0;
		
		currentlyMonitoringSkinnyCall->myRTPCall->currentFwdPPSCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->previousFwdPPSCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->currentRevPPSCtr = 0;
		currentlyMonitoringSkinnyCall->myRTPCall->previousRevPPSCtr = 0;

#endif
	}
}

#ifdef HAVE_LIB_VLC

int openMonitorStreamFiles(){

	int i = 0;
	for(i = 0; i < NUM_MONITOR_CALLS; i++){

		monitorFP[i] = fopen(monitorFileName[i], "w+");
		if(monitorFP[i] == NULL){

			INSTANT_USER_MSG("Error opening file %s: %s \n", monitorFileName[i], strerror(errno));

			i -= 1;
			while(i > 0){
		        if(fclose(monitorFP[i]) < 0) {
					INSTANT_USER_MSG("Error closing file %s: %s", monitorFileName[i], strerror(errno) );
					SIPLOG("%s:%d - Error closing file %s: %s", __FILE__, __LINE__, monitorFileName[i], strerror(errno) );
				}
				i -= 1;
			}
			return -1;	
		}
	}
	return 0;
}


int openMonitorStreamFilesSkinny(){

	int i = 0;
	for(i = 0; i < NUM_MONITOR_CALLS; i++){

		monitorFPSkinny[i] = fopen(monitorFileNameSkinny[i], "w+");
		if(monitorFPSkinny[i] == NULL){

			INSTANT_USER_MSG("Error opening file %s \n", monitorFileNameSkinny[i]);

			i -= 1;
			while(i > 0){
				(monitorFPSkinny[i]);
				i -= 1;
			}
			return -1;	
		}
	}
	return 0;
}

/*static void raise(libvlc_exception_t * ex)
{
    if (libvlc_exception_raised (ex))
    {
         fprintf (stderr, "error: %s\n", libvlc_exception_get_message(ex));
         return;
    }
}*/
		
void startLiveMonitorPlayer(int fileType,int *vlcStartFlag){

	/* FileType , 0 - fwd video, 1 - rev video */
	
	vlcStartObj[fileType].fileType = fileType;
	vlcStartObj[fileType].vlcStartFlag = vlcStartFlag;
	ec_thread_new("vlcPlayer","vlc player for live monitoring", &startVlcPlayer, &vlcStartObj[fileType]);
}

void *startVlcPlayer(void *args){

	ec_thread_init();
	
	struct vlcStartStruct *vlcStartObjTemp;
	vlcStartObjTemp = (struct vlcStartStruct *) args;
	VLCCOUNTER_LOCK;
	int fileType = vlcStartObjTemp->fileType;
	int *vlcStartFlag = vlcStartObjTemp->vlcStartFlag;
	
	char fileName[32] = {'\0'};
	
	strncpy(fileName,monitorFileName[fileType],sizeof(fileName));

	SIPLOG("[UCSniff-vlc-debugs] : New thread created for monitoring file %s \n", fileName);

	const char * const vlc_args[] = {
    	//"-vvv",
		"--h264-fps=25",	
	    "-I", "ncurses", /* Don't use any interface */
	    	//"--sout '#standard{mux=\"avi\",access=\"file\",dst=\"./test.avi\"}'",
	    "--ignore-config", /* Don't use VLC's config */
    };

   	libvlc_media_player_t *mp;
   	libvlc_media_t *m;		

	if(startVlcCounter == 0){
		//libvlc_exception_init (&ex);
		SIPLOG("[UCSniff-vlc-debugs]: libvlc is initialized \n");	
    	/* init vlc modules, should be done only once */
    	inst = libvlc_new (sizeof(vlc_args) / sizeof(vlc_args[0]), vlc_args);
    	//raise (&ex);
	if(inst == NULL)
		SIPLOG("[UCSniff-vlc-debugs]: New instance could not be created \n");
		startVlcCounter += 1;
	}
	SIPLOG("[UCSniff-vlc-debugs]: New instance created \n");
	threadCounter += 1;

    /* Create a new item */
	m = libvlc_media_new_path(inst,fileName);
//	libvlc_media_add_option(m,":sout=#std{access=file,mux=mp4,dst=\"./test.avi\"}");

	//libvlc_media_add_option(m,":sout=#transcode{vcodec=h264,width=352,height=288,scale=1,venc=x264,acodec=none}:std{access=file,mux=mp4,dst=\"./test.avi\"}");
    //raise (&ex);

	SIPLOG("[UCSniff-vlc-debugs]: New vlc media item created \n");

	/* XXX: demo art and meta information fetching */

    /* Create a media player playing environement */
    mp = libvlc_media_player_new_from_media (m);
    //raise (&ex);
	
	SIPLOG("[UCSniff-vlc-debugs]: Media player environment created \n");

    /* No need to keep the media now */
    libvlc_media_release (m);

	SIPLOG("[UCSniff-vlc-debugs]: Releasing temporary media \n");

    /* play the media_player */

	SIPLOG("[UCSniff-vlc-debugs]: Before playing the media player \n");
	libvlc_media_player_play (mp);

while(1){
	if(libvlc_media_player_is_playing(mp))
	{
		break;
	}
}
    //raise (&ex);

    /* Play the video clip, until it ends */
		
	VLCSTARTED_LOCK;

	*vlcStartFlag = 1;

	VLCSTARTED_UNLOCK;
	
	VLCCOUNTER_UNLOCK;

   while(1){

		PLAYERSTOP_LOCK;

		if(stopPlayers == 1){
			PLAYERSTOP_UNLOCK;
		   	break;
		}

		PLAYERSTOP_UNLOCK;
   }

   	VLCCOUNTER_LOCK;

    /* Stop playing */
	SIPLOG("[UCSniff-vlc-debugs]: Stopping the media player \n");
    libvlc_media_player_stop (mp);
	//raise(&ex);

    /* Free the media_player */
    libvlc_media_player_release (mp);
	//raise(&ex);
	SIPLOG("[UCSniff-vlc-debugs]: Media player released \n");
	

	threadCounter -= 1;
	VLCCOUNTER_UNLOCK;
	pthread_exit(NULL);
}

void startLiveMonitorPlayerSkinny(int fileType,int *vlcStartFlag){

	/* FileType , 0 - fwd video, 1 - rev video */
	
	vlcStartObj[fileType].fileType = fileType;
	vlcStartObj[fileType].vlcStartFlag = vlcStartFlag;
	//if(fileType == 0)
	ec_thread_new("vlcPlayer","vlc player for live monitoring", &startVlcPlayerSkinny, &vlcStartObj[fileType]);
}

void *startVlcPlayerSkinny(void *args){

	ec_thread_init();
	//printf("Vlc player is gonna start for skinny protocol \n");
	
	struct vlcStartStruct *vlcStartObjTemp;
	vlcStartObjTemp = (struct vlcStartStruct *) args;
	VLCCOUNTER_SKINNY_LOCK;
	int fileType = vlcStartObjTemp->fileType;
	int *vlcStartFlag = vlcStartObjTemp->vlcStartFlag;
	
	char fileName[32] = {'\0'};
	
	strncpy(fileName,monitorFileNameSkinny[fileType],sizeof(fileName));

	SIPLOG("[UCSniff-vlc-debugs] : New thread created for monitoring file %s \n", fileName);

	const char * const vlc_args[] = {
    		//"-vvv",
		"--h264-fps=30",	
	    	"-I", "ncurses", /* Don't use any interface */
	    	"--ignore-config", /* Don't use VLC's config */
		//"--sout",
    };

   	libvlc_media_player_t *mp;
   	libvlc_media_t *m;		

	if(startVlcCounter_Skinny == 0){
		//libvlc_exception_init (&ex);
		SIPLOG("[UCSniff-vlc-debugs]: libvlc is initialized \n");	
    	/* init vlc modules, should be done only once */
    	inst_Skinny = libvlc_new (sizeof(vlc_args) / sizeof(vlc_args[0]), vlc_args);
    	//raise (&ex);
	if(inst_Skinny == NULL)
		//printf(" new instance could not be created \n");
		startVlcCounter_Skinny += 1;
	}
	threadCounter_Skinny += 1;

    /* Create a new item */
	m = libvlc_media_new_path(inst_Skinny,fileName);
    //raise (&ex);

	SIPLOG("[UCSniff-vlc-debugs]: New vlc media item created \n");

	/* XXX: demo art and meta information fetching */

    /* Create a media player playing environement */
    mp = libvlc_media_player_new_from_media (m);
    //raise (&ex);
	
	SIPLOG("[UCSniff-vlc-debugs]: Media player environment created \n");

    /* No need to keep the media now */
    libvlc_media_release (m);

	SIPLOG("[UCSniff-vlc-debugs]: Releasing temporary media \n");

    /* play the media_player */

	SIPLOG("[UCSniff-vlc-debugs]: Before playing the media player \n");
	libvlc_media_player_play (mp);

while(1){
	if(libvlc_media_player_is_playing(mp))
	{
		//printf(" Windows are gonna pop up \n");
		break;
	}
}
    //raise (&ex);

    /* Play the video clip, until it ends */
		
	VLCSTARTED_SKINNY_LOCK;

	*vlcStartFlag = 1;

	VLCSTARTED_SKINNY_UNLOCK;
	
	VLCCOUNTER_SKINNY_UNLOCK;

   while(1){

		PLAYERSTOP_SKINNY_LOCK;

		if(stopPlayers_Skinny == 1){
			PLAYERSTOP_SKINNY_UNLOCK;
		   	break;
		}

		PLAYERSTOP_SKINNY_UNLOCK;
   }

   	VLCCOUNTER_SKINNY_LOCK;

    /* Stop playing */
	SIPLOG("[UCSniff-vlc-debugs]: Stopping the media player \n");
    libvlc_media_player_stop (mp);
	//raise(&ex);

    /* Free the media_player */
    libvlc_media_player_release (mp);
	//raise(&ex);
	SIPLOG("[UCSniff-vlc-debugs]: Media player released \n");
	

	threadCounter_Skinny -= 1;
	VLCCOUNTER_SKINNY_UNLOCK;
	pthread_exit(NULL);
}

void closeLibVlc(){

	if(startVlcCounter == 1){
		SIPLOG("[UCSniff-vlc-debugs]: Before releasing libvlc \n");
		libvlc_release(inst);
		//raise(&ex);
	}
	startVlcCounter = 0;
	if(startVlcCounter_Skinny == 1){
		SIPLOG("[UCSniff-vlc-debugs]: Before releasing libvlc \n");
		libvlc_release(inst_Skinny);
		//raise(&ex);
	}
	startVlcCounter_Skinny = 0;
}

int createSDPFile(int fileType, char *rtpmap, char *fmtp, char *payloadType){

	char portAndPayloadStr[32] = {'\0'};

	/* SDP Version */	
	fwrite("v=0\n", sizeof(char), strlen("v=0\n"), monitorFP[fileType]);
	/* Owner info */
	fwrite("o=2007 8000 8000 IN IP4 0.0.0.0\n", sizeof(char), strlen("o=2007 8000 8000 IN IP4 0.0.0.0\n"), monitorFP[fileType]);
	/*Session info */
	fwrite("s=SIP Call\n", sizeof(char), strlen("s=SIP Call\n"), monitorFP[fileType]);
	/* */
	fwrite("t=0 0\n", sizeof(char), strlen("t=0 0\n"), monitorFP[fileType]);
	/*Port and Payload Type */
	strncpy(portAndPayloadStr, "m=video ", strlen("m=video "));

	switch(fileType){

		case FWDVIDEOFT:
			strcat(portAndPayloadStr,FWDVIDEOPORT);
			break;
		case REVVIDEOFT:
			strcat(portAndPayloadStr,REVVIDEOPORT);
			break;
	}

	char pType[4] = {'\0'};
	snprintf(pType,sizeof(pType),"%d",*payloadType);

	strcat(portAndPayloadStr," RTP/AVP ");
	strcat(portAndPayloadStr,pType);
	strcat(portAndPayloadStr," \n");

	fwrite(portAndPayloadStr,sizeof(char),strlen(portAndPayloadStr),monitorFP[fileType]);

	/* Connection IP */
	fwrite("c=IN IP4 ", sizeof(char), strlen("c=IN IP4 "), monitorFP[fileType]);
	fwrite(myIP, sizeof(char), strlen(myIP), monitorFP[fileType]);
	fwrite("\n", sizeof(char), strlen("\n"),monitorFP[fileType]);

	/* RTPMAP */
	if(rtpmap != NULL){
		fwrite(rtpmap, sizeof(char), strlen(rtpmap), monitorFP[fileType]);
		fwrite("\n", sizeof(char), strlen("\n"),monitorFP[fileType]);
	}

	/* FMTP */
/*	if(fmtp != NULL){
		fwrite(fmtp, sizeof(char), strlen(fmtp), monitorFP[fileType]);
		fwrite("\n", sizeof(char), strlen("\n"),monitorFP[fileType]);
	}*/
	
	if(fclose(monitorFP[fileType]) < 0) {
		INSTANT_USER_MSG("Error closing SDP file: %s", strerror(errno) );
		SIPLOG("%s:%d - Error closing SDP file: %s", __FILE__, __LINE__, strerror(errno) );
	}

	return 0;
}


int createSDPFileSkinny(int fileType, char *payloadType){
	char portAndPayloadStr[32] = {'\0'};

	/* SDP Version */	
	fwrite("v=0\n", sizeof(char), strlen("v=0\n"), monitorFPSkinny[fileType]);
	/* Owner info */
	fwrite("o=2007 8000 8000 IN IP4 0.0.0.0\n", sizeof(char), strlen("o=2007 8000 8000 IN IP4 0.0.0.0\n"), monitorFPSkinny[fileType]);
	/*Session info */
	fwrite("s=Skinny Call\n", sizeof(char), strlen("s=Skinny Call\n"), monitorFPSkinny[fileType]);
	/* */
	fwrite("t=0 0\n", sizeof(char), strlen("t=0 0\n"), monitorFPSkinny[fileType]);
	/*Port and Payload Type */
	strncpy(portAndPayloadStr, "m=video ", strlen("m=video "));

	switch(fileType){

		case FWDVIDEOFT:
			strcat(portAndPayloadStr,FWDVIDEOPORTSKINNY);
			break;
		case REVVIDEOFT:
			strcat(portAndPayloadStr,REVVIDEOPORTSKINNY);
			break;
	}

	char pType[4] = {'\0'};
	snprintf(pType,sizeof(pType),"%d",*payloadType);

	strcat(portAndPayloadStr," RTP/AVP ");
	strcat(portAndPayloadStr,pType);
	strcat(portAndPayloadStr," \n");

	fwrite(portAndPayloadStr,sizeof(char),strlen(portAndPayloadStr),monitorFPSkinny[fileType]);

	/* Connection IP */
	fwrite("c=IN IP4 ", sizeof(char), strlen("c=IN IP4 "), monitorFPSkinny[fileType]);
	fwrite(myIP, sizeof(char), strlen(myIP), monitorFPSkinny[fileType]);
	fwrite("\n", sizeof(char), strlen("\n"),monitorFPSkinny[fileType]);

      fwrite("a=rtpmap:97 H264/90000", sizeof(char), strlen("a=rtpmap:97 H264/90000"), monitorFPSkinny[fileType]);
	fwrite("\n", sizeof(char), strlen("\n"),monitorFPSkinny[fileType]);

	/* FMTP */
/*	if(fmtp != NULL){
		fwrite(fmtp, sizeof(char), strlen(fmtp), monitorFP[fileType]);
		fwrite("\n", sizeof(char), strlen("\n"),monitorFP[fileType]);
	}*/
	
	fflush(monitorFPSkinny[fileType]);
	fclose(monitorFPSkinny[fileType]);
	return 0;
}

#endif
