/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2008 VIPER Lab
    Jason Ostrom <jostrom@viperlab.net>
    Arjun Sambamoorthy <arjun@viperlab.net>

    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <targets.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <tftp_client.h>
#include <client.h>
#include <malloc.h>
#include <sox.h>
#include <ec.h>
#include <ec_decode.h>
#include <ec.h>
#include <ec_dissect.h>
#include <ffmpeg.h>
#include <ec_siplog.h>
#include <ec_voipglobals.h>
#include <ec_voiplogfiles.h>
#ifndef OS_WINDOWS
#include <arpa/inet.h>
#endif

struct targets targettab[1024];
struct singletarget target1;
//int flush;
int ip_yes;
// Temporary

typedef struct skinnycall SkinnyCall;
typedef SkinnyCall *SkinnyCallPtr;

int targetcount = 0;
FILE *singletarget;
char *singletargetfile = "singletarget_log";

// Declare functions
int targetfileadd(char *ip, char *extension, char *deviceprotocol);
int checkdigitstring(char str[24]);
int targetfileadddirectory(char ip[MAX_ASCII_ADDR_LEN], char extension[24], char dirname[64], char deviceprotocol[10]);
void printtargetwait(void);
void updatetargetsfileip(char ip[MAX_ASCII_ADDR_LEN],char extension[24]);
void updatetargetsfiledirname(int index, char ip[MAX_ASCII_ADDR_LEN], char dirname[64]);
int targetlookup(SkinnyCallPtr *sPtr, int passthrupartyid, char ip[MAX_ASCII_ADDR_LEN]);
void initializeTargetTab(int);
void printcorrect();
int checkstr( char *ext);
int checkpro( char *pro);
int isvalidip( char *ip);

// from vc_main.c
extern FILE *ftargets;
extern char *targetsfile;

// from vc_scan.c
extern int host_count;

// from ec_sccp.c
extern int rtp_counter;
extern int callcount;
extern char *lookupdialingname (int conferenceid, SkinnyCallPtr *sPtr);
extern char *lookupdialedname (int conferenceid, SkinnyCallPtr *sPtr);
extern FUNC_DECODER(dissector_sccp);
extern FUNC_DECODER(dissector_rtp_skinny);
extern FUNC_DECODER(dissector_rtp_sip);
extern struct digitbuffer partygetstrings(char remainingStr[24], int splength);

//extern struct rtp_data rtp[1024];
extern SkinnyCallPtr first_skinnycall;

struct sip_uri uri_obj;

//local function definitions
int check_targettab(char *);

int check_local_ip(char *ip){

	/* In monitor mode, the host list will be empty, so local ip always returns 1, FIX ME 
	if(monitor_yes)
		return 1;
	*/

	struct hosts_list *hl;
        char iptemp[MAX_ASCII_ADDR_LEN];

        LIST_FOREACH(hl, &GBL_HOSTLIST, next) {

                ip_addr_ntoa(&hl->ip, iptemp);

                int compare = strcmp(iptemp, ip);
                if (compare == 0) {

                        return 1;

                }
        }

        return 0;
} 

int check_targettab(char *extension){

	int i = 0;
	
	for(i = 0; i < targetcount; i++){

		if(strcmp(targettab[i].extension,extension) == 0)
			return i;
	}
	return -1;
}

void initializeTargetTab(int count){

	memset(targettab[count].ip,'\0',sizeof(targettab[count].ip));
	memset(targettab[count].mac,'\0',sizeof(targettab[count].mac));
	memset(targettab[count].extension,'\0',sizeof(targettab[count].extension));
	memset(targettab[count].dirname,'\0',sizeof(targettab[count].dirname));
	memset(targettab[count].protocol,'\0',sizeof(targettab[count].protocol));
	memset(targettab[count].ua,'\0',sizeof(targettab[count].ua));
	memset(targettab[count].misc,'\0',sizeof(targettab[count].misc));

}
	
void updatetargetsfileip(char ip[MAX_ASCII_ADDR_LEN],char extension[24]){

	SIPLOG("Inside updatetargetsfileip\n");

	struct target_file{

                char putstring[124];

                struct target_file *next;
        };

        struct target_file *first = NULL;
        struct target_file *hl, *var, *tmp;

        char *token;
        char ip_string[MAX_ASCII_ADDR_LEN];
        char ext_string[24];
        char dirname_final[64];
        char protocol_final[10];
        char *tempname;
        char *tempprotocol;

        int extension_found = 0;
        char *search = ",";
        int str_len;
        char s[124];

	
        ftargets = fopen(targetsfile, "r+");
        if(ftargets == NULL) {
		SIPLOG("Error opening file targets.txt\n");
                INSTANT_USER_MSG("Error opening file targets.txt\n");
                return;
        }

	while (fgets(s, 124, ftargets) != NULL) {

                hl = calloc(sizeof(struct target_file),1);
                hl -> next = first;
                first = hl;
                str_len = strlen(s);
                token = strtok(s,search);
                strncpy(ip_string,token,sizeof(ip_string));

                token = strtok(NULL,search);
                strncpy(ext_string,token,sizeof(ext_string));

                tempname = strtok(NULL,search);
                tempprotocol = strtok(NULL,search);

                int compare_str = strcmp(ext_string,extension);
                if (compare_str == 0) {

                        if (tempprotocol == NULL) {

                                strncpy(protocol_final,tempname,sizeof(protocol_final));
                                strncpy(hl->putstring,ip,MAX_ASCII_ADDR_LEN);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,ext_string);
                                strcat(hl->putstring,",,");
                                strcat(hl->putstring,protocol_final);

                        } else {

                                strncpy(dirname_final,tempname,sizeof(dirname_final));
                                strncpy(protocol_final,tempprotocol,sizeof(protocol_final));
                                strncpy(hl->putstring,ip,MAX_ASCII_ADDR_LEN);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,ext_string);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,dirname_final);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,protocol_final);
                        }

                        extension_found = 1;

                } else {

                        if(tempprotocol == NULL){

                                strncpy(protocol_final,tempname,sizeof(protocol_final));
                                strncpy(hl->putstring,ip_string,MAX_ASCII_ADDR_LEN);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,ext_string);
                                strcat(hl->putstring,",,");
                                strcat(hl->putstring,protocol_final);
                        }
                        else{
                                strncpy(dirname_final,tempname,sizeof(dirname_final));
                                strncpy(protocol_final,tempprotocol,sizeof(protocol_final));
                                strncpy(hl->putstring,ip_string,MAX_ASCII_ADDR_LEN);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,ext_string);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,dirname_final);
                                strcat(hl->putstring,",");
                                strcat(hl->putstring,protocol_final);
                        }
                }
        }

	if(extension_found != 1) {
		SIPLOG("Extension %s could not be found in Targets file, 'targets.txt'\n",extension);
                INSTANT_USER_MSG("Extension %s could not be found in Targets file, 'targets.txt'\n",extension);
                fclose(ftargets);
                return;
        }
        else if(extension_found == 1){

                fclose(ftargets);
                FILE *fp = fopen(targetsfile,"w+");
                if(fp == NULL){
			SIPLOG("Unable to open %s\n",targetsfile);
                        INSTANT_USER_MSG("Unable to open %s\n",targetsfile);
			SIPLOG("Targets list not updated\n");
                        INSTANT_USER_MSG("Targets list not updated\n");
                        return;
                }

                var = hl;
                while(var != NULL){
                        fprintf(fp,"%s",var->putstring);
                        tmp = var;
                        var = var->next;
                        free(tmp);
                }
                fclose(fp);
		SIPLOG("Targets file has been successfully updated\n");
        }
}

void updatetargetsfiledirname(int index, char ip[MAX_ASCII_ADDR_LEN], char dirname[64]){

}

void printtargetwait(void) {

	INSTANT_USER_MSG("Listening for new calls to or from target %s (Extension %s, IP %s)\n",
			target1.dirname,
			target1.extension,
			target1.ip);
	if(voipdebug) {
		fprintf(sccpfile,"Listening for new calls to or from target %s (Extension %s, IP %s)\n",
			target1.dirname,
			target1.extension,
			target1.ip);
		fflush(sccpfile);
	}

}

void targetmode_init() {

	if (target_yes != 1) {
		return;
	}

	if (targetcount == 0) {
		INSTANT_USER_MSG("No targets have been previously discovered in Targets file, targets.txt\n");
		INSTANT_USER_MSG("Please run UCSniff in learning mode, or manually edit targets.txt\n");
		exit(1);
	} else {
		INSTANT_USER_MSG("UCSniff running in target mode.  Parsed %d previously discovered targets\n",targetcount);
	}

	singletarget = fopen(singletargetfile, "a+");
	if (singletarget == NULL) {
		INSTANT_USER_MSG("Error: unable to open the file %s\n",singletargetfile);
		exit (8);
	}	

	INSTANT_USER_MSG("In this mode, you select one IP Phone Endpoint (User / Extension), and all calls to or from this endpoint are targeted for eavesdropping\n\n");
	INSTANT_USER_MSG("Displaying the discovered targets list:\n");

	print_targets();

	// Now read the one target selected
	int i = targetcount;
	INSTANT_USER_MSG("\nPlease select one endpoint (1 - %d) from the discovered targets list:\n",i);

	//char *t1;		
	//t1 = getchar();
	char *tmpinput;
	int y = 0;
	tmpinput = calloc(20, sizeof(char));
	for (tmpinput[y] = getchar(); tmpinput[y]!='\n'; tmpinput[y]=getchar()) {
		y+=1;
	}

	long selection = atoi(tmpinput);

	while ((selection < 1)||(selection > i)) {

		INSTANT_USER_MSG("Please select endpoint between (1 - %d)\n",i);
		int w = 0;
		for (tmpinput[w] = getchar(); tmpinput[w]!='\n'; tmpinput[w]=getchar()) {

			w+=1;
		}

		selection = atoi(tmpinput);
	}

	free(tmpinput);

	// copy in target selection
	int index = selection - 1;
	strcpy(target1.ip,targettab[index].ip);
	strcpy(target1.extension,targettab[index].extension);
	strcpy(target1.dirname,targettab[index].dirname);
	strcpy(target1.protocol,targettab[index].protocol);

	INSTANT_USER_MSG("\nTarget selected for single user eavesdropping:\n");
        INSTANT_USER_MSG("%s    %s      %s      %s\n",
                        target1.extension,
                        target1.dirname,
                        target1.ip,
                        target1.protocol);


}

void print_targets() {

	int i;
	int count = 1;
	int extension_len = 12;
	int dirname_len = 22;
	int ip_len = 16;

	if (targetcount == 0) {
		INSTANT_USER_MSG("No targets discovered\n\n");
		return;
	}

	INSTANT_USER_MSG("---------------------------------------------------------------------\n");
	INSTANT_USER_MSG("     Extension          Name                   IP          Protocol\n");
	INSTANT_USER_MSG("---------------------------------------------------------------------\n");

	for (i = 0; i < targetcount; i++) {
		INSTANT_USER_MSG("%d)  ",count);
		int extlength = strlen(targettab[i].extension);
		int a;
		for (a = 0; a < extension_len; a++) {
			INSTANT_USER_MSG("%c",targettab[i].extension[a]);
		}

		if (extlength < 12) {
			int remaining = extension_len - extlength;
			int b;
			for (b = 0; b < remaining; b++) {
				INSTANT_USER_MSG(" ");
			}
		}

		INSTANT_USER_MSG("   ");	

		int dirnamelength = strlen(targettab[i].dirname);
		for (a = 0; a < dirname_len; a++) {
			INSTANT_USER_MSG("%c",targettab[i].dirname[a]);
		}

		// print remaining blanks
		if (dirnamelength < dirname_len) {
			int remaining = dirname_len - dirnamelength;
			int b;
			for (b = 0; b < remaining; b++) {
				INSTANT_USER_MSG(" ");
			}
		}

		int iplength = strlen(targettab[i].ip);
		for (a = 0; a < ip_len; a++) {
			INSTANT_USER_MSG("%c",targettab[i].ip[a]);
		}

		if (iplength < ip_len) {
			int remaining = ip_len - iplength;
			int b;
			for (b = 0; b < remaining; b++) {
				INSTANT_USER_MSG(" ");
			}
		}

		INSTANT_USER_MSG("  ");

		INSTANT_USER_MSG("%s",targettab[i].protocol);
		int strLen = strlen(targettab[i].protocol);
		//printf("strLen:  %d  ",strLen);

		count ++;

		int compare1 = strcmp(targettab[i].protocol,"sip");
		int compare2 = strcmp(targettab[i].protocol,"sccp");
		
		if ((strLen == 4) && (compare2 == 0)) {
			//targettab[i].protocol[4] = '\n';
			INSTANT_USER_MSG("\n");
		}
		
		if ((strLen == 3) && (compare1 == 0)) {
			INSTANT_USER_MSG("\n");
		}

	}

	INSTANT_USER_MSG("---------------------------------------------------------------------\n\n");

}

int parse_targets() {

	char *token;
	//char token[64];
	char *tempname;
	//char tempname[64];
	char *tempprotocol;
	//char tempprotocol[64];

	ftargets = fopen(targetsfile, "r");
	if (ftargets == NULL) {
		DEBUG_MSG("File %s can't be opened for reading in working directory\n",targetsfile);
		return 0;
	}

	// Now that the file has been opened
	// Read through the file, one line at a time
	// Load the entries into memory,
	// into the targettab array
	// each IP is an entry and the 'targetcount' variable
	// is the index
	// targetcount is then used for subsequent adds to the target file
	//
	int close;
	int length = MAX_ASCII_ADDR_LEN + MAX_ASCII_ADDR_LEN + 210;
	char temp[length];

	while (fgets(temp, length, ftargets) != NULL) {

		char *search = ",";

		token = strtok(temp,search);
		strcpy(targettab[targetcount].ip,token);
		ip_yes = isvalidip(targettab[targetcount].ip);
		if(ip_yes != 1){
			printf("IP Address '%s' is not valid\n",targettab[targetcount].ip);
			printcorrect();
			exit(1);
		}

		token = strtok(NULL,search);
		strcpy(targettab[targetcount].extension, token);
		int retval = checkstr(targettab[targetcount].extension);

		tempname = strtok(NULL, search);
		tempprotocol = strtok(NULL, search);

		if(tempprotocol == NULL) {
			strcpy(targettab[targetcount].dirname,"Unknown");
			//strcpy(targettab[targetcount].protocol,tempname);
			strncpy(targettab[targetcount].protocol,tempname,sizeof(targettab[targetcount].protocol));
			retval = checkpro(tempname);
		} else {
			strcpy(targettab[targetcount].dirname,tempname);
			retval = checkstr(targettab[targetcount].dirname);
			strncpy(targettab[targetcount].protocol,tempprotocol,sizeof(targettab[targetcount].protocol));
			retval = checkpro(tempprotocol);
		}

		targetcount++;

	}	

	DEBUG_MSG("Parsed %d entries in Targets file, targets.txt\n",targetcount);
	
	close = fclose(ftargets);

	return targetcount; 
}

int okay_targethandler(sipDB* currentSipCall, u_char *ptr){

	if(uri_obj.kpml_enabled == 0){

		if((!strcmp(target1.extension,currentSipCall->src_extension)) || (!strcmp(target1.extension,currentSipCall->dst_extension)))
			return 1;
		else{
			int return_remote = decode_remote_party(ptr);
			if(return_remote == 0){
				if((strcmp(currentSipCall->dst_extension,uri_obj.dst_extension) != 0) && (!strcmp(target1.extension,uri_obj.dst_extension)))
					return 1;
			}
		}
	}
	else if(uri_obj.kpml_enabled == 1){
		if(!strcmp(currentSipCall->src_extension,target1.extension))
			return 1;
	}
	return 0;
}			

void sip_targetlookup(sipDB* currentSipCall){

	if(monitor_yes == 1)
		return;

	SIPLOG("Inside sip_targetlookup\n");

	int target_return = check_targettab(currentSipCall->src_extension);
	if(target_return < 0){

		/* Adds a new entry to the target structure  --> This is the Dialing party extension
		   Also checks whether the IP address is from the local vlan or remote vlan, adds only if the IP address is from the local vlan */ 

			int return_val = check_local_ip(currentSipCall->media_src_ip);
			if(return_val == 1){

				initializeTargetTab(targetcount);
				memcpy(targettab[targetcount].extension,currentSipCall->src_extension,sizeof(targettab[targetcount].extension));
				memcpy(targettab[targetcount].ip,currentSipCall->media_src_ip,MAX_ASCII_ADDR_LEN);
				memcpy(targettab[targetcount].dirname,currentSipCall->src_username,sizeof(targettab[targetcount].dirname));
				memcpy(targettab[targetcount].protocol,"sip",sizeof(targettab[targetcount].protocol));
				memcpy(targettab[targetcount].ua,currentSipCall->user_agent,sizeof(targettab[targetcount].ua));
				if(strlen(targettab[targetcount].dirname) > 0)
					targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,targettab[targetcount].protocol);
				else
					targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].protocol);
				
				INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);

				SIPLOG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
							targettab[targetcount].ip,
							targettab[targetcount].extension,
							targettab[targetcount].dirname);
				++targetcount;
			}
	}
	else{
		/* checks whether the ip has changed, if it has changed get it replaced else just let it remain the same */

		if(strcmp(targettab[target_return].ip,currentSipCall->media_src_ip) != 0){

			int return_val = check_local_ip(currentSipCall->media_src_ip);
			if(return_val == 1){

                        	INSTANT_USER_MSG("Updated target extension %s from IP '%s' to '%s'\n",
                                	                                        targettab[target_return].extension,
                                        	                                targettab[target_return].ip,
                                                	                        currentSipCall->media_src_ip);

				SIPLOG("Updated target extension %s from IP '%s' to '%s'\n",
										targettab[target_return].extension,
										targettab[target_return].ip,
										currentSipCall->media_src_ip);	

                   		memset(targettab[target_return].ip,'\0',sizeof(targettab[target_return].ip));
                        	memcpy(targettab[target_return].ip,currentSipCall->media_src_ip,MAX_ASCII_ADDR_LEN);
                        	updatetargetsfileip(currentSipCall->media_src_ip,targettab[target_return].extension);
			}
                }
	}

	int target_return1 = 0;
	if(currentSipCall->remoteParty)
		target_return1 = check_targettab(currentSipCall->remote_extension);
	else
		target_return1 = check_targettab(currentSipCall->dst_extension);
	if(target_return1 < 0){

		/* Adds a new entry to the target structure  --> This is the Dialed party extension
		   Adds only if the IP address belongs to the local vlan or lan segment */

		int return_val = check_local_ip(currentSipCall->media_dst_ip);
		if(return_val == 1){ 

			initializeTargetTab(targetcount);

			if(currentSipCall->remoteParty)
				memcpy(targettab[targetcount].extension,currentSipCall->remote_extension,sizeof(targettab[targetcount].extension));
			else
				memcpy(targettab[targetcount].extension,currentSipCall->dst_extension,sizeof(targettab[targetcount].extension));
        	        memcpy(targettab[targetcount].ip,currentSipCall->media_dst_ip,MAX_ASCII_ADDR_LEN);
               		memcpy(targettab[targetcount].dirname,currentSipCall->dst_username,sizeof(targettab[targetcount].dirname));
                	memcpy(targettab[targetcount].protocol,"sip",sizeof(targettab[targetcount].protocol));
                	memcpy(targettab[targetcount].ua,currentSipCall->user_agent,sizeof(targettab[targetcount].ua));
			if(strlen(targettab[targetcount].dirname) > 0){
				targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,targettab[targetcount].protocol);
			}
			else
				targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].protocol);
		
			INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);		
			
			SIPLOG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
							targettab[targetcount].ip,
							targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);        		

	        	++targetcount;
		}
	}
	else{
		/* checks whether the ip has changed, if it has changed get it replaced else just let it remain the same */

		if(strcmp(targettab[target_return1].ip,currentSipCall->media_dst_ip) != 0){

			int return_val = check_local_ip(currentSipCall->media_dst_ip);
			if(return_val == 1){

                  	      INSTANT_USER_MSG("Updated target extension %s from IP '%s' to '%s'\n",
                        	                                                targettab[target_return1].extension,
                                	                                        targettab[target_return1].ip,
                                        	                                currentSipCall->media_dst_ip);
				
			        SIPLOG("Updated target extension %s from IP '%s' to '%s'\n",				
										 targettab[target_return1].extension,
                                                                                targettab[target_return1].ip,
                                                                                currentSipCall->media_dst_ip);

                        	memset(targettab[target_return1].ip,'\0',sizeof(targettab[target_return1].ip));
                        	memcpy(targettab[target_return1].ip,currentSipCall->media_dst_ip,MAX_ASCII_ADDR_LEN);
                        	updatetargetsfileip(currentSipCall->media_dst_ip,targettab[target_return1].extension);
			}
                }
	}
	return;
}

int targetlookup(SkinnyCallPtr *sPtr, int passthrupartyid, char ip[MAX_ASCII_ADDR_LEN]) {

        SkinnyCallPtr current_skinnycall;
        current_skinnycall = *sPtr;

        if (target_yes == 1) {
                return 0;
        }

        char *null_string = "NULL";
        char *deviceprotocol = "sccp";

        while (current_skinnycall != NULL) {

                if(passthrupartyid == current_skinnycall->passthrupartyid1) {


                        if(targetcount == 0) {

                                // Check to make sure ip is local
                                int is_local = check_local_ip(current_skinnycall->dialingip);
                                if ((is_local == 1)||(monitor_yes == 1)) {

                                        // Copy dialing IP into targettab
                                        strcpy(targettab[targetcount].ip,current_skinnycall->dialingip);

                                        // Copy dialing number into targettab
                                        strcpy(targettab[targetcount].extension,current_skinnycall->dialingnumber);

                                        // Copy protocol
                                        strcpy(targettab[targetcount].protocol, deviceprotocol);

                                        // Lookup directory dialing name
                                        char *temp1 = lookupdialingname(passthrupartyid, &first_skinnycall);
                                        int compare = strcmp(temp1,null_string);
                                        if (compare != 0) {
                                                // copy dialing name into targetcount, if there
                                                strcpy(targettab[targetcount].dirname,temp1);
                                                // print to stdout
                                                INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);
                                                // output to targets file
                                                targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,deviceprotocol);

                                        } else {

						INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension);
                                                targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,deviceprotocol);
                                        }

                                        // increment targetcount.  Since targetcount was 0 before, we know the dialed IP Phone
                                        // needs to be added in as well
                                        targetcount++;

                                } else {
                                        //printf("IP %s is not local, not updating targets\n",ip);
                                }

				is_local = check_local_ip(current_skinnycall->dialedip);
				//printf("TEST is_local:  %d, current_skinnycall->dialedip is %s\n",is_local,current_skinnycall->dialedip);
                                if ((is_local == 1)||(monitor_yes == 1)) {

					// Check to make sure the dialed number is numeric
					int check = checkdigitstring(current_skinnycall->dialednumber);

					if (check == 1) {
                                        	// Copy dialed IP into targettab
                                        	strcpy(targettab[targetcount].ip,current_skinnycall->dialedip);
                                        
						// Copy dialed number into targettab
						// Only copy if all numeric digits
                                        	strcpy(targettab[targetcount].extension,current_skinnycall->dialednumber);
						if(voipdebug){
							fprintf(sccpfile,"Copying dialed number of %s into targettab\n",current_skinnycall->dialednumber);
							fflush(sccpfile);
						}
                                        
						// Copy device protocol
                                        	strcpy(targettab[targetcount].protocol, deviceprotocol);

                                        	char *temp2 = lookupdialedname(passthrupartyid, &first_skinnycall);
                                        	int compare = strcmp(temp2,null_string);

						if (compare != 0) {
                                                	strcpy(targettab[targetcount].dirname,temp2);
                                                	INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name:  %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);
                                                	targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,deviceprotocol);

                                        	} else {
                                                	INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension);
                                                	targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,deviceprotocol);
                                        	}
                                        	
						targetcount++;


					} else {
						INSTANT_USER_MSG("Dialed String is not fit for updating targets file\n");
					}

                                } else {

                                        //printf("IP %s is not local, not updating targets\n",current_skinnycall->ip2);

                                }
			
			// targetcount greater than 0
                        } else {

				// This section actually looks up the extension to see if the IP address needs to be updated
                                int b;
                                int new_ext_dialing = 1;
                                int new_ext_dialed = 1;
                                int compare_ext;
                                // Loop through targettab for extensions
                                for (b = 0; b < targetcount; b++) {

					compare_ext = strcmp(targettab[b].extension,current_skinnycall->dialingnumber); 
                                        if (compare_ext == 0) {
						// Only proceed if this is an existing extension in targettab
						
						// compare dialing IP
                                                int compare_ip = strcmp(targettab[b].ip,current_skinnycall->dialingip);

						// check to see if Dialing IP is in local ARP Scanned Hosts
                                        	int is_local = check_local_ip(current_skinnycall->dialingip);

                                                if ((compare_ip == 0)&&(is_local == 1)){

							// IP Address the same for this extension,
							// IP Address is local
							//printf("IP address the same for extension.\n");

                                                } else if ((compare_ip != 0)&&(is_local == 1)) {
							// IP Address differs and IP Address is local

							/* Only update if IP Address is not equal to 'IP Unknown' */
							if (strcmp("IP Unknown",current_skinnycall->dialingip) != 0) {

                                                        	INSTANT_USER_MSG("Updated target extension %s from IP '%s' to '%s'\n",
									targettab[b].extension,
									targettab[b].ip,
									current_skinnycall->dialingip);

								// Copy it in
                                                        	strcpy(targettab[b].ip, current_skinnycall->dialingip);

								// Update targets text file
                                                        	updatetargetsfileip(current_skinnycall->dialingip,targettab[b].extension);
							} else {
								INSTANT_USER_MSG("Not updating targets entry because dialing IP Address is unknown\n");
							}

                                                } else if ((compare_ip != 0)&&(monitor_yes == 1)) {
						
							INSTANT_USER_MSG("IP Differs, but running in Monitor Mode, so we don't know if local or remote\n");	
							// IP Address differs but we don't know if it is local 	
							
						} else {


						}
	
						//Updating the directory name for the user, if applicable
                                                char *temp1 = lookupdialingname(passthrupartyid, &first_skinnycall);
                                                int compare_dir = strcmp(temp1,null_string);
                                                // If the string is not null
                                                if (compare_dir != 0) {

                                                        //Check to see if it is the same, or needs to be updated
                                                        int compare_strval = strcmp(targettab[b].dirname,temp1);
                                                        if (compare_strval == 0) {
                                                                // Directory name the same for extension 
                                                        } else {
                                                                INSTANT_USER_MSG("Updated target directory name for extension %s to '%s'\n",
                                                                                targettab[b].extension,
                                                                                temp1);

								// Update directory name
                                                                strcpy(targettab[b].dirname,temp1);

                                                                // updatetargetsfiledirname
                                                                updatetargetsfiledirname(b,ip,temp1);
                                                        }
                                                }

                                                //newipdialing = 0;
                                                new_ext_dialing = 0;

                                        }

                                        //is_local = check_local_ip(current_skinnycall->dialedip);
					compare_ext = strcmp(targettab[b].extension,current_skinnycall->dialednumber);
					// Compare dialed extension to what is already in targettab
                                        if (compare_ext == 0) {
                                                
						// Compare dialed IP	
						int compare_ip = strcmp(targettab[b].ip,current_skinnycall->dialedip);
						
						// check to see if Dialed IP is in local ARP Scanned Hosts
						// int is_local = check_local_ip(current_skinnycall->dialingip);
						int is_local = check_local_ip(current_skinnycall->dialedip);

                                                if ((compare_ip == 0)&&(is_local == 1)){
							
							// IP Address the same for this extension,
							// IP Address is local
							//printf("IP address the same for extension.\n");

                                                } else if ((compare_ip != 0)&&(is_local == 1)) {
							// IP Address differs and IP Address is local

							/* Only update if IP is not 'IP Unknown' */
							if (strcmp("IP Unknown",current_skinnycall->dialedip) != 0) {


                                                        	INSTANT_USER_MSG("Updated target extension %s from IP '%s' to '%s'\n",
									targettab[b].extension,
									targettab[b].ip,
									current_skinnycall->dialedip);

								// Copy it in
                                                        	//strcpy(targettab[b].ip, current_skinnycall->dialingip);
                                                        	strcpy(targettab[b].ip, current_skinnycall->dialedip);
                                                       
						        	// Update targets text file	
								updatetargetsfileip(current_skinnycall->dialedip,targettab[b].extension);

							} else {

								INSTANT_USER_MSG("Not updating targets entry because dialed IP Address is unknown\n");
								
							}

                                                } else if ((compare_ip != 0)&&(monitor_yes == 1)) {
							
							INSTANT_USER_MSG("IP Differs, but running in Monitor Mode, so we don't know if local or remote\n");	
							// IP Address differs but we don't know if it is local 	
                                                
						} else {

						}


						//Updating the directory name for the user, if applicable
                                                char *temp2 = lookupdialedname(passthrupartyid, &first_skinnycall);
                                                int compare_dir = strcmp(temp2,null_string);
                                                if (compare_dir != 0) {

                                                        int compare_strval = strcmp(targettab[b].dirname,temp2);

                                                        if (compare_strval == 0) {
                                                                // Directory name the same for extension 
                                                        } else {

                                                                INSTANT_USER_MSG("Updated target directory name for extension %s to '%s'\n",
                                                                                targettab[b].extension,
                                                                                temp2);

                                                                
								// Update directory name
								strcpy(targettab[b].dirname,temp2);
                                                                
								// updatetargetsfiledirname
                                                                updatetargetsfiledirname(b,current_skinnycall->dialedip,temp2);

                                                        }

                                                }

						//newipdialed = 0;
						new_ext_dialed = 0;

                                        }

                                }

                                // This section adds a new extension to targettab
                                // Must be a new extension Found if compare = 1
                                int is_local = check_local_ip(current_skinnycall->dialingip);
                                if ((new_ext_dialing == 1)&&((is_local == 1)||(monitor_yes == 1))) {

					/* Don't add in new target entry if extension string is 'Unknown' */
					if (strcmp("Unknown",current_skinnycall->dialingnumber) != 0) {

                                        	strcpy(targettab[targetcount].ip,current_skinnycall->dialingip);
                                        	strcpy(targettab[targetcount].extension,current_skinnycall->dialingnumber);
						//printf("TEST copying %s into targettab\n",current_skinnycall->dialingnumber);
                                        	strcpy(targettab[targetcount].protocol, deviceprotocol);
                                        	char *temp1 = lookupdialingname(passthrupartyid, &first_skinnycall);
                                        	int compare = strcmp(temp1,null_string);
                                        	if (compare != 0) {
                                                	strcpy(targettab[targetcount].dirname,temp1);

                                                	INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name %s\n",
                                                        	targettab[targetcount].ip,
                                                        	targettab[targetcount].extension,
                                                        	targettab[targetcount].dirname);
                                                	targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,deviceprotocol);

                                        	} else {
                                                	INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s\n",
                                                        	targettab[targetcount].ip,
                                                        	targettab[targetcount].extension);
                                                	targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,deviceprotocol);
                                        	}

                                        	targetcount++;
					
					} else {
						INSTANT_USER_MSG("Not creating new target entry because dialingnumber is Unknown\n");
	
					}

                                }

				is_local = check_local_ip(current_skinnycall->dialedip);
                                if ((new_ext_dialed == 1)&&((is_local == 1)||(monitor_yes == 1))) {

                                        strcpy(targettab[targetcount].ip,current_skinnycall->dialedip);
                                        strcpy(targettab[targetcount].extension,current_skinnycall->dialednumber);
                                        strcpy(targettab[targetcount].protocol, deviceprotocol);
                                        char *temp2 = lookupdialedname(passthrupartyid, &first_skinnycall);
                                        int compare = strcmp(temp2,null_string);
                                        if (compare != 0) {

                                                strcpy(targettab[targetcount].dirname,temp2);

                                                INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s and name %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension,
                                                        targettab[targetcount].dirname);
                                                targetfileadddirectory(targettab[targetcount].ip,targettab[targetcount].extension,targettab[targetcount].dirname,deviceprotocol);


                                        } else {

                                                INSTANT_USER_MSG("Mapped new target entry:  (IP: %s) --> extension %s\n",
                                                        targettab[targetcount].ip,
                                                        targettab[targetcount].extension);
                                                targetfileadd(targettab[targetcount].ip,targettab[targetcount].extension,deviceprotocol);
                                        }


                                        targetcount++;
                                }

                        }

                        current_skinnycall = current_skinnycall->next_skinnycall;

                } else {

                        current_skinnycall = current_skinnycall->next_skinnycall;

                }

        }
		return 0;
}
				
int targetfileadddirectory(char ip[MAX_ASCII_ADDR_LEN], char extension[24], char dirname[64], char deviceprotocol[10]) {

	ftargets = fopen(targetsfile,"a+");
	if(ftargets == NULL){
		INSTANT_USER_MSG("Error: Opening targets.txt file \n");
		return 0;
	}

	fprintf(ftargets,"%s,%s,%s,%s\n",ip,extension,dirname,deviceprotocol);
	fflush(ftargets);
	fclose(ftargets);
	return 1;

}

int targetfileadd(char *ip, char *extension, char *deviceprotocol) {

	ftargets = fopen(targetsfile,"a+");
	if(ftargets == NULL){
		INSTANT_USER_MSG("Error: Opening targets.txt file \n");
		return -1;
	}
	fprintf(ftargets,"%s,%s,,%s\n",ip,extension,deviceprotocol);
	//int flush = fflush(ftargets);
	//return flush;
	fclose(ftargets);
	return 1;

}

int sip_targethandler(int message_type, u_char *bytes, int length, char *src_ip, char *dst_ip){

		return 0;

}// end of sip_targethandler

void print_targets_summary(void){

	if (targetcount == 0) {
		
		return;
	}

	double percent = (double) targetcount / host_count * 100;

	INSTANT_USER_MSG("\nMapped %d targets out of %d potential hosts in source network ~ (%.2f Percent)\n\n",targetcount,host_count,percent);

}
void print_targets_count(void){

	INSTANT_USER_MSG("%d Targets Total\n",targetcount);
	INSTANT_USER_MSG("\n\n");

}

int check_rtp_entry(char *ip_addr){

        int i = 0;

        for( i = 0; i < rtp_counter; i++){

                if(strcmp(ip_addr,rtp[i].ip_addr) == 0)
                        return i;
        }
        return -1;
}

int check_rtp_hdr(char *ip_addr, int media_port, int *video){

        int i = 0;

        for( i = 0; i < rtp_counter; i++){

                if((strcmp(ip_addr,rtp[i].ip_addr) == 0)) {
                        if ((rtp[i].port == media_port)) {
                                *video = 0;
                                return i;
                        }
                        else if(rtp[i].vport == media_port) {
                                *video = 1;
                                return i;
                        }
                }
        }
        return -1;
}

int checkdigitstring(char str[24]){

	// return 1 if all digits are decimal digits.  If not, return 0
	
	int retval = 0;
	int check = 0;
	int len = strlen(str);
	int i;
	if (len == 0) {
		return 0;
	}
	for (i = 0; i < len; i++) {

		check = isdigit(str[i]);
		if(check == 0) {
			//printf("not a decimal digit, exiting\n");
			return retval;
		}

	}
	
	return 1;
	
}
int isvalidip( char *ip) {

#ifndef OS_WINDOWS
	struct sockaddr_in sa;
	int result = inet_pton(AF_INET, ip, &(sa.sin_addr));
	return result != 0;
#else
	return 1;
#endif

}
void printcorrect(){
	printf("File targets.txt must have correct format.  Here is an example:\n\n");
	printf("172.16.30.8,8000,Jason,sccp\n");
	printf("172.16.30.9,8001,Arjun,sccp\n");
	printf("172.16.30.10,8002,Mike,sip\n");
	printf("172.16.30.11,8003,,sip\n");
	printf("172.16.30.12,8004,Harsh,sip\n");
	printf("172.16.30.13,8005,Abhijeet,sccp\n\n");
	printf("Correct the file 'targets.txt' and launch again.  UCSniff will now exit.\n\n");
}
int checkstr( char *ext){
	int i;
	int len = strlen(ext);
	//printf("length of extension:  %d\n",len);
	if (len < 1 || len > 64) {
			printf("String length must be between 1 - 64 characters.  Length is %d\n",len);
			printcorrect();
			exit(1);
	}
	for (i = 0; i < len; i++ ){
		if(isalnum(ext[i]) == 0){
			if(isprint(ext[i]) == 0){
				int blah = i + 1;
				printf("Character %d is not printable in string %s.\n",blah,ext);
				printcorrect();
				exit(1);
			}	
		}
	}
	return 1;
}
int checkpro( char *pro){
        int i;
        int len = strlen(pro);
	int lencheck = len - 1;

	//printf("Inside checkpro, len:  %d, lencheck:  %d\n",len, lencheck);
        if (len < 1 || len > 9) {
                        printf("Protocol string length must be between 1 - 8 characters.  Length is %d.  Should be 'sccp' or 'sip.\n",lencheck);
                        printcorrect();
                        exit(1);
        }

	for (i = 0; i < lencheck; i++){
		if(isalpha(pro[i]) == 0){
			printf("Character '%c' in protocol value %s is not alpha.  Should be 'sccp' or 'sip'\n",pro[i],pro);
			printcorrect();
			exit(1);

		}


	}
	return 0;
}
