/*
 * ACE - Automated Corporate Enumerator
 * Copyright (C) 2008 Sipera VIPER Lab
 * Author:  Jason Ostrom
 * Chief Developer:  Arjun Sambamoorthy
 *
 * This file is part of ACE.
 *
 * ACE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * ACE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ec.h"
#include "includes.h"
#include "tftp_client.h"
#include "client.h"
#include "ec_voipglobals.h"

#ifndef OS_WINDOWS

#include "pathnames.h"
#include "signals.h"
#include "errno.h"
#include "syslog.h"
#include "sys/ioctl.h"
#include "sys/stat.h"

#endif

#ifdef OS_WINDOWS

#include "winsock.h"
#include "windef.h"

extern CHAR *networkDevice;
extern WCHAR *wTCPNetworkDevice;

#endif

//From old ace main.c
FILE *fp;
char route_buf[64];

static void initializeGlobalVariables();

static void initializeGlobalVariables(){

	server_iis = 0;
	server_apache = 0;
	verbosity = 0;
	http_port = HTTP_PORT;
	directory_start = 0;
	directory_end = 0;
	directory_total = 0;
	structure_size = 0;

	memset(cookie,'\0',sizeof(cookie));	
	memset(route_buf,'\0',sizeof(route_buf));

	memset(directory_buf,'\0',sizeof(directory_buf));
	memset(auth_buf,'\0',sizeof(auth_buf));
	memset(info_buf,'\0',sizeof(info_buf));
	memset(services_buf,'\0',sizeof(services_buf));
	memset(directory_buf1,'\0',sizeof(directory_buf1));
	memset(directory_ip,'\0',sizeof(directory_ip));
	memset(directory_url,'\0',sizeof(directory_url));
	memset(prompt_string,'\0',sizeof(prompt_string));
	memset(ucm_ip,'\0',sizeof(ucm_ip));
}


int directory_download(){ 

	initializeGlobalVariables();

	if(monitor_yes == 1)
		return -1;

	if(ace_yes == 0)
		return 0;

	//Check to see if they did or did not get the TFTP Server IP Address via Option 150 from the vlan_interface_init() function
	if((vlan_yes == 1) && (tftp_ip_yes != 1)&&(tftp_yes != 1)) {
		INSTANT_USER_MSG("TFTP Server IP Address not automatically received in Option 150 of DHCP Message\n");	
		INSTANT_USER_MSG("Try setting the TFTP Server IP Address with the (-t <ip>) Option\n");	
		INSTANT_USER_MSG("Example:  ace -i eth0 -v 96 -a -m 00:1F:6C:7E:CF:6C -t 172.16.100.1 // //\n");
		return -1;
	}

	if((cdp_mode_yes == 1) && (tftp_ip_yes != 1)&&(tftp_yes != 1)) {
		INSTANT_USER_MSG("TFTP Server IP Address not automatically received in Option 150 of DHCP Message\n");	
		INSTANT_USER_MSG("Try setting the TFTP Server IP Address with the (-t <ipa>) Option\n");	
		INSTANT_USER_MSG("Example:  ace -i eth0 -c 1 -a -m 00:1F:6C:7E:CF:6C -t 172.16.100.1 // //\n");
		return -1;
	}

	if ((tftp_yes != 1)&&(vlan_yes != 1)&&(cdp_mode_yes != 1)) {
		if (ace_yes == 0) {
			return 0;
		} else {
#ifndef OS_WINDOWS
			dont_set_ip = 1;
			dhcpclientcall(IfName_temp);
#endif
#ifdef OS_WINDOWS

			dont_set_ip = 1;
			int retdhcp = getOption150AndRouterIP(networkDevice,tftp_server_ip,router_ip);
			if(retdhcp < 0)
				return -1;
			
#endif
			if(tftp_ip_yes != 1) {
				INSTANT_USER_MSG("TFTP Server IP Address not automatically received in Option 150 of DHCP Message\n");	
				INSTANT_USER_MSG("Try setting the TFTP Server IP Address with the (-t <ip>) Option\n");	
				INSTANT_USER_MSG("Example:  ace -i eth0 -a -m 00:1F:6C:7E:CF:6C -t 172.16.100.1 // //\n");
				return -1;

			}
		}
	}

	if (ace_yes == 1) {

#ifndef OS_WINDOWS
		strcpy(route_buf,"route add -host ");
		strcat(route_buf,tftp_server_ip);
		strcat(route_buf," gw ");
		strcat(route_buf,router_ip);

		fp = popen(route_buf,"r");
		if(fp == NULL){
			INSTANT_USER_MSG("Error:  adding host route in routing table\n");
			return -1;
		}

		pclose(fp);
#endif

#ifdef OS_WINDOWS

		
		dont_set_ip = 1;
		int retdhcp = getOption150AndRouterIP(networkDevice,tftp_server_ip,router_ip);
		if(retdhcp < 0)
			return -1;
		int retRoute = addRoute(wTCPNetworkDevice, tftp_server_ip, router_ip);
			if(retRoute < 0)
				return -1;
#endif

		/* TFTP Handler */
		int tftp_return = tftp_handler(mac_name,tftp_server_ip);
		if(tftp_return < 0) {
			INSTANT_USER_MSG("Error:  tftp_handler\n");
			return -1;
		}

		/*Parses directory URL to get the IP Address */
		parse_url(directory_buf);

		/*Calls the http handler */
		int httpRet = http_handler();
		if(httpRet < 0)
			return -1;
		/* Prints the corporate data to file directory-users.txt */
		print_corporate_data();

	}

	return 0;
}


// This function converts ":" Deliminated MAC address in string format.(Removes : delimiter)
void mac2string (char *mac, char *s)
{
	while(*mac){
                if ( *mac == ':'){
                        *mac++;
                }
                *s = *mac;
                *mac++;
                *s++;
        }
        *s = '\0';
}



int send_ack_packet(int sock, struct sockaddr_in *saddr, short block, short port){

	int send_err = 0;
	struct tftp_ack tftp_ack_packet;
	
	tftp_ack_packet.opcode = htons(ACK);
        tftp_ack_packet.block = block;
        saddr->sin_port = port;

        send_err = sendto(sock,&tftp_ack_packet,sizeof(struct tftp_ack),0,(struct sockaddr *)saddr,sizeof(struct sockaddr_in));
        if(send_err < 0){
        	perror("Error while sending TFTP ACK \n");
                return -1;
        }
	return 0;

}
	

int  recv_request_reply(int sock, struct sockaddr_in *saddr, char *file_name) {

	int recv_err = 0, data_size = DATA_SIZE;
	struct tftp_recv tftp_recv_packet;
	struct sockaddr_in saddr1;
	FILE *fp = NULL;
	int size_of = sizeof(saddr1);
	tftp_recv_packet.data[512] = '\0';

	while(data_size == 512) {  // Data size less than 512 implies last data packet

		memset(tftp_recv_packet.data,'\0',sizeof(tftp_recv_packet.data));
        	recv_err = recvfrom(sock,&tftp_recv_packet,sizeof(tftp_recv_packet),0,(struct sockaddr*)&saddr1,&size_of);
        	if(recv_err < 0){
                	perror("Error while receiving\n");
                	return -1;
        	}

        	if(tftp_recv_packet.opcode == htons(ERROR_CODE)){

			INSTANT_USER_MSG("***********************************\n");
                	INSTANT_USER_MSG("Received Error message:  %s \n",tftp_recv_packet.data);
                	data_size = strlen(tftp_recv_packet.data);
			return -1;
        	}
		else if(tftp_recv_packet.opcode == htons(DATA_PACKET)){

                	data_size = strlen(tftp_recv_packet.data);

			/* Writes Data into the file */
                	if(ntohs(tftp_recv_packet.block) == 1){

                        	fp = fopen(file_name,"w+");
                        	if(fp == NULL){
                                	perror("Opening File \n");
                                	return -1;
                        	}
                	}
                	fputs(tftp_recv_packet.data,fp);
                	memset(tftp_recv_packet.data,'\0',512);

			/* Send ACK packet */
			int send_err = send_ack_packet(sock,saddr,tftp_recv_packet.block,saddr1.sin_port);
			if(send_err < 0)
				return -1;
        	}

     	}
	fclose(fp);	
	return 0;
		

}

int send_tftp_request(struct tftp *tftp_packet, char *file_name, char *mode, int sock, struct sockaddr_in *saddr){

	int send_err = 0;
	memset(tftp_packet->file_name,'\0',sizeof(tftp_packet->file_name));

	tftp_packet->opcode = htons(READ_REQUEST);
        memcpy(tftp_packet->file_name,file_name,strlen(file_name));
        memcpy(tftp_packet->file_name + strlen(file_name) + 1, mode, strlen(mode));
        strcpy(tftp_packet->file_name + strlen(file_name) + 1 + strlen(mode),"\0");

        send_err = sendto(sock,tftp_packet,sizeof(struct tftp)-80+2+strlen(file_name)+strlen("octet"),0,(struct sockaddr*)saddr,sizeof(struct sockaddr_in));
        if(send_err < 0){
                perror("Send Error \n");
               	return -1;
        }
	else{
		INSTANT_USER_MSG("TFTP request for file %s sent\n",file_name);
	}
	return 0;
	
}
	

void intialize_server_socket(struct sockaddr_in *saddr,char *tftp_server_ip,int port) {

	saddr->sin_port = htons(port);
	saddr->sin_family = AF_INET;
	saddr->sin_addr.s_addr = inet_addr(tftp_server_ip);
}

int create_socket(){

	int sock = 0;
#ifdef OS_WINDOWS

	WORD wVersionRequested;
	WSADATA wsaData;
	
	wVersionRequested = MAKEWORD(2,2);
	int wsa_error = WSAStartup(wVersionRequested, &wsaData);
	if(wsa_error != 0){

		INSTANT_USER_MSG("WSAStartup failed with error: %d\n",wsa_error);
		return -1;
	}

	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if( sock == INVALID_SOCKET){
		INSTANT_USER_MSG("Error: Windows socket creation, error %d\n", WSAGetLastError());
		return -1;
	}
#else
	sock = socket(AF_INET,SOCK_DGRAM,0);
	if(sock < 0){
		INSTANT_USER_MSG("Socket Creation \n");
		return -1;
	}
#endif
	else{
		return sock;
	}
}

int create_file_from_mac(char *file_name, char *real_filename){

	char temp_file[64];

	if(strlen(file_name) != 12 ){
		INSTANT_USER_MSG("MAC Address should be 6 Bytes\n");
		INSTANT_USER_MSG("Example:  ace -m 00:1E:F7:28:9C:8e\n");
		return -1;
	}
	strcpy(temp_file,"SEP");
	strcat(temp_file,file_name);
	strcat(temp_file,".cnf.xml");
	
	strcpy(real_filename,temp_file);
	return 0;
}

int tftp_handler(char *mac, char *tftp_server_ip){


	//INSTANT_USER_MSG("Running TFTP Handler\n");
	int sock = 0,send_err = 0;
	char mode[16], real_filename[64];
	char mac_string[18];
	struct sockaddr_in saddr;
	struct tftp tftp_packet;
	
	memset(mac_string,'\0',sizeof(mac_string));
	mac2string(mac,mac_string);
	
	int createRet = create_file_from_mac(mac_string,real_filename);
	if(createRet < 0)
		return -1;
	
	/*Creates tftp socket and initializes the server address */
	sock = create_socket();
	if(sock < 0){
		return -1;
	}
	intialize_server_socket(&saddr,tftp_server_ip,TFTP_SERVER_PORT);

	/*file transfer mode requested - octet*/
	memset(mode,'\0',sizeof(mode));
	strncpy(mode,"octet",strlen("octet"));
	
	/* Sends TFTP request */	
	send_err = send_tftp_request(&tftp_packet,real_filename,mode,sock,&saddr);
	if(send_err < 0)
		return -1;	

	/*Receive reply for TFTP request and sends ACK for data packetS received */
	int recv_err;
	recv_err = recv_request_reply(sock,&saddr,real_filename);

	if(recv_err < 0){
		close(sock);
		INSTANT_USER_MSG("TFTP configuration file not found\n");
		return -1;
	}
	else if(recv_err >= 0){
		close(sock);
	}
	
	/* Extracts directory, authentication, services, informational URL from config file */
	int return_url = xml_config_parser(real_filename);
	if(return_url < 0){
		INSTANT_USER_MSG("Directory url not found in xml configuration file. Probably no corporate directory has been setup. \n");
		return -1;
	}

	/* check for asp or jsp */
	int return_val = check_file_format(directory_buf);
	if(return_val < 0 ){
		INSTANT_USER_MSG("File format not handled or wrong file format\n");
		return -1;
	}
	
	return 0;	

} 			

	
