/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2009 Sipera Systems, Inc. 
    Jason Ostrom <jostrom@viperlab.net>
    Arjun Sambamoorthy <arjun@viperlab.net>

    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ec.h>
#include <ec_version.h>
#include <ec_globals.h>
#include <ec_signals.h>
#include <ec_parser.h>
#include <ec_threads.h>
#include <ec_capture.h>
#include <ec_dispatcher.h>
#include <ec_send.h>
#include <ec_plugins.h>
#include <ec_format.h>
#include <ec_fingerprint.h>
#include <ec_manuf.h>
#include <ec_services.h>
#include <ec_http.h>
#include <ec_scan.h>
#include <ec_ui.h>
#include <ec_conf.h>
#include <ec_mitm.h>
#include <ec_sslwrap.h>
#include <ec_garpb.h>
#include <ec_voipglobals.h>
#include <ec_voiplogfiles.h>
#include <tftp_client.h>
#include <voiphop.h>
#include <targets.h>
#include <client.h>
#include <queue.h>

#ifdef HAVE_LIB_VLC

#include <ec_liveMonitor.h>

#endif

#ifdef OS_WINDOWS

#include <winsock2.h>
#include <ws2tcpip.h>
#include <iphlpapi.h>

void getDefaultRoute();

#endif

/* global vars */

struct Ifacelist *ifaceHead;
struct Ifacelist *ifaceTail;
static int ucsniffStarted;

/* Mutex variable for accessing live monitor */
extern pthread_mutex_t livemonitor_mutex;
#define LIVEMONITOR_LOCK    do{ pthread_mutex_lock(&livemonitor_mutex); } while(0)
#define LIVEMONITOR_UNLOCK  do{ pthread_mutex_unlock(&livemonitor_mutex); } while(0)
/* protos */

void startUCSniff();
void stopUCSniff();
void clean_exit(int errcode);
//static void drop_privs(void);
static void time_check(void);
static void initVoipGlobals(void);
void getInterfaceList();

extern void *startUCSniffJUCEGUI(void *);
extern void (*getFromQueue(void))();
extern void send_close();

// from targets.c
extern int target_yes;
extern char* singletargetfile;
extern struct singletarget target1;
extern FILE *singletarget;

#ifdef HAVE_JUCE_GUI

extern struct Queue *queueHead;
extern struct Queue *queueTail;

#endif

char defaultGatewayMac[MAX_ASCII_ADDR_LEN];

// from monitor.c
void monitor_mode_init(void);

#ifdef OS_WINDOWS

CHAR *networkDevice = NULL;
CHAR *v_NetworkDevice = NULL;
WCHAR *wTCPNetworkDevice = NULL;

void createWindowsInterfaceNames();

#endif

#ifdef HAVE_JUCE_GUI

void getInterfaceList(){

	ifaceHead = NULL;
	ifaceTail = NULL;
	struct Ifacelist *currentlist = NULL;

	pcap_if_t *alldevs, *d;
	char errbuf[PCAP_ERRBUF_SIZE];
	if (pcap_findalldevs(&alldevs, errbuf) == -1)
        {
            printf("Error in pcap_findalldevs: %s\n", errbuf);
            exit(1);
        }

	/* Print the list */
        for(d=alldevs; d; d=d->next)
        {

			if(ifaceHead == NULL){
				currentlist = (struct Ifacelist *) malloc(1*sizeof(struct Ifacelist));
				if(currentlist == NULL){
					printf("Not enough memory available for allocation\n");
					exit(1);
				}
				ifaceHead = ifaceTail = currentlist;
				currentlist -> next = NULL;
			}
			else{
				currentlist = NULL;
				currentlist = (struct Ifacelist *) malloc(1*sizeof(struct Ifacelist));
				if(currentlist == NULL){
					printf("Not enough memory available for allocation\n");
					exit(1);
				}
				ifaceTail->next = currentlist;
				ifaceTail = currentlist;
				currentlist->next = NULL;
			}
		
			currentlist->name = (char *)calloc((strlen(d->name) + 1),sizeof(char));
			if(currentlist->name == NULL){
				printf("Not enough memory available for allocation\n");
				exit(1);
			}
			memcpy(currentlist->name,d->name,strlen(d->name));

			if(d->description == NULL){
				
				currentlist->description = (char *)calloc((strlen(d->name) + MAX_ASCII_ADDR_LEN + 4 + 1),sizeof(char));
				if(currentlist->description == NULL){
					printf("Not enough memory available for allocation\n");
					exit(1);
				}
				memcpy(currentlist->description,d->name,strlen(d->name));
			}
			else{
				currentlist->description = (char *)calloc((strlen(d->description) + MAX_ASCII_ADDR_LEN + 4 + 1) , sizeof(char));
				if(currentlist->description == NULL){
					printf("Not enough memory available for allocation\n");
					exit(1);
				}

				strncpy(currentlist->description,d->description,strlen(d->description));
			}

			pcap_addr_t *t;
			int i = 0;
			char ipaddress[MAX_ASCII_ADDR_LEN] = {'\0'};
			for( t = d->addresses; t; t = t->next){
			
				if(t->addr != NULL){
					
					if(i == 1){

						char *myip = inet_ntoa( ((struct sockaddr_in *)t->addr)->sin_addr );
						strcpy(ipaddress,"  (");
						strcpy(ipaddress + 3, myip);
						strcpy(ipaddress + 3 + strlen(myip),")");
						memcpy(currentlist->description + strlen(currentlist->description), ipaddress, strlen(ipaddress));
						memset(ipaddress,'\0',sizeof(ipaddress));
						break;
					}
					i += 1;
				}
			}
        }
}

#endif

int main(int argc, char *argv[]) {

   /*
    * Alloc the global structures
    * We can access these structs via the macro in ec_globals.h
    */
  
   globals_alloc();
   initVoipGlobals();
   initVoipLogFiles();
  
   GBL_PROGRAM = strdup(EC_PROGRAM);
   GBL_VERSION = strdup(EC_VERSION);
   SAFE_CALLOC(GBL_DEBUG_FILE, strlen(EC_PROGRAM) + strlen(EC_VERSION) + strlen("_debug.log") + 1, sizeof(char));
   sprintf(GBL_DEBUG_FILE, "%s%s_debug.log", GBL_PROGRAM, EC_VERSION);
   
   DEBUG_INIT();
   DEBUG_MSG("main -- here we go !!");
   
   /* register the main thread as "init" */  
   ec_thread_register(EC_PTHREAD_SELF, "init", "initialization phase");
   
   /* activate the signal handler */
   signal_handler();
   
   printf("%s ", EC_PROGRAM);
   printf("%s ", EC_VERSION);
   printf("starting\n");

   parse_targets();

   /* getopt related parsing...  */
   parse_options(argc, argv);

   if(guimode){

#ifdef HAVE_JUCE_GUI

		queueHead = NULL;
		queueTail = NULL;

		void (*tempFuncPtr)() = NULL;

		pthread_t threads[0];

		int rc = pthread_create(&threads[0],NULL,startUCSniffJUCEGUI,NULL);
		if(rc){
			printf("Error Creating thread for JUCE GUI : %d\n",rc);
			exit(1);
		}

		for(;;){

			tempFuncPtr = getFromQueue();
			if(tempFuncPtr != NULL){
				(*tempFuncPtr)();
			}

			if(guiShutdown){
				break;
			}
		
		}
#endif
	}
	else if(textmode){
	
		startUCSniff();
	}

   //DEBUG_CLOSE();
//printf("%s:%d - %s calling pthread_exit\n", __FILE__, __LINE__, __func__);
//	pthread_exit(NULL);
	return 0;
}

void startUCSniff(){

	if(ucsniffStarted == 1)
		return;

	if(learning_yes != 1 && target_yes != 1 && monitor_yes != 1){
		INSTANT_USER_MSG("Please select one of the following modes , learning / target / monitor \n");
		return;
	}

	if(guimode){

		int i = 0;
	
		struct Ifacelist *currentiface = ifaceHead;
		while(currentiface != NULL){

			i += 1;
			if( i == ifaceType){

#ifdef OS_WINDOWS
				IfName_temp = (char *)malloc((strlen(currentiface->name)+6)*sizeof(char));
				if(IfName_temp == NULL){
					INSTANT_USER_MSG("Error: allocating memory for the interface name\n");
					return;
				}	 
				memset(IfName_temp,'\0',strlen(currentiface->name)+6);
				strcpy(IfName_temp,(currentiface->name));

				char *start = strstr(IfName_temp,"{");
				char *end = strchr(IfName_temp,'\0');


				int length = end - start;
	
				int buf = length + 1;
				char tempDevice[buf];
			
				memcpy(tempDevice,start,length);
				memset(IfName_temp,'\0',strlen(currentiface->name)+6);

				strcpy(IfName_temp,"\\\\Device\\\\NPF_");
				char *tempBuf = IfName_temp;
				tempBuf += strlen("\\\\Device\\\\NPF_");
				strcpy(tempBuf,tempDevice);
				IfName_temp_len = strlen(currentiface->name) + 6;
#else
				IfName_temp = (char *)malloc((strlen(currentiface->name)+1)*sizeof(char));
				if(IfName_temp == NULL){
					INSTANT_USER_MSG("Error: allocating memory for the interface name\n");
					return;
				}
				memset(IfName_temp,'\0',strlen(currentiface->name)+1);
				strcpy(IfName_temp,currentiface->name);
#endif
				
				GBL_OPTIONS->iface = strdup(IfName_temp);
				intfd = 1;
				break;
			}
			currentiface = currentiface->next;
		}
	}

	if(intfd != 1){
		INSTANT_USER_MSG("Please select a interface \n");
		return;
	}
	if(guimode){

		if(ace_yes == 1 && mac_yes == 1){

			bzero(mac_name,64);
		  	strncpy(mac_name,macString,strlen(macString));
		  	int mac_err = check_mac_format(mac_name);
		  	if(mac_err < 0) {
			  	INSTANT_USER_MSG("MAC Address must be specified in 00:00:00:00:00:00 format\n"); 
			  	return;
		  	}
		}

		if(cdp_mode_yes == 1){
           
			GBL_OPTIONS->cdp = cdp_mode_type;
		}
		  
		if (monitor_yes == 1) {
           
	   		GBL_OPTIONS->unoffensive = 1;
 
	   		if (IfName_temp == NULL) {
		   		INSTANT_USER_MSG("In Monitor Mode, must specify interface.\n");
		   		//INSTANT_USER_MSG("Example:  ucsniff -i eth0 -M\n");
		   		return;
	   		}
			compile_display_filter();
   		}

		if ((ace_yes == 1)&&(mac_yes != 1)) {
			INSTANT_USER_MSG("MAC Address option (-m <mac>) is mandatory when (-a) option for directory download is enabled\n");
			return;	
   		}

		if (tftp_modify_yes == 1) {
        		if(( target_yes != 1)&&(garpdb_yes != 1)) {
                		INSTANT_USER_MSG("TFTP Modify option can only be enabled along with target mode and GARP Disable Bypass\n");
                		return;
        		}
   		}

		if (monitor_yes != 1) {
   			GBL_OPTIONS->mitm = 1;
   			char *arp_remote_string = "arp:remote";
   			char *tmp = malloc(10);
   			memcpy(tmp,arp_remote_string,10);
   			if (mitm_set(tmp) != ESUCCESS) {
				FATAL_ERROR("MITM method 'arp:remote' not supported...\n");
   			}

			if(target_yes != 1){
				GBL_OPTIONS->target1 = strdup("//");
				GBL_OPTIONS->target2 = strdup("//");
				/* create the list form the TARGET format (MAC/IPrange/PORTrange) */
				compile_display_filter();
			}
   		}
		if(target_yes == 1){

			if(!strlen(target1.ip)){
				INSTANT_USER_MSG( "In Target Mode, the targets list cannot be empty, please select a target\n");
				return;
			}

			char *tar1string;
			tar1string = (char *)malloc(sizeof(target1.ip) + 2);
			strcpy(tar1string,"/");
			strcat(tar1string,target1.ip);
			strcat(tar1string,"/");
			char *tar2string = "//";
        		GBL_OPTIONS->target1 = tar1string;
        		GBL_OPTIONS->target2 = tar2string;

			compile_display_filter();

			singletarget = fopen(singletargetfile, "a+");
			if (singletarget == NULL) {
				INSTANT_USER_MSG( "Error: unable to open the file %s\n",singletargetfile);
				return;
			}
		}

		GBL_OPTIONS->superquiet = 1;
    	GBL_OPTIONS->quiet = 1;

   		if (GBL_SNIFF->start == NULL)
      		set_unified_sniff();

	} // GUI mode

    if(target_yes == 1 && textmode){

		singletarget = fopen(singletargetfile, "a+");
		if (singletarget == NULL) {
			INSTANT_USER_MSG( "Error: unable to open the file %s\n",singletargetfile);
			return;
		}
	}

   /* Test to see if DHCP Option 150 should be parsed */
   if (tftp_yes == 1) {
	   dont_parse_option150 = 1;
    }

   // Handle Monitor Mode, if specified
   monitor_mode_init();
   
#ifdef OS_WINDOWS

   createWindowsInterfaceNames();

#endif

   // Set VLAN interface or Auto Discover via CDP
   if(monitor_yes != 1){

	 int returnVlan = vlan_interface_init();
	 if(returnVlan < 0)
		  return;

	 vlan_id = returnVlan;

	// Ace function to download corporate directory
	int aceRet = directory_download();
	if(aceRet < 0)
		return;

	aceSuccess = 1;
	
   }
   /* check the date */
   //time_check();

   /* load the configuration file */
   load_conf();
  
   /* 
    * get the list of available interfaces 
    * 
    * this function will not return if the -I option was
    * specified on command line. it will instead print the
    * list and exit
    */
   //capture_getifs();
  
	if(textmode){ 
   		/* initialize the user interface */
   		ui_init();
	}
  
   /* initialize libpcap */
   capture_init();

   /* initialize libnet (the function contain all the checks) */
   send_init();
   /* get hardware infos */
   get_hw_info();

   /* Save IP info to myIP variable */
   ip_addr_ntoa(&GBL_IFACE->ip,myIP);

   /* 
    * always disable the kernel ip forwarding (except when reading from file).
    * the forwarding will be done by ettercap.
    */
   if (!GBL_OPTIONS->read && !GBL_OPTIONS->unoffensive && !GBL_OPTIONS->only_mitm)
      disable_ip_forward();

   /* Open VoIP log files */
   openVoipLogFiles();
      

/***** !! NO PRIVS AFTER THIS POINT !! *****/

   /* set the encoding for the UTF-8 visualization */
   set_utf8_encoding((u_char*)GBL_CONF->utf8_encoding);
  
   /* print all the buffered messages */
   if (GBL_UI->type == UI_TEXT)
     USER_MSG("\n");
   
   if(textmode)
     ui_msg_flush(MSG_ALL);

/**** INITIALIZATION PHASE TERMINATED ****/
   
   /* 
    * we are interested only in the mitm attack i
    * if entered, this function will not return...
    */
   if (GBL_OPTIONS->only_mitm)
      only_mitm();
   
   /* create the dispatcher thread */
   ec_thread_new("top_half", "dispatching module", &top_half, NULL);

   /* this thread becomes the UI then displays it */
   ec_thread_register(EC_PTHREAD_SELF, GBL_PROGRAM, "the user interface");


   if(textmode){
   	ui_start();
	/* Builds the host list */
    	GBL_UI->type = UI_TEXT;
   }

   if(guimode)
   	build_hosts_list();

#ifdef OS_WINDOWS
	
   /* Get the default route, because libnet (windows) has a bug forwarding remote destination packets */
   /* If the dest addr belongs to a remote network, forward it to the default gateway */
   /* Assumption: Hopefully there are no other routes in the routing table */

   getDefaultRoute();

#endif

   if(guimode){

		/* start the mitm attack */
   		mitm_start();

   		/* start the sniffing method */
   		EXECUTE(GBL_SNIFF->start);

   		if(garpb_yes == 1){

			GBL_CONF->arp_poison_request = 1;
	   		//arp_poison();
   		}

		ucsniffInstantCtr += 1;
   		mitmSuccess = 1;
		ucsniffStarted = 1;
	}
 }

 void stopUCSniff(){

/******************************************** 
 * reached only when the UI is shutted down 
 ********************************************/

   /* Check if UCSniff is started in GUI mode */
   if(guimode){
	if(!ucsniffStarted){
		INSTANT_USER_MSG("UCSniff not started. \n");
		return;
	}
   }	

   /* flush the exit message */
   if(textmode)
   	ui_msg_flush(MSG_ALL);
   
   /* stop the mitm attack */
   mitm_stop();

   /* terminate the sniffing engine */
   EXECUTE(GBL_SNIFF->cleanup);
  
#ifndef OS_WINDOWS   
   /* Release any currently monitoring stream */
   LIVEMONITOR_LOCK;
   stopCurrentlyMonitoringStream();
   LIVEMONITOR_UNLOCK;
#endif   

   /* Free any SIP/RTP calls */
	closeSipRTPCall();

   /* kill all the running threads but the current */
   ec_thread_kill_all();

   /* Close the send libnet */
   send_close();
  
   /* Close the capture structures */
   capture_close();

   /* Close voip log files */
   closeVoipLogFiles();

   /* Close lib vlc */
#ifdef HAVE_LIB_VLC
   closeLibVlc();
#endif

   /* clean up the UI */
   if(textmode)
   	ui_cleanup();
	
   /* Set ucsniffStarted to zero */
   if(guimode)
	ucsniffStarted = 0;

   return;
}


/* 
 * drop root privs 
 */
static void drop_privs(void)
{
   u_int uid, gid;
   char *var;

#ifdef OS_WINDOWS
   /* do not drop privs under windows */
   return;
#endif
   
   /* are we root ? */
   if (getuid() != 0)
      return;

   /* get the env variable for the UID to drop privs to */
   var = getenv("EC_UID");
   
   /* if the EC_UID variable is not set, default to GBL_CONF->ec_uid (nobody) */
   if (var != NULL)
      uid = atoi(var);
   else
      uid = GBL_CONF->ec_uid;
   
   /* get the env variable for the GID to drop privs to */
   var = getenv("EC_GID");
   
   /* if the EC_UID variable is not set, default to GBL_CONF->ec_gid (nobody) */
   if (var != NULL)
      gid = atoi(var);
   else
      gid = GBL_CONF->ec_gid;
   
   DEBUG_MSG("drop_privs: setuid(%d) setgid(%d)", uid, gid);
   
   /* drop to a good uid/gid ;) */
   if ( setgid(gid) < 0 )
      ERROR_MSG("setgid()");
   
   if ( setuid(uid) < 0 )
      ERROR_MSG("setuid()");

   DEBUG_MSG("privs: UID: %d %d  GID: %d %d", (int)getuid(), (int)geteuid(), (int)getgid(), (int)getegid() );
   USER_MSG("Privileges dropped to UID %d GID %d...\n\n", (int)getuid(), (int)getgid() ); 
}


/*
 * cleanly exit from the program
 */

void clean_exit(int errcode)
{
   DEBUG_MSG("clean_exit: %d", errcode);
  
   INSTANT_USER_MSG("\nTerminating %s...\n", GBL_PROGRAM);

   /* kill all the running threads but the current */
   ec_thread_kill_all();

   /* close the UI */
   ui_cleanup();

   /* call all the ATEXIT functions */
   exit(errcode);
}


static void time_check(void)
{
   /* 
    * a nice easter egg... 
    * just to waste some time of code reviewers... ;) 
    *
    * and no, you can't simply remove this code, you'll break the license...
    *
    * trust me, it's not evil ;) only a boring afternoon, and nothing to do...
    */
   time_t K9=time(NULL);char G5P[1<<6],*o=G5P,*O;uint U4M, _,__=0; char dMG[]= 
   "\n*\n^1U4Mm\x04wW#K\x2e\x0e+X\x7f\f,N'U!I-L5?";struct{char X5T[7];int dMG;
   int U4M;} X5T[]={{"N!WwFr", 0x414c6f52,0},{"S6FfUe", 0x4e614741,0}};sprintf
   (G5P,"%s",ctime(&K9));o+=4;O=strchr(o+4,' ');*O=0; for(U4M=(1<<5)-(1<<2)+1;
   U4M>0;U4M--)dMG[U4M]=dMG[U4M]^dMG[U4M-1];for(U4M=0;U4M<sizeof(X5T)/sizeof(*
   X5T);U4M++){for(_=(1<<2)+1; _>0;_--)X5T[U4M].X5T[_]=X5T[U4M].X5T[_]^X5T[U4M
   ].X5T[_-1];if(!strcmp(X5T[U4M].X5T,o)){char T0Q[]="\n\0O!M4\x14r\x1doO;T0Q"
   "(\bm\x19m\bz\x19x\b(A2\x12s\x1d=X5T=Q&G5Pp\x03l\n~\th\x1a\x7f_dMG\x06hH-@"
   "!H$\x04s\x1av\x1a:X=\x1d|\f|\x0ek\ba\0t\x11u[u[{^-m\fb\x16\x7f\x19v\x04oA"
   "\x2e\\;1;K9\\/\\|9w#f4\x1a\x34\x1a\x1a";for(_=(1<<7)-(1<<3)-(1<<2)+1;_>0;_
   --)T0Q[_]=T0Q[_]^T0Q[_-1];write(1,dMG,1);while(__++<1<<5)printf("%c",(1<<5)
   +(1<<3)+(1<<1));X5T[U4M].dMG=ntohl(X5T[U4M].dMG);printf(dMG,&X5T[U4M].dMG);
   while(--__) printf("%c",(1<<6)-(1<<4)-(1<<3)+(1<<1)); printf(T0Q,&X5T[U4M].
   dMG);getchar();break;}}
}

void initVoipGlobals(void){

   ucsniffInstantCtr = 0;
   ucsniffStarted = 0;
   aceSuccess  = 0;
   mitmSuccess = 0;
   guiShutdown = 0;
   ifaceType = 0;
   numHosts = 0;
   learning_yes = 0;
   garpb_yes = 0;
   garpdb_yes = 0;
   tftp_modify_yes = 0;

   vlan_id = 0;
   tftp_yes = 0;
   vlan_yes = 0;
   cdp_mode_yes = 0;
   cdp_mode_type = 0;
   monitor_yes = 0;
   voipdebug = 0;

   intfd = 0;
   mac_yes = 0;
   ace_yes = 0;
   target_yes = 0;
   guimode = 0;
   textmode = 1;   /* The default mode is the text mode */

}

#ifdef OS_WINDOWS
void createWindowsInterfaceNames(){


	char *start = strstr(IfName_temp,"{");
	char *end = strchr(IfName_temp,'\0');


	int length = end - start;

	int buf = length + 1;
	//CHAR networkDevice[buf];
	networkDevice = (CHAR *) calloc(buf,sizeof(CHAR));
	if(networkDevice == NULL){
		INSTANT_USER_MSG("Error: Allocated memory for networkDevice\n");
		exit(-1);
	}
	memcpy(networkDevice,start,length);

	int v_netdevLength = strlen("\\DEVICE\\") + length + 1;
	v_NetworkDevice = (CHAR *)malloc(v_netdevLength * sizeof(CHAR));
	if(v_NetworkDevice == NULL){
		INSTANT_USER_MSG("Error: Allocated memory for networkDevice\n");
		exit(-1);
	}
	CHAR *tempDevice = v_NetworkDevice;
	memset(tempDevice,'\0',v_netdevLength * sizeof(CHAR));
	memcpy(tempDevice,"\\DEVICE\\",strlen("\\DEVICE\\"));
	tempDevice += strlen("\\DEVICE\\");
	memcpy(tempDevice,start,length);

	int tcpNetLength = v_netdevLength + strlen("TCPIP_");
	CHAR *tcpNetworkDevice = (CHAR *) malloc(tcpNetLength * sizeof(CHAR));
	CHAR *temptcpNetworkDevice = tcpNetworkDevice;
	memset(temptcpNetworkDevice,'\0',tcpNetLength);

	memcpy(temptcpNetworkDevice,"\\DEVICE\\TCPIP_",strlen("\\DEVICE\\TCPIP_"));
	temptcpNetworkDevice += strlen("\\DEVICE\\TCPIP_");
	memcpy(temptcpNetworkDevice,start,length);
	INSTANT_USER_MSG("tcpnetworkDevice is %s \n", tcpNetworkDevice);
	//WCHAR wTCPNetworkDevice[tcpNetLength+1];
	wTCPNetworkDevice = (WCHAR *) calloc(tcpNetLength+1,sizeof(WCHAR));
	if(wTCPNetworkDevice == NULL){
		INSTANT_USER_MSG("Error: Allocated memory for networkDevice\n");
		exit(-1);
	}
	
	MultiByteToWideChar(CP_ACP,0,tcpNetworkDevice,strlen(tcpNetworkDevice)+1,wTCPNetworkDevice,((tcpNetLength+1)*sizeof(WCHAR))/sizeof(WCHAR));
	

}

void getDefaultRoute(){

	PMIB_IPFORWARDTABLE pIpForwardTable;
    DWORD dwSize = 0;
    DWORD dwRetVal = 0;

    char szDestIp[128];
    char szMaskIp[128];
    char szGatewayIp[128];

	char macAddr[MAX_ASCII_ADDR_LEN];

    struct in_addr IpAddr;

    int i;

    pIpForwardTable =
        (MIB_IPFORWARDTABLE *) malloc(sizeof (MIB_IPFORWARDTABLE));
    if (pIpForwardTable == NULL) {
        INSTANT_USER_MSG("Error allocating memory, IPFORWARDTABLE\n");
        return;
    }

    if (GetIpForwardTable(pIpForwardTable, &dwSize, 0) ==
        ERROR_INSUFFICIENT_BUFFER) {
        free(pIpForwardTable);
        pIpForwardTable = (MIB_IPFORWARDTABLE *) malloc(dwSize);
        if (pIpForwardTable == NULL) {
            INSTANT_USER_MSG("Error allocating memory, IPFORWARDTABLE\n");
            return;
        }
    }

    /* Note that the IPv4 addresses returned in 
     * GetIpForwardTable entries are in network byte order 
     */
    if ((dwRetVal = GetIpForwardTable(pIpForwardTable, &dwSize, 0)) == NO_ERROR) {

        for (i = 0; i < (int) pIpForwardTable->dwNumEntries; i++) {
            /* Convert IPv4 addresses to strings */

            IpAddr.S_un.S_addr =
                (u_long) pIpForwardTable->table[i].dwForwardDest;
            strncpy(szDestIp,inet_ntoa(IpAddr),sizeof (szDestIp));

			if(strcmp(szDestIp,"0.0.0.0") != 0){
				continue;
			}

            IpAddr.S_un.S_addr =
                (u_long) pIpForwardTable->table[i].dwForwardMask;
            strncpy(szMaskIp,  inet_ntoa(IpAddr),sizeof (szMaskIp));

			if(strcmp(szMaskIp,"0.0.0.0") != 0){
				continue;
			}

            IpAddr.S_un.S_addr =
                (u_long) pIpForwardTable->table[i].dwForwardNextHop;
            strncpy(szGatewayIp, inet_ntoa(IpAddr), sizeof (szGatewayIp));

			if(!(strcmp(szDestIp,"0.0.0.0")) && (!strcmp(szMaskIp,"0.0.0.0")) && ((IpAddr.S_un.S_addr & GBL_OPTIONS->netmaskInt32) == GBL_OPTIONS->networkInt32)){

				memcpy(GBL_OPTIONS->defaultGatewayIP, szGatewayIp, sizeof(GBL_OPTIONS->defaultGatewayIP) - 1);

				struct hosts_list *hl;
				char temp1[MAX_ASCII_ADDR_LEN];

				LIST_FOREACH(hl, &GBL_HOSTLIST, next) {
      
					memset(temp1,'\0',sizeof(temp1));
					ip_addr_ntoa(&hl->ip,temp1);
					if(!strcmp(GBL_OPTIONS->defaultGatewayIP,temp1)){
						memset(defaultGatewayMac, '\0', sizeof(defaultGatewayMac));
						mac_addr_ntoa(hl->mac,defaultGatewayMac);
						
						//strncpy(GBL_OPTIONS->defaultGatewayMac, macAddr, MAX_ASCII_ADDR_LEN-1);
						break;
					}
				}
				break;
			}
        }
        free(pIpForwardTable);
    } else {
        free(pIpForwardTable);
    }
}

}
#endif



/* EOF */

// vim:ts=3:expandtab

