/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2009 Sipera Systems, Inc. 
    Jason Ostrom <jostrom@viperlab.net>
    Arjun Sambamoorthy <arjun@viperlab.net>

    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ec.h>
#include <ec_capture.h>
#include <ec_send.h>
#include <ec_threads.h>
#include <ec_inject.h>
#include <ec_conntrack.h>
#include <ec_sslwrap.h>
#include <targets.h>
#include <ec_voipglobals.h>
#include <ec_voiplogfiles.h>

/* proto */
void start_unified_sniff(void);
void stop_unified_sniff(void);
void forward_unified_sniff(struct packet_object *po);
void unified_check_forwarded(struct packet_object *po);
void unified_set_forwardable(struct packet_object *po);
int flush;

extern int target_yes;
extern struct singletarget target1; 
extern int tftp_modify_yes;
extern int garpdb_yes;

/*******************************************/

/*
 * creates the threads for capturing 
 */
void start_unified_sniff(void)
{
   DEBUG_MSG("start_unified_sniff");
   
   if (GBL_SNIFF->active == 1) {
      USER_MSG("Unified sniffing already started...\n");
      return;
   }
   
   USER_MSG("\nStarting Unified sniffing...\n\n");
   if (tftp_modify_yes == 1) {
        USER_MSG("\nUCSniff running in TFTP Modify mode with GARP Disablement Bypass, against targeted user\n\n");
   } else if (garpdb_yes == 1) {
        USER_MSG("\nUCSniff running with GARP Disablement Bypass enabled\n\n");
   }
   USER_MSG("Warning:  Please ensure that you hit 'q' when you are finished with this program.\n");
   USER_MSG("Warning:  'q' re-ARPs the victims.  Failure to do so before program exit will result in a DoS.\n\n");

   if(target_yes == 1) {
      USER_MSG("Listening for new calls to or from target %s (Extension %s, IP %s)\n",target1.dirname,target1.extension,target1.ip);

	   if(voipdebug){
       		fprintf(sccpfile,"Listening for new calls to or from target %s (Extension %s, IP %s)\n",target1.dirname,target1.extension,target1.ip);
		   flush = fflush(sccpfile);
	   }
   }
   
   /* create the timeouter thread */
   if (!GBL_OPTIONS->read) { 
      pthread_t pid;
      
      pid = ec_thread_getpid("timer");
      if (pthread_equal(pid, EC_PTHREAD_NULL)) {
         ec_thread_new("timer", "conntrack timeouter", &conntrack_timeouter, NULL);
      }
   }

   /* create the thread for packet capture */
   ec_thread_new("capture", "pcap handler and packet decoder", &capture, GBL_OPTIONS->iface);

   /* start ssl_wrapper thread */
   if (!GBL_OPTIONS->read && !GBL_OPTIONS->unoffensive && !GBL_OPTIONS->only_mitm)
      ec_thread_new("sslwrap", "wrapper for ssl connections", &sslw_start, NULL);

   GBL_SNIFF->active = 1;
}


/*
 * kill the capturing threads, but leave untouched the others
 */
void stop_unified_sniff(void)
{
   pthread_t pid;
   
   DEBUG_MSG("stop_unified_sniff");
   
   if (GBL_SNIFF->active == 0) {
      USER_MSG("Unified sniffing is not running...\n");
      return;
   }
  
   /* get the pid and kill it */
   pid = ec_thread_getpid("capture");
   if (!pthread_equal(pid, EC_PTHREAD_NULL))
      ec_thread_destroy(pid);
   
   pid = ec_thread_getpid("sslwrap");
   if (!pthread_equal(pid, EC_PTHREAD_NULL))
      ec_thread_destroy(pid);

   USER_MSG("Unified sniffing was stopped.\n");

   GBL_SNIFF->active = 0;
}


void forward_unified_sniff(struct packet_object *po)
{
	
   /* if it was not initialized, no packet are forwardable */
   if (!GBL_LNET->lnet)
      return;

   /* if the interface was not configured, no packet are forwardable */
   if (!GBL_IFACE->configured)
      return;
   
   /* if unoffensive is set, don't forward any packet */
   if (GBL_OPTIONS->unoffensive || GBL_OPTIONS->read)
      return;

   /* 
    * forward the packet to Layer 3, the kernel
    * will route them to the correct destination (host or gw)
    */

   /* don't forward dropped packets */
   if ((po->flags & PO_DROPPED) == 0){
      send_to_L3(po);
	   //send_to_NewLayer(po);
   }

    /* 
     * if the packet was modified and it exceeded the mtu,
     * we have to inject the exceeded data
     */
    if (po->DATA.inject)
    {
       inject_buffer(po);
    } 
}

/*
 * check if the packet has been forwarded by us
 * the source mac address is our, but the ip address is different
 */
void unified_check_forwarded(struct packet_object *po) 
{
   /* the interface was not configured, the packets are not forwardable */
   if (!GBL_IFACE->configured)
      return;
   
   /* 
    * dont sniff forwarded packets (equal mac, different ip) 
    * but only if we are on live connections
    */
   if ( GBL_CONF->skip_forwarded && !GBL_OPTIONS->read &&
        !memcmp(GBL_IFACE->mac, po->L2.src, MEDIA_ADDR_LEN) &&
        ip_addr_cmp(&GBL_IFACE->ip, &po->L3.src) ) {
      po->flags |= PO_FORWARDED;
   }
}

/* 
 * if the dest mac address of the packet is
 * the same of GBL_IFACE->mac but the dest ip is
 * not the same as GBL_IFACE->ip, the packet is not
 * for us and we can do mitm on it before forwarding.
 */
void unified_set_forwardable(struct packet_object *po)
{
   /* 
    * if the mac is our, but the ip is not...
    * it has to be forwarded
    */
   if (!memcmp(GBL_IFACE->mac, po->L2.dst, MEDIA_ADDR_LEN) &&
       memcmp(GBL_IFACE->mac, po->L2.src, MEDIA_ADDR_LEN) &&
       ip_addr_cmp(&GBL_IFACE->ip, &po->L3.dst) ) {
      po->flags |= PO_FORWARDABLE;
   }
   
}


/* EOF */

// vim:ts=3:expandtab

